@@ -18,14 +18,14 @@ Revision History:
 #ifndef API_AST_MAP_H_
 #define API_AST_MAP_H_
 
-#include"api_util.h"
-#include"obj_hashtable.h"
+#include "api/api_util.h"
+#include "util/obj_hashtable.h"
 
 struct Z3_ast_map_ref : public api::object {
     ast_manager &            m;
     obj_map<ast, ast*> m_map;
-    Z3_ast_map_ref(ast_manager & _m):m(_m) {}
-    virtual ~Z3_ast_map_ref();
+    Z3_ast_map_ref(api::context& c, ast_manager & _m): api::object(c), m(_m) {}
+    ~Z3_ast_map_ref() override;
 };
 
 inline Z3_ast_map_ref * to_ast_map(Z3_ast_map v) { return reinterpret_cast<Z3_ast_map_ref *>(v); }
