@@ -18,12 +18,16 @@ Revision History:
 #ifndef API_AST_VECTOR_H_
 #define API_AST_VECTOR_H_
 
-#include"api_util.h"
+#include "api/api_util.h"
+
+namespace api {
+    class context;
+};
 
 struct Z3_ast_vector_ref : public api::object {
     ast_ref_vector  m_ast_vector;
-    Z3_ast_vector_ref(ast_manager & m):m_ast_vector(m) {}
-    virtual ~Z3_ast_vector_ref() {}
+    Z3_ast_vector_ref(api::context& c, ast_manager & m): api::object(c), m_ast_vector(m) {}
+    ~Z3_ast_vector_ref() override {}
 };
 
 inline Z3_ast_vector_ref * to_ast_vector(Z3_ast_vector v) { return reinterpret_cast<Z3_ast_vector_ref *>(v); }
