@@ -160,69 +160,40 @@ extern "C" {
         LOG_Z3_mk_list_sort(c, name, elem_sort, nil_decl, is_nil_decl, cons_decl, is_cons_decl, head_decl, tail_decl);
         RESET_ERROR_CODE();
         ast_manager& m = mk_c(c)->m();
+        func_decl_ref nil(m), is_nil(m), cons(m), is_cons(m), head(m), tail(m);        
         datatype_util& dt_util = mk_c(c)->dtutil();
         mk_c(c)->reset_last_result();
-        datatype_util data_util(m);
-        accessor_decl* head_tail[2] = {
-            mk_accessor_decl(m, symbol("head"), type_ref(to_sort(elem_sort))),
-            mk_accessor_decl(m, symbol("tail"), type_ref(0))
-        };
-        constructor_decl* constrs[2] = {
-            mk_constructor_decl(symbol("nil"), symbol("is_nil"), 0, nullptr),
-            // Leo: SMT 2.0 document uses 'insert' instead of cons
-            mk_constructor_decl(symbol("cons"), symbol("is_cons"), 2, head_tail)
-        };
-
-        sort_ref_vector sorts(m);
-        {
-            datatype_decl * decl = mk_datatype_decl(dt_util, to_symbol(name), 0, nullptr, 2, constrs);
-            bool is_ok = mk_c(c)->get_dt_plugin()->mk_datatypes(1, &decl, 0, nullptr, sorts);
-            del_datatype_decl(decl);
-
-            if (!is_ok) {
-                SET_ERROR_CODE(Z3_INVALID_ARG, nullptr);
-                RETURN_Z3(nullptr);
-            }
+        sort_ref s = dt_util.mk_list_datatype(to_sort(elem_sort), to_symbol(name), cons, is_cons, head, tail, nil, is_nil);
+        
+        if (!s) {
+            SET_ERROR_CODE(Z3_INVALID_ARG, nullptr);
+            RETURN_Z3(nullptr);
         }
-        sort * s = sorts.get(0);
 
         mk_c(c)->save_multiple_ast_trail(s);
-        ptr_vector<func_decl> const& cnstrs = *data_util.get_datatype_constructors(s);
-        SASSERT(cnstrs.size() == 2);
-        func_decl* f;
         if (nil_decl) {
-            f = cnstrs[0];
-            mk_c(c)->save_multiple_ast_trail(f);
-            *nil_decl = of_func_decl(f);
+            mk_c(c)->save_multiple_ast_trail(nil);
+            *nil_decl = of_func_decl(nil);
         }
         if (is_nil_decl) {
-            f = data_util.get_constructor_is(cnstrs[0]);
-            mk_c(c)->save_multiple_ast_trail(f);
-            *is_nil_decl = of_func_decl(f);
+            mk_c(c)->save_multiple_ast_trail(is_nil);
+            *is_nil_decl = of_func_decl(is_nil);
         }
         if (cons_decl) {
-            f = cnstrs[1];
-            mk_c(c)->save_multiple_ast_trail(f);
-            *cons_decl = of_func_decl(f);
+            mk_c(c)->save_multiple_ast_trail(cons);
+            *cons_decl = of_func_decl(cons);
         }
         if (is_cons_decl) {
-            f = data_util.get_constructor_is(cnstrs[1]);
-            mk_c(c)->save_multiple_ast_trail(f);
-            *is_cons_decl = of_func_decl(f);
+            mk_c(c)->save_multiple_ast_trail(is_cons);
+            *is_cons_decl = of_func_decl(is_cons);
         }
         if (head_decl) {
-            ptr_vector<func_decl> const& acc = *data_util.get_constructor_accessors(cnstrs[1]);
-            SASSERT(acc.size() == 2);
-            f = (acc)[0];
-            mk_c(c)->save_multiple_ast_trail(f);
-            *head_decl = of_func_decl(f);
+            mk_c(c)->save_multiple_ast_trail(head);
+            *head_decl = of_func_decl(head);
         }
         if (tail_decl) {
-            ptr_vector<func_decl> const& acc = *data_util.get_constructor_accessors(cnstrs[1]);
-            SASSERT(acc.size() == 2);
-            f = (acc)[1];
-            mk_c(c)->save_multiple_ast_trail(f);
-            *tail_decl = of_func_decl(f);
+            mk_c(c)->save_multiple_ast_trail(tail);
+            *tail_decl = of_func_decl(tail);
         }
         RETURN_Z3_mk_list_sort(of_sort(s));
         Z3_CATCH_RETURN(nullptr);
@@ -609,7 +580,7 @@ extern "C" {
         expr* args[2] = { _t, _v };
         sort* domain[2] = { m.get_sort(_t), m.get_sort(_v) };
         parameter param(_f);
-        func_decl * d = m.mk_func_decl(mk_c(c)->get_array_fid(), OP_DT_UPDATE_FIELD, 1, &param, 2, domain);
+        func_decl * d = m.mk_func_decl(mk_c(c)->get_dt_fid(), OP_DT_UPDATE_FIELD, 1, &param, 2, domain);
         app* r = m.mk_app(d, 2, args);
         mk_c(c)->save_ast_trail(r);
         check_sorts(c, r);
