@@ -15,7 +15,6 @@ Author:
 Revision History:
 
 --*/
-#include<iostream>
 #include "api/z3.h"
 #include "api/api_log_macros.h"
 #include "api/api_context.h"
@@ -163,14 +162,16 @@ extern "C" {
         model * _m = to_model_ref(m);
         params_ref p;
         ast_manager& mgr = mk_c(c)->m();
-        _m->set_solver(alloc(api::seq_expr_solver, mgr, p));
+        if (!_m->has_solver()) {
+            _m->set_solver(alloc(api::seq_expr_solver, mgr, p));
+        }
         expr_ref result(mgr);
         model::scoped_model_completion _scm(*_m, model_completion);
         result = (*_m)(to_expr(t));
         mk_c(c)->save_ast_trail(result.get());
         *v = of_ast(result.get());
         RETURN_Z3_model_eval true;
-        Z3_CATCH_RETURN(0);
+        Z3_CATCH_RETURN(false);
     }
 
     unsigned Z3_API Z3_model_get_num_sorts(Z3_context c, Z3_model m) {
@@ -479,7 +480,7 @@ extern "C" {
             model_v2_pp(buffer, *(to_model_ref(m)), p.partial());
             result = buffer.str();
         }
-        return mk_c(c)->mk_external_string(result);
+        return mk_c(c)->mk_external_string(std::move(result));
         Z3_CATCH_RETURN(nullptr);
     }
 
