@@ -62,7 +62,6 @@ extern "C" {
         }
         sort * _ty = to_sort(ty);
         bool is_float = mk_c(c)->fpautil().is_float(_ty);
-        std::string fixed_num;
         char const* m = n;
         while (*m) {
             if (!(('0' <= *m && *m <= '9') ||
@@ -239,13 +238,21 @@ extern "C" {
         expr* e = to_expr(a);
         fpa_util & fu = mk_c(c)->fpautil();
         scoped_mpf tmp(fu.fm());
-        if (!mk_c(c)->fpautil().is_numeral(e, tmp) ||
-            tmp.get().get_ebits() > 11 ||
-            tmp.get().get_sbits() > 53) {
-            SET_ERROR_CODE(Z3_INVALID_ARG, nullptr);
-            return NAN;
+        if (mk_c(c)->fpautil().is_numeral(e, tmp)) {
+            if (tmp.get().get_ebits() > 11 ||
+                tmp.get().get_sbits() > 53) {
+                SET_ERROR_CODE(Z3_INVALID_ARG, nullptr);
+                return NAN;
+            }
+            return fu.fm().to_double(tmp);
         }
-        return fu.fm().to_double(tmp);
+        rational r;
+        arith_util & u = mk_c(c)->autil();
+        if (u.is_numeral(e, r)) {
+            return r.get_double();
+        }
+        SET_ERROR_CODE(Z3_INVALID_ARG, nullptr);
+        return 0.0;
     }
 
     Z3_string Z3_API Z3_get_numeral_decimal_string(Z3_context c, Z3_ast a, unsigned precision) {
