@@ -18,11 +18,11 @@ Revision History:
 
 --*/
 #include<iostream>
-#include"z3.h"
-#include"api_log_macros.h"
-#include"api_context.h"
-#include"api_util.h"
-#include"params.h"
+#include "api/z3.h"
+#include "api/api_log_macros.h"
+#include "api/api_context.h"
+#include "api/api_util.h"
+#include "util/params.h"
 
 extern "C" {
     
@@ -30,11 +30,11 @@ extern "C" {
         Z3_TRY;
         LOG_Z3_mk_params(c);
         RESET_ERROR_CODE();
-        Z3_params_ref * p = alloc(Z3_params_ref);
+        Z3_params_ref * p = alloc(Z3_params_ref, *mk_c(c));
         mk_c(c)->save_object(p);
         Z3_params r = of_params(p);
         RETURN_Z3(r);
-        Z3_CATCH_RETURN(0);
+        Z3_CATCH_RETURN(nullptr);
     }
     
     /**
@@ -62,11 +62,11 @@ extern "C" {
     /**
        \brief Add a Boolean parameter \c k with value \c v to the parameter set \c p.
     */
-    void Z3_API Z3_params_set_bool(Z3_context c, Z3_params p, Z3_symbol k, Z3_bool v) {
+    void Z3_API Z3_params_set_bool(Z3_context c, Z3_params p, Z3_symbol k, bool v) {
         Z3_TRY;
         LOG_Z3_params_set_bool(c, p, k, v);
         RESET_ERROR_CODE();
-        to_params(p)->m_params.set_bool(norm_param_name(to_symbol(k)).c_str(), v != 0);
+        to_params(p)->m_params.set_bool(norm_param_name(to_symbol(k)).c_str(), v);
         Z3_CATCH;
     }
 
@@ -171,12 +171,25 @@ extern "C" {
         LOG_Z3_param_descrs_get_name(c, p, i);
         RESET_ERROR_CODE();
         if (i >= to_param_descrs_ptr(p)->size()) {
-            SET_ERROR_CODE(Z3_IOB);
-            RETURN_Z3(0);
+            SET_ERROR_CODE(Z3_IOB, nullptr);
+            RETURN_Z3(nullptr);
         }
         Z3_symbol result = of_symbol(to_param_descrs_ptr(p)->get_param_name(i));
         return result;
-        Z3_CATCH_RETURN(0);
+        Z3_CATCH_RETURN(nullptr);
+    }
+
+    Z3_string Z3_API Z3_param_descrs_get_documentation(Z3_context c, Z3_param_descrs p, Z3_symbol s) {
+        Z3_TRY;
+        LOG_Z3_param_descrs_get_documentation(c, p, s);
+        RESET_ERROR_CODE();
+        char const* result = to_param_descrs_ptr(p)->get_descr(to_symbol(s));
+        if (result == nullptr) {
+            SET_ERROR_CODE(Z3_IOB, nullptr);
+            RETURN_Z3(nullptr);
+        }
+        return mk_c(c)->mk_external_string(result);
+        Z3_CATCH_RETURN(nullptr);
     }
 
     Z3_string Z3_API Z3_param_descrs_to_string(Z3_context c, Z3_param_descrs p) {
