@@ -19,19 +19,18 @@ Notes:
 #ifndef API_POLYNOMIAL_H_
 #define API_POLYNOMIAL_H_
 
-#include"polynomial.h"
+#include "math/polynomial/polynomial.h"
 
 namespace api {
     
-    class pmanager {
+    class pmanager final {
         unsynch_mpz_manager m_nm;
         polynomial::manager m_pm;
         // TODO: add support for caching expressions -> polynomial and back
     public:
-        pmanager();
-        virtual ~pmanager();
+        pmanager(reslimit& lim) : m_pm(lim, m_nm) {}
+        ~pmanager() {}
         polynomial::manager & pm() { return m_pm; }
-        void set_cancel(bool f);
     };
 
 };
