@@ -15,33 +15,32 @@ Author:
 Revision History:
 
 --*/
-#include<iostream>
-#include"z3.h"
-#include"api_log_macros.h"
-#include"api_context.h"
-#include"api_util.h"
-#include"pattern_validation.h"
-#include"expr_abstract.h"
+#include "api/z3.h"
+#include "api/api_log_macros.h"
+#include "api/api_context.h"
+#include "api/api_util.h"
+#include "parsers/util/pattern_validation.h"
+#include "ast/expr_abstract.h"
 
 extern "C" {
 
     Z3_ast Z3_API Z3_mk_quantifier(
-        Z3_context c, 
-        Z3_bool is_forall, 
-        unsigned weight, 
-        unsigned num_patterns, Z3_pattern const patterns[], 
-        unsigned num_decls, Z3_sort const sorts[], 
-        Z3_symbol const decl_names[], 
-        Z3_ast body) 
+        Z3_context c,
+        bool is_forall,
+        unsigned weight,
+        unsigned num_patterns, Z3_pattern const patterns[],
+        unsigned num_decls, Z3_sort const sorts[],
+        Z3_symbol const decl_names[],
+        Z3_ast body)
     {
         return Z3_mk_quantifier_ex(
-            c, 
-            is_forall, 
+            c,
+            is_forall,
             weight,
-            0,
-            0,
+            nullptr,
+            nullptr,
             num_patterns, patterns,
-            0, 0,
+            0, nullptr,
             num_decls, sorts,
             decl_names,
             body
@@ -50,34 +49,39 @@ extern "C" {
     }
 
     Z3_ast mk_quantifier_ex_core(
-        Z3_context c, 
-        Z3_bool is_forall, 
-        unsigned weight, 
+        Z3_context c,
+        bool is_forall,
+        unsigned weight,
         Z3_symbol quantifier_id,
         Z3_symbol skolem_id,
-        unsigned num_patterns, Z3_pattern const patterns[], 
-        unsigned num_no_patterns, Z3_ast const no_patterns[], 
-        unsigned num_decls, Z3_sort const sorts[], 
-        Z3_symbol const decl_names[], 
+        unsigned num_patterns, Z3_pattern const patterns[],
+        unsigned num_no_patterns, Z3_ast const no_patterns[],
+        unsigned num_decls, Z3_sort const sorts[],
+        Z3_symbol const decl_names[],
         Z3_ast body) {
         Z3_TRY;
         RESET_ERROR_CODE();
         if (!mk_c(c)->m().is_bool(to_expr(body))) {
-            SET_ERROR_CODE(Z3_SORT_ERROR);
+            SET_ERROR_CODE(Z3_SORT_ERROR, nullptr);
+            return nullptr;
         }
         if (num_patterns > 0 && num_no_patterns > 0) {
-            SET_ERROR_CODE(Z3_INVALID_USAGE);
+            SET_ERROR_CODE(Z3_INVALID_USAGE, nullptr);
+            return nullptr;
         }
         expr * const* ps = reinterpret_cast<expr * const*>(patterns);
         expr * const* no_ps = reinterpret_cast<expr * const*>(no_patterns);
-        pattern_validator v(mk_c(c)->m());
-        for (unsigned i = 0; i < num_patterns; i++) {
-            if (!v(num_decls, ps[i])) {
-                SET_ERROR_CODE(Z3_INVALID_PATTERN);
-                return 0;
+        symbol qid = to_symbol(quantifier_id);
+        bool is_rec = mk_c(c)->m().rec_fun_qid() == qid;
+        if (!is_rec) {
+            pattern_validator v(mk_c(c)->m());
+            for (unsigned i = 0; i < num_patterns; i++) {
+                if (!v(num_decls, ps[i], 0, 0)) {
+                    SET_ERROR_CODE(Z3_INVALID_PATTERN, nullptr);
+                    return nullptr;
+                }
             }
         }
-
         sort* const* ts = reinterpret_cast<sort * const*>(sorts);
         svector<symbol> names;
         for (unsigned i = 0; i < num_decls; ++i) {
@@ -86,10 +90,10 @@ extern "C" {
         expr_ref result(mk_c(c)->m());
         if (num_decls > 0) {
             result = mk_c(c)->m().mk_quantifier(
-                (0 != is_forall), 
-                names.size(), ts, names.c_ptr(), to_expr(body),            
-                weight, 
-                to_symbol(quantifier_id),
+                is_forall ? forall_k : exists_k,
+                names.size(), ts, names.c_ptr(), to_expr(body),
+                weight,
+                qid,
                 to_symbol(skolem_id),
                 num_patterns, ps,
                 num_no_patterns, no_ps
@@ -100,49 +104,104 @@ extern "C" {
         }
         mk_c(c)->save_ast_trail(result.get());
         return of_ast(result.get());
-        Z3_CATCH_RETURN(0);
+        Z3_CATCH_RETURN(nullptr);
     }
 
     Z3_ast Z3_API Z3_mk_quantifier_ex(
-        Z3_context c, 
-        Z3_bool is_forall, 
-        unsigned weight, 
+        Z3_context c,
+        bool is_forall,
+        unsigned weight,
         Z3_symbol quantifier_id,
         Z3_symbol skolem_id,
-        unsigned num_patterns, Z3_pattern const patterns[], 
-        unsigned num_no_patterns, Z3_ast const no_patterns[], 
-        unsigned num_decls, Z3_sort const sorts[], 
-        Z3_symbol const decl_names[], 
+        unsigned num_patterns, Z3_pattern const patterns[],
+        unsigned num_no_patterns, Z3_ast const no_patterns[],
+        unsigned num_decls, Z3_sort const sorts[],
+        Z3_symbol const decl_names[],
         Z3_ast body)
     {
-        LOG_Z3_mk_quantifier_ex(c, is_forall, weight, quantifier_id, skolem_id, num_patterns, patterns, 
+        LOG_Z3_mk_quantifier_ex(c, is_forall, weight, quantifier_id, skolem_id, num_patterns, patterns,
                                 num_no_patterns, no_patterns, num_decls, sorts, decl_names, body);
-        Z3_ast r = mk_quantifier_ex_core(c, is_forall, weight, quantifier_id, skolem_id, num_patterns, patterns, 
+        Z3_ast r = mk_quantifier_ex_core(c, is_forall, weight, quantifier_id, skolem_id, num_patterns, patterns,
                                          num_no_patterns, no_patterns, num_decls, sorts, decl_names, body);
         RETURN_Z3(r);
     }
-    
-    Z3_ast Z3_API Z3_mk_forall(Z3_context c, 
-                               unsigned weight, 
-                               unsigned num_patterns, Z3_pattern const patterns[], 
-                               unsigned num_decls, Z3_sort const types[], 
-                               Z3_symbol const decl_names[], 
+
+    Z3_ast Z3_API Z3_mk_forall(Z3_context c,
+                               unsigned weight,
+                               unsigned num_patterns, Z3_pattern const patterns[],
+                               unsigned num_decls, Z3_sort const types[],
+                               Z3_symbol const decl_names[],
                                Z3_ast body) {
-        return Z3_mk_quantifier(c, 1, weight, num_patterns, patterns, num_decls, types, decl_names, body);
+        return Z3_mk_quantifier(c, true, weight, num_patterns, patterns, num_decls, types, decl_names, body);
     }
-    
-    Z3_ast Z3_API Z3_mk_exists(Z3_context c, 
-                               unsigned weight, 
-                               unsigned num_patterns, Z3_pattern const patterns[], 
-                               unsigned num_decls, Z3_sort const types[], 
-                               Z3_symbol const decl_names[], 
+
+    Z3_ast Z3_API Z3_mk_exists(Z3_context c,
+                               unsigned weight,
+                               unsigned num_patterns, Z3_pattern const patterns[],
+                               unsigned num_decls, Z3_sort const types[],
+                               Z3_symbol const decl_names[],
                                Z3_ast body) {
-        return Z3_mk_quantifier(c, 0, weight, num_patterns, patterns, num_decls, types, decl_names, body);
+        return Z3_mk_quantifier(c, false, weight, num_patterns, patterns, num_decls, types, decl_names, body);
+    }
+
+    Z3_ast Z3_API Z3_mk_lambda(Z3_context c,
+                               unsigned num_decls, Z3_sort const types[],
+                               Z3_symbol const decl_names[],
+                               Z3_ast body) {
+
+        Z3_TRY;
+        LOG_Z3_mk_lambda(c, num_decls, types, decl_names, body);
+        RESET_ERROR_CODE();
+        expr_ref result(mk_c(c)->m());
+        if (num_decls == 0) {
+            SET_ERROR_CODE(Z3_INVALID_USAGE, nullptr);
+            RETURN_Z3(nullptr);
+        }
+
+        sort* const* ts = reinterpret_cast<sort * const*>(types);
+        svector<symbol> names;
+        for (unsigned i = 0; i < num_decls; ++i) {
+            names.push_back(to_symbol(decl_names[i]));
+        }
+        result = mk_c(c)->m().mk_lambda(names.size(), ts, names.c_ptr(), to_expr(body));
+        mk_c(c)->save_ast_trail(result.get());
+        return of_ast(result.get());
+        Z3_CATCH_RETURN(nullptr);
+    }
+
+    Z3_ast Z3_API Z3_mk_lambda_const(Z3_context c, 
+                                     unsigned num_decls, Z3_app const vars[],
+                                     Z3_ast body) {
+
+        Z3_TRY;
+        LOG_Z3_mk_lambda_const(c, num_decls, vars, body);
+        RESET_ERROR_CODE();
+        if (num_decls == 0) {
+            SET_ERROR_CODE(Z3_INVALID_USAGE, nullptr);
+            RETURN_Z3(nullptr);
+        }
+
+        svector<symbol>  _names;
+        ptr_vector<sort> _vars;
+        ptr_vector<expr> _args;
+        for (unsigned i = 0; i < num_decls; ++i) {
+            app* a = to_app(vars[i]);
+            _names.push_back(to_app(a)->get_decl()->get_name());
+            _args.push_back(a);
+            _vars.push_back(mk_c(c)->m().get_sort(a));
+        }
+        expr_ref result(mk_c(c)->m());        
+        expr_abstract(mk_c(c)->m(), 0, num_decls, _args.c_ptr(), to_expr(body), result);
+        
+        result = mk_c(c)->m().mk_lambda(_vars.size(), _vars.c_ptr(), _names.c_ptr(), result);
+        mk_c(c)->save_ast_trail(result.get());
+        return of_ast(result.get());
+        Z3_CATCH_RETURN(nullptr);
     }
 
 
-    Z3_ast Z3_API Z3_mk_quantifier_const_ex(Z3_context c, 
-                                            Z3_bool is_forall,
+    Z3_ast Z3_API Z3_mk_quantifier_const_ex(Z3_context c,
+                                            bool is_forall,
                                             unsigned weight,
                                             Z3_symbol quantifier_id,
                                             Z3_symbol skolem_id,
@@ -161,21 +220,26 @@ extern "C" {
         svector<Z3_sort> types;
         ptr_vector<expr> bound_asts;
         if (num_patterns > 0 && num_no_patterns > 0) {
-            SET_ERROR_CODE(Z3_INVALID_USAGE);
+            SET_ERROR_CODE(Z3_INVALID_USAGE, nullptr);
+            RETURN_Z3(nullptr);
+        }
+        if (num_bound == 0) {
+            SET_ERROR_CODE(Z3_INVALID_USAGE, "number of bound variables is 0");
+            RETURN_Z3(nullptr);
         }
         for (unsigned i = 0; i < num_bound; ++i) {
             app* a = to_app(bound[i]);
             if (a->get_kind() != AST_APP) {
-                SET_ERROR_CODE(Z3_INVALID_ARG);
-                RETURN_Z3(0);
+                SET_ERROR_CODE(Z3_INVALID_ARG, nullptr);
+                RETURN_Z3(nullptr);
             }
             symbol s(to_app(a)->get_decl()->get_name());
             names.push_back(of_symbol(s));
             types.push_back(of_sort(mk_c(c)->m().get_sort(a)));
             bound_asts.push_back(a);
             if (a->get_family_id() != null_family_id || a->get_num_args() != 0) {
-                SET_ERROR_CODE(Z3_INVALID_ARG);
-                RETURN_Z3(0);
+                SET_ERROR_CODE(Z3_INVALID_ARG, nullptr);
+                RETURN_Z3(nullptr);
             }
         }
         // Abstract patterns
@@ -186,7 +250,7 @@ extern "C" {
             app* pat = to_pattern(patterns[i]);
             SASSERT(mk_c(c)->m().is_pattern(pat));
             expr_abstract(mk_c(c)->m(), 0, num_bound, bound_asts.c_ptr(), pat, result);
-            SASSERT(result.get()->get_kind() == AST_APP);        
+            SASSERT(result.get()->get_kind() == AST_APP);
             pinned.push_back(result.get());
             SASSERT(mk_c(c)->m().is_pattern(result.get()));
             _patterns.push_back(of_pattern(result.get()));
@@ -195,45 +259,45 @@ extern "C" {
         for (unsigned i = 0; i < num_no_patterns; ++i) {
             expr_ref result(mk_c(c)->m());
             if (!is_app(to_expr(no_patterns[i]))) {
-                SET_ERROR_CODE(Z3_INVALID_ARG);
-                RETURN_Z3(0);
+                SET_ERROR_CODE(Z3_INVALID_ARG, nullptr);
+                RETURN_Z3(nullptr);
             }
             app* pat = to_app(to_expr(no_patterns[i]));
             expr_abstract(mk_c(c)->m(), 0, num_bound, bound_asts.c_ptr(), pat, result);
-            SASSERT(result.get()->get_kind() == AST_APP);        
+            SASSERT(result.get()->get_kind() == AST_APP);
             pinned.push_back(result.get());
             _no_patterns.push_back(of_ast(result.get()));
         }
         expr_ref abs_body(mk_c(c)->m());
         expr_abstract(mk_c(c)->m(), 0, num_bound, bound_asts.c_ptr(), to_expr(body), abs_body);
 
-        Z3_ast result = mk_quantifier_ex_core(c, is_forall, weight, 
+        Z3_ast result = mk_quantifier_ex_core(c, is_forall, weight,
                                               quantifier_id,
                                               skolem_id,
-                                              num_patterns, _patterns.c_ptr(), 
+                                              num_patterns, _patterns.c_ptr(),
                                               num_no_patterns, _no_patterns.c_ptr(),
-                                              names.size(), types.c_ptr(), names.c_ptr(), 
+                                              names.size(), types.c_ptr(), names.c_ptr(),
                                               of_ast(abs_body.get()));
         RETURN_Z3(result);
-        Z3_CATCH_RETURN(0);
+        Z3_CATCH_RETURN(nullptr);
     }
 
-    Z3_ast Z3_API Z3_mk_quantifier_const(Z3_context c, 
-                                         Z3_bool is_forall,
+    Z3_ast Z3_API Z3_mk_quantifier_const(Z3_context c,
+                                         bool is_forall,
                                          unsigned weight,
                                          unsigned num_bound,
                                          Z3_app const bound[],
                                          unsigned num_patterns,
                                          Z3_pattern const patterns[],
                                          Z3_ast body) {
-        return Z3_mk_quantifier_const_ex(c, is_forall, weight, 0, 0, 
-                                         num_bound, bound, 
+        return Z3_mk_quantifier_const_ex(c, is_forall, weight, nullptr, nullptr,
+                                         num_bound, bound,
                                          num_patterns, patterns,
-                                         0, 0,
+                                         0, nullptr,
                                          body);
     }
 
-    Z3_ast Z3_API Z3_mk_forall_const(Z3_context c, 
+    Z3_ast Z3_API Z3_mk_forall_const(Z3_context c,
                                      unsigned weight,
                                      unsigned num_bound,
                                      Z3_app const bound[],
@@ -243,7 +307,7 @@ extern "C" {
         return Z3_mk_quantifier_const(c, true, weight, num_bound, bound, num_patterns, patterns, body);
     }
 
-    Z3_ast Z3_API Z3_mk_exists_const(Z3_context c, 
+    Z3_ast Z3_API Z3_mk_exists_const(Z3_context c,
                                      unsigned weight,
                                      unsigned num_bound,
                                      Z3_app const bound[],
@@ -252,23 +316,23 @@ extern "C" {
                                      Z3_ast body) {
         return Z3_mk_quantifier_const(c, false, weight, num_bound, bound, num_patterns, patterns, body);
     }
-    
+
     Z3_pattern Z3_API Z3_mk_pattern(Z3_context c, unsigned num_patterns, Z3_ast const terms[]) {
         Z3_TRY;
         LOG_Z3_mk_pattern(c, num_patterns, terms);
         RESET_ERROR_CODE();
         for (unsigned i = 0; i < num_patterns; ++i) {
             if (!is_app(to_expr(terms[i]))) {
-                SET_ERROR_CODE(Z3_INVALID_ARG);
-                RETURN_Z3(0);
+                SET_ERROR_CODE(Z3_INVALID_ARG, nullptr);
+                RETURN_Z3(nullptr);
             }
         }
-        app* a = mk_c(c)->m().mk_pattern(num_patterns, reinterpret_cast<app*const*>(to_exprs(terms)));
+        app* a = mk_c(c)->m().mk_pattern(num_patterns, reinterpret_cast<app*const*>(to_exprs(num_patterns, terms)));
         mk_c(c)->save_ast_trail(a);
         RETURN_Z3(of_pattern(a));
-        Z3_CATCH_RETURN(0);
+        Z3_CATCH_RETURN(nullptr);
     }
-    
+
     Z3_ast Z3_API Z3_mk_bound(Z3_context c, unsigned index, Z3_sort ty) {
         Z3_TRY;
         LOG_Z3_mk_bound(c, index, ty);
@@ -276,24 +340,34 @@ extern "C" {
         ast* a = mk_c(c)->m().mk_var(index, to_sort(ty));
         mk_c(c)->save_ast_trail(a);
         RETURN_Z3(of_ast(a));
-        Z3_CATCH_RETURN(0);
+        Z3_CATCH_RETURN(nullptr);
     }
 
-    Z3_bool Z3_API Z3_is_quantifier_forall(Z3_context c, Z3_ast a) {
+    bool Z3_API Z3_is_quantifier_forall(Z3_context c, Z3_ast a) {
         Z3_TRY;
         LOG_Z3_is_quantifier_forall(c, a);
         RESET_ERROR_CODE();
-        ast * _a = to_ast(a);
-        if (_a->get_kind() == AST_QUANTIFIER) {
-            return to_quantifier(_a)->is_forall();
-        }
-        else {
-            SET_ERROR_CODE(Z3_SORT_ERROR);
-            return Z3_FALSE;
-        }
-        Z3_CATCH_RETURN(Z3_FALSE);
+        return ::is_forall(to_ast(a));
+        Z3_CATCH_RETURN(false);
     }
 
+    bool Z3_API Z3_is_quantifier_exists(Z3_context c, Z3_ast a) {
+        Z3_TRY;
+        LOG_Z3_is_quantifier_exists(c, a);
+        RESET_ERROR_CODE();
+        return ::is_exists(to_ast(a));
+        Z3_CATCH_RETURN(false);
+    }
+
+    bool Z3_API Z3_is_lambda(Z3_context c, Z3_ast a) {
+        Z3_TRY;
+        LOG_Z3_is_lambda(c, a);
+        RESET_ERROR_CODE();
+        return ::is_lambda(to_ast(a));
+        Z3_CATCH_RETURN(false);
+    }
+
+
     unsigned Z3_API Z3_get_quantifier_weight(Z3_context c, Z3_ast a) {
         Z3_TRY;
         LOG_Z3_get_quantifier_weight(c, a);
@@ -303,7 +377,7 @@ extern "C" {
             return to_quantifier(_a)->get_weight();
         }
         else {
-            SET_ERROR_CODE(Z3_SORT_ERROR);
+            SET_ERROR_CODE(Z3_SORT_ERROR, nullptr);
             return 0;
         }
         Z3_CATCH_RETURN(0);
@@ -318,7 +392,7 @@ extern "C" {
             return to_quantifier(_a)->get_num_patterns();
         }
         else {
-            SET_ERROR_CODE(Z3_SORT_ERROR);
+            SET_ERROR_CODE(Z3_SORT_ERROR, nullptr);
             return 0;
         }
         Z3_CATCH_RETURN(0);
@@ -334,10 +408,10 @@ extern "C" {
             RETURN_Z3(r);
         }
         else {
-            SET_ERROR_CODE(Z3_SORT_ERROR);
-            RETURN_Z3(0);
+            SET_ERROR_CODE(Z3_SORT_ERROR, nullptr);
+            RETURN_Z3(nullptr);
         }
-        Z3_CATCH_RETURN(0);
+        Z3_CATCH_RETURN(nullptr);
     }
 
 
@@ -350,7 +424,7 @@ extern "C" {
             return to_quantifier(_a)->get_num_no_patterns();
         }
         else {
-            SET_ERROR_CODE(Z3_SORT_ERROR);
+            SET_ERROR_CODE(Z3_SORT_ERROR, nullptr);
             return 0;
         }
         Z3_CATCH_RETURN(0);
@@ -366,10 +440,10 @@ extern "C" {
             RETURN_Z3(r);
         }
         else {
-            SET_ERROR_CODE(Z3_SORT_ERROR);
-            RETURN_Z3(0);
+            SET_ERROR_CODE(Z3_SORT_ERROR, nullptr);
+            RETURN_Z3(nullptr);
         }
-        Z3_CATCH_RETURN(0);
+        Z3_CATCH_RETURN(nullptr);
     }
 
     Z3_symbol Z3_API Z3_get_quantifier_bound_name(Z3_context c, Z3_ast a, unsigned i) {
@@ -381,10 +455,10 @@ extern "C" {
             return of_symbol(to_quantifier(_a)->get_decl_names()[i]);
         }
         else {
-            SET_ERROR_CODE(Z3_SORT_ERROR);
-            return 0;
+            SET_ERROR_CODE(Z3_SORT_ERROR, nullptr);
+            return nullptr;
         }
-        Z3_CATCH_RETURN(0);
+        Z3_CATCH_RETURN(nullptr);
     }
 
     Z3_sort Z3_API Z3_get_quantifier_bound_sort(Z3_context c, Z3_ast a, unsigned i) {
@@ -397,10 +471,10 @@ extern "C" {
             RETURN_Z3(r);
         }
         else {
-            SET_ERROR_CODE(Z3_SORT_ERROR);
-            RETURN_Z3(0);
+            SET_ERROR_CODE(Z3_SORT_ERROR, nullptr);
+            RETURN_Z3(nullptr);
         }
-        Z3_CATCH_RETURN(0);
+        Z3_CATCH_RETURN(nullptr);
     }
 
     Z3_ast Z3_API Z3_get_quantifier_body(Z3_context c, Z3_ast a) {
@@ -413,10 +487,10 @@ extern "C" {
             RETURN_Z3(r);
         }
         else {
-            SET_ERROR_CODE(Z3_SORT_ERROR);
-            RETURN_Z3(0);
+            SET_ERROR_CODE(Z3_SORT_ERROR, nullptr);
+            RETURN_Z3(nullptr);
         }
-        Z3_CATCH_RETURN(0);
+        Z3_CATCH_RETURN(nullptr);
     }
 
     unsigned Z3_API Z3_get_quantifier_num_bound(Z3_context c, Z3_ast a) {
@@ -429,9 +503,9 @@ extern "C" {
             return to_quantifier(_a)->get_num_decls();
         }
         else {
-            SET_ERROR_CODE(Z3_SORT_ERROR);
+            SET_ERROR_CODE(Z3_SORT_ERROR, nullptr);
             return 0;
-        }        
+        }
         Z3_CATCH_RETURN(0);
     }
 
@@ -444,8 +518,8 @@ extern "C" {
             return _p->get_num_args();
         }
         else {
-            SET_ERROR_CODE(Z3_SORT_ERROR);
-            return 0;            
+            SET_ERROR_CODE(Z3_SORT_ERROR, nullptr);
+            return 0;
         }
         Z3_CATCH_RETURN(0);
     }
@@ -460,55 +534,20 @@ extern "C" {
             RETURN_Z3(r);
         }
         else {
-            SET_ERROR_CODE(Z3_SORT_ERROR);
-            RETURN_Z3(0);
+            SET_ERROR_CODE(Z3_SORT_ERROR, nullptr);
+            RETURN_Z3(nullptr);
         }
-        Z3_CATCH_RETURN(0);
+        Z3_CATCH_RETURN(nullptr);
     }
 
-    Z3_func_decl Z3_API Z3_mk_injective_function(Z3_context c, 
-                                                 Z3_symbol s, 
-                                                 unsigned domain_size, 
-                                                 Z3_sort const domain[],
-                                                 Z3_sort range) {
-        Z3_TRY;
-        LOG_Z3_mk_injective_function(c, s, domain_size, domain, range);
-        RESET_ERROR_CODE(); 
-        ast_manager & m = mk_c(c)->m();
-        mk_c(c)->reset_last_result();
-        sort* range_ = to_sort(range);
-        func_decl* d = m.mk_func_decl(to_symbol(s), domain_size, to_sorts(domain), range_);
-        expr_ref_vector args(m);
-        expr_ref fn(m), body(m);
-        vector<symbol> names;
-        for (unsigned i = 0; i < domain_size; ++i) {
-            unsigned idx = domain_size-i-1;
-            args.push_back(m.mk_var(idx, to_sort(domain[i])));
-            names.push_back(symbol(idx));
-        }
-        fn = m.mk_app(d, args.size(), args.c_ptr());
-
-        for (unsigned i = 0; i < domain_size; ++i) {
-            expr* arg = args[i].get();
-            sort* dom = m.get_sort(arg);
-            func_decl* inv = m.mk_fresh_func_decl(symbol("inv"), to_symbol(s), 1, &range_, dom);
-            body = m.mk_eq(m.mk_app(inv, fn.get()), arg);
-            body = m.mk_forall(args.size(), to_sorts(domain), names.c_ptr(), body.get());
-            mk_c(c)->save_multiple_ast_trail(body.get());
-            mk_c(c)->assert_cnstr(body.get());
-        }
-        mk_c(c)->save_multiple_ast_trail(d);       
-        RETURN_Z3(of_func_decl(d));
-        Z3_CATCH_RETURN(0);
-    }
-
-    Z3_ast Z3_API Z3_pattern_to_ast(Z3_context c, Z3_pattern p) { 
+    Z3_ast Z3_API Z3_pattern_to_ast(Z3_context c, Z3_pattern p) {
         RESET_ERROR_CODE();
-        return (Z3_ast)(p); 
-    }    
+        return (Z3_ast)(p);
+    }
 
     Z3_API char const * Z3_pattern_to_string(Z3_context c, Z3_pattern p) {
         return Z3_ast_to_string(c, reinterpret_cast<Z3_ast>(p));
     }
-    
+
 };
+
