@@ -21,6 +21,7 @@ Revision History:
 #include "util/cancel_eh.h"
 #include "util/file_path.h"
 #include "util/scoped_timer.h"
+#include "util/file_path.h"
 #include "ast/ast_pp.h"
 #include "api/z3.h"
 #include "api/api_log_macros.h"
@@ -30,11 +31,12 @@ Revision History:
 #include "api/api_model.h"
 #include "api/api_stats.h"
 #include "api/api_ast_vector.h"
-#include "solver/tactic2solver.h"
-#include "util/file_path.h"
+#include "model/model_params.hpp"
 #include "smt/smt_solver.h"
 #include "smt/smt_implied_equalities.h"
 #include "solver/smt_logics.h"
+#include "solver/tactic2solver.h"
+#include "solver/solver_params.hpp"
 #include "cmd_context/cmd_context.h"
 #include "parsers/smt2/smt2parser.h"
 #include "sat/dimacs.h"
@@ -44,6 +46,92 @@ Revision History:
 
 extern "C" {
 
+    void solver2smt2_pp::assert_expr(expr* e) {
+        m_pp_util.collect(e);
+        m_pp_util.display_decls(m_out);
+        m_pp_util.display_assert(m_out, e, true);
+    }
+
+    void solver2smt2_pp::assert_expr(expr* e, expr* t) {
+        m_pp_util.collect(e);
+        m_pp_util.collect(t);
+        m_pp_util.display_decls(m_out);
+        m_pp_util.display_assert_and_track(m_out, e, t, true);
+        m_tracked.push_back(t);
+    }
+
+    void solver2smt2_pp::push() {
+        m_out << "(push)\n";
+        m_pp_util.push();
+        m_tracked_lim.push_back(m_tracked.size());
+    }
+
+    void solver2smt2_pp::pop(unsigned n) {
+        m_out << "(pop " << n << ")\n";
+        m_pp_util.pop(n);
+        m_tracked.shrink(m_tracked_lim[m_tracked_lim.size() - n]);
+        m_tracked_lim.shrink(m_tracked_lim.size() - n);
+    }
+
+    void solver2smt2_pp::reset() {
+        m_out << "(reset)\n";
+        m_pp_util.reset();        
+    }
+
+    void solver2smt2_pp::check(unsigned n, expr* const* asms) {
+        m_out << "(check-sat";        
+        for (unsigned i = 0; i < n; ++i) {
+            m_pp_util.display_expr(m_out << "\n", asms[i]);            
+        }
+        for (expr* e : m_tracked) {
+            m_pp_util.display_expr(m_out << "\n", e);
+        }
+        m_out << ")\n";
+        m_out.flush();
+    }
+
+    void solver2smt2_pp::get_consequences(expr_ref_vector const& assumptions, expr_ref_vector const& variables) {
+        m_out << "(get-consequences (";
+        for (expr* f : assumptions) {
+            m_out << "\n";
+            m_pp_util.display_expr(m_out, f);
+        }
+        m_out << ") (";
+        for (expr* f : variables) {
+            m_out << "\n";
+            m_pp_util.display_expr(m_out, f);
+        }
+        m_out << ")\n";
+        m_out.flush();
+    }
+
+    solver2smt2_pp::solver2smt2_pp(ast_manager& m, char const* file): 
+        m_pp_util(m), m_out(file), m_tracked(m) {
+        if (!m_out) {
+            throw default_exception("could not open " + std::string(file) + " for output");
+        }
+    }
+
+    void Z3_solver_ref::set_eh(event_handler* eh) {
+        std::lock_guard<std::mutex> lock(m_mux);
+        m_eh = eh;
+    }
+
+    void Z3_solver_ref::set_cancel() {
+        std::lock_guard<std::mutex> lock(m_mux);
+        if (m_eh) (*m_eh)(API_INTERRUPT_EH_CALLER);
+    }
+
+    void Z3_solver_ref::assert_expr(expr * e) {
+        if (m_pp) m_pp->assert_expr(e);
+        m_solver->assert_expr(e);
+    }
+
+    void Z3_solver_ref::assert_expr(expr * e, expr* t) {
+        if (m_pp) m_pp->assert_expr(e, t);
+        m_solver->assert_expr(e, t);
+    }
+
     static void init_solver_core(Z3_context c, Z3_solver _s) {
         Z3_solver_ref * s = to_solver(_s);
         bool proofs_enabled, models_enabled, unsat_core_enabled;
@@ -154,7 +242,7 @@ extern "C" {
         if (!initialized)
             init_solver(c, s);
         for (expr * e : ctx->assertions()) {
-            to_solver_ref(s)->assert_expr(e);
+            to_solver(s)->assert_expr(e);
         }
         to_solver_ref(s)->set_model_converter(ctx->get_model_converter());
     }
@@ -177,7 +265,7 @@ extern "C" {
         goal g(m);            
         s2g(solver, a2b, to_solver_ref(s)->get_params(), g, mc);
         for (unsigned i = 0; i < g.size(); ++i) {
-            to_solver_ref(s)->assert_expr(g.form(i));
+            to_solver(s)->assert_expr(g.form(i));
         }
     }
 
@@ -261,10 +349,13 @@ extern "C" {
         RESET_ERROR_CODE();
 
         symbol logic = to_param_ref(p).get_sym("smt.logic", symbol::null);
+        symbol smt2log = to_param_ref(p).get_sym("solver.smtlib2_log", symbol::null);
         if (logic != symbol::null) {
             to_solver(s)->m_logic = logic;
         }
-
+        if (smt2log != symbol::null && !to_solver(s)->m_pp) {
+            to_solver(s)->m_pp = alloc(solver2smt2_pp, mk_c(c)->m(), smt2log.str().c_str());
+        }
         if (to_solver(s)->m_solver) {
             bool old_model = to_solver(s)->m_params.get_bool("model", true);
             bool new_model = to_param_ref(p).get_bool("model", true);
@@ -277,6 +368,7 @@ extern "C" {
             to_solver_ref(s)->updt_params(to_param_ref(p));
         }
         to_solver(s)->m_params.append(to_param_ref(p));
+        
         Z3_CATCH;
     }
     
@@ -302,9 +394,14 @@ extern "C" {
         RESET_ERROR_CODE();
         init_solver(c, s);
         to_solver_ref(s)->push();
+        if (to_solver(s)->m_pp) to_solver(s)->m_pp->push();
         Z3_CATCH;
     }
 
+    void Z3_API Z3_solver_interrupt(Z3_context c, Z3_solver s) {
+        to_solver(s)->set_cancel();
+    }
+
     void Z3_API Z3_solver_pop(Z3_context c, Z3_solver s, unsigned n) {
         Z3_TRY;
         LOG_Z3_solver_pop(c, s, n);
@@ -314,8 +411,10 @@ extern "C" {
             SET_ERROR_CODE(Z3_IOB, nullptr);
             return;
         }
-        if (n > 0)
+        if (n > 0) {
             to_solver_ref(s)->pop(n);
+            if (to_solver(s)->m_pp) to_solver(s)->m_pp->pop(n);
+        }
         Z3_CATCH;
     }
 
@@ -324,6 +423,7 @@ extern "C" {
         LOG_Z3_solver_reset(c, s);
         RESET_ERROR_CODE();
         to_solver(s)->m_solver = nullptr;
+        if (to_solver(s)->m_pp) to_solver(s)->m_pp->reset();
         Z3_CATCH;
     }
     
@@ -342,7 +442,7 @@ extern "C" {
         RESET_ERROR_CODE();
         init_solver(c, s);
         CHECK_FORMULA(a,);
-        to_solver_ref(s)->assert_expr(to_expr(a));
+        to_solver(s)->assert_expr(to_expr(a));
         Z3_CATCH;
     }
 
@@ -353,7 +453,7 @@ extern "C" {
         init_solver(c, s);
         CHECK_FORMULA(a,);
         CHECK_FORMULA(p,);
-        to_solver_ref(s)->assert_expr(to_expr(a), to_expr(p));
+        to_solver(s)->assert_expr(to_expr(a), to_expr(p));
         Z3_CATCH;
     }
 
@@ -449,28 +549,40 @@ extern "C" {
                 return Z3_L_UNDEF;
             }
         }
-        expr * const * _assumptions = to_exprs(assumptions);
-        unsigned timeout     = to_solver(s)->m_params.get_uint("timeout", mk_c(c)->get_timeout());
+        expr * const * _assumptions = to_exprs(num_assumptions, assumptions);
+        solver_params sp(to_solver(s)->m_params);
+        unsigned timeout     = mk_c(c)->get_timeout();
+        timeout              = to_solver(s)->m_params.get_uint("timeout", timeout);
+        timeout              = sp.timeout() != UINT_MAX ? sp.timeout() : timeout;
         unsigned rlimit      = to_solver(s)->m_params.get_uint("rlimit", mk_c(c)->get_rlimit());
         bool     use_ctrl_c  = to_solver(s)->m_params.get_bool("ctrl_c", true);
         cancel_eh<reslimit> eh(mk_c(c)->m().limit());
+        to_solver(s)->set_eh(&eh);
         api::context::set_interruptable si(*(mk_c(c)), eh);
-        lbool result;
+        lbool result = l_undef;
         {
             scoped_ctrl_c ctrlc(eh, false, use_ctrl_c);
             scoped_timer timer(timeout, &eh);
             scoped_rlimit _rlimit(mk_c(c)->m().limit(), rlimit);
             try {
+                if (to_solver(s)->m_pp) to_solver(s)->m_pp->check(num_assumptions, _assumptions); 
                 result = to_solver_ref(s)->check_sat(num_assumptions, _assumptions);
             }
             catch (z3_exception & ex) {
                 to_solver_ref(s)->set_reason_unknown(eh);
+                to_solver(s)->set_eh(nullptr);
                 if (!mk_c(c)->m().canceled()) {
                     mk_c(c)->handle_exception(ex);
                 }
                 return Z3_L_UNDEF;
             }
+            catch (...) {
+                to_solver_ref(s)->set_reason_unknown(eh);
+                to_solver(s)->set_eh(nullptr);
+                return Z3_L_UNDEF;
+            }
         }
+        to_solver(s)->set_eh(nullptr);
         if (result == l_undef) {
             to_solver_ref(s)->set_reason_unknown(eh);
         }
@@ -507,7 +619,8 @@ extern "C" {
             RETURN_Z3(nullptr);
         }
         if (_m) {
-            if (mk_c(c)->params().m_model_compress) _m->compress();
+            model_params mp(to_solver_ref(s)->get_params());
+            if (mp.compact()) _m->compress();
         }
         Z3_model_ref * m_ref = alloc(Z3_model_ref, *mk_c(c)); 
         m_ref->m_model = _m;
@@ -605,7 +718,7 @@ extern "C" {
         RESET_ERROR_CODE();
         CHECK_SEARCHING(c);
         init_solver(c, s);
-        lbool result = smt::implied_equalities(m, *to_solver_ref(s), num_terms, to_exprs(terms), class_ids);
+        lbool result = smt::implied_equalities(m, *to_solver_ref(s), num_terms, to_exprs(num_terms, terms), class_ids);
         return static_cast<Z3_lbool>(result); 
         Z3_CATCH_RETURN(Z3_L_UNDEF);
     }
@@ -645,21 +758,27 @@ extern "C" {
         unsigned rlimit      = to_solver(s)->m_params.get_uint("rlimit", mk_c(c)->get_rlimit());
         bool     use_ctrl_c  = to_solver(s)->m_params.get_bool("ctrl_c", true);
         cancel_eh<reslimit> eh(mk_c(c)->m().limit());
+        to_solver(s)->set_eh(&eh);
         api::context::set_interruptable si(*(mk_c(c)), eh);
         {
             scoped_ctrl_c ctrlc(eh, false, use_ctrl_c);
             scoped_timer timer(timeout, &eh);
             scoped_rlimit _rlimit(mk_c(c)->m().limit(), rlimit);
             try {
+                if (to_solver(s)->m_pp) to_solver(s)->m_pp->get_consequences(_assumptions, _variables); 
                 result = to_solver_ref(s)->get_consequences(_assumptions, _variables, _consequences);
             }
             catch (z3_exception & ex) {
+                to_solver(s)->set_eh(nullptr);
                 to_solver_ref(s)->set_reason_unknown(eh);
                 _assumptions.finalize(); _consequences.finalize(); _variables.finalize();
                 mk_c(c)->handle_exception(ex);
                 return Z3_L_UNDEF;
             }
+            catch (...) {
+            }
         }
+        to_solver(s)->set_eh(nullptr);
         if (result == l_undef) {
             to_solver_ref(s)->set_reason_unknown(eh);
         }
@@ -687,6 +806,7 @@ extern "C" {
         unsigned rlimit      = to_solver(s)->m_params.get_uint("rlimit", mk_c(c)->get_rlimit());
         bool     use_ctrl_c  = to_solver(s)->m_params.get_bool("ctrl_c", true);
         cancel_eh<reslimit> eh(mk_c(c)->m().limit());
+        to_solver(s)->set_eh(&eh);
         api::context::set_interruptable si(*(mk_c(c)), eh);
         {
             scoped_ctrl_c ctrlc(eh, false, use_ctrl_c);
@@ -696,10 +816,14 @@ extern "C" {
                 result.append(to_solver_ref(s)->cube(vars, cutoff));
             }
             catch (z3_exception & ex) {
+                to_solver(s)->set_eh(nullptr);
                 mk_c(c)->handle_exception(ex);
                 return nullptr;
             }
+            catch (...) {
+            }
         }
+        to_solver(s)->set_eh(nullptr);
         Z3_ast_vector_ref * v = alloc(Z3_ast_vector_ref, *mk_c(c), mk_c(c)->m());
         mk_c(c)->save_object(v);
         for (expr* e : result) {
