@@ -18,12 +18,13 @@ Revision History:
 #ifndef API_STATS_H_
 #define API_STATS_H_
 
-#include"api_util.h"
-#include"statistics.h"
+#include "api/api_util.h"
+#include "util/statistics.h"
 
 struct Z3_stats_ref : public api::object {
     statistics m_stats;
-    virtual ~Z3_stats_ref() {}
+    Z3_stats_ref(api::context& c): api::object(c) {}
+    ~Z3_stats_ref() override {}
 };
 
 inline Z3_stats_ref * to_stats(Z3_stats s) { return reinterpret_cast<Z3_stats_ref *>(s); }
