@@ -1039,7 +1039,7 @@ struct
     let get_big_int (x:expr) =
       if is_int_numeral x then
         let s = (Z3native.get_numeral_string (Expr.gc x) x) in
-        Big_int.big_int_of_string s
+        Z.of_string s
       else
         raise (Error "Conversion failed.")
 
@@ -1063,7 +1063,7 @@ struct
     let get_ratio x =
       if is_rat_numeral x then
         let s = Z3native.get_numeral_string (Expr.gc x) x in
-        Ratio.ratio_of_string s
+        Q.of_string s
       else
         raise (Error "Conversion failed.")
 
@@ -1546,9 +1546,8 @@ struct
   end
 
   let get_const_interp (x:model) (f:func_decl) =
-    if FuncDecl.get_arity f <> 0 ||
-       (sort_kind_of_int (Z3native.get_sort_kind (FuncDecl.gc f) (Z3native.get_range (FuncDecl.gc f) f))) = ARRAY_SORT then
-      raise (Error "Non-zero arity functions and arrays have FunctionInterpretations as a model. Use FuncInterp.")
+    if FuncDecl.get_arity f <> 0 then
+      raise (Error "Non-zero arity functions have FunctionInterpretations as a model. Use FuncInterp.")
     else
       let np = Z3native.model_get_const_interp (gc x) x f  in
       if Z3native.is_null_ast np then
