@@ -16,13 +16,14 @@ Author:
 Revision History:
 
 --*/
-#include "arith_decl_plugin.h"
-#include "bv_decl_plugin.h"
-#include "datatype_decl_plugin.h"
-#include "array_decl_plugin.h"
-#include "format.h"
-#include "ast_translation.h"
-#include "ast_ll_pp.h"
+#include "ast/arith_decl_plugin.h"
+#include "ast/bv_decl_plugin.h"
+#include "ast/datatype_decl_plugin.h"
+#include "ast/array_decl_plugin.h"
+#include "ast/format.h"
+#include "ast/ast_translation.h"
+#include "ast/ast_ll_pp.h"
+#include "ast/ast_pp.h"
 
 ast_translation::~ast_translation() {
     reset_cache();
@@ -37,11 +38,9 @@ void ast_translation::cleanup() {
 }
 
 void ast_translation::reset_cache() {
-    obj_map<ast, ast*>::iterator it  = m_cache.begin();
-    obj_map<ast, ast*>::iterator end = m_cache.end();
-    for (; it != end; ++it) {
-        m_from_manager.dec_ref(it->m_key);
-        m_to_manager.dec_ref(it->m_value);
+    for (auto & kv : m_cache) {
+        m_from_manager.dec_ref(kv.m_key);
+        m_to_manager.dec_ref(kv.m_value);
     }
     m_cache.reset();
 }
@@ -49,9 +48,10 @@ void ast_translation::reset_cache() {
 void ast_translation::cache(ast * s, ast * t) {
     SASSERT(!m_cache.contains(s));
     if (s->get_ref_count() > 1) {
-        m_cache.insert(s, t);
         m_from_manager.inc_ref(s);
         m_to_manager.inc_ref(t);
+        m_cache.insert(s, t);
+        ++m_insert_count;
     }
 }
 
@@ -77,10 +77,16 @@ void ast_translation::push_frame(ast * n) {
 }
 
 bool ast_translation::visit(ast * n) {        
-    ast * r;
-    if (n->get_ref_count() > 1 && m_cache.find(n, r)) {
-        m_result_stack.push_back(r);
-        return true;
+    if (n->get_ref_count() > 1) {
+        ast * r;
+        if (m_cache.find(n, r)) {
+            m_result_stack.push_back(r);
+            ++m_hit_count;
+            return true;
+        }
+        else {
+            ++m_miss_count;
+        }
     }
     push_frame(n);
     return false;
@@ -110,7 +116,7 @@ void ast_translation::copy_params(decl * d, unsigned rpos, buffer<parameter> & p
 void ast_translation::mk_sort(sort * s, frame & fr) {
     sort_info * si     = s->get_info();
     sort * new_s;
-    if (si == 0) {
+    if (si == nullptr) {
         // TODO: investigate: this branch is probably unreachable.
         // It became unreachable after we started using mk_uninterpreted_sort for creating uninterpreted sorts,
         // and mk_uninterpreted_sort actually creates a user_sort.
@@ -141,7 +147,7 @@ void ast_translation::mk_func_decl(func_decl * f, frame & fr) {
     sort ** new_domain = reinterpret_cast<sort**>(m_result_stack.c_ptr() + fr.m_rpos + num_extra);
     sort *  new_range  = static_cast<sort*>(m_result_stack.back());  
     func_decl * new_f;
-    if (fi == 0) {
+    if (fi == nullptr) {
         new_f = m_to_manager.mk_func_decl(f->get_name(),
                                           f->get_arity(),
                                           new_domain,
@@ -162,7 +168,7 @@ void ast_translation::mk_func_decl(func_decl * f, frame & fr) {
         new_fi.set_chainable(fi->is_chainable());
         new_fi.set_pairwise(fi->is_pairwise());
         new_fi.set_injective(fi->is_injective());
-        new_fi.set_skolem(fi->is_skolem()); 
+/// TBD        new_fi.set_skolem(fi->is_skolem()); 
         new_fi.set_idempotent(fi->is_idempotent());
 
         new_f = m_to_manager.mk_func_decl(f->get_name(),
@@ -184,24 +190,37 @@ void ast_translation::mk_func_decl(func_decl * f, frame & fr) {
 }
 
 ast * ast_translation::process(ast const * _n) {
+    if (!_n) return nullptr;
     SASSERT(m_result_stack.empty());
     SASSERT(m_frame_stack.empty());
     SASSERT(m_extra_children_stack.empty());
     
+    ++m_num_process;
+    if (m_num_process > (1 << 14)) {
+        reset_cache();
+        m_num_process = 0;
+    }
     if (!visit(const_cast<ast*>(_n))) {
         while (!m_frame_stack.empty()) {
         loop:
+            ++m_loop_count;
             frame & fr = m_frame_stack.back();
             ast * n = fr.m_n;
             ast * r;         
             TRACE("ast_translation", tout << mk_ll_pp(n, m_from_manager, false) << "\n";);
-            if (fr.m_idx == 0 && n->get_ref_count() > 1 && m_cache.find(n, r)) {
-                SASSERT(m_result_stack.size() == fr.m_rpos);
-                m_result_stack.push_back(r);
-                m_extra_children_stack.shrink(fr.m_cpos);
-                m_frame_stack.pop_back();
-                TRACE("ast_translation", tout << "hit\n";);
-                continue;
+            if (fr.m_idx == 0 && n->get_ref_count() > 1) {
+                if (m_cache.find(n, r)) {
+                    SASSERT(m_result_stack.size() == fr.m_rpos);
+                    m_result_stack.push_back(r);
+                    m_extra_children_stack.shrink(fr.m_cpos);
+                    m_frame_stack.pop_back();
+                    TRACE("ast_translation", tout << "hit\n";);
+                    m_hit_count++;
+                    continue;
+                }
+                else {
+                    m_miss_count++;
+                }
             }
             switch (n->get_kind()) {
             case AST_VAR: {
@@ -228,7 +247,7 @@ ast * ast_translation::process(ast const * _n) {
                 while (fr.m_idx <= num) {
                     expr * arg = to_app(n)->get_arg(fr.m_idx - 1);
                     fr.m_idx++;
-                    if (!visit(arg))
+                    if (!visit(arg)) 
                         goto loop;
                 }
                 func_decl * new_f   = to_func_decl(m_result_stack[fr.m_rpos]);
@@ -260,7 +279,7 @@ ast * ast_translation::process(ast const * _n) {
                 expr **  pats         = reinterpret_cast<expr**>(m_result_stack.c_ptr() + fr.m_rpos + num_decls + 1);
                 unsigned num_no_pats  = to_quantifier(n)->get_num_no_patterns();
                 expr **  no_pats      = pats + num_pats;
-                quantifier * new_q    = m_to_manager.mk_quantifier(to_quantifier(n)->is_forall(),
+                quantifier * new_q    = m_to_manager.mk_quantifier(to_quantifier(n)->get_kind(),
                                                                    num_decls,
                                                                    dsorts,
                                                                    dnames,
@@ -321,7 +340,7 @@ ast * ast_translation::process(ast const * _n) {
 }
 
 expr_dependency * expr_dependency_translation::operator()(expr_dependency * d) {
-    if (d == 0)
+    if (d == nullptr)
         return d;
     m_buffer.reset();
     m_translation.from().linearize(d, m_buffer);
