@@ -29,6 +29,10 @@ class decl_collector {
     ptr_vector<sort>      m_sorts;
     ptr_vector<func_decl> m_decls;
     ast_mark              m_visited;
+    ast_ref_vector        m_trail;
+    unsigned_vector       m_trail_lim;
+    unsigned_vector       m_sorts_lim;
+    unsigned_vector       m_decls_lim;
     family_id             m_basic_fid;
     family_id             m_dt_fid;
     datatype_util         m_dt_util;
@@ -47,12 +51,16 @@ public:
     decl_collector(ast_manager & m);
     ast_manager & m() { return m_manager; }
 
+    void reset() { m_sorts.reset(); m_decls.reset(); m_visited.reset(); m_trail.reset(); }
     void visit_func(func_decl* n);
     void visit(ast * n);
     void visit(unsigned n, expr* const* es);
     void visit(expr_ref_vector const& es);
 
-    void order_deps();
+    void push();
+    void pop(unsigned n);
+
+    void order_deps(unsigned n);
 
     unsigned get_num_sorts() const { return m_sorts.size(); }
     unsigned get_num_decls() const { return m_decls.size(); }
