@@ -19,9 +19,9 @@ Revision History:
 #ifndef DL_DECL_PLUGIN_H_
 #define DL_DECL_PLUGIN_H_
 
-#include"ast.h"
-#include "arith_decl_plugin.h"
-#include "bv_decl_plugin.h"
+#include "ast/ast.h"
+#include "ast/arith_decl_plugin.h"
+#include "ast/bv_decl_plugin.h"
 
 namespace datalog {
 
@@ -50,7 +50,6 @@ namespace datalog {
         OP_DL_LT,
         OP_DL_REP,
         OP_DL_ABS,
-        OP_DL_MIN,
         LAST_RA_OP
     };
     
@@ -72,7 +71,6 @@ namespace datalog {
         symbol m_lt_sym;
         symbol m_le_sym;
         symbol m_rule_sym;
-        symbol m_min_sym;
 
         bool check_bounds(char const* msg, unsigned low, unsigned up, unsigned val) const;
         bool check_domain(unsigned low, unsigned up, unsigned val) const;
@@ -96,73 +94,17 @@ namespace datalog {
         func_decl * mk_compare(decl_kind k, symbol const& sym, sort*const* domain);
         func_decl * mk_clone(sort* r);
         func_decl * mk_rule(unsigned arity);
-        func_decl * mk_min(decl_kind k, unsigned num_parameters, parameter const * parameters);
                                 
         sort * mk_finite_sort(unsigned num_params, parameter const* params);
         sort * mk_relation_sort(unsigned num_params, parameter const* params);
         sort * mk_rule_sort();
 
     public:
-        /**
-        Is \c decl a min aggregation function?
-        */
-        static bool is_aggregate(const func_decl* const decl)
-        {
-            return decl->get_decl_kind() == OP_DL_MIN;
-        }
-
-        /**
-        \pre: is_aggregate(aggregate)
-
-        \returns function declaration of predicate which is subject to min aggregation function
-        */
-        static func_decl * min_func_decl(const func_decl* const aggregate)
-        {
-            SASSERT(is_aggregate(aggregate));
-            parameter const & relation_parameter = aggregate->get_parameter(0);
-            return to_func_decl(relation_parameter.get_ast());
-        }
-
-        /**
-        \pre: is_aggregate(aggregate)
-
-        \returns column identifier (starting at zero) which is minimized by aggregation function
-        */
-        static unsigned min_col(const func_decl* const aggregate)
-        {
-            SASSERT(is_aggregate(aggregate));
-            return (unsigned)aggregate->get_parameter(1).get_int();
-        }
-
-        /**
-        \pre: is_aggregate(aggregate)
-
-        \returns column identifiers for the "group by" in the given min aggregation function
-        */
-        static unsigned_vector group_by_cols(const func_decl* const aggregate)
-        {
-            SASSERT(is_aggregate(aggregate));
-            unsigned _min_col = min_col(aggregate);
-            if (aggregate->get_arity() == 0U)
-                return unsigned_vector();
-
-            unsigned col_num = 0;
-            unsigned_vector cols(aggregate->get_arity() - 1U);
-            for (unsigned i = 0; i < cols.size(); ++i, ++col_num)
-            {
-                if (col_num == _min_col)
-                    ++col_num;
-
-                cols[i] = col_num;
-            }
-
-            return cols;
-        }
 
         dl_decl_plugin();
-        virtual ~dl_decl_plugin() {}
+        ~dl_decl_plugin() override {}
 
-        virtual decl_plugin * mk_fresh() { return alloc(dl_decl_plugin); }
+        decl_plugin * mk_fresh() override { return alloc(dl_decl_plugin); }
         
         //
         // Contract for sort DL_RELATION_SORT 
@@ -174,25 +116,25 @@ namespace datalog {
         //   parameters[0]     - name
         //   parameters[1]     - uint64
         //
-        virtual sort * mk_sort(decl_kind k, unsigned num_parameters, parameter const * parameters);
+        sort * mk_sort(decl_kind k, unsigned num_parameters, parameter const * parameters) override;
         
         //
         // Contract for func_decl:
         //   parameters[0]     - array sort
         // Contract for OP_DL_CONSTANT:
-        //   parameters[0]     - rational containing uint64 with constant value
+        //   parameters[0]     - rational containing uint64_t with constant value
         //   parameters[1]     - a DL_FINITE_SORT sort of the constant
         // Contract for others:
         //   no parameters
-        virtual func_decl * mk_func_decl(decl_kind k, unsigned num_parameters, parameter const * parameters, 
-                                         unsigned arity, sort * const * domain, sort * range);
+        func_decl * mk_func_decl(decl_kind k, unsigned num_parameters, parameter const * parameters,
+                                 unsigned arity, sort * const * domain, sort * range) override;
         
-        virtual void get_op_names(svector<builtin_name> & op_names, symbol const & logic);
+        void get_op_names(svector<builtin_name> & op_names, symbol const & logic) override;
         
-        virtual void get_sort_names(svector<builtin_name> & sort_names, symbol const & logic);
+        void get_sort_names(svector<builtin_name> & sort_names, symbol const & logic) override;
 
-        virtual bool is_value(app * e) const { return is_app_of(e, m_family_id, OP_DL_CONSTANT); }
-        virtual bool is_unique_value(app * e) const { return is_value(e); }
+        bool is_value(app * e) const override { return is_app_of(e, m_family_id, OP_DL_CONSTANT); }
+        bool is_unique_value(app * e) const override { return is_value(e); }
 
     };
 
@@ -224,7 +166,7 @@ namespace datalog {
         dl_decl_util(ast_manager& m);
         // create a constant belonging to a given finite domain.
         // the options include the DL_FINITE_SORT, BV_SORT, and BOOL_SORT
-        app* mk_numeral(uint64 value, sort* s);
+        app* mk_numeral(uint64_t value, sort* s);
 
         app* mk_lt(expr* a, expr* b);
 
@@ -234,19 +176,19 @@ namespace datalog {
 
         bool is_numeral(const expr* c) const { return is_app_of(c, m_fid, OP_DL_CONSTANT); }
 
-        bool is_numeral(const expr* e, uint64& v) const;
+        bool is_numeral(const expr* e, uint64_t& v) const;
 
         //
         // Utilities for extracting constants 
         // from bit-vectors and finite domains.
         // 
-        bool is_numeral_ext(expr* c, uint64& v) const;
+        bool is_numeral_ext(expr* c, uint64_t& v) const;
 
         bool is_numeral_ext(expr* c) const;        
 
-        sort* mk_sort(const symbol& name, uint64  domain_size);
+        sort* mk_sort(const symbol& name, uint64_t  domain_size);
 
-        bool try_get_size(const sort *, uint64& size) const;
+        bool try_get_size(const sort *, uint64_t& size) const;
 
         bool is_finite_sort(sort* s) const { 
             return is_sort_of(s, m_fid, DL_FINITE_SORT); 
@@ -262,9 +204,9 @@ namespace datalog {
 
         sort* mk_rule_sort();
 
-        app* mk_rule(symbol const& name, unsigned num_args = 0, expr* const* args = 0);
+        app* mk_rule(symbol const& name, unsigned num_args = 0, expr* const* args = nullptr);
 
-        app* mk_fact(symbol const& name) { return mk_rule(name, 0, 0); }
+        app* mk_fact(symbol const& name) { return mk_rule(name, 0, nullptr); }
 
         ast_manager& get_manager() const { return m; }
 
