@@ -17,120 +17,118 @@ Author:
 Revision History:
 
 --*/
-#include"macro_util.h"
-#include"occurs.h"
-#include"arith_simplifier_plugin.h"
-#include"basic_simplifier_plugin.h"
-#include"bv_simplifier_plugin.h"
-#include"var_subst.h"
-#include"ast_pp.h"
-#include"ast_ll_pp.h"
-#include"ast_util.h"
-#include"for_each_expr.h"
-#include"well_sorted.h"
+#include "ast/macros/macro_util.h"
+#include "ast/occurs.h"
+#include "ast/ast_util.h"
+#include "ast/rewriter/var_subst.h"
+#include "ast/ast_pp.h"
+#include "ast/ast_ll_pp.h"
+#include "ast/for_each_expr.h"
+#include "ast/well_sorted.h"
+#include "ast/rewriter/bool_rewriter.h"
 
-macro_util::macro_util(ast_manager & m, simplifier & s):
+macro_util::macro_util(ast_manager & m):
     m_manager(m),
-    m_simplifier(s),
-    m_arith_simp(0),
-    m_bv_simp(0),
-    m_basic_simp(0), 
-    m_forbidden_set(0),
-    m_curr_clause(0) {
+    m_bv(m),
+    m_arith(m),
+    m_arith_rw(m),
+    m_bv_rw(m),
+    m_forbidden_set(nullptr),
+    m_curr_clause(nullptr) {
 }
 
-arith_simplifier_plugin * macro_util::get_arith_simp() const {
-    if (m_arith_simp == 0) {
-        const_cast<macro_util*>(this)->m_arith_simp = static_cast<arith_simplifier_plugin*>(m_simplifier.get_plugin(m_manager.mk_family_id("arith")));
-    }
-    SASSERT(m_arith_simp != 0);
-    return m_arith_simp;
-}
-
-bv_simplifier_plugin * macro_util::get_bv_simp() const {
-    if (m_bv_simp == 0) {
-        const_cast<macro_util*>(this)->m_bv_simp = static_cast<bv_simplifier_plugin*>(m_simplifier.get_plugin(m_manager.mk_family_id("bv")));
-    }
-    SASSERT(m_bv_simp != 0);
-    return m_bv_simp;
-}
-
-basic_simplifier_plugin * macro_util::get_basic_simp() const {
-    if (m_basic_simp == 0) {
-        const_cast<macro_util*>(this)->m_basic_simp = static_cast<basic_simplifier_plugin*>(m_simplifier.get_plugin(m_manager.get_basic_family_id()));
-    }
-    SASSERT(m_basic_simp != 0);
-    return m_basic_simp;
-}
 
 bool macro_util::is_bv(expr * n) const {
-    return get_bv_simp()->is_bv(n); 
+    return m_bv.is_bv(n);
 }
 
 bool macro_util::is_bv_sort(sort * s) const {
-    return get_bv_simp()->is_bv_sort(s); 
+    return m_bv.is_bv_sort(s);
 }
 
 bool macro_util::is_add(expr * n) const {
-    return get_arith_simp()->is_add(n) || get_bv_simp()->is_add(n); 
+    return m_arith.is_add(n) || m_bv.is_bv_add(n);
 }
 
 bool macro_util::is_times_minus_one(expr * n, expr * & arg) const {
-    return get_arith_simp()->is_times_minus_one(n, arg) || get_bv_simp()->is_times_minus_one(n, arg);
+    return m_arith_rw.is_times_minus_one(n, arg) || m_bv_rw.is_times_minus_one(n, arg);
 }
 
-bool macro_util::is_le(expr * n) const { 
-    return get_arith_simp()->is_le(n) || get_bv_simp()->is_le(n); 
+bool macro_util::is_le(expr * n) const {
+    return m_arith.is_le(n) || m_bv.is_bv_ule(n) || m_bv.is_bv_sle(n);
 }
 
 bool macro_util::is_le_ge(expr * n) const {
-    return get_arith_simp()->is_le_ge(n) || get_bv_simp()->is_le_ge(n);
+    return m_arith.is_ge(n) || m_arith.is_le(n) || m_bv.is_bv_ule(n) || m_bv.is_bv_sle(n);
 }
 
-poly_simplifier_plugin * macro_util::get_poly_simp_for(sort * s) const {
-    if (is_bv_sort(s))
-        return get_bv_simp();
-    else
-        return get_arith_simp();
+bool macro_util::is_var_plus_ground(expr * n, bool & inv, var * & v, expr_ref & t) {
+    return m_arith_rw.is_var_plus_ground(n, inv, v, t) || m_bv_rw.is_var_plus_ground(n, inv, v, t);
+}
+
+bool macro_util::is_zero_safe(expr * n) const {
+    if (m_bv_rw.is_bv(n)) {
+        return m_bv.is_zero(n);
+    }
+    else {
+        return m_arith_rw.is_zero(n);
+    }
 }
 
 app * macro_util::mk_zero(sort * s) const {
-    poly_simplifier_plugin * ps = get_poly_simp_for(s);
-    ps->set_curr_sort(s);
-    return ps->mk_zero();
+    if (m_bv.is_bv_sort(s)) {
+        return m_bv.mk_numeral(rational(0), s);
+    }
+    else {
+        return m_arith.mk_numeral(rational(0), s);
+    }
 }
 
 void macro_util::mk_sub(expr * t1, expr * t2, expr_ref & r) const {
     if (is_bv(t1)) {
-        get_bv_simp()->mk_sub(t1, t2, r);
+        m_bv_rw.mk_sub(t1, t2, r);
     }
     else {
-        get_arith_simp()->mk_sub(t1, t2, r);
+        m_arith_rw.mk_sub(t1, t2, r);
     }
 }
 
 void macro_util::mk_add(expr * t1, expr * t2, expr_ref & r) const {
     if (is_bv(t1)) {
-        get_bv_simp()->mk_add(t1, t2, r);
+        m_bv_rw.mk_add(t1, t2, r);
     }
     else {
-        get_arith_simp()->mk_add(t1, t2, r);
+        m_arith_rw.mk_add(t1, t2, r);
     }
 }
 
 void macro_util::mk_add(unsigned num_args, expr * const * args, sort * s, expr_ref & r) const {
-    if (num_args == 0) {
+    switch (num_args) {
+    case 0: 
         r = mk_zero(s);
-        return;
+        break;
+    case 1:
+        r = args[0];
+        break;
+    default:
+        if (m_bv.is_bv_sort(s)) {
+            r = args[0];
+            while (num_args >= 2) {
+                --num_args;
+                ++args;
+                r = m_bv.mk_bv_add(r, args[0]);
+            }
+        }
+        else {
+            r = m_arith.mk_add(num_args, args);
+        }
+        break;
     }
-    poly_simplifier_plugin * ps = get_poly_simp_for(s);
-    ps->set_curr_sort(s);
-    ps->mk_add(num_args, args, r);
 }
 
 /**
    \brief Return true if \c n is an application of the form
-   
+
    (f x_{k_1}, ..., x_{k_n})
 
    where f is uninterpreted
@@ -147,7 +145,7 @@ bool macro_util::is_macro_head(expr * n, unsigned num_decls) const {
         var2pos.resize(num_decls, -1);
         for (unsigned i = 0; i < num_decls; i++) {
             expr * c = to_app(n)->get_arg(i);
-            if (!is_var(c)) 
+            if (!is_var(c))
                 return false;
             unsigned idx = to_var(c)->get_idx();
             if (idx >= num_decls || var2pos[idx] != -1)
@@ -161,12 +159,12 @@ bool macro_util::is_macro_head(expr * n, unsigned num_decls) const {
 
 /**
    \brief Return true if n is of the form
-   
+
    (= (f x_{k_1}, ..., x_{k_n}) t)    OR
-   (iff (f x_{k_1}, ..., x_{k_n}) t) 
+   (iff (f x_{k_1}, ..., x_{k_n}) t)
 
    where
-   
+
    is_macro_head((f x_{k_1}, ..., x_{k_n})) returns true   AND
    t does not contain f                                    AND
    f is not in forbidden_set
@@ -176,11 +174,12 @@ bool macro_util::is_macro_head(expr * n, unsigned num_decls) const {
    def  will contain t
 
 */
-bool macro_util::is_left_simple_macro(expr * n, unsigned num_decls, app * & head, expr * & def) const {
-    if (m_manager.is_eq(n) || m_manager.is_iff(n)) {
+bool macro_util::is_left_simple_macro(expr * n, unsigned num_decls, app_ref & head, expr_ref & def) const {
+    if (m_manager.is_eq(n)) {
         expr * lhs = to_app(n)->get_arg(0);
         expr * rhs = to_app(n)->get_arg(1);
-        if (is_macro_head(lhs, num_decls) && !is_forbidden(to_app(lhs)->get_decl()) && !occurs(to_app(lhs)->get_decl(), rhs)) {
+        if (is_macro_head(lhs, num_decls) && !is_forbidden(to_app(lhs)->get_decl()) &&
+            !occurs(to_app(lhs)->get_decl(), rhs)) {
             head = to_app(lhs);
             def  = rhs;
             return true;
@@ -189,14 +188,15 @@ bool macro_util::is_left_simple_macro(expr * n, unsigned num_decls, app * & head
     return false;
 }
 
+
 /**
    \brief Return true if n is of the form
-   
+
    (= t (f x_{k_1}, ..., x_{k_n}))    OR
    (iff t (f x_{k_1}, ..., x_{k_n}))
 
    where
-   
+
    is_macro_head((f x_{k_1}, ..., x_{k_n})) returns true   AND
    t does not contain f                                    AND
    f is not in forbidden_set
@@ -206,11 +206,12 @@ bool macro_util::is_left_simple_macro(expr * n, unsigned num_decls, app * & head
    def  will contain t
 
 */
-bool macro_util::is_right_simple_macro(expr * n, unsigned num_decls, app * & head, expr * & def) const {
-    if (m_manager.is_eq(n) || m_manager.is_iff(n)) {
+bool macro_util::is_right_simple_macro(expr * n, unsigned num_decls, app_ref & head, expr_ref & def) const {
+    if (m_manager.is_eq(n)) {
         expr * lhs = to_app(n)->get_arg(0);
         expr * rhs = to_app(n)->get_arg(1);
-        if (is_macro_head(rhs, num_decls) && !is_forbidden(to_app(rhs)->get_decl()) && !occurs(to_app(rhs)->get_decl(), lhs)) {
+        if (is_macro_head(rhs, num_decls) && !is_forbidden(to_app(rhs)->get_decl()) &&
+            !occurs(to_app(rhs)->get_decl(), lhs)) {
             head = to_app(rhs);
             def  = lhs;
             return true;
@@ -245,18 +246,17 @@ bool macro_util::poly_contains_head(expr * n, func_decl * f, expr * exception) c
 
 bool macro_util::is_arith_macro(expr * n, unsigned num_decls, app_ref & head, expr_ref & def, bool & inv) const {
     // TODO: obsolete... we should move to collect_arith_macro_candidates
-    arith_simplifier_plugin * as = get_arith_simp();
-    if (!m_manager.is_eq(n) && !as->is_le(n) && !as->is_ge(n))
+    if (!m_manager.is_eq(n) && !m_arith.is_le(n) && !m_arith.is_ge(n))
         return false;
     expr * lhs = to_app(n)->get_arg(0);
     expr * rhs = to_app(n)->get_arg(1);
 
-    if (!as->is_numeral(rhs))
+    if (!m_arith.is_numeral(rhs))
         return false;
-  
+
     inv = false;
     ptr_buffer<expr> args;
-    expr * h = 0;
+    expr * h = nullptr;
     unsigned lhs_num_args;
     expr * const * lhs_args;
     if (is_add(lhs)) {
@@ -270,15 +270,15 @@ bool macro_util::is_arith_macro(expr * n, unsigned num_decls, app_ref & head, ex
     for (unsigned i = 0; i < lhs_num_args; i++) {
         expr * arg = lhs_args[i];
         expr * neg_arg;
-        if (h == 0 && 
-            is_macro_head(arg, num_decls) && 
-            !is_forbidden(to_app(arg)->get_decl()) && 
+        if (h == nullptr &&
+            is_macro_head(arg, num_decls) &&
+            !is_forbidden(to_app(arg)->get_decl()) &&
             !poly_contains_head(lhs, to_app(arg)->get_decl(), arg)) {
             h = arg;
         }
-        else if (h == 0 && as->is_times_minus_one(arg, neg_arg) &&
-                 is_macro_head(neg_arg, num_decls) && 
-                 !is_forbidden(to_app(neg_arg)->get_decl()) && 
+        else if (h == nullptr && m_arith_rw.is_times_minus_one(arg, neg_arg) &&
+                 is_macro_head(neg_arg, num_decls) &&
+                 !is_forbidden(to_app(neg_arg)->get_decl()) &&
                  !poly_contains_head(lhs, to_app(neg_arg)->get_decl(), arg)) {
             h = neg_arg;
             inv = true;
@@ -287,23 +287,24 @@ bool macro_util::is_arith_macro(expr * n, unsigned num_decls, app_ref & head, ex
             args.push_back(arg);
         }
     }
-    if (h == 0)
+    if (h == nullptr)
         return false;
     head = to_app(h);
     expr_ref tmp(m_manager);
-    as->mk_add(args.size(), args.c_ptr(), tmp);
+    tmp = m_arith.mk_add(args.size(), args.c_ptr());
     if (inv)
-        as->mk_sub(tmp, rhs, def);
+        mk_sub(tmp, rhs, def);
     else
-        as->mk_sub(rhs, tmp, def);
+        mk_sub(rhs, tmp, def);
+    TRACE("macro_util", tout << def << "\n";);
     return true;
 }
 
 /**
    \brief Auxiliary function for is_pseudo_predicate_macro. It detects the pattern (= (f X) t)
 */
-bool macro_util::is_pseudo_head(expr * n, unsigned num_decls, app * & head, app * & t) {
-    if (!m_manager.is_eq(n)) 
+bool macro_util::is_pseudo_head(expr * n, unsigned num_decls, app_ref & head, app_ref & t) {
+    if (!m_manager.is_eq(n))
         return false;
     expr * lhs = to_app(n)->get_arg(0);
     expr * rhs = to_app(n)->get_arg(1);
@@ -330,26 +331,25 @@ bool macro_util::is_pseudo_head(expr * n, unsigned num_decls, app * & head, app
 
 /**
    \brief Returns true if n if of the form (forall (X) (iff (= (f X) t) def[X]))
-   where t is a ground term, (f X) is the head. 
+   where t is a ground term, (f X) is the head.
 */
-bool macro_util::is_pseudo_predicate_macro(expr * n, app * & head, app * & t, expr * & def) {
-    if (!is_quantifier(n) || !to_quantifier(n)->is_forall())
+bool macro_util::is_pseudo_predicate_macro(expr * n, app_ref & head, app_ref & t, expr_ref & def) {
+    if (!is_forall(n)) 
         return false;
     TRACE("macro_util", tout << "processing: " << mk_pp(n, m_manager) << "\n";);
     expr * body        = to_quantifier(n)->get_expr();
     unsigned num_decls = to_quantifier(n)->get_num_decls();
-    if (!m_manager.is_iff(body))
+    expr * lhs, *rhs;
+    if (!m_manager.is_iff(body, lhs, rhs))
         return false;
-    expr * lhs = to_app(body)->get_arg(0);
-    expr * rhs = to_app(body)->get_arg(1);
-    if (is_pseudo_head(lhs, num_decls, head, t) && 
-        !is_forbidden(head->get_decl()) && 
+    if (is_pseudo_head(lhs, num_decls, head, t) &&
+        !is_forbidden(head->get_decl()) &&
         !occurs(head->get_decl(), rhs)) {
         def = rhs;
         return true;
     }
-    if (is_pseudo_head(rhs, num_decls, head, t) && 
-        !is_forbidden(head->get_decl()) && 
+    if (is_pseudo_head(rhs, num_decls, head, t) &&
+        !is_forbidden(head->get_decl()) &&
         !occurs(head->get_decl(), lhs)) {
         def = lhs;
         return true;
@@ -360,7 +360,7 @@ bool macro_util::is_pseudo_predicate_macro(expr * n, app * & head, app * & t, ex
 /**
    \brief A quasi-macro head is of the form f[X_1, ..., X_n],
    where n == num_decls, f[X_1, ..., X_n] is a term starting with symbol f, f is uninterpreted,
-   contains all universally quantified variables as arguments. 
+   contains all universally quantified variables as arguments.
    Note that, some arguments of f[X_1, ..., X_n] may not be variables.
 
    Examples of quasi-macros:
@@ -402,7 +402,7 @@ bool macro_util::is_quasi_macro_head(expr * n, unsigned num_decls) const {
    \brief Convert a quasi-macro head into a macro head, and store the conditions under
    which it is valid in cond.
 */
-void macro_util::quasi_macro_head_to_macro_head(app * qhead, unsigned num_decls, app_ref & head, expr_ref & cond) const {
+void macro_util::quasi_macro_head_to_macro_head(app * qhead, unsigned & num_decls, app_ref & head, expr_ref & cond) const {
     unsigned num_args = qhead->get_num_args();
     sbuffer<bool> found_vars;
     found_vars.resize(num_decls, false);
@@ -426,8 +426,9 @@ void macro_util::quasi_macro_head_to_macro_head(app * qhead, unsigned num_decls,
         new_args.push_back(new_var);
         new_conds.push_back(new_cond);
     }
-    get_basic_simp()->mk_and(new_conds.size(), new_conds.c_ptr(), cond);
+    bool_rewriter(m_manager).mk_and(new_conds.size(), new_conds.c_ptr(), cond);
     head = m_manager.mk_app(qhead->get_decl(), new_args.size(), new_args.c_ptr());
+    num_decls = next_var_idx;
 }
 
 /**
@@ -437,10 +438,10 @@ void macro_util::quasi_macro_head_to_macro_head(app * qhead, unsigned num_decls,
 
    See normalize_expr
 */
-void macro_util::mk_macro_interpretation(app * head, expr * def, expr_ref & interp) const {
+void macro_util::mk_macro_interpretation(app * head, unsigned num_decls, expr * def, expr_ref & interp) const {
     SASSERT(is_macro_head(head, head->get_num_args()));
     SASSERT(!occurs(head->get_decl(), def));
-    normalize_expr(head, def, interp);
+    normalize_expr(head, num_decls, def, interp);
 }
 
 /**
@@ -453,42 +454,38 @@ void macro_util::mk_macro_interpretation(app * head, expr * def, expr_ref & inte
    f(x_1, x_2) --> f(x_0, x_1)
    f(x_3, x_2) --> f(x_0, x_1)
 */
-void macro_util::normalize_expr(app * head, expr * t, expr_ref & norm_t) const {
-    expr_ref_buffer  var_mapping(m_manager);
+void macro_util::normalize_expr(app * head, unsigned num_decls, expr * t, expr_ref & norm_t) const {
+    expr_ref_buffer var_mapping(m_manager);
+    var_mapping.resize(num_decls);
     bool changed = false;
     unsigned num_args = head->get_num_args();
-    unsigned max      = num_args;
-    for (unsigned i = 0; i < num_args; i++) {
-        var * v     = to_var(head->get_arg(i));
-        if (v->get_idx() >= max)
-            max = v->get_idx() + 1;
-    }
-    TRACE("normalize_expr_bug",
+    TRACE("macro_util",
           tout << "head: " << mk_pp(head, m_manager) << "\n";
           tout << "applying substitution to:\n" << mk_bounded_pp(t, m_manager) << "\n";);
     for (unsigned i = 0; i < num_args; i++) {
-        var * v     = to_var(head->get_arg(i));
-        if (v->get_idx() != i) {
+        var * v = to_var(head->get_arg(i));
+        unsigned vi = v->get_idx();
+        SASSERT(vi < num_decls);
+        if (vi != i) {
             changed = true;
-            var * new_var = m_manager.mk_var(i, v->get_sort());
-            CTRACE("normalize_expr_bug", v->get_idx() >= num_args, tout << mk_pp(v, m_manager) << ", num_args: " << num_args << "\n";);
-            SASSERT(v->get_idx() < max);
-            var_mapping.setx(max - v->get_idx() - 1, new_var);
-        }
-        else {
-            var_mapping.setx(max - i - 1, v);
+            var_ref new_var(m_manager.mk_var(i, v->get_sort()), m_manager);
+            var_mapping.setx(num_decls - vi - 1, new_var);
         }
+        else
+            var_mapping.setx(num_decls - i - 1, v);
     }
+
     if (changed) {
         // REMARK: t may have nested quantifiers... So, I must use the std order for variable substitution.
-        var_subst subst(m_manager); 
-        TRACE("macro_util_bug",
+        var_subst subst(m_manager, true);
+        TRACE("macro_util",
               tout << "head: " << mk_pp(head, m_manager) << "\n";
-              tout << "applying substitution to:\n" << mk_ll_pp(t, m_manager) << "\nsubstitituion:\n";
+              tout << "applying substitution to:\n" << mk_ll_pp(t, m_manager) << "\nsubstitution:\n";
               for (unsigned i = 0; i < var_mapping.size(); i++) {
-                  tout << "#" << i << " -> " << mk_pp(var_mapping[i], m_manager) << "\n";
+                  if (var_mapping[i] != 0)
+                      tout << "#" << i << " -> " << mk_ll_pp(var_mapping[i], m_manager);
               });
-        subst(t, var_mapping.size(), var_mapping.c_ptr(), norm_t);
+        norm_t = subst(t, var_mapping.size(), var_mapping.c_ptr());
     }
     else {
         norm_t = t;
@@ -497,8 +494,8 @@ void macro_util::normalize_expr(app * head, expr * t, expr_ref & norm_t) const {
 
 // -----------------------------
 //
-// "Hint" support 
-// See comment at is_hint_atom 
+// "Hint" support
+// See comment at is_hint_atom
 // for a definition of what a hint is.
 //
 // -----------------------------
@@ -510,7 +507,7 @@ bool is_hint_head(expr * n, ptr_buffer<var> & vars) {
         return false;
     unsigned num_args = to_app(n)->get_num_args();
     for (unsigned i = 0; i < num_args; i++) {
-        expr * arg = to_app(n)->get_arg(i); 
+        expr * arg = to_app(n)->get_arg(i);
         if (is_var(arg))
             vars.push_back(to_var(arg));
     }
@@ -546,7 +543,7 @@ bool vars_of_is_subset(expr * n, ptr_buffer<var> const & vars) {
             }
         }
         else {
-            SASSERT(is_quantifier(curr)); 
+            SASSERT(is_quantifier(curr));
             return false; // do no support nested quantifier... being conservative.
         }
     }
@@ -554,7 +551,7 @@ bool vars_of_is_subset(expr * n, ptr_buffer<var> const & vars) {
 }
 
 /**
-   \brief (= lhs rhs) is a hint atom if 
+   \brief (= lhs rhs) is a hint atom if
    lhs is of the form (f t_1 ... t_n)
    and all variables occurring in rhs are direct arguments of lhs.
 */
@@ -565,7 +562,7 @@ bool is_hint_atom(expr * lhs, expr * rhs) {
     return !occurs(to_app(lhs)->get_decl(), rhs) && vars_of_is_subset(rhs, vars);
 }
 
-void hint_to_macro_head(ast_manager & m, app * head, unsigned num_decls, app_ref & new_head) {
+void hint_to_macro_head(ast_manager & m, app * head, unsigned & num_decls, app_ref & new_head) {
     unsigned num_args = head->get_num_args();
     ptr_buffer<expr> new_args;
     sbuffer<bool> found_vars;
@@ -587,21 +584,22 @@ void hint_to_macro_head(ast_manager & m, app * head, unsigned num_decls, app_ref
         new_args.push_back(new_var);
     }
     new_head = m.mk_app(head->get_decl(), new_args.size(), new_args.c_ptr());
+    num_decls = next_var_idx;
 }
 
 /**
    \brief Return true if n can be viewed as a polynomial "hint" based on head.
    That is, n (but the monomial exception) only uses the variables in head, and does not use
-   head->get_decl(). 
+   head->get_decl().
    is_hint_head(head, vars) must also return true
 */
 bool macro_util::is_poly_hint(expr * n, app * head, expr * exception) {
-    TRACE("macro_util_hint", tout << "is_poly_hint n:\n" << mk_pp(n, m_manager) << "\nhead:\n" << mk_pp(head, m_manager) << "\nexception:\n";
+    TRACE("macro_util", tout << "is_poly_hint n:\n" << mk_pp(n, m_manager) << "\nhead:\n" << mk_pp(head, m_manager) << "\nexception:\n";
           if (exception) tout << mk_pp(exception, m_manager); else tout << "<null>";
           tout << "\n";);
     ptr_buffer<var> vars;
     if (!is_hint_head(head, vars)) {
-        TRACE("macro_util_hint", tout << "failed because head is not hint head\n";);
+        TRACE("macro_util", tout << "failed because head is not hint head\n";);
         return false;
     }
     func_decl * f = head->get_decl();
@@ -618,13 +616,13 @@ bool macro_util::is_poly_hint(expr * n, app * head, expr * exception) {
     for (unsigned i = 0; i < num_args; i++) {
         expr * arg = args[i];
         if (arg != exception && (occurs(f, arg) || !vars_of_is_subset(arg, vars))) {
-            TRACE("macro_util_hint", tout << "failed because of:\n" << mk_pp(arg, m_manager) << "\n";);
+            TRACE("macro_util", tout << "failed because of:\n" << mk_pp(arg, m_manager) << "\n";);
             return false;
         }
     }
-    TRACE("macro_util_hint", tout << "succeeded\n";);
+    TRACE("macro_util", tout << "succeeded\n";);
     return true;
-    
+
 }
 
 // -----------------------------
@@ -663,19 +661,19 @@ void macro_util::macro_candidates::insert(func_decl * f, expr * def, expr * cond
 //
 // -----------------------------
 
-void macro_util::insert_macro(app * head, expr * def, expr * cond, bool ineq, bool satisfy_atom, bool hint, macro_candidates & r) {
+void macro_util::insert_macro(app * head, unsigned num_decls, expr * def, expr * cond, bool ineq, bool satisfy_atom, bool hint, macro_candidates & r) {
     expr_ref norm_def(m_manager);
     expr_ref norm_cond(m_manager);
-    normalize_expr(head, def, norm_def);
-    if (cond != 0)
-        normalize_expr(head, cond, norm_cond);
+    normalize_expr(head, num_decls, def, norm_def);
+    if (cond != nullptr)
+        normalize_expr(head, num_decls, cond, norm_cond);
     else if (!hint)
         norm_cond = m_manager.mk_true();
     SASSERT(!hint || norm_cond.get() == 0);
     r.insert(head->get_decl(), norm_def.get(), norm_cond.get(), ineq, satisfy_atom, hint);
 }
 
-void macro_util::insert_quasi_macro(app * head, unsigned num_decls, expr * def, expr * cond, bool ineq, bool satisfy_atom, 
+void macro_util::insert_quasi_macro(app * head, unsigned num_decls, expr * def, expr * cond, bool ineq, bool satisfy_atom,
                                     bool hint, macro_candidates & r) {
     if (!is_macro_head(head, head->get_num_args())) {
         app_ref  new_head(m_manager);
@@ -683,23 +681,26 @@ void macro_util::insert_quasi_macro(app * head, unsigned num_decls, expr * def,
         expr_ref new_cond(m_manager);
         if (!hint) {
             quasi_macro_head_to_macro_head(head, num_decls, new_head, extra_cond);
-            if (cond == 0)
+            if (cond == nullptr)
                 new_cond = extra_cond;
             else
-                get_basic_simp()->mk_and(cond, extra_cond, new_cond);
+                bool_rewriter(m_manager).mk_and(cond, extra_cond, new_cond);
         }
         else {
             hint_to_macro_head(m_manager, head, num_decls, new_head);
+            TRACE("macro_util",
+                tout << "hint macro head: " << mk_ismt2_pp(new_head, m_manager) << std::endl;
+                tout << "hint macro def: " << mk_ismt2_pp(def, m_manager) << std::endl; );
         }
-        insert_macro(new_head, def, new_cond, ineq, satisfy_atom, hint, r);
+        insert_macro(new_head, num_decls, def, new_cond, ineq, satisfy_atom, hint, r);
     }
     else {
-        insert_macro(head, def, cond, ineq, satisfy_atom, hint, r);
+        insert_macro(head, num_decls, def, cond, ineq, satisfy_atom, hint, r);
     }
 }
 
 bool macro_util::rest_contains_decl(func_decl * f, expr * except_lit) {
-    if (m_curr_clause == 0)
+    if (m_curr_clause == nullptr)
         return false;
     SASSERT(is_clause(m_manager, m_curr_clause));
     unsigned num_lits = get_clause_num_literals(m_manager, m_curr_clause);
@@ -712,23 +713,22 @@ bool macro_util::rest_contains_decl(func_decl * f, expr * except_lit) {
 }
 
 void macro_util::get_rest_clause_as_cond(expr * except_lit, expr_ref & extra_cond) {
-    if (m_curr_clause == 0)
+    if (m_curr_clause == nullptr)
         return;
     SASSERT(is_clause(m_manager, m_curr_clause));
-    basic_simplifier_plugin * bs = get_basic_simp();
     expr_ref_buffer neg_other_lits(m_manager);
     unsigned num_lits = get_clause_num_literals(m_manager, m_curr_clause);
     for (unsigned i = 0; i < num_lits; i++) {
         expr * l = get_clause_literal(m_manager, m_curr_clause, i);
         if (l != except_lit) {
             expr_ref neg_l(m_manager);
-            bs->mk_not(l, neg_l);
+            bool_rewriter(m_manager).mk_not(l, neg_l);
             neg_other_lits.push_back(neg_l);
         }
     }
     if (neg_other_lits.empty())
         return;
-    get_basic_simp()->mk_and(neg_other_lits.size(), neg_other_lits.c_ptr(), extra_cond);
+    bool_rewriter(m_manager).mk_and(neg_other_lits.size(), neg_other_lits.c_ptr(), extra_cond);
 }
 
 void macro_util::collect_poly_args(expr * n, expr * exception, ptr_buffer<expr> & args) {
@@ -777,8 +777,8 @@ void macro_util::collect_arith_macro_candidates(expr * lhs, expr * rhs, expr * a
         if (!is_app(arg))
             continue;
         func_decl * f = to_app(arg)->get_decl();
-        
-        bool _is_arith_macro = 
+
+        bool _is_arith_macro =
             is_quasi_macro_head(arg, num_decls) &&
             !is_forbidden(f) &&
             !poly_contains_head(lhs, f, arg) &&
@@ -794,19 +794,19 @@ void macro_util::collect_arith_macro_candidates(expr * lhs, expr * rhs, expr * a
             mk_sub(rhs, rest, def);
             // If is_poly_hint, rhs may contain variables that do not occur in to_app(arg).
             // So, we should re-check.
-            if (!_is_poly_hint || is_poly_hint(def, to_app(arg), 0))
+            if (!_is_poly_hint || is_poly_hint(def, to_app(arg), nullptr))
                 add_arith_macro_candidate(to_app(arg), num_decls, def, atom, is_ineq, _is_poly_hint, r);
         }
         else if (is_times_minus_one(arg, neg_arg) && is_app(neg_arg)) {
             f = to_app(neg_arg)->get_decl();
-            bool _is_arith_macro = 
+            bool _is_arith_macro =
                 is_quasi_macro_head(neg_arg, num_decls) &&
                 !is_forbidden(f) &&
                 !poly_contains_head(lhs, f, arg) &&
                 !occurs(f, rhs) &&
                 !rest_contains_decl(f, atom);
             bool _is_poly_hint   = !_is_arith_macro && is_poly_hint(lhs, to_app(neg_arg), arg);
-            
+
             if (_is_arith_macro || _is_poly_hint) {
                 collect_poly_args(lhs, arg, args);
                 expr_ref rest(m_manager);
@@ -815,7 +815,7 @@ void macro_util::collect_arith_macro_candidates(expr * lhs, expr * rhs, expr * a
                 mk_sub(rest, rhs, def);
                 // If is_poly_hint, rhs may contain variables that do not occur in to_app(neg_arg).
                 // So, we should re-check.
-                if (!_is_poly_hint || is_poly_hint(def, to_app(neg_arg), 0))
+                if (!_is_poly_hint || is_poly_hint(def, to_app(neg_arg), nullptr))
                     add_arith_macro_candidate(to_app(neg_arg), num_decls, def, atom, is_ineq, _is_poly_hint, r);
             }
         }
@@ -823,7 +823,7 @@ void macro_util::collect_arith_macro_candidates(expr * lhs, expr * rhs, expr * a
 }
 
 void macro_util::collect_arith_macro_candidates(expr * atom, unsigned num_decls, macro_candidates & r) {
-    TRACE("macro_util_hint", tout << "collect_arith_macro_candidates:\n" << mk_pp(atom, m_manager) << "\n";);
+    TRACE("macro_util", tout << "collect_arith_macro_candidates:\n" << mk_pp(atom, m_manager) << "\n";);
     if (!m_manager.is_eq(atom) && !is_le_ge(atom))
         return;
     expr * lhs = to_app(atom)->get_arg(0);
@@ -836,45 +836,47 @@ void macro_util::collect_arith_macro_candidates(expr * atom, unsigned num_decls,
 /**
    \brief Collect macro candidates for atom \c atom.
    The candidates are stored in \c r.
-   
+
    The following post-condition holds:
 
    for each i in [0, r.size() - 1]
       we have a conditional macro of the form
-      
+
       r.get_cond(i) IMPLIES f(x_1, ..., x_n) = r.get_def(i)
-      
+
       where
          f == r.get_fs(i) .., x_n), f is uninterpreted and x_1, ..., x_n are variables.
          r.get_cond(i) and r.get_defs(i) do not contain f or variables not in {x_1, ..., x_n}
 
       The idea is to use r.get_defs(i) as the interpretation for f in a model M whenever r.get_cond(i)
-   
-   Given a model M and values { v_1, ..., v_n } 
+
+   Given a model M and values { v_1, ..., v_n }
    Let M' be M{x_1 -> v_1, ..., v_n -> v_n}
-   
+
       Note that M'(f(x_1, ..., x_n)) = M(f)(v_1, ..., v_n)
-      
+
       Then, IF we have that M(f)(v_1, ..., v_n) = M'(r.get_def(i)) AND
                             M'(r.get_cond(i))   = true
             THEN M'(atom) = true
 
       That is, if the conditional macro is used then the atom is satisfied when  M'(r.get_cond(i)) = true
-      
+
       IF r.is_ineq(i) = false, then
       M(f)(v_1, ..., v_n) ***MUST BE*** M'(r.get_def(i)) whenever M'(r.get_cond(i)) = true
- 
+
       IF r.satisfy_atom(i) = true, then we have the stronger property:
 
       Then, IF we have that (M'(r.get_cond(i)) = true IMPLIES M(f)(v_1, ..., v_n) = M'(r.get_def(i)))
             THEN M'(atom) = true
 */
 void macro_util::collect_macro_candidates_core(expr * atom, unsigned num_decls, macro_candidates & r) {
-    if (m_manager.is_eq(atom) || m_manager.is_iff(atom)) {
-        expr * lhs = to_app(atom)->get_arg(0);
-        expr * rhs = to_app(atom)->get_arg(1);
-        if (is_quasi_macro_head(lhs, num_decls) && 
-            !is_forbidden(to_app(lhs)->get_decl()) && 
+    expr* lhs, *rhs;
+
+    TRACE("macro_util", tout << "Candidate check for: " << mk_ismt2_pp(atom, m_manager) << std::endl;);
+
+    if (m_manager.is_eq(atom, lhs, rhs) || m_manager.is_iff(atom, lhs, rhs)) {
+        if (is_quasi_macro_head(lhs, num_decls) &&
+            !is_forbidden(to_app(lhs)->get_decl()) &&
             !occurs(to_app(lhs)->get_decl(), rhs) &&
             !rest_contains_decl(to_app(lhs)->get_decl(), atom)) {
             expr_ref cond(m_manager);
@@ -882,12 +884,11 @@ void macro_util::collect_macro_candidates_core(expr * atom, unsigned num_decls,
             insert_quasi_macro(to_app(lhs), num_decls, rhs, cond, false, true, false, r);
         }
         else if (is_hint_atom(lhs, rhs)) {
-            insert_quasi_macro(to_app(lhs), num_decls, rhs, 0, false, true, true, r);
+            insert_quasi_macro(to_app(lhs), num_decls, rhs, nullptr, false, true, true, r);
         }
 
-        
-        if (is_quasi_macro_head(rhs, num_decls) && 
-            !is_forbidden(to_app(rhs)->get_decl()) && 
+        if (is_quasi_macro_head(rhs, num_decls) &&
+            !is_forbidden(to_app(rhs)->get_decl()) &&
             !occurs(to_app(rhs)->get_decl(), lhs) &&
             !rest_contains_decl(to_app(rhs)->get_decl(), atom)) {
             expr_ref cond(m_manager);
@@ -895,7 +896,7 @@ void macro_util::collect_macro_candidates_core(expr * atom, unsigned num_decls,
             insert_quasi_macro(to_app(rhs), num_decls, lhs, cond, false, true, false, r);
         }
         else if (is_hint_atom(rhs, lhs)) {
-            insert_quasi_macro(to_app(rhs), num_decls, lhs, 0, false, true, true, r);
+            insert_quasi_macro(to_app(rhs), num_decls, lhs, nullptr, false, true, true, r);
         }
     }
 
@@ -903,7 +904,7 @@ void macro_util::collect_macro_candidates_core(expr * atom, unsigned num_decls,
 }
 
 void macro_util::collect_macro_candidates(expr * atom, unsigned num_decls, macro_candidates & r) {
-    m_curr_clause = 0;
+    m_curr_clause = nullptr;
     r.reset();
     collect_macro_candidates_core(atom, num_decls, r);
 }
@@ -920,7 +921,7 @@ void macro_util::collect_macro_candidates(quantifier * q, macro_candidates & r)
         unsigned num_lits = get_clause_num_literals(m_manager, n);
         for (unsigned i = 0; i < num_lits; i++)
             collect_macro_candidates_core(get_clause_literal(m_manager, n, i), num_decls, r);
-        m_curr_clause = 0;
+        m_curr_clause = nullptr;
     }
     else {
         collect_macro_candidates_core(n, num_decls, r);
