@@ -20,9 +20,9 @@ Notes:
 #ifndef NNF_H_
 #define NNF_H_
 
-#include"ast.h"
-#include"params.h"
-#include"defined_names.h"
+#include "ast/ast.h"
+#include "util/params.h"
+#include "ast/normal_forms/defined_names.h"
 
 class nnf {
     struct imp;
@@ -44,10 +44,6 @@ public:
     */
     static void get_param_descrs(param_descrs & r);
 
-    void cancel() { set_cancel(true); }
-    void reset_cancel() { set_cancel(false); }
-    void set_cancel(bool f);
-
     void reset();
     void reset_cache();
 };
