@@ -19,8 +19,8 @@ Revision History:
 #ifndef NUM_OCCURS_H_
 #define NUM_OCCURS_H_
 
-#include"ast.h"
-#include"obj_hashtable.h"
+#include "ast/ast.h"
+#include "util/obj_hashtable.h"
 
 /**
    \brief Functor for computing the number of occurrences of each sub-expression in a expression F.
