@@ -19,7 +19,7 @@ Revision History:
 #ifndef PATTERN_INFERENCE_PARAMS_H_
 #define PATTERN_INFERENCE_PARAMS_H_
 
-#include"params.h"
+#include "util/params.h"
 
 enum arith_pattern_inference_kind {
     AP_NO,           // do not infer patterns with arithmetic terms
@@ -46,6 +46,8 @@ struct pattern_inference_params {
     }
 
     void updt_params(params_ref const & _p);
+
+    void display(std::ostream & out) const;
 };
 
 #endif /* PATTERN_INFERENCE_PARAMS_H_ */
