@@ -19,7 +19,6 @@ Revision History:
 #include "ast/rewriter/bit_blaster/bit_blaster_tpl.h"
 #include "util/rational.h"
 #include "ast/ast_pp.h"
-#include "util/cooperate.h"
 #include "util/common_msgs.h"
 #include "ast/rewriter/rewriter_types.h"
 
@@ -30,7 +29,6 @@ void bit_blaster_tpl<Cfg>::checkpoint() {
         throw rewriter_exception(Z3_MAX_MEMORY_MSG);
     if (m().canceled())
         throw rewriter_exception(m().limit().get_cancel_msg());
-    cooperate("bit-blaster");
 }
 
 /**
@@ -75,9 +73,23 @@ bool bit_blaster_tpl<Cfg>::is_minus_one(unsigned sz, expr * const * bits) const
 static void _num2bits(ast_manager & m, rational const & v, unsigned sz, expr_ref_vector & out_bits) {
     SASSERT(v.is_nonneg());
     rational aux = v;
-    rational two(2);
+    rational two(2), base32(1ull << 32ull, rational::ui64());
     for (unsigned i = 0; i < sz; i++) {
-        if ((aux % two).is_zero())
+        if (i + 32 < sz) {
+            unsigned u = (aux % base32).get_unsigned();
+            for (unsigned j = 0; j < 32; ++j) {
+                if (0 != (u & (1 << j))) {
+                    out_bits.push_back(m.mk_true());
+                }
+                else {
+                    out_bits.push_back(m.mk_false());
+                }
+            }
+            aux = div(aux, base32);
+            i += 31;
+            continue;
+        }
+        else if ((aux % two).is_zero())
             out_bits.push_back(m.mk_false());
         else
             out_bits.push_back(m.mk_true());
