@@ -22,7 +22,7 @@ Revision History:
 #ifndef EXPR_SAFE_REPLACE_H_
 #define EXPR_SAFE_REPLACE_H_
 
-#include "ast.h"
+#include "ast/ast.h"
 
 class expr_safe_replace {
     ast_manager& m;
@@ -42,6 +42,8 @@ public:
 
     void operator()(expr* src, expr_ref& e);
 
+    void operator()(expr_ref_vector& es);
+
     void apply_substitution(expr* s, expr* def, expr_ref& t);
 
     void reset();
