@@ -28,7 +28,7 @@ Notes:
 #include "util/uint_set.h"
 #include "util/gparams.h"
 
-const unsigned g_primes[7] = { 2, 3, 5, 7, 11, 13, 17};
+static const unsigned g_primes[7] = { 2, 3, 5, 7, 11, 13, 17};
 
 
 struct pb2bv_rewriter::imp {
@@ -161,7 +161,6 @@ struct pb2bv_rewriter::imp {
             }
 
             if (m_pb_solver == "segmented") {
-                expr_ref result(m);
                 switch (is_le) {
                 case l_true:  return mk_seg_le(k);
                 case l_false: return mk_seg_ge(k);
@@ -169,6 +168,11 @@ struct pb2bv_rewriter::imp {
                 }
             }
 
+            if (m_pb_solver == "binary_merge") {
+                expr_ref result = binary_merge(is_le, k);
+                if (result) return result;
+            }
+
             // fall back to divide and conquer encoding.
             SASSERT(k.is_pos());
             expr_ref zero(m), bound(m);
@@ -366,7 +370,7 @@ struct pb2bv_rewriter::imp {
         rational         m_min_cost;
         vector<rational> m_base;
 
-        void create_basis(vector<rational> const& seq, rational carry_in, rational cost) {
+        void create_basis(vector<rational> const& seq, rational const& carry_in, rational const& cost) {
             if (cost >= m_min_cost) {
                 return;
             }
@@ -494,6 +498,37 @@ struct pb2bv_rewriter::imp {
             return true;
         }
 
+        /**
+           \brief binary merge encoding.
+        */
+        expr_ref binary_merge(lbool is_le, rational const& k) {
+            expr_ref result(m);
+            unsigned_vector coeffs;
+            for (rational const& c : m_coeffs) {
+                if (c.is_unsigned()) {
+                    coeffs.push_back(c.get_unsigned());
+                }
+                else {
+                    return result;
+                }
+            }
+            if (!k.is_unsigned()) {
+                return result;
+            }
+            switch (is_le) {
+            case l_true: 
+                result = m_sort.le(k.get_unsigned(), coeffs.size(), coeffs.c_ptr(), m_args.c_ptr());
+                break;
+            case l_false:
+                result = m_sort.ge(k.get_unsigned(), coeffs.size(), coeffs.c_ptr(), m_args.c_ptr());
+                break;
+            case l_undef:
+                result = m_sort.eq(k.get_unsigned(), coeffs.size(), coeffs.c_ptr(), m_args.c_ptr());
+                break;
+            }
+            return result;
+        }
+
         /**
            \brief Segment based encoding.
            The PB terms are partitoned into segments, such that each segment contains arguments with the same cofficient.
@@ -891,6 +926,8 @@ struct pb2bv_rewriter::imp {
 
         void set_cardinality_encoding(sorting_network_encoding enc) { m_sort.cfg().m_encoding = enc; }
 
+        void set_min_arity(unsigned ma) { m_min_arity = ma; }
+
     };
 
     struct card2bv_rewriter_cfg : public default_rewriter_cfg {
@@ -905,6 +942,7 @@ struct pb2bv_rewriter::imp {
         void keep_cardinality_constraints(bool f) { m_r.keep_cardinality_constraints(f); }
         void set_pb_solver(symbol const& s) { m_r.set_pb_solver(s); }
         void set_cardinality_encoding(sorting_network_encoding enc) { m_r.set_cardinality_encoding(enc); }
+        void set_min_arity(unsigned ma) { m_r.set_min_arity(ma); }
 
     };
     
@@ -917,6 +955,7 @@ struct pb2bv_rewriter::imp {
         void keep_cardinality_constraints(bool f) { m_cfg.keep_cardinality_constraints(f); }
         void set_pb_solver(symbol const& s) { m_cfg.set_pb_solver(s); }
         void set_cardinality_encoding(sorting_network_encoding e) { m_cfg.set_cardinality_encoding(e); }
+        void set_min_arity(unsigned ma) { m_cfg.set_min_arity(ma); }
         void rewrite(bool full, expr* e, expr_ref& r, proof_ref& p) {
             expr_ref ee(e, m());
             if (m_cfg.m_r.mk_app(full, e, r)) {
@@ -947,6 +986,15 @@ struct pb2bv_rewriter::imp {
         return gparams::get_module("sat").get_sym("pb.solver", symbol("solver"));
     }
 
+    unsigned min_arity() const {
+        params_ref const& p = m_params;
+        unsigned r = p.get_uint("sat.pb.min_arity", UINT_MAX);
+        if (r != UINT_MAX) return r;
+        r = p.get_uint("pb.min_arity", UINT_MAX);
+        if (r != UINT_MAX) return r;
+        return gparams::get_module("sat").get_uint("pb.min_arity", 9);
+    }
+
     sorting_network_encoding cardinality_encoding() const {
         symbol enc = m_params.get_sym("cardinality.encoding", symbol());
         if (enc == symbol()) {
@@ -976,6 +1024,7 @@ struct pb2bv_rewriter::imp {
         m_rw.keep_cardinality_constraints(keep_cardinality());
         m_rw.set_pb_solver(pb_solver());
         m_rw.set_cardinality_encoding(cardinality_encoding());
+        m_rw.set_min_arity(min_arity());
     }
 
     void collect_param_descrs(param_descrs& r) const {
