@@ -38,9 +38,10 @@ void rewriter_tpl<Config>::process_var(var * v) {
     if (!ProofGen) {
         // bindings are only used when Proof Generation is not enabled.
         unsigned idx = v->get_idx();
+
         if (idx < m_bindings.size()) {
             unsigned index = m_bindings.size() - idx - 1;
-            var * r = (var*)(m_bindings[index]);
+            expr * r = m_bindings[index];
             if (r != nullptr) {
                 CTRACE("rewriter", v->get_sort() != m().get_sort(r),
                        tout << expr_ref(v, m()) << ":" << sort_ref(v->get_sort(), m()) << " != " << expr_ref(r, m()) << ":" << sort_ref(m().get_sort(r), m());
@@ -50,11 +51,18 @@ void rewriter_tpl<Config>::process_var(var * v) {
                 if (!is_ground(r) && m_shifts[index] != m_bindings.size()) {
 
                     unsigned shift_amount = m_bindings.size() - m_shifts[index];
+                    expr* c = get_cached(r, shift_amount);
+                    if (c) {
+                        result_stack().push_back(c);
+                        set_new_child_flag(v);
+                        return;
+                    }
                     expr_ref tmp(m());
                     m_shifter(r, shift_amount, tmp);
                     result_stack().push_back(tmp);
                     TRACE("rewriter", tout << "shift: " << shift_amount << " idx: " << idx << " --> " << tmp << "\n";
                           display_bindings(tout););
+                    cache_shifted_result(r, shift_amount, tmp);                    
                 }
                 else {
                     result_stack().push_back(r);
@@ -72,27 +80,42 @@ void rewriter_tpl<Config>::process_var(var * v) {
 
 template<typename Config>
 template<bool ProofGen>
-void rewriter_tpl<Config>::process_const(app * t) {
+bool rewriter_tpl<Config>::process_const(app * t0) {
+    app_ref t(t0, m());
+    bool retried = false;
+ retry:
     SASSERT(t->get_num_args() == 0);
     br_status st = m_cfg.reduce_app(t->get_decl(), 0, nullptr, m_r, m_pr);
     SASSERT(st != BR_DONE || m().get_sort(m_r) == m().get_sort(t));
-    SASSERT(st == BR_FAILED || st == BR_DONE);
-    if (st == BR_DONE) {
+    switch (st) {
+    case BR_FAILED:
+        if (!retried) {
+            result_stack().push_back(t);
+            if (ProofGen)
+                result_pr_stack().push_back(nullptr); // implicit reflexivity
+            return true;
+        }
+        m_r = t;
+        // fall through
+    case BR_DONE:
         result_stack().push_back(m_r.get());
         if (ProofGen) {
             if (m_pr)
                 result_pr_stack().push_back(m_pr);
             else
-                result_pr_stack().push_back(m().mk_rewrite(t, m_r));
+                result_pr_stack().push_back(m().mk_rewrite(t0, m_r));
             m_pr = nullptr;
         }
         m_r = nullptr;
-        set_new_child_flag(t);
-    }
-    else {
-        result_stack().push_back(t);
-        if (ProofGen)
-            result_pr_stack().push_back(nullptr); // implicit reflexivity
+        set_new_child_flag(t0);
+        return true;
+    default: 
+        if (is_app(m_r) && to_app(m_r)->get_num_args() == 0) {
+            t = to_app(m_r);
+            retried = true;
+            goto retry;
+        }
+        return false;
     }
 }
 
@@ -107,6 +130,7 @@ void rewriter_tpl<Config>::process_const(app * t) {
 template<typename Config>
 template<bool ProofGen>
 bool rewriter_tpl<Config>::visit(expr * t, unsigned max_depth) {
+ // retry:
     TRACE("rewriter_visit", tout << "visiting\n" << mk_ismt2_pp(t, m()) << "\n";);
     expr *  new_t = nullptr;
     proof * new_t_pr = nullptr;
@@ -156,15 +180,14 @@ bool rewriter_tpl<Config>::visit(expr * t, unsigned max_depth) {
     switch (t->get_kind()) {
     case AST_APP:
         if (to_app(t)->get_num_args() == 0) {
-            process_const<ProofGen>(to_app(t));
-            return true;
-        }
-        else {
-            if (max_depth != RW_UNBOUNDED_DEPTH)
-                max_depth--;
-            push_frame(t, c, max_depth);
-            return false;
+            if (process_const<ProofGen>(to_app(t))) 
+                return true; 
+            t = m_r;
         }
+        if (max_depth != RW_UNBOUNDED_DEPTH)
+            max_depth--;
+        push_frame(t, c, max_depth);
+        return false;        
     case AST_VAR:
         process_var<ProofGen>(to_var(t));
         return true;
@@ -216,7 +239,7 @@ bool rewriter_tpl<Config>::constant_fold(app * t, frame & fr) {
 template<typename Config>
 template<bool ProofGen>
 void rewriter_tpl<Config>::process_app(app * t, frame & fr) {
-    SASSERT(t->get_num_args() > 0);
+    // SASSERT(t->get_num_args() > 0);
     SASSERT(!frame_stack().empty());
     switch (fr.m_state) {
     case PROCESS_CHILDREN: {
@@ -380,7 +403,6 @@ void rewriter_tpl<Config>::process_app(app * t, frame & fr) {
                     TRACE("get_macro", display_bindings(tout););
                     begin_scope();
                     m_num_qvars += num_args;
-                    //m_num_qvars = 0;
                     m_root      = def;
                     push_frame(def, false, RW_UNBOUNDED_DEPTH);
                     return;
@@ -480,7 +502,7 @@ void rewriter_tpl<Config>::process_quantifier(quantifier * q, frame & fr) {
             m_root       = q->get_expr();
             unsigned sz = m_bindings.size();
             for (unsigned i = 0; i < num_decls; i++) {
-                m_bindings.push_back(0);
+                m_bindings.push_back(nullptr);
                 m_shifts.push_back(sz);
             }
         }
