@@ -19,8 +19,8 @@ Notes:
 #ifndef REWRITER_TYPES_H_
 #define REWRITER_TYPES_H_
 
-#include"z3_exception.h"
-#include"common_msgs.h"
+#include "util/z3_exception.h"
+#include "util/common_msgs.h"
 
 /**
    \brief Builtin rewrite result status
@@ -46,7 +46,7 @@ inline br_status unsigned2br_status(unsigned u) {
 
 class rewriter_exception : public default_exception {
 public:                                                
-    rewriter_exception(char const * msg):default_exception(msg) {}
+    rewriter_exception(std::string && msg) : default_exception(std::move(msg)) {}
 };
 
 #endif
