@@ -16,7 +16,6 @@ Author:
 Notes:
 
 --*/
-#include "util/cooperate.h"
 #include "ast/rewriter/th_rewriter.h"
 #include "ast/rewriter/rewriter_params.hpp"
 #include "ast/rewriter/bool_rewriter.h"
@@ -32,6 +31,7 @@ Notes:
 #include "ast/rewriter/var_subst.h"
 #include "ast/expr_substitution.h"
 #include "ast/ast_smt2_pp.h"
+#include "ast/ast_pp.h"
 #include "ast/ast_util.h"
 #include "ast/well_sorted.h"
 
@@ -107,7 +107,6 @@ struct th_rewriter_cfg : public default_rewriter_cfg {
     bool cache_all_results() const { return m_cache_all; }
 
     bool max_steps_exceeded(unsigned num_steps) const {
-        cooperate("simplifier");
         if (memory::get_allocation_size() > m_max_memory)
             throw rewriter_exception(Z3_MAX_MEMORY_MSG);
         return num_steps > m_max_steps;
@@ -184,7 +183,6 @@ struct th_rewriter_cfg : public default_rewriter_cfg {
                     st = m_ar_rw.mk_eq_core(args[0], args[1], result);
                 else if (s_fid == m_seq_rw.get_fid())
                     st = m_seq_rw.mk_eq_core(args[0], args[1], result);
-
                 if (st != BR_FAILED)
                     return st;
             }
@@ -559,9 +557,69 @@ struct th_rewriter_cfg : public default_rewriter_cfg {
         return BR_DONE;
     }
 
+    void count_down_subterm_references(expr * e, map<expr *, unsigned, ptr_hash<expr>, ptr_eq<expr>> & reference_map) {
+        if (is_app(e)) {
+            app * a = to_app(e);
+            for (unsigned i = 0; i < a->get_num_args(); ++i) {
+                expr * child = a->get_arg(i);
+                unsigned countdown = reference_map.get(child, child->get_ref_count()) - 1;
+                reference_map.insert(child,  countdown);
+                if (countdown == 0)
+                    count_down_subterm_references(child, reference_map);
+            }
+        }
+    }
+
+    void log_rewrite_axiom_instantiation(func_decl * f, unsigned num, expr * const * args, expr_ref & result, proof_ref & result_pr) {
+        family_id fid = f->get_family_id();
+        if (fid == m_b_rw.get_fid()) {
+            decl_kind k = f->get_decl_kind();
+            if (k == OP_EQ) {
+                SASSERT(num == 2);
+                fid = m().get_sort(args[0])->get_family_id();
+            }
+            else if (k == OP_ITE) {
+                SASSERT(num == 3);
+                fid = m().get_sort(args[1])->get_family_id();
+            }
+        }
+        app_ref tmp(m());
+        tmp = m().mk_app(f, num, args);
+        m().trace_stream() << "[inst-discovered] theory-solving " << static_cast<void *>(nullptr) << " " << m().get_family_name(fid) << "# ; #" << tmp->get_id() << "\n";
+        if (m().proofs_enabled())
+            result_pr = m().mk_rewrite(tmp, result);
+        tmp = m().mk_eq(tmp, result);
+        m().trace_stream() << "[instance] " << static_cast<void *>(nullptr) << " #" << tmp->get_id() << "\n";
+
+        // Make sure that both the result term and equality were newly introduced.
+        if (tmp->get_ref_count() == 1) {
+            if (result->get_ref_count() == 1) {
+                map<expr *, unsigned, ptr_hash<expr>, ptr_eq<expr>> reference_map;
+                count_down_subterm_references(result, reference_map);
+
+                // Any term that was newly introduced by the rewrite step is only referenced within / reachable from the result term.
+                for (auto kv : reference_map) {
+                    if (kv.m_value == 0) {
+                        m().trace_stream() << "[attach-enode] #" << kv.m_key->get_id() << " 0\n";
+                    }
+                }
+
+                m().trace_stream() << "[attach-enode] #" << result->get_id() << " 0\n";
+            }
+            m().trace_stream() << "[attach-enode] #" << tmp->get_id() << " 0\n";
+        }
+        m().trace_stream() << "[end-of-instance]\n";
+        m().trace_stream().flush();
+    }
+
     br_status reduce_app(func_decl * f, unsigned num, expr * const * args, expr_ref & result, proof_ref & result_pr) {
         result_pr = nullptr;
         br_status st = reduce_app_core(f, num, args, result);
+
+        if (st != BR_FAILED && m().has_trace_stream()) {
+            log_rewrite_axiom_instantiation(f, num, args, result, result_pr);
+        }
+
         if (st != BR_DONE && st != BR_FAILED) {
             CTRACE("th_rewriter_step", st != BR_FAILED,
                    tout << f->get_name() << "\n";
@@ -638,6 +696,13 @@ struct th_rewriter_cfg : public default_rewriter_cfg {
                 p1 = m().mk_pull_quant(old_q, q1);
             }
         }
+        else if (
+                 old_q->get_kind() == lambda_k &&
+                 is_ground(new_body)) {
+            result = m_ar_rw.util().mk_const_array(old_q->get_sort(), new_body);
+            result_pr = nullptr;
+            return true;
+        }
         else {
             ptr_buffer<expr> new_patterns_buf;
             ptr_buffer<expr> new_no_patterns_buf;
@@ -654,10 +719,10 @@ struct th_rewriter_cfg : public default_rewriter_cfg {
             TRACE("reduce_quantifier", tout << mk_ismt2_pp(old_q, m()) << "\n----->\n" << mk_ismt2_pp(q1, m()) << "\n";);
             SASSERT(is_well_sorted(m(), q1));
         }
-
         SASSERT(m().get_sort(old_q) == m().get_sort(q1));
         result = elim_unused_vars(m(), q1, params_ref());
 
+
         TRACE("reduce_quantifier", tout << "after elim_unused_vars:\n" << result << "\n";);
 
         result_pr = nullptr;
@@ -816,3 +881,13 @@ expr_ref th_rewriter::mk_app(func_decl* f, unsigned num_args, expr* const* args)
 void th_rewriter::set_solver(expr_solver* solver) {
     m_imp->set_solver(solver);
 }
+
+
+bool th_rewriter::reduce_quantifier(quantifier * old_q, 
+                                    expr * new_body, 
+                                    expr * const * new_patterns, 
+                                    expr * const * new_no_patterns,
+                                    expr_ref & result,
+                                    proof_ref & result_pr) {
+    return m_imp->cfg().reduce_quantifier(old_q, new_body, new_patterns, new_no_patterns, result, result_pr);
+}
