@@ -41,7 +41,7 @@ public:
     static void get_param_descrs(param_descrs & r);
     unsigned get_cache_size() const;
     unsigned get_num_steps() const;
-
+   
     void operator()(expr_ref& term);
     void operator()(expr * t, expr_ref & result);
     void operator()(expr * t, expr_ref & result, proof_ref & result_pr);
@@ -49,6 +49,13 @@ public:
 
     expr_ref mk_app(func_decl* f, unsigned num_args, expr* const* args);
 
+    bool reduce_quantifier(quantifier * old_q, 
+                           expr * new_body, 
+                           expr * const * new_patterns, 
+                           expr * const * new_no_patterns,
+                           expr_ref & result,
+                           proof_ref & result_pr);
+
     void cleanup();
     void reset();
 
