@@ -21,13 +21,11 @@ Revision History:
 #include "util/ref_util.h"
 
 inline void shared_occs::insert(expr * t) {
-    obj_hashtable<expr>::entry * dummy;
-    if (m_shared.insert_if_not_there_core(t, dummy))
-        m.inc_ref(t);
+    m_shared.reserve(t->get_id() + 1);
+    m_shared[t->get_id()] = t;
 }
 
 void shared_occs::reset() { 
-    dec_ref_collection_values(m, m_shared);
     m_shared.reset(); 
 }
 
@@ -132,9 +130,15 @@ void shared_occs::operator()(expr * t) {
 }
 
 void shared_occs::display(std::ostream & out, ast_manager & m) const {
-    iterator it =  begin_shared();
-    iterator end = end_shared();
-    for (; it != end; ++it) {
-        out << mk_ismt2_pp(*it, m) << "\n";
+    for (expr* s : m_shared) {
+        if (s) {
+            out << mk_ismt2_pp(s, m) << "\n";
+        }
     }
 }
+
+unsigned shared_occs::num_shared() const{ 
+    unsigned count = 0;
+    for (expr* s : m_shared) if (s) count++;
+    return count;
+}
