@@ -1674,9 +1674,9 @@ void cmd_context::display_dimacs() {
 void cmd_context::display_model(model_ref& mdl) {
     if (mdl) {
         if (m_mc0) (*m_mc0)(mdl);
-        if (m_params.m_model_compress) mdl->compress();
-        add_declared_functions(*mdl);
         model_params p;
+        if (p.compact()) mdl->compress();
+        add_declared_functions(*mdl);
         if (p.v1() || p.v2()) {
             std::ostringstream buffer;
             model_v2_pp(buffer, *mdl, p.partial());
@@ -1912,6 +1912,19 @@ void cmd_context::validate_model() {
 }
 
 void cmd_context::analyze_failure(model_evaluator& ev, expr* a, bool expected_value) {
+    expr* c = nullptr, *t = nullptr, *e = nullptr;
+    if (m().is_not(a, e)) {
+        analyze_failure(ev, e, !expected_value);
+        return;
+    }
+    if (!expected_value && m().is_or(a)) {
+        for (expr* arg : *to_app(a)) {
+            if (ev.is_true(arg)) {
+                analyze_failure(ev, arg, false);
+                return;
+            }
+        }
+    }
     if (expected_value && m().is_and(a)) {
         for (expr* arg : *to_app(a)) {
             if (ev.is_false(arg)) {
@@ -1920,36 +1933,27 @@ void cmd_context::analyze_failure(model_evaluator& ev, expr* a, bool expected_va
             }
         }
     }
-    expr* c = nullptr, *t = nullptr, *e = nullptr;
     if (expected_value && m().is_ite(a, c, t, e)) {
         if (ev.is_true(c) && ev.is_false(t)) {
-            analyze_failure(ev, t, true);
+            if (!m().is_true(c)) analyze_failure(ev, c, false);
+            if (!m().is_false(t)) analyze_failure(ev, t, true);
             return;
         }
         if (ev.is_false(c) && ev.is_false(e)) {
-            analyze_failure(ev, e, true);
+            if (!m().is_false(c)) analyze_failure(ev, c, true);
+            if (!m().is_false(e)) analyze_failure(ev, e, true);
             return;
         }
     }
-    if (m().is_not(a, e)) {
-        analyze_failure(ev, e, !expected_value);
-        return;
-    }
-    if (!expected_value && m().is_or(a)) {
-        for (expr* arg : *to_app(a)) {
-            if (ev.is_false(arg)) {
-                analyze_failure(ev, arg, false);
-                return;
-            }
-        }
-    }
     if (!expected_value && m().is_ite(a, c, t, e)) {
         if (ev.is_true(c) && ev.is_true(t)) {
-            analyze_failure(ev, t, false);
+            if (!m().is_true(c)) analyze_failure(ev, c, false);
+            if (!m().is_true(t)) analyze_failure(ev, t, false);
             return;
         }
         if (ev.is_false(c) && ev.is_true(e)) {
-            analyze_failure(ev, e, false);
+            if (!m().is_false(c)) analyze_failure(ev, c, true);
+            if (!m().is_true(e)) analyze_failure(ev, e, false);
             return;
         }
     }
@@ -2180,7 +2184,7 @@ void cmd_context::dt_eh::operator()(sort * dt, pdecl* pd) {
             m_owner.insert(a);
         }
     }
-    if (!m_owner.m_scopes.empty()) {
+    if (!m_owner.m_scopes.empty() && !m_owner.m_global_decls) {
         m_owner.pm().inc_ref(pd);
         m_owner.m_psort_inst_stack.push_back(pd);
     }
