@@ -37,11 +37,11 @@ Notes:
 #ifndef HEAP_TRIE_H_
 #define HEAP_TRIE_H_
 
-#include "map.h"
-#include "vector.h"
-#include "buffer.h"
-#include "statistics.h"
-#include "small_object_allocator.h"
+#include "util/map.h"
+#include "util/vector.h"
+#include "util/buffer.h"
+#include "util/statistics.h"
+#include "util/small_object_allocator.h"
 
 
 template<typename Key, typename KeyLE, typename KeyHash, typename Value>
@@ -81,14 +81,14 @@ class heap_trie {
         Value m_value;
     public:
         leaf(): node(leaf_t) {}
-        virtual ~leaf() {}
+        ~leaf() override {}
         Value const& get_value() const { return m_value; }
         void set_value(Value const& v) { m_value = v; }
-        virtual void display(std::ostream& out, unsigned indent) const {
+        void display(std::ostream& out, unsigned indent) const override {
             out << " value: " << m_value;
         }
-        virtual unsigned num_nodes() const { return 1; }
-        virtual unsigned num_leaves() const { return this->ref_count()>0?1:0; }
+        unsigned num_nodes() const override { return 1; }
+        unsigned num_leaves() const override { return this->ref_count()>0?1:0; }
     };
 
     typedef buffer<std::pair<Key,node*>, true, 2> children_t;
@@ -99,7 +99,7 @@ class heap_trie {
     public:
         trie(): node(trie_t) {}
 
-        virtual ~trie() {
+        ~trie() override {
         }
 
         node* find_or_insert(Key k, node* n) {
@@ -137,7 +137,7 @@ class heap_trie {
         children_t const& nodes() const { return m_nodes; }
         children_t & nodes() { return m_nodes; }
 
-        virtual void display(std::ostream& out, unsigned indent) const {
+        void display(std::ostream& out, unsigned indent) const override {
             for (unsigned j = 0; j < m_nodes.size(); ++j) {
                 if (j != 0 || indent > 0) {
                     out << "\n";
@@ -151,7 +151,7 @@ class heap_trie {
             }
         }
 
-        virtual unsigned num_nodes() const {
+        unsigned num_nodes() const override {
             unsigned sz = 1;
             for (unsigned j = 0; j < m_nodes.size(); ++j) {
                 sz += m_nodes[j].second->num_nodes();
@@ -159,7 +159,7 @@ class heap_trie {
             return sz;
         }
 
-        virtual unsigned num_leaves() const {
+        unsigned num_leaves() const override {
             unsigned sz = 0;
             for (unsigned j = 0; j < m_nodes.size(); ++j) {
                 sz += m_nodes[j].second->num_leaves();
@@ -195,9 +195,9 @@ public:
         m_le(le),
         m_num_keys(0),
         m_do_reshuffle(4),
-        m_root(0),
-        m_spare_leaf(0),
-        m_spare_trie(0)
+        m_root(nullptr),
+        m_spare_leaf(nullptr),
+        m_spare_trie(nullptr)
     {}
 
     ~heap_trie() {
@@ -283,7 +283,7 @@ public:
         ++m_stats.m_num_removes;
         // assumption: key is in table.
         node* n = m_root;
-        node* m;
+        node* m = nullptr;
         for (unsigned i = 0; i < num_keys(); ++i) {
             n->dec_ref();
             VERIFY (to_trie(n)->find(get_key(keys, i), m));
