@@ -234,6 +234,11 @@ public:
 
     bool check_invariant(interval const & n) const;
 
+    /**
+       \brief b <- k
+    */
+    void set(numeral const& k, interval & b);
+
     /**
        \brief b <- -a
     */
