@@ -19,7 +19,7 @@ Notes:
 #ifndef POLYNOMIAL_CACHE_H_
 #define POLYNOMIAL_CACHE_H_
 
-#include"polynomial.h"
+#include "math/polynomial/polynomial.h"
 
 namespace polynomial {
 
