@@ -332,7 +332,7 @@ namespace simplex {
         SASSERT(well_formed());
         while ((v = select_var_to_fix()) != null_var) {
             TRACE("simplex", display(tout << "v" << v << "\n"););
-            if (m_cancel || num_iterations > m_max_iterations) {
+            if (!m_limit.inc() || num_iterations > m_max_iterations) {
                 return l_undef;
             }
             check_blands_rule(v, num_repeated);
@@ -540,7 +540,7 @@ namespace simplex {
         var_t max    = get_num_vars();
         var_t result = max;
         row r = row(m_vars[x_i].m_base2row);
-        int n;
+        int n = 0;
         unsigned best_col_sz = UINT_MAX;
         int best_so_far    = INT_MAX;
         
@@ -634,7 +634,7 @@ namespace simplex {
         //
         //   max { c*x | A*x = 0 and l <= x <= u }
         //
-        // start with feasible assigment 
+        // start with feasible assignment
         // A*x0 = 0 and l <= x0 <= u
         //
         // Identify pivot: i, j: such that x_i is base,
@@ -670,7 +670,7 @@ namespace simplex {
         bool inc_x_i, inc_x_j;
 
         while (true) {
-            if (m_cancel) {
+            if (!m_limit.inc()) {
                 return l_undef;
             }
             select_pivot_primal(v, x_i, x_j, a_ij, inc_x_i, inc_x_j);
@@ -745,7 +745,7 @@ namespace simplex {
             numeral const& base_coeff = vs.m_base_coeff;
             SASSERT(!m.is_zero(coeff));
             bool base_to_lower = (m.is_pos(coeff) != m.is_pos(base_coeff)) == to_lower;
-            eps_numeral const* bound = 0;
+            eps_numeral const* bound = nullptr;
             if (!base_to_lower && vs.m_upper_valid) {
                 bound = &vs.m_upper;
             }
@@ -998,6 +998,7 @@ namespace simplex {
     template<typename Ext>
     bool simplex<Ext>::well_formed_row(row const& r) const {
         var_t s = m_row2base[r.id()];
+        (void)s;
         SASSERT(m_vars[s].m_base2row == r.id());
         SASSERT(m_vars[s].m_is_base);
         // SASSERT(m.is_neg(m_vars[s].m_base_coeff));
