@@ -21,8 +21,8 @@ Notes:
 #ifndef SPARSE_MATRIX_DEF_H_
 #define SPARSE_MATRIX_DEF_H_
 
-#include "sparse_matrix.h"
-#include "uint_set.h"
+#include "math/simplex/sparse_matrix.h"
+#include "util/uint_set.h"
 
 namespace simplex {
 
@@ -233,7 +233,7 @@ namespace simplex {
                 return it;
             }
         }
-        return 0;
+        return nullptr;
     }
 
     template<typename Ext>
@@ -509,12 +509,13 @@ namespace simplex {
                 dead.insert(i);
                 continue;
             }
-            SASSERT(!vars.contains(e.m_var));
-            SASSERT(!m.is_zero(e.m_coeff));            
-            SASSERT(e.m_var != dead_id);
-            col_entry const& c = m_columns[e.m_var].m_entries[e.m_col_idx];
-            SASSERT((unsigned)c.m_row_id == row_id);
-            SASSERT((unsigned)c.m_row_idx == i);
+            DEBUG_CODE(
+                SASSERT(!vars.contains(e.m_var));
+                SASSERT(!m.is_zero(e.m_coeff));            
+                SASSERT(e.m_var != dead_id);
+                col_entry const& c = m_columns[e.m_var].m_entries[e.m_col_idx];
+                SASSERT((unsigned)c.m_row_id == row_id);
+                SASSERT((unsigned)c.m_row_idx == i););
             vars.insert(e.m_var);
         }                  
         int idx = r.m_first_free_idx;
@@ -541,10 +542,11 @@ namespace simplex {
                 continue;
             }
             SASSERT(!rows.contains(c.m_row_id));
-            _row const& row = m_rows[c.m_row_id];
-            _row_entry const& r = row.m_entries[c.m_row_idx];
-            SASSERT(r.m_var == v); 
-            SASSERT((unsigned)r.m_col_idx == i);
+            DEBUG_CODE(
+                _row const& row = m_rows[c.m_row_id];
+                _row_entry const& r = row.m_entries[c.m_row_idx];
+                SASSERT(r.m_var == v); 
+                SASSERT((unsigned)r.m_col_idx == i););
             rows.insert(c.m_row_id);
         }                           
         int idx = col.m_first_free_idx;
