@@ -71,6 +71,8 @@ public:
     void unregister_decl(func_decl * d);
 
     virtual expr * get_some_value(sort * s) = 0;
+    virtual expr * get_fresh_value(sort * s) = 0;
+    virtual bool get_some_values(sort * s, expr_ref & v1, expr_ref & v2) = 0;
 
     expr * get_some_const_interp(func_decl * d) { 
         expr * r = get_const_interp(d); 
