@@ -20,18 +20,19 @@ Revision History:
 #ifndef DL_RULE_H_
 #define DL_RULE_H_
 
-#include"ast.h"
-#include"dl_costs.h"
-#include"dl_util.h"
-#include"used_vars.h"
-#include"proof_converter.h"
-#include"model_converter.h"
-#include"ast_counter.h"
-#include"rewriter.h"
-#include"hnf.h"
-#include"qe_lite.h"
-#include"var_subst.h"
-#include"datatype_decl_plugin.h"
+#include "ast/ast.h"
+#include "muz/base/dl_costs.h"
+#include "muz/base/dl_util.h"
+#include "ast/used_vars.h"
+#include "tactic/proof_converter.h"
+#include "tactic/model_converter.h"
+#include "ast/rewriter/ast_counter.h"
+#include "ast/rewriter/rewriter.h"
+#include "muz/base/hnf.h"
+#include "qe/qe_lite.h"
+#include "ast/rewriter/var_subst.h"
+#include "ast/datatype_decl_plugin.h"
+#include "ast/rewriter/label_rewriter.h"
 
 namespace datalog {
 
@@ -53,7 +54,7 @@ namespace datalog {
         bool m_found;
         func_decl* m_func;
         uninterpreted_function_finder_proc(ast_manager& m): 
-            m(m), m_dt(m), m_dl(m), m_found(false), m_func(0) {}
+            m(m), m_dt(m), m_dl(m), m_found(false), m_func(nullptr) {}
         void operator()(var * n) { }
         void operator()(quantifier * n) { }
         void operator()(app * n) {
@@ -70,7 +71,7 @@ namespace datalog {
                 }
             }
         }
-        void reset() { m_found = false; m_func = 0; }
+        void reset() { m_found = false; m_func = nullptr; }
 
         bool found(func_decl*& f) const { f = m_func; return m_found; }
     };
@@ -81,37 +82,39 @@ namespace datalog {
         quantifier_finder_proc() : m_exist(false), m_univ(false) {}
         void operator()(var * n) { }
         void operator()(quantifier * n) {
-            if (n->is_forall()) {
-                m_univ = true;
-            }
-            else {
-                SASSERT(n->is_exists());
-                m_exist = true;
+            switch (n->get_kind()) {
+            case forall_k: m_univ = true; break;
+            case exists_k: m_exist = true; break;
+            case lambda_k: UNREACHABLE();
             }
         }
         void operator()(app * n) { }
         void reset() { m_exist = m_univ = false; }
     };
 
+    struct fd_finder_proc {
+        ast_manager& m;
+        bv_util      m_bv;
+        bool         m_is_fd;
+        fd_finder_proc(ast_manager& m): m(m), m_bv(m), m_is_fd(true) {}
+
+        bool is_fd() const { return m_is_fd; }
+        bool is_fd(sort* s) { return m.is_bool(s) || m_bv.is_bv_sort(s); }
+        void operator()(var* n) { m_is_fd &= is_fd(n->get_sort()); }
+        void operator()(quantifier* ) { m_is_fd = false; }
+        void operator()(app* n) { m_is_fd &= is_fd(n->get_decl()->get_range()); }
+        void reset() { m_is_fd = true; }
+    };
+
 
     /**
        \brief Manager for the \c rule class
 
-       \remark \c rule_manager objects are interchangable as long as they
+       \remark \c rule_manager objects are interchangeable as long as they
          contain the same \c ast_manager object.
     */
     class rule_manager
     {
-        class remove_label_cfg : public default_rewriter_cfg {
-            family_id m_label_fid;
-        public:        
-            remove_label_cfg(ast_manager& m): m_label_fid(m.get_label_family_id()) {}
-            virtual ~remove_label_cfg();
-            
-            br_status reduce_app(func_decl * f, unsigned num, expr * const * args, expr_ref & result, 
-                                 proof_ref & result_pr);
-        };
-    
         ast_manager&         m;
         context&             m_ctx;
         rule_counter         m_counter;
@@ -124,11 +127,11 @@ namespace datalog {
         svector<bool>        m_neg;
         hnf                  m_hnf;
         qe_lite              m_qe;
-        remove_label_cfg               m_cfg;
-        rewriter_tpl<remove_label_cfg> m_rwr;
+        label_rewriter       m_rwr;
         mutable uninterpreted_function_finder_proc m_ufproc;
         mutable quantifier_finder_proc m_qproc;
         mutable expr_sparse_mark       m_visited;
+        mutable fd_finder_proc         m_fd_proc;
 
 
         // only the context can create a rule_manager
@@ -219,7 +222,7 @@ namespace datalog {
            
            \remark A tail may contain negation. tail[i] is assumed to be negated if is_neg != 0 && is_neg[i] == true
         */
-        rule * mk(app * head, unsigned n, app * const * tail, bool const * is_neg = 0, 
+        rule * mk(app * head, unsigned n, app * const * tail, bool const * is_neg = nullptr,
                   symbol const& name = symbol::null, bool normalize = true);
 
         /**
@@ -277,6 +280,7 @@ namespace datalog {
         bool has_uninterpreted_non_predicates(rule const& r, func_decl*& f) const;
         void has_quantifiers(rule const& r, bool& existential, bool& universal) const;
         bool has_quantifiers(rule const& r) const;
+        bool is_finite_domain(rule const& r) const;
 
     };
 
@@ -308,7 +312,7 @@ namespace datalog {
         
         static unsigned get_obj_size(unsigned n) { return sizeof(rule) + n * sizeof(app *); }
 
-        rule() : m_ref_cnt(0) {}
+        rule() : m_ref_cnt(0), m_name(symbol::null) {}
         ~rule() {}
 
         void deallocate(ast_manager & m);
@@ -346,13 +350,6 @@ namespace datalog {
 
         bool is_neg_tail(unsigned i) const { SASSERT(i < m_tail_size); return GET_TAG(m_tail[i]) == 1; }
 
-        /**
-           A predicate P(Xj) can be annotated by adding an interpreted predicate of the form ((_ min P N) ...)
-           where N is the column number that should be used for the min aggregation function.
-           Such an interpreted predicate is an example for which this function returns true.
-        */
-        bool is_min_tail(unsigned i) const { return dl_decl_plugin::is_aggregate(get_tail(i)->get_decl()); }
-
         /**
         Check whether predicate p is in the interpreted tail.
         
@@ -372,7 +369,12 @@ namespace datalog {
 
         void display(context & ctx, std::ostream & out) const;
 
-        symbol const& name() const { return m_name; }
+        /**
+           \brief Return the name(s) associated with this rule. Plural for preprocessed (e.g. obtained by inlining) rules.
+
+           This possibly returns a ";"-separated list of names.
+        */
+        symbol const& name() const { return m_name; } ;
 
         unsigned hash() const;
 
