@@ -23,18 +23,22 @@ Revision History:
 #ifdef _WINDOWS
 #include <windows.h>
 #endif
-#include"ast_pp.h"
-#include"bool_rewriter.h"
-#include"for_each_expr.h"
-#include"scoped_proof.h"
-#include"dl_context.h"
-#include"dl_rule.h"
-#include"dl_util.h"
-#include"stopwatch.h"
+#include "ast/ast_pp.h"
+#include "ast/rewriter/bool_rewriter.h"
+#include "ast/for_each_expr.h"
+#include "ast/scoped_proof.h"
+#include "muz/base/dl_context.h"
+#include "muz/base/dl_rule.h"
+#include "muz/base/dl_util.h"
+#include "util/stopwatch.h"
+#ifndef __STDC_FORMAT_MACROS
+#define __STDC_FORMAT_MACROS
+#endif
+#include <inttypes.h>
 
 namespace datalog {
 
-    verbose_action::verbose_action(char const* msg, unsigned lvl): m_lvl(lvl), m_sw(0) {
+    verbose_action::verbose_action(char const* msg, unsigned lvl): m_lvl(lvl), m_sw(nullptr) {
         IF_VERBOSE(m_lvl, 
                    (verbose_stream() << msg << "...").flush(); 
                    m_sw = alloc(stopwatch); 
@@ -87,7 +91,7 @@ namespace datalog {
             else {
                 SASSERT(is_var(arg));
                 int vidx      = to_var(arg)->get_idx();
-                var * new_var = 0;
+                var * new_var = nullptr;
                 if (!varidx2var.find(vidx, new_var)) {
                     new_var = m.mk_var(next_idx, to_var(arg)->get_sort());
                     next_idx++;
@@ -111,8 +115,7 @@ namespace datalog {
         expr_ref tmp(m);
         for (unsigned i = 0; i < tgt.size(); ++i) {
             if (tgt[i].get()) {
-                vs(tgt[i].get(), sub.size(), sub.c_ptr(), tmp);
-                tgt[i] = tmp;
+                tgt[i] = vs(tgt[i].get(), sub.size(), sub.c_ptr());
             }
             else {
                 tgt[i] = sub[i];
@@ -165,7 +168,7 @@ namespace datalog {
             }
 
             expr * arg = f->get_arg(i);
-            uint64 sym_num;
+            uint64_t sym_num;
             SASSERT(is_app(arg));
             VERIFY( ctx.get_decl_util().is_numeral_ext(to_app(arg), sym_num) );
             relation_sort sort = pred_decl->get_domain(i);            
@@ -277,16 +280,21 @@ namespace datalog {
         return get_max_var(has_var);
     }
 
-    void del_rule(horn_subsume_model_converter* mc, rule& r) {
+    void del_rule(horn_subsume_model_converter* mc, rule& r, bool unreachable) {
         if (mc) {
             ast_manager& m = mc->get_manager();
             expr_ref_vector body(m);
-            for (unsigned i = 0; i < r.get_tail_size(); ++i) {
-                if (r.is_neg_tail(i)) {
-                    body.push_back(m.mk_not(r.get_tail(i)));
-                }
-                else {
-                    body.push_back(r.get_tail(i));
+            if (unreachable) {
+                body.push_back(m.mk_false());
+            }
+            else {
+                for (unsigned i = 0; i < r.get_tail_size(); ++i) {
+                    if (r.is_neg_tail(i)) {
+                        body.push_back(m.mk_not(r.get_tail(i)));
+                    }
+                    else {
+                        body.push_back(r.get_tail(i));
+                    }
                 }
             }
             TRACE("dl_dr", 
@@ -380,24 +388,32 @@ namespace datalog {
     public:
         skip_model_converter() {}
  
-        virtual model_converter * translate(ast_translation & translator) { 
+        model_converter * translate(ast_translation & translator) override { 
             return alloc(skip_model_converter);
         }
 
+        void operator()(model_ref&) override {}
+
+        void display(std::ostream & out) override { }
+
+        void get_units(obj_map<expr, bool>& units) override {}
+
     };
 
     model_converter* mk_skip_model_converter() { return alloc(skip_model_converter); }
 
     class skip_proof_converter : public proof_converter {
-        virtual void operator()(ast_manager & m, unsigned num_source, proof * const * source, proof_ref & result) {
+
+        proof_ref operator()(ast_manager & m, unsigned num_source, proof * const * source) override {
             SASSERT(num_source == 1);
-            result = source[0];
+            return proof_ref(source[0], m);
         }
 
-        virtual proof_converter * translate(ast_translation & translator) {
+        proof_converter * translate(ast_translation & translator) override {
             return alloc(skip_proof_converter);
         }
 
+        void display(std::ostream & out) override { out << "(skip-proof-converter)\n"; }
     };
 
     proof_converter* mk_skip_proof_converter() { return alloc(skip_proof_converter); }
@@ -423,7 +439,7 @@ namespace datalog {
 
         unsigned tgt_sz = max_var_idx+1;
         unsigned tgt_ofs = tgt_sz-1;
-        tgt.resize(tgt_sz, 0);
+        tgt.resize(tgt_sz, nullptr);
         for(unsigned i=0; i<src_sz; i++) {
             expr * e = src[src_ofs-i];
             if(!e) {
@@ -441,7 +457,7 @@ namespace datalog {
         out << "(";
         for(int i=len-1; i>=0; i--) {
             out << (len-1-i) <<"->";
-            if(cont.get(i)==0) {
+            if(cont.get(i)==nullptr) {
                 out << "{none}";
             }
             else {
@@ -508,10 +524,9 @@ namespace datalog {
     }
 
     void collect_and_transform(const unsigned_vector & src, const unsigned_vector & translation, 
-            unsigned_vector & res) {
-        unsigned n = src.size();
-        for(unsigned i=0; i<n; i++) {
-            unsigned translated = translation[src[i]];
+                               unsigned_vector & res) {
+        for (unsigned s : src) {
+            unsigned translated = translation[s];
             if(translated!=UINT_MAX) {
                 res.push_back(translated);
             }
@@ -520,10 +535,8 @@ namespace datalog {
 
 
     void transform_set(const unsigned_vector & map, const idx_set & src, idx_set & result) {
-        idx_set::iterator it = src.begin();
-        idx_set::iterator end = src.end();
-        for(; it!=end; ++it) {
-            result.insert(map[*it]);
+        for (unsigned s : src) {
+            result.insert(map[s]);
         }
     }
 
@@ -542,7 +555,7 @@ namespace datalog {
     //
     // -----------------------------------
 
-    void get_file_names(std::string directory, std::string extension, bool traverse_subdirs, 
+    void get_file_names(std::string directory, const std::string & extension, bool traverse_subdirs,
             string_vector & res) {
 
         if(directory[directory.size()-1]!='\\' && directory[directory.size()-1]!='/') {
@@ -590,7 +603,7 @@ namespace datalog {
 #endif
     }
 
-    bool file_exists(std::string name) {
+    bool file_exists(const std::string & name) {
         struct stat st;
         if(stat(name.c_str(),&st) == 0) {
             return true;
@@ -598,7 +611,7 @@ namespace datalog {
         return false;
     }
 
-    bool is_directory(std::string name) {
+    bool is_directory(const std::string & name) {
         if(!file_exists(name)) {
             return false;
         }
@@ -607,20 +620,20 @@ namespace datalog {
         return (status.st_mode&S_IFDIR)!=0;
     }
 
-    std::string get_file_name_without_extension(std::string name) {
+    std::string get_file_name_without_extension(const std::string & name) {
         size_t slash_index = name.find_last_of("\\/");
-        size_t dot_index = name.rfind(".");
+        size_t dot_index = name.rfind('.');
         size_t ofs = (slash_index==std::string::npos) ? 0 : slash_index+1;
         size_t count = (dot_index!=std::string::npos && dot_index>ofs) ? 
             (dot_index-ofs) : std::string::npos;
         return name.substr(ofs, count);
     }
 
-    bool string_to_uint64(const char * s, uint64 & res) {
+    bool string_to_uint64(const char * s, uint64_t & res) {
 #if _WINDOWS
-        int converted = sscanf_s(s, "%I64u", &res);
+        int converted = sscanf_s(s, "%" SCNu64, &res);
 #else
-        int converted = sscanf(s, "%llu", &res);
+        int converted = sscanf(s, "%" SCNu64, &res);
 #endif
         if(converted==0) {
             return false;
@@ -629,9 +642,9 @@ namespace datalog {
         return true;
     }
 
-    bool read_uint64(const char * & s, uint64 & res) {
-        static const uint64 max_but_one_digit = ULLONG_MAX/10;
-        static const uint64 max_but_one_digit_safe = (ULLONG_MAX-9)/10;
+    bool read_uint64(const char * & s, uint64_t & res) {
+        static const uint64_t max_but_one_digit = ULLONG_MAX/10;
+        static const uint64_t max_but_one_digit_safe = (ULLONG_MAX-9)/10;
 
         if(*s<'0' || *s>'9') {
             return false;
@@ -659,7 +672,7 @@ namespace datalog {
         return true;
     }
 
-    std::string to_string(uint64 num) {
+    std::string to_string(uint64_t num) {
         std::stringstream stm;
         stm<<num;
         return stm.str();
