@@ -11,7 +11,7 @@ Abstract:
 
 Author:
 
-    Leonardo de Moura (leonardo) 2010-05-20.
+    Krystof Hoder 2010
 
 Revision History:
 
@@ -19,17 +19,19 @@ Revision History:
 #ifndef DL_UTIL_H_
 #define DL_UTIL_H_
 
-#include"ast.h"
-#include"hashtable.h"
-#include"obj_hashtable.h"
-#include"uint_set.h"
-#include"horn_subsume_model_converter.h"
-#include"replace_proof_converter.h"
-#include"substitution.h"
-#include"ast_counter.h"
-#include"statistics.h"
-#include"lbool.h"
-#include"qe_util.h"
+#include<vector>
+#include "ast/ast.h"
+#include "util/hashtable.h"
+#include "util/obj_hashtable.h"
+#include "util/uint_set.h"
+#include "tactic/horn_subsume_model_converter.h"
+#include "tactic/replace_proof_converter.h"
+#include "ast/substitution/substitution.h"
+#include "ast/rewriter/ast_counter.h"
+#include "util/statistics.h"
+#include "util/stopwatch.h"
+#include "util/lbool.h"
+#include "util/container_util.h"
 
 namespace datalog {
 
@@ -51,6 +53,7 @@ namespace datalog {
         ~verbose_action();
     };
 
+    typedef ref_vector<rule, rule_manager> rule_ref_vector;
     enum PDR_CACHE_MODE {
         NO_CACHE,
         HASH_CACHE,
@@ -68,7 +71,7 @@ namespace datalog {
     typedef idx_set var_idx_set;
     typedef u_map<var *> varidx2var_map;
     typedef obj_hashtable<func_decl> func_decl_set; //!< Rule dependencies.
-    typedef vector<std::string> string_vector;
+    typedef std::vector<std::string> string_vector;
     
     bool contains_var(expr * trm, unsigned var_idx);
 
@@ -94,7 +97,7 @@ namespace datalog {
        \brief Auxiliary function used to create a tail based on \c pred for a new rule.
        The variables in \c pred are re-assigned using \c next_idx and \c varidx2var.
        A variable is considered non-local to the rule if it is in the set \c non_local_vars.
-       Non-local variables are coppied to new_rule_args, and their sorts to \c new_rule_domain.
+       Non-local variables are copied to new_rule_args, and their sorts to \c new_rule_domain.
        The new predicate is stored in \c new_pred.
     */
     void mk_new_rule_tail(ast_manager & m, app * pred, 
@@ -273,7 +276,7 @@ namespace datalog {
         }
         unsigned n = container.size();
         unsigned ofs = 1;
-        int r_i = 1;
+        unsigned r_i = 1;
         for(unsigned i=removed_cols[0]+1; i<n; i++) {
             if(r_i!=removed_col_cnt && removed_cols[r_i]==i) {
                 r_i++;
@@ -287,7 +290,7 @@ namespace datalog {
     }
 
     template<class T>
-    void project_out_vector_columns(T & container, const unsigned_vector removed_cols) {
+    void project_out_vector_columns(T & container, const unsigned_vector & removed_cols) {
         project_out_vector_columns(container, removed_cols.size(), removed_cols.c_ptr());
     }
 
@@ -339,7 +342,7 @@ namespace datalog {
     }
 
     template<class T>
-    void permutate_by_cycle(T & container, const unsigned_vector permutation_cycle) {
+    void permutate_by_cycle(T & container, const unsigned_vector & permutation_cycle) {
         permutate_by_cycle(container, permutation_cycle.size(), permutation_cycle.c_ptr());
     }
 
@@ -351,7 +354,7 @@ namespace datalog {
         unsigned get_max_rule_var(const rule& r);
     };
 
-    void del_rule(horn_subsume_model_converter* mc, rule& r);
+    void del_rule(horn_subsume_model_converter* mc, rule& r, bool unreachable);
 
     void resolve_rule(rule_manager& rm,
                       replace_proof_converter* pc, rule const& r1, rule const& r2, unsigned idx, 
@@ -379,129 +382,6 @@ namespace datalog {
     */
     void apply_subst(expr_ref_vector& tgt, expr_ref_vector const& sub);
 
-    // -----------------------------------
-    //
-    // container functions
-    //
-    // -----------------------------------
-
-    template<class Set1, class Set2>
-    void set_intersection(Set1 & tgt, const Set2 & src) {
-        svector<typename Set1::data> to_remove;
-        typename Set1::iterator vit = tgt.begin();
-        typename Set1::iterator vend = tgt.end();
-        for(;vit!=vend;++vit) {
-            typename Set1::data itm=*vit;
-            if(!src.contains(itm)) {
-                to_remove.push_back(itm);
-            }
-        }
-        while(!to_remove.empty()) {
-            tgt.remove(to_remove.back());
-            to_remove.pop_back();
-        }
-    }
-
-    template<class Set>
-    void set_difference(Set & tgt, const Set & to_remove) {
-        typename Set::iterator vit = to_remove.begin();
-        typename Set::iterator vend = to_remove.end();
-        for(;vit!=vend;++vit) {
-            typename Set::data itm=*vit;
-            tgt.remove(itm);
-        }
-    }
-
-    template<class Set1, class Set2>
-    void set_union(Set1 & tgt, const Set2 & to_add) {
-        typename Set2::iterator vit = to_add.begin();
-        typename Set2::iterator vend = to_add.end();
-        for(;vit!=vend;++vit) {
-            typename Set1::data itm=*vit;
-            tgt.insert(itm);
-        }
-    }
-
-    void idx_set_union(idx_set & tgt, const idx_set & src);
-
-    template<class T>
-    void unite_disjoint_maps(T & tgt, const T & src) {
-        typename T::iterator it = src.begin();
-        typename T::iterator end = src.end();
-        for(; it!=end; ++it) {
-            SASSERT(!tgt.contains(it->m_key));
-            tgt.insert(it->m_key, it->m_value);
-        }
-    }
-
-    template<class T, class U>
-    void collect_map_range(T & acc, const U & map) {
-        typename U::iterator it = map.begin();
-        typename U::iterator end = map.end();
-        for(; it!=end; ++it) {
-            acc.push_back(it->m_value);
-        }
-    }
-
-
-    template<class T>
-    void print_container(const T & begin, const T & end, std::ostream & out) {
-        T it = begin;
-        out << "(";
-        bool first = true;
-        for(; it!=end; ++it) {
-            if(first) { first = false; } else { out << ","; }
-            out << (*it);
-        }
-        out << ")";
-    }
-
-    template<class T>
-    void print_container(const T & cont, std::ostream & out) {
-        print_container(cont.begin(), cont.end(), out);
-    }
-
-    template<class T, class M>
-    void print_container(const ref_vector<T,M> & cont, std::ostream & out) {
-        print_container(cont.c_ptr(), cont.c_ptr() + cont.size(), out);
-    }
-
-    template<class T>
-    void print_map(const T & cont, std::ostream & out) {
-        typename T::iterator it = cont.begin();
-        typename T::iterator end = cont.end();
-        out << "(";
-        bool first = true;
-        for(; it!=end; ++it) {
-            if(first) { first = false; } else { out << ","; }
-            out << it->m_key << "->" << it->m_value;
-        }
-        out << ")";
-    }
-
-    template<class It, class V> 
-    unsigned find_index(const It & begin, const It & end, const V & val) {
-        unsigned idx = 0;
-        It it = begin;
-        for(; it!=end; it++, idx++) {
-            if(*it==val) {
-                return idx;
-            }
-        }
-        return UINT_MAX;
-    }
-
-    template<class T, class U>
-    bool containers_equal(const T & begin1, const T & end1, const U & begin2, const U & end2) {
-        T it1 = begin1;
-        U it2 = begin2;
-        for(; it1!=end1 && it2!=end2; ++it1, ++it2) {
-            if(*it1!=*it2) { 
-                return false;
-            }
-        }
-        return it1==end1 && it2==end2;
-    }
 
     template<class T, class U>
     bool vectors_equal(const T & c1, const U & c2) {
@@ -519,6 +399,8 @@ namespace datalog {
         return true;
     }
 
+    void idx_set_union(idx_set & tgt, const idx_set & src);
+
     template<class T>
     struct default_obj_chash {
         unsigned operator()(T const& cont, unsigned i) const {
@@ -591,7 +473,7 @@ namespace datalog {
 
 
     /**
-       \brief Remove the first occurence of \c el from \c v and return \c true. If
+       \brief Remove the first occurrence of \c el from \c v and return \c true. If
        \c el is not present in \c v, return \c false. The order of elements in \c v
        is not preserved.
      */
@@ -696,13 +578,13 @@ namespace datalog {
     //
     // -----------------------------------
 
-    void get_file_names(std::string directory, std::string extension, bool traverse_subdirs, 
+    void get_file_names(std::string directory, const std::string & extension, bool traverse_subdirs, 
         string_vector & res);
 
-    bool file_exists(std::string name);
-    bool is_directory(std::string name);
+    bool file_exists(const std::string & name);
+    bool is_directory(const std::string & name);
 
-    std::string get_file_name_without_extension(std::string name);
+    std::string get_file_name_without_extension(const std::string & name);
 
     // -----------------------------------
     //
@@ -720,8 +602,8 @@ namespace datalog {
        \brief If it is possible to convert the beginning of \c s to uint64,
        store the result of conversion and return true; otherwise return false.
      */
-    bool string_to_uint64(const char * s, uint64 & res);
-    std::string to_string(uint64 num);
+    bool string_to_uint64(const char * s, uint64_t & res);
+    std::string to_string(uint64_t num);
     /**
        \brief Read the sequence of decimal digits starting at \c s and interpret it as
        uint64. If successful, \c res will contain the read number and \c s will point 
@@ -730,7 +612,7 @@ namespace datalog {
        overflows, \c points to the character which caused the overflow and false is 
        returned.
      */
-    bool read_uint64(const char * & s, uint64 & res);
+    bool read_uint64(const char * & s, uint64_t & res);
 };
 
 #endif /* DL_UTIL_H_ */
