@@ -11,7 +11,7 @@ Copyright (c) 2015 Microsoft Corporation
  
  Abstract:
  
-     Horn normal form convertion.
+     Horn normal form conversion.
 
  Author:
  
@@ -25,10 +25,10 @@ Copyright (c) 2015 Microsoft Corporation
 #ifndef HNF_H_
 #define HNF_H_
  
-#include"ast.h"
-#include"params.h"
-#include"defined_names.h"
-#include"proof_converter.h"
+#include "ast/ast.h"
+#include "util/params.h"
+#include "ast/normal_forms/defined_names.h"
+#include "tactic/proof_converter.h"
  
 class hnf {
     class imp;
@@ -43,9 +43,6 @@ class hnf {
                     proof_ref_vector& ps               // [OUT] proofs of rs
                     );
 
-    void cancel() { set_cancel(true); }
-    void reset_cancel() { set_cancel(false); }
-    void set_cancel(bool f);
     void set_name(symbol const& name);    
     void reset();
     func_decl_ref_vector const& get_fresh_predicates();
