@@ -19,10 +19,10 @@ Revision History:
 #ifndef CLP_CONTEXT_H_
 #define CLP_CONTEXT_H_
 
-#include "ast.h"
-#include "lbool.h"
-#include "statistics.h"
-#include "dl_engine_base.h"
+#include "ast/ast.h"
+#include "util/lbool.h"
+#include "util/statistics.h"
+#include "muz/base/dl_engine_base.h"
 
 namespace datalog {
     class context;
@@ -32,14 +32,12 @@ namespace datalog {
         imp* m_imp;
     public:
         clp(context& ctx);
-        ~clp();
-        virtual lbool query(expr* query);
-        virtual void cancel();
-        virtual void cleanup();
-        virtual void reset_statistics();
-        virtual void collect_statistics(statistics& st) const;
-        virtual void display_certificate(std::ostream& out) const;        
-        virtual expr_ref get_answer();
+        ~clp() override;
+        lbool query(expr* query) override;
+        void reset_statistics() override;
+        void collect_statistics(statistics& st) const override;
+        void display_certificate(std::ostream& out) const override;
+        expr_ref get_answer() override;
     };
 };
 
