@@ -19,8 +19,8 @@ Revision History:
 #ifndef DATALOG_PARSER_H_
 #define DATALOG_PARSER_H_
 
-#include "ast.h"
-#include "dl_context.h"
+#include "ast/ast.h"
+#include "muz/base/dl_context.h"
 
 namespace datalog {
 
