@@ -19,14 +19,15 @@ Revision History:
 
 
 #include <sstream>
-#include"ref_vector.h"
-#include"dl_context.h"
-#include"rel_context.h"
-#include"dl_rule.h"
-#include"dl_util.h"
-#include"dl_compiler.h"
-#include"ast_pp.h"
-#include"ast_smt2_pp.h"
+#include "util/ref_vector.h"
+#include "muz/base/dl_context.h"
+#include "muz/rel/rel_context.h"
+#include "muz/base/dl_rule.h"
+#include "muz/base/dl_util.h"
+#include "muz/rel/dl_compiler.h"
+#include "ast/ast_pp.h"
+// include"ast_smt2_pp.h"
+#include "ast/ast_util.h"
 
 
 namespace datalog {
@@ -73,12 +74,6 @@ namespace datalog {
             vars.get_cols2(), removed_cols.size(), removed_cols.c_ptr(), result));
     }
 
-    void compiler::make_min(reg_idx source, reg_idx & target, const unsigned_vector & group_by_cols,
-        const unsigned min_col, instruction_block & acc) {
-        target = get_register(m_reg_signatures[source], true, source);
-        acc.push_back(instruction::mk_min(source, target, group_by_cols, min_col));
-    }
-
     void compiler::make_filter_interpreted_and_project(reg_idx src, app_ref & cond,
             const unsigned_vector & removed_cols, reg_idx & result, bool reuse, instruction_block & acc) {
         SASSERT(!removed_cols.empty());
@@ -191,10 +186,11 @@ namespace datalog {
         }
     }
 
-    void compiler::make_add_unbound_column(rule* compiled_rule, unsigned col_idx, func_decl* pred, reg_idx src, const relation_sort s, reg_idx & result,
+    void compiler::make_add_unbound_column(rule* compiled_rule, unsigned col_idx, func_decl* pred, reg_idx src, const relation_sort& s, reg_idx & result,
             bool & dealloc, instruction_block & acc) {
         
-        TRACE("dl", tout << "Adding unbound column " << mk_pp(pred, m_context.get_manager()) << "\n";);
+        TRACE("dl", tout << "Adding unbound column " << mk_pp(pred, m_context.get_manager()) 
+              << " " << m_context.get_rel_context()->get_rmanager().to_nice_string(s) << "\n";);
             IF_VERBOSE(3, { 
                     expr_ref e(m_context.get_manager()); 
                     m_context.get_rule_manager().to_formula(*compiled_rule, e); 
@@ -334,13 +330,15 @@ namespace datalog {
                 continue;
             }
             unsigned bound_column_index;
-            if(acis[i].kind!=ACK_UNBOUND_VAR || !handled_unbound.find(acis[i].var_index,bound_column_index)) {
+            if(acis[i].kind != ACK_UNBOUND_VAR || !handled_unbound.find(acis[i].var_index,bound_column_index)) {
                 bound_column_index=curr_sig->size();
                 if(acis[i].kind==ACK_CONSTANT) {
                     make_add_constant_column(head_pred, curr, acis[i].domain, acis[i].constant, curr, dealloc, acc);
                 }
                 else {
                     SASSERT(acis[i].kind==ACK_UNBOUND_VAR);
+                    TRACE("dl", tout << head_pred->get_name() << " index: " << i 
+                          << " " << m_context.get_rel_context()->get_rmanager().to_nice_string(acis[i].domain) << "\n";);
                     make_add_unbound_column(compiled_rule, i, head_pred, curr, acis[i].domain, curr, dealloc, acc);
                     handled_unbound.insert(acis[i].var_index,bound_column_index);
                 }
@@ -446,29 +444,7 @@ namespace datalog {
         get_local_indexes_for_projection(t2, counter, t1->get_num_args(), res);
     }
 
-    void compiler::find_min_aggregates(const rule * r, ptr_vector<func_decl>& min_aggregates) {
-        unsigned ut_len = r->get_uninterpreted_tail_size();
-        unsigned ft_len = r->get_tail_size(); // full tail
-        func_decl * aggregate;
-        for (unsigned tail_index = ut_len; tail_index < ft_len; ++tail_index) {
-            aggregate = r->get_tail(tail_index)->get_decl();
-            if (dl_decl_plugin::is_aggregate(aggregate)) {
-                min_aggregates.push_back(aggregate);
-            }
-        }
-    }
 
-    bool compiler::prepare_min_aggregate(const func_decl * decl, const ptr_vector<func_decl>& min_aggregates,
-        unsigned_vector & group_by_cols, unsigned & min_col) {
-        for (unsigned i = 0; i < min_aggregates.size(); ++i) {
-            if (dl_decl_plugin::min_func_decl(min_aggregates[i]) == decl) {
-                group_by_cols = dl_decl_plugin::group_by_cols(min_aggregates[i]);
-                min_col = dl_decl_plugin::min_col(min_aggregates[i]);
-                return true;
-            }
-        }
-        return false;
-    }
 
     void compiler::compile_rule_evaluation_run(rule * r, reg_idx head_reg, const reg_idx * tail_regs, 
             reg_idx delta_reg, bool use_widening, instruction_block & acc) {
@@ -490,17 +466,11 @@ namespace datalog {
 
         //used to save on filter_identical instructions where the check is already done 
         //by the join operation
-        unsigned second_tail_arg_ofs;
+        unsigned second_tail_arg_ofs = 0;
 
         // whether to dealloc the previous result
         bool dealloc = true;
 
-        // setup information for min aggregation
-        ptr_vector<func_decl> min_aggregates;
-        find_min_aggregates(r, min_aggregates);
-        unsigned_vector group_by_cols;
-        unsigned min_col;
-
         if(pt_len == 2) {
             reg_idx t1_reg=tail_regs[0];
             reg_idx t2_reg=tail_regs[1];
@@ -509,14 +479,6 @@ namespace datalog {
             SASSERT(m_reg_signatures[t1_reg].size()==a1->get_num_args());
             SASSERT(m_reg_signatures[t2_reg].size()==a2->get_num_args());
 
-            if (prepare_min_aggregate(a1->get_decl(), min_aggregates, group_by_cols, min_col)) {
-                make_min(t1_reg, single_res, group_by_cols, min_col, acc);
-            }
-
-            if (prepare_min_aggregate(a2->get_decl(), min_aggregates, group_by_cols, min_col)) {
-                make_min(t2_reg, single_res, group_by_cols, min_col, acc);
-            }
-
             variable_intersection a1a2(m_context.get_manager());
             a1a2.populate(a1,a2);
 
@@ -558,9 +520,6 @@ namespace datalog {
             single_res = tail_regs[0];
             dealloc = false;
 
-            if (prepare_min_aggregate(a->get_decl(), min_aggregates, group_by_cols, min_col)) {
-                make_min(single_res, single_res, group_by_cols, min_col, acc);
-            }
 
             SASSERT(m_reg_signatures[single_res].size() == a->get_num_args());
 
@@ -643,15 +602,15 @@ namespace datalog {
         // add unbounded columns for interpreted filter
         unsigned ut_len = r->get_uninterpreted_tail_size();
         unsigned ft_len = r->get_tail_size(); // full tail
-        ptr_vector<expr> tail;
+        expr_ref_vector tail(m);
         for (unsigned tail_index = ut_len; tail_index < ft_len; ++tail_index) {
-            if (!r->is_min_tail(tail_index))
-                tail.push_back(r->get_tail(tail_index));
+            tail.push_back(r->get_tail(tail_index));
         }
 
         expr_ref_vector binding(m);
         if (!tail.empty()) {
-            app_ref filter_cond(tail.size() == 1 ? to_app(tail.back()) : m.mk_and(tail.size(), tail.c_ptr()), m);
+            expr_ref filter_cond = mk_and(tail);
+            m_free_vars.reset();
             m_free_vars(filter_cond);
             // create binding
             binding.resize(m_free_vars.size()+1);
@@ -666,6 +625,7 @@ namespace datalog {
                 } else {
                     // we have an unbound variable, so we add an unbound column for it
                     relation_sort unbound_sort = m_free_vars[v];
+                    TRACE("dl", tout << "unbound: " << v << "\n" << filter_cond << " " << mk_pp(unbound_sort, m) << "\n";);
                     make_add_unbound_column(r, 0, head_pred, filtered_res, unbound_sort, filtered_res, dealloc, acc);
 
                     src_col = single_res_expr.size();
@@ -765,8 +725,7 @@ namespace datalog {
                 }
             }
 
-            expr_ref renamed(m);
-            m_context.get_var_subst()(filter_cond, binding.size(), binding.c_ptr(), renamed);
+            expr_ref renamed = m_context.get_var_subst()(filter_cond, binding.size(), binding.c_ptr());
             app_ref app_renamed(to_app(renamed), m);
             if (remove_columns.empty()) {
                 if (!dealloc)
@@ -1160,7 +1119,7 @@ namespace datalog {
         //and clear local deltas
         make_inloop_delta_transition(global_head_deltas, global_tail_deltas, local_deltas, *loop_body);
 
-        loop_body->set_observer(0);
+        loop_body->set_observer(nullptr);
         acc.push_back(instruction::mk_while_loop(loop_control_regs.size(),
             loop_control_regs.c_ptr(), loop_body));
     }
@@ -1356,7 +1315,7 @@ namespace datalog {
         
         pred2idx empty_pred2idx_map;
 
-        compile_strats(m_rule_set.get_stratifier(), static_cast<pred2idx *>(0), 
+        compile_strats(m_rule_set.get_stratifier(), static_cast<pred2idx *>(nullptr),
             empty_pred2idx_map, true, execution_code);
 
 
@@ -1371,7 +1330,7 @@ namespace datalog {
             termination_code.push_back(instruction::mk_store(m_context.get_manager(), pred, reg));
         }
 
-        acc.set_observer(0);
+        acc.set_observer(nullptr);
 
         TRACE("dl", execution_code.display(execution_context(m_context), tout););
     }
