@@ -225,8 +225,8 @@ namespace datalog {
 
            The newline character at the end should not be printed.
         */
-        virtual void display_head_impl(execution_context const & ctx, std::ostream & out) const {
-            out << "<instruction>";
+        virtual std::ostream& display_head_impl(execution_context const & ctx, std::ostream & out) const {
+            return out << "<instruction>";
         }
         /**
            \brief If relevant, output the body of the current instruction.
