@@ -20,9 +20,9 @@ Revision History:
 #define DL_RELATION_MANAGER_H_
 
 
-#include"map.h"
-#include"vector.h"
-#include"dl_base.h"
+#include "util/map.h"
+#include "util/vector.h"
+#include "muz/rel/dl_base.h"
 
 namespace datalog {
 
@@ -109,8 +109,8 @@ namespace datalog {
     public:
         relation_manager(context & ctx) : 
           m_context(ctx), 
-          m_favourite_table_plugin(0),
-          m_favourite_relation_plugin(0),
+          m_favourite_table_plugin(nullptr),
+          m_favourite_relation_plugin(nullptr),
           m_next_table_fid(0),
           m_next_relation_fid(0) {}
 
@@ -225,8 +225,6 @@ namespace datalog {
             relation_fact & to);
 
 
-        void set_cancel(bool f);
-
 
         // -----------------------------------
         //
@@ -251,14 +249,11 @@ namespace datalog {
             return mk_join_fn(t1, t2, cols1.size(), cols1.c_ptr(), cols2.c_ptr(), allow_product_relation);
         }
 
-        table_min_fn * mk_min_fn(const table_base & t,
-            unsigned_vector & group_by_cols, const unsigned col);
-
         /**
             \brief Return functor that transforms a table into one that lacks columns listed in
             \c removed_cols array.
 
-            The \c removed_cols cotains columns of table \c t in strictly ascending order.
+            The \c removed_cols contains columns of table \c t in strictly ascending order.
             */
         relation_transformer_fn * mk_project_fn(const relation_base & t, unsigned col_cnt, 
             const unsigned * removed_cols);
@@ -268,7 +263,7 @@ namespace datalog {
         }
 
         /**
-            \brief Return an operation that is a composition of a join an a project operation.
+            \brief Return an operation that is a composition of a join and a project operation.
         */
         relation_join_fn * mk_join_project_fn(const relation_base & t1, const relation_base & t2,
                 unsigned joined_col_cnt, const unsigned * cols1, const unsigned * cols2, 
@@ -294,7 +289,7 @@ namespace datalog {
         relation_transformer_fn * mk_permutation_rename_fn(const relation_base & t, 
             const unsigned * permutation);
         relation_transformer_fn * mk_permutation_rename_fn(const relation_base & t, 
-                const unsigned_vector permutation) {
+                const unsigned_vector & permutation) {
             SASSERT(t.get_signature().size()==permutation.size());
             return mk_permutation_rename_fn(t, permutation.c_ptr());
         }
@@ -336,7 +331,7 @@ namespace datalog {
             const relation_base * delta);
 
         relation_union_fn * mk_union_fn(const relation_base & tgt, const relation_base & src) { 
-            return mk_union_fn(tgt, src, static_cast<relation_base *>(0));
+            return mk_union_fn(tgt, src, static_cast<relation_base *>(nullptr));
         }
 
         /**
@@ -348,7 +343,7 @@ namespace datalog {
 
         relation_mutator_fn * mk_filter_identical_fn(const relation_base & t, unsigned col_cnt, 
             const unsigned * identical_cols);
-        relation_mutator_fn * mk_filter_identical_fn(const relation_base & t, const unsigned_vector identical_cols) {
+        relation_mutator_fn * mk_filter_identical_fn(const relation_base & t, const unsigned_vector & identical_cols) {
             return mk_filter_identical_fn(t, identical_cols.size(), identical_cols.c_ptr());
         }
 
@@ -425,7 +420,7 @@ namespace datalog {
             \brief Return functor that transforms a table into one that lacks columns listed in
             \c removed_cols array.
 
-            The \c removed_cols cotains columns of table \c t in strictly ascending order.
+            The \c removed_cols contains columns of table \c t in strictly ascending order.
 
             If a project operation removes a non-functional column, all functional columns become 
             non-functional (so that none of the values in functional columns are lost)
@@ -438,7 +433,7 @@ namespace datalog {
         }
 
         /**
-            \brief Return an operation that is a composition of a join an a project operation.
+            \brief Return an operation that is a composition of a join and a project operation.
 
             This operation is equivalent to the two operations performed separately, unless functional 
             columns are involved.
@@ -473,7 +468,7 @@ namespace datalog {
            of column number.
         */
         table_transformer_fn * mk_permutation_rename_fn(const table_base & t, const unsigned * permutation);
-        table_transformer_fn * mk_permutation_rename_fn(const table_base & t, const unsigned_vector permutation) {
+        table_transformer_fn * mk_permutation_rename_fn(const table_base & t, const unsigned_vector & permutation) {
             SASSERT(t.get_signature().size()==permutation.size());
             return mk_permutation_rename_fn(t, permutation.c_ptr());
         }
@@ -515,7 +510,7 @@ namespace datalog {
             const table_base * delta);
 
         table_union_fn * mk_union_fn(const table_base & tgt, const table_base & src) { 
-            return mk_union_fn(tgt, src, static_cast<table_base *>(0));
+            return mk_union_fn(tgt, src, static_cast<table_base *>(nullptr));
         }
 
         /**
@@ -527,7 +522,7 @@ namespace datalog {
 
         table_mutator_fn * mk_filter_identical_fn(const table_base & t, unsigned col_cnt, 
             const unsigned * identical_cols);
-        table_mutator_fn * mk_filter_identical_fn(const table_base & t, const unsigned_vector identical_cols) {
+        table_mutator_fn * mk_filter_identical_fn(const table_base & t, const unsigned_vector & identical_cols) {
             return mk_filter_identical_fn(t, identical_cols.size(), identical_cols.c_ptr());
         }
 
@@ -614,7 +609,7 @@ namespace datalog {
         std::string to_nice_string(const relation_element & el) const;
         /**
            This one may give a nicer representation of \c el than the 
-           \c to_nice_string(const relation_element & el) function, by unsing the information about the sort
+           \c to_nice_string(const relation_element & el) function, by using the information about the sort
            of the element.
         */
         std::string to_nice_string(const relation_sort & s, const relation_element & el) const;
