@@ -20,37 +20,36 @@ Revision History:
 --*/
 
 
-#include"rel_context.h"
-#include"stopwatch.h"
-#include"dl_context.h"
-#include"dl_compiler.h"
-#include"dl_instruction.h"
-#include"dl_mk_explanations.h"
-#include"dl_mk_magic_sets.h"
-#include"dl_product_relation.h"
-#include"dl_bound_relation.h"
-#include"dl_interval_relation.h"
-#include"karr_relation.h"
-#include"dl_finite_product_relation.h"
-#include"udoc_relation.h"
-#include"check_relation.h"
-#include"dl_lazy_table.h"
-#include"dl_sparse_table.h"
-#include"dl_table.h"
-#include"dl_table_relation.h"
-#include"aig_exporter.h"
-#include"dl_mk_simple_joins.h"
-#include"dl_mk_similarity_compressor.h"
-#include"dl_mk_unbound_compressor.h"
-#include"dl_mk_subsumption_checker.h"
-#include"dl_mk_partial_equiv.h"
-#include"dl_mk_coi_filter.h"
-#include"dl_mk_filter_rules.h"
-#include"dl_mk_rule_inliner.h"
-#include"dl_mk_interp_tail_simplifier.h"
-#include"dl_mk_bit_blast.h"
-#include"dl_mk_separate_negated_tails.h"
-#include"ast_util.h"
+#include "muz/rel/rel_context.h"
+#include "util/stopwatch.h"
+#include "muz/base/dl_context.h"
+#include "muz/rel/dl_compiler.h"
+#include "muz/rel/dl_instruction.h"
+#include "muz/rel/dl_mk_explanations.h"
+#include "muz/transforms/dl_mk_magic_sets.h"
+#include "muz/rel/dl_product_relation.h"
+#include "muz/rel/dl_bound_relation.h"
+#include "muz/rel/dl_interval_relation.h"
+#include "muz/rel/karr_relation.h"
+#include "muz/rel/dl_finite_product_relation.h"
+#include "muz/rel/udoc_relation.h"
+#include "muz/rel/check_relation.h"
+#include "muz/rel/dl_lazy_table.h"
+#include "muz/rel/dl_sparse_table.h"
+#include "muz/rel/dl_table.h"
+#include "muz/rel/dl_table_relation.h"
+#include "muz/rel/aig_exporter.h"
+#include "muz/rel/dl_mk_simple_joins.h"
+#include "muz/rel/dl_mk_similarity_compressor.h"
+#include "muz/transforms/dl_mk_unbound_compressor.h"
+#include "muz/transforms/dl_mk_subsumption_checker.h"
+#include "muz/transforms/dl_mk_coi_filter.h"
+#include "muz/transforms/dl_mk_filter_rules.h"
+#include "muz/transforms/dl_mk_rule_inliner.h"
+#include "muz/transforms/dl_mk_interp_tail_simplifier.h"
+#include "muz/transforms/dl_mk_bit_blast.h"
+#include "muz/transforms/dl_mk_separate_negated_tails.h"
+#include "ast/ast_util.h"
 
 
 namespace datalog {
@@ -97,7 +96,7 @@ namespace datalog {
           m(ctx.get_manager()),
           m_rmanager(ctx),
           m_answer(m), 
-          m_last_result_relation(0),
+          m_last_result_relation(nullptr),
           m_ectx(ctx),
           m_sw(0) {
 
@@ -108,7 +107,6 @@ namespace datalog {
         rm.register_plugin(alloc(sparse_table_plugin, rm));
         rm.register_plugin(alloc(hashtable_table_plugin, rm));
         rm.register_plugin(alloc(bitvector_table_plugin, rm));
-        rm.register_plugin(alloc(equivalence_table_plugin, rm));
         rm.register_plugin(lazy_table_plugin::mk_sparse(rm));
 
         // register plugins for builtin relations
@@ -123,7 +121,7 @@ namespace datalog {
     rel_context::~rel_context() {
         if (m_last_result_relation) {
             m_last_result_relation->deallocate();
-            m_last_result_relation = 0;
+            m_last_result_relation = nullptr;
         }        
     }
 
@@ -151,6 +149,7 @@ namespace datalog {
             m_context.ensure_closed();
             transform_rules();
             if (m_context.canceled()) {
+                TRACE("dl", tout << "canceled\n";);
                 result = l_undef;
                 break;
             }
@@ -191,6 +190,7 @@ namespace datalog {
             IF_VERBOSE(10, m_ectx.report_big_relations(1000, verbose_stream()););
 
             if (m_context.canceled()) {
+                TRACE("dl", tout << "canceled\n";);
                 result = l_undef;
                 break;
             }
@@ -206,6 +206,7 @@ namespace datalog {
             }
             if (timeout_after_this_round) {
                 m_context.set_status(TIMEOUT);
+                TRACE("dl", tout << "timeout\n";);
                 result = l_undef;
                 break;
             }
@@ -214,7 +215,7 @@ namespace datalog {
                 SASSERT(remaining_time_limit>restart_time);
                 remaining_time_limit -= restart_time;
             }
-            uint64 new_restart_time = static_cast<uint64>(restart_time)*m_context.initial_restart_timeout();
+            uint64_t new_restart_time = static_cast<uint64_t>(restart_time)*m_context.initial_restart_timeout();
             if (new_restart_time > UINT_MAX) {
                 restart_time = UINT_MAX;
             }
@@ -272,6 +273,7 @@ namespace datalog {
             if (some_non_empty) {
                 m_answer = mk_and(m, ans.size(), ans.c_ptr());
                 if (is_approx) {
+                    TRACE("dl", tout << "approx\n";);
                     res = l_undef;
                     m_context.set_status(APPROX);
                 }
@@ -286,32 +288,24 @@ namespace datalog {
             m_answer = m.mk_false();
             break;
         case l_undef:
+            TRACE("dl", tout << "saturation in undef\n";);
             break;
         }
         return res;
     }
 
-#define _MIN_DONE_ 1
-
     void rel_context::transform_rules() {
         rule_transformer transf(m_context);
-#ifdef _MIN_DONE_
         transf.register_plugin(alloc(mk_coi_filter, m_context));
-#endif
         transf.register_plugin(alloc(mk_filter_rules, m_context));        
         transf.register_plugin(alloc(mk_simple_joins, m_context));
         if (m_context.unbound_compressor()) {
             transf.register_plugin(alloc(mk_unbound_compressor, m_context));
         }
-#ifdef _MIN_DONE_
         if (m_context.similarity_compressor()) {
             transf.register_plugin(alloc(mk_similarity_compressor, m_context)); 
         }
-#endif
-        transf.register_plugin(alloc(mk_partial_equivalence_transformer, m_context));
-#ifdef _MIN_DONE_
         transf.register_plugin(alloc(mk_rule_inliner, m_context));
-#endif
         transf.register_plugin(alloc(mk_interp_tail_simplifier, m_context));
         transf.register_plugin(alloc(mk_separate_negated_tails, m_context));
 
@@ -375,6 +369,7 @@ namespace datalog {
                 m_last_result_relation->to_formula(m_answer);
                 if (!m_last_result_relation->is_precise()) {
                     m_context.set_status(APPROX);
+                    TRACE("dl", tout << "approx\n";);
                     res = l_undef;
                 }
             }
@@ -520,9 +515,6 @@ namespace datalog {
         get_rmanager().set_predicate_kind(pred, target_kind);
     }
 
-    void rel_context::set_cancel(bool f) {
-        get_rmanager().set_cancel(f);        
-    }
 
     void rel_context::setup_default_relation() {
         if (m_context.default_relation() == symbol("doc")) {
