@@ -19,7 +19,7 @@ Revision History:
 #ifndef DL_MK_DIFFERENT_SYMBOLIC_H_
 #define DL_MK_DIFFERENT_SYMBOLIC_H_
 
-#include"dl_rule_transformer.h"
+#include "muz/base/dl_rule_transformer.h"
 
 namespace datalog {
 
