@@ -19,14 +19,15 @@ Revision History:
 
 
 #include <sstream>
-#include"ast_pp.h"
-#include"bool_rewriter.h"
-#include"rewriter.h"
-#include"rewriter_def.h"
-#include"dl_mk_rule_inliner.h"
-#include"dl_mk_interp_tail_simplifier.h"
-#include"ast_util.h"
+#include "ast/ast_pp.h"
+#include "ast/rewriter/bool_rewriter.h"
+#include "ast/rewriter/rewriter.h"
+#include "ast/rewriter/rewriter_def.h"
+#include "muz/transforms/dl_mk_rule_inliner.h"
+#include "muz/transforms/dl_mk_interp_tail_simplifier.h"
+#include "ast/ast_util.h"
 
+#include "muz/base/fp_params.hpp"
 namespace datalog {
 
     // -----------------------------------
@@ -45,7 +46,7 @@ namespace datalog {
     bool mk_interp_tail_simplifier::rule_substitution::unify(expr * e1, expr * e2) {
         SASSERT(m_rule);
 
-        //we need to apply the current substitution in order to ensure the unifier 
+        //we need to apply the current substitution in order to ensure the unifier
         //works in an incremental way
         expr_ref e1_s(m);
         expr_ref e2_s(m);
@@ -84,7 +85,7 @@ namespace datalog {
         mk_rule_inliner::remove_duplicate_tails(m_tail, m_neg);
 
         SASSERT(m_tail.size() == m_neg.size());
-        res = m_context.get_rule_manager().mk(m_head, m_tail.size(), m_tail.c_ptr(), m_neg.c_ptr());
+        res = m_context.get_rule_manager().mk(m_head, m_tail.size(), m_tail.c_ptr(), m_neg.c_ptr(),m_rule->name());
         res->set_accounting_parent_object(m_context, m_rule);
         res->norm_vars(res.get_manager());
     }
@@ -221,20 +222,20 @@ namespace datalog {
         */
         app * detect_equivalence(const arg_pair& p1, const arg_pair& p2, bool inside_disjunction)
         {
-            if (m.is_not(p1.first)==m.is_not(p2.first)) { return 0; }
-            if (m.is_not(p1.second)==m.is_not(p2.second)) { return 0; }
+            if (m.is_not(p1.first)==m.is_not(p2.first)) { return nullptr; }
+            if (m.is_not(p1.second)==m.is_not(p2.second)) { return nullptr; }
 
-            expr * first_bare = 0;
-            if (m.is_not(p1.first, first_bare) && p2.first!=first_bare) { return 0; }
-            if (m.is_not(p2.first, first_bare) && p1.first!=first_bare) { return 0; }
+            expr * first_bare = nullptr;
+            if (m.is_not(p1.first, first_bare) && p2.first!=first_bare) { return nullptr; }
+            if (m.is_not(p2.first, first_bare) && p1.first!=first_bare) { return nullptr; }
             SASSERT(first_bare);
 
-            expr * second_bare = 0;
-            if (m.is_not(p1.second, second_bare) && p2.second!=second_bare) { return 0; }
-            if (m.is_not(p2.second, second_bare) && p1.second!=second_bare) { return 0; }
+            expr * second_bare = nullptr;
+            if (m.is_not(p1.second, second_bare) && p2.second!=second_bare) { return nullptr; }
+            if (m.is_not(p2.second, second_bare) && p1.second!=second_bare) { return nullptr; }
             SASSERT(second_bare);
 
-            if (!m.is_bool(first_bare) || !m.is_bool(second_bare)) { return 0; }
+            if (!m.is_bool(first_bare) || !m.is_bool(second_bare)) { return nullptr; }
 
             //both negations are in the same pair
             bool negs_together = m.is_not(p1.first)==m.is_not(p1.second);
@@ -250,7 +251,7 @@ namespace datalog {
         bool detect_equivalences(expr_ref_vector& v, bool inside_disjunction)
         {
             bool have_pair = false;
-            unsigned prev_pair_idx;
+            unsigned prev_pair_idx = 0;
             arg_pair ap;
 
             unsigned read_idx = 0;
@@ -260,14 +261,14 @@ namespace datalog {
 
                 arg_pair new_ap;
                 if (match_arg_pair(e, new_ap, inside_disjunction)) {
-                    app * neq = 0;
+                    app * neq = nullptr;
                     if (have_pair) {
                         neq = detect_equivalence(ap, new_ap, inside_disjunction);
                     }
                     if (neq) {
                         have_pair = false;
                         v[prev_pair_idx] = neq;
-                        
+
                         read_idx++;
                         continue;
                     }
@@ -293,29 +294,28 @@ namespace datalog {
 
         //bool detect_same_variable_conj_pairs
 
-        br_status reduce_app(func_decl * f, unsigned num, expr * const * args, expr_ref & result, 
+        br_status reduce_app(func_decl * f, unsigned num, expr * const * args, expr_ref & result,
             proof_ref & result_pr)
         {
-
             if (m.is_not(f) && (m.is_and(args[0]) || m.is_or(args[0]))) {
-                SASSERT(num==1);
+                SASSERT(num == 1);
                 expr_ref tmp(m);
                 app* a = to_app(args[0]);
                 m_app_args.reset();
-                for (unsigned i = 0; i < a->get_num_args(); ++i) {
-                    m_brwr.mk_not(a->get_arg(i), tmp);
+                for (expr* arg : *a) {
+                    m_brwr.mk_not(arg, tmp);
                     m_app_args.push_back(tmp);
                 }
                 if (m.is_and(args[0])) {
-                    result = m.mk_or(m_app_args.size(), m_app_args.c_ptr());
+                    result = mk_or(m_app_args);
                 }
                 else {
-                    result = m.mk_and(m_app_args.size(), m_app_args.c_ptr());
+                    result = mk_and(m_app_args);
                 }
                 return BR_REWRITE2;
             }
-            if (!m.is_and(f) && !m.is_or(f)) { 
-                return BR_FAILED; 
+            if (!m.is_and(f) && !m.is_or(f)) {
+                return BR_FAILED;
             }
             if (num == 0) {
                 if (m.is_and(f)) {
@@ -375,7 +375,7 @@ namespace datalog {
               m_simp(ctx.get_rewriter()),
               a(m),
               m_rule_subst(ctx),
-              m_tail(m), 
+              m_tail(m),
               m_itail_members(m),
               m_conj(m) {
         m_cfg = alloc(normalizer_cfg, m);
@@ -386,7 +386,7 @@ namespace datalog {
         dealloc(m_rw);
         dealloc(m_cfg);
     }
-    
+
 
     void mk_interp_tail_simplifier::simplify_expr(app * a, expr_ref& res)
     {
@@ -394,14 +394,12 @@ namespace datalog {
         m_simp(a, simp1_res);
         (*m_rw)(simp1_res.get(), res);
 
-        /*if (simp1_res.get()!=res.get()) {
-            std::cout<<"pre norm:\n"<<mk_pp(simp1_res.get(),m)<<"post norm:\n"<<mk_pp(res.get(),m)<<"\n";
-        }*/
-
         m_simp(res.get(), res);
     }
 
     bool mk_interp_tail_simplifier::propagate_variable_equivalences(rule * r, rule_ref& res) {
+        if (!m_context.get_params ().xform_tail_simplifier_pve ())
+            return false;
         unsigned u_len = r->get_uninterpreted_tail_size();
         unsigned len = r->get_tail_size();
         if (u_len == len) {
@@ -539,7 +537,7 @@ namespace datalog {
         simplify_expr(itail.get(), simp_res);
 
         modified |= itail.get() != simp_res.get();
-        
+
         if (m.is_false(simp_res)) {
             TRACE("dl", r->display(m_context, tout << "rule is infeasible\n"););
             return false;
@@ -561,7 +559,7 @@ namespace datalog {
             }
 
             SASSERT(m_tail.size() == m_tail_neg.size());
-            res = m_context.get_rule_manager().mk(head, m_tail.size(), m_tail.c_ptr(), m_tail_neg.c_ptr());
+            res = m_context.get_rule_manager().mk(head, m_tail.size(), m_tail.c_ptr(), m_tail_neg.c_ptr(), r->name());
             res->set_accounting_parent_object(m_context, r);
         }
         else {
@@ -570,7 +568,7 @@ namespace datalog {
 
         rule_ref pro_var_eq_result(m_context.get_rule_manager());
         if (propagate_variable_equivalences(res, pro_var_eq_result)) {
-            SASSERT(rule_counter().get_max_rule_var(*r.get())==0 || 
+            SASSERT(rule_counter().get_max_rule_var(*r.get())==0 ||
                     rule_counter().get_max_rule_var(*r.get()) > rule_counter().get_max_rule_var(*pro_var_eq_result.get()));
             r = pro_var_eq_result;
             goto start;
@@ -603,18 +601,20 @@ namespace datalog {
 
     rule_set * mk_interp_tail_simplifier::operator()(rule_set const & source) {
         if (source.get_num_rules() == 0) {
-            return 0;
+            return nullptr;
         }
 
         rule_set * res = alloc(rule_set, m_context);
         if (transform_rules(source, *res)) {
             res->inherit_predicates(source);
+            TRACE("dl",
+                  source.display(tout);
+                  res->display(tout););
         } else {
             dealloc(res);
-            res = 0;
+            res = nullptr;
         }
         return res;
     }
-  
-};
 
+};
