@@ -19,8 +19,8 @@ Revision History:
 #ifndef DL_MK_LOOP_COUNTER_H_
 #define DL_MK_LOOP_COUNTER_H_
 
-#include"dl_rule_transformer.h"
-#include"arith_decl_plugin.h"
+#include "muz/base/dl_rule_transformer.h"
+#include "ast/arith_decl_plugin.h"
 
 namespace datalog {
 
@@ -36,9 +36,9 @@ namespace datalog {
         app_ref del_arg(app* fn);        
     public:
         mk_loop_counter(context & ctx, unsigned priority = 33000);
-        ~mk_loop_counter();
+        ~mk_loop_counter() override;
         
-        rule_set * operator()(rule_set const & source);
+        rule_set * operator()(rule_set const & source) override;
 
         func_decl* get_old(func_decl* f) const { return m_new2old.find(f); }
 
