@@ -19,11 +19,11 @@ Revision History:
 #ifndef DL_MK_UNFOLD_H_
 #define DL_MK_UNFOLD_H_
 
-#include"dl_context.h"
-#include"dl_rule_set.h"
-#include"uint_set.h"
-#include"dl_rule_transformer.h"
-#include"dl_mk_rule_inliner.h"
+#include "muz/base/dl_context.h"
+#include "muz/base/dl_rule_set.h"
+#include "util/uint_set.h"
+#include "muz/base/dl_rule_transformer.h"
+#include "muz/transforms/dl_mk_rule_inliner.h"
 
 namespace datalog {
 
@@ -44,7 +44,7 @@ namespace datalog {
          */
         mk_unfold(context & ctx);
         
-        rule_set * operator()(rule_set const & source);
+        rule_set * operator()(rule_set const & source) override;
     };
 
 };
