@@ -21,7 +21,7 @@ Revision History:
 #ifndef DL_TRANSFORMS_H_
 #define DL_TRANSFORMS_H_
 
-#include "dl_context.h"
+#include "muz/base/dl_context.h"
 
 namespace datalog {
     void apply_default_transformation(context& ctx);
