@@ -16,9 +16,9 @@ Author:
 Revision History:
 
 --*/
-#include"nlsat_interval_set.h"
-#include"algebraic_numbers.h"
-#include"buffer.h"
+#include "nlsat/nlsat_interval_set.h"
+#include "math/polynomial/algebraic_numbers.h"
+#include "util/buffer.h"
 
 namespace nlsat {
 
@@ -28,6 +28,7 @@ namespace nlsat {
         unsigned  m_lower_inf:1;
         unsigned  m_upper_inf:1;
         literal   m_justification;
+        clause const* m_clause;
         anum      m_lower;
         anum      m_upper;
     };
@@ -102,14 +103,14 @@ namespace nlsat {
     
     // Check if the intervals are valid, ordered, and are disjoint.
     bool check_interval_set(anum_manager & am, unsigned sz, interval const * ints) {
-        for (unsigned i = 0; i < sz; i++) {
-            interval const & curr = ints[i];
-            SASSERT(check_interval(am, curr));
-            if (i < sz - 1) {
-                interval const & next = ints[i+1];
-                SASSERT(check_no_overlap(am, curr, next));
-            }
-        }
+        DEBUG_CODE(
+            for (unsigned i = 0; i < sz; i++) {
+                interval const & curr = ints[i];
+                SASSERT(check_interval(am, curr));
+                if (i < sz - 1) {
+                    SASSERT(check_no_overlap(am, curr, ints[i+1]));
+                }
+            });
         return true;
     }
 
@@ -122,7 +123,7 @@ namespace nlsat {
     }
     
     void interval_set_manager::del(interval_set * s) {
-        if (s == 0)
+        if (s == nullptr)
             return;
         unsigned num    = s->m_num_intervals;
         unsigned obj_sz = interval_set::get_obj_size(num);
@@ -147,7 +148,7 @@ namespace nlsat {
     
     interval_set * interval_set_manager::mk(bool lower_open, bool lower_inf, anum const & lower, 
                                             bool upper_open, bool upper_inf, anum const & upper,
-                                            literal justification) {
+                                            literal justification, clause const* cls) {
         void * mem = m_allocator.allocate(interval_set::get_obj_size(1));
         interval_set * new_set = new (mem) interval_set();
         new_set->m_num_intervals = 1;
@@ -159,6 +160,7 @@ namespace nlsat {
         i->m_upper_open = upper_open;
         i->m_upper_inf  = upper_inf;
         i->m_justification = justification;
+        i->m_clause = cls;
         if (!lower_inf)
             m_am.set(i->m_lower, lower);
         if (!upper_inf)
@@ -270,9 +272,9 @@ namespace nlsat {
 
     interval_set * interval_set_manager::mk_union(interval_set const * s1, interval_set const * s2) {
         TRACE("nlsat_interval", tout << "mk_union\ns1: "; display(tout, s1); tout << "\ns2: "; display(tout, s2); tout << "\n";);
-        if (s1 == 0 || s1 == s2)
+        if (s1 == nullptr || s1 == s2)
             return const_cast<interval_set*>(s2);
-        if (s2 == 0)
+        if (s2 == nullptr)
             return const_cast<interval_set*>(s1);
         if (s1->m_full)
             return const_cast<interval_set*>(s1);
@@ -514,22 +516,22 @@ namespace nlsat {
     }
 
     bool interval_set_manager::is_full(interval_set const * s) {
-        if (s == 0)
+        if (s == nullptr)
             return false;
         return s->m_full == 1;
     }
 
     unsigned interval_set_manager::num_intervals(interval_set const * s) const {
-        if (s == 0) return 0;
+        if (s == nullptr) return 0;
         return s->m_num_intervals;
     }
     
     bool interval_set_manager::subset(interval_set const * s1, interval_set const * s2) {
         if (s1 == s2)
             return true;
-        if (s1 == 0)
+        if (s1 == nullptr)
             return true;
-        if (s2 == 0)
+        if (s2 == nullptr)
             return false;
         if (s2->m_full)
             return true;
@@ -616,7 +618,7 @@ namespace nlsat {
     }
 
     bool interval_set_manager::set_eq(interval_set const * s1, interval_set const * s2) {
-        if (s1 == 0 || s2 == 0)
+        if (s1 == nullptr || s2 == nullptr)
             return s1 == s2;
         if (s1->m_full || s2->m_full)
             return s1->m_full == s2->m_full;
@@ -625,7 +627,7 @@ namespace nlsat {
     }
         
     bool interval_set_manager::eq(interval_set const * s1, interval_set const * s2) {
-        if (s1 == 0 || s2 == 0)
+        if (s1 == nullptr || s2 == nullptr)
             return s1 == s2;
         if (s1->m_num_intervals != s2->m_num_intervals)
             return false;
@@ -644,8 +646,9 @@ namespace nlsat {
         return true;
     }
 
-    void interval_set_manager::get_justifications(interval_set const * s, literal_vector & js) {
+    void interval_set_manager::get_justifications(interval_set const * s, literal_vector & js, ptr_vector<clause>& clauses) {
         js.reset();
+        clauses.reset();
         unsigned num = num_intervals(s);
         for (unsigned i = 0; i < num; i++) {
             literal l     = s->m_intervals[i].m_justification;
@@ -654,6 +657,9 @@ namespace nlsat {
                 continue;
             m_already_visited.setx(lidx, true, false);
             js.push_back(l);
+            if (s->m_intervals[i].m_clause) {
+                clauses.push_back(const_cast<clause*>(s->m_intervals[i].m_clause));
+            }
         }
         for (unsigned i = 0; i < num; i++) {
             literal l     = s->m_intervals[i].m_justification;
@@ -672,14 +678,14 @@ namespace nlsat {
         return new_set;
     }
 
-    void interval_set_manager::peek_in_complement(interval_set const * s, anum & w, bool randomize) {
+    void interval_set_manager::peek_in_complement(interval_set const * s, bool is_int, anum & w, bool randomize) {
         SASSERT(!is_full(s));
-        if (s == 0) {
+        if (s == nullptr) {
             if (randomize) {
                 int num = m_rand() % 2 == 0 ? 1 : -1;
 #define MAX_RANDOM_DEN_K 4
                 int den_k = (m_rand() % MAX_RANDOM_DEN_K);
-                int den   = 1 << den_k;
+                int den   = is_int ? 1 : (1 << den_k);
                 scoped_mpq _w(m_am.qm());
                 m_am.qm().set(_w, num, den);
                 m_am.set(w, _w);
@@ -743,10 +749,10 @@ namespace nlsat {
         m_am.set(w, s->m_intervals[irrational_i].m_upper);
     }
 
-    void interval_set_manager::display(std::ostream & out, interval_set const * s) const {
-        if (s == 0) {
+    std::ostream& interval_set_manager::display(std::ostream & out, interval_set const * s) const {
+        if (s == nullptr) {
             out << "{}";
-            return;
+            return out;
         }
         out << "{";
         for (unsigned i = 0; i < s->m_num_intervals; i++) {
@@ -757,6 +763,7 @@ namespace nlsat {
         out << "}";
         if (s->m_full)
             out << "*";
+        return out;
     }
 
 };
