@@ -21,15 +21,17 @@ Author:
 Notes:
 
 --*/
-#include"goal2nlsat.h"
-#include"goal.h"
-#include"goal_util.h"
-#include"nlsat_solver.h"
-#include"expr2polynomial.h"
-#include"expr2var.h"
-#include"arith_decl_plugin.h"
-#include"tactic.h"
-#include"ast_smt2_pp.h"
+#include "nlsat/tactic/goal2nlsat.h"
+#include "tactic/goal.h"
+#include "tactic/goal_util.h"
+#include "nlsat/nlsat_solver.h"
+#include "ast/expr2polynomial.h"
+#include "ast/expr2var.h"
+#include "ast/arith_decl_plugin.h"
+#include "tactic/tactic.h"
+#include "ast/ast_pp.h"
+#include "math/polynomial/polynomial.h"
+#include "math/polynomial/algebraic_numbers.h"
 
 struct goal2nlsat::imp {
     struct nlsat_expr2polynomial : public expr2polynomial {
@@ -39,11 +41,11 @@ struct goal2nlsat::imp {
             m_solver(s) {
         }
 
-        virtual bool is_int(polynomial::var x) const {
+        bool is_int(polynomial::var x) const override {
             return m_solver.is_int(x);
         }
 
-        virtual polynomial::var mk_var(bool is_int) {
+        polynomial::var mk_var(bool is_int) override {
             return m_solver.mk_var(is_int);
         }
     };
@@ -61,7 +63,6 @@ struct goal2nlsat::imp {
     unsigned long long        m_max_memory;
     bool                      m_factor;
 
-    volatile bool             m_cancel;
 
     imp(ast_manager & _m, params_ref const & p, nlsat::solver & s, expr2var & a2b, expr2var & t2x):
         m(_m),
@@ -73,7 +74,6 @@ struct goal2nlsat::imp {
         m_t2x(t2x),
         m_expr2poly(m_solver, m, m_solver.pm(), &m_t2x) {
         updt_params(p);
-        m_cancel = false;
     }
 
     void updt_params(params_ref const & p) {
@@ -82,11 +82,6 @@ struct goal2nlsat::imp {
         m_fparams.updt_params(p);
     }
 
-    void set_cancel(bool f) {
-        m_cancel = f;
-        m_pm.set_cancel(f);
-    }
-
     nlsat::atom::kind flip(nlsat::atom::kind k) {
         switch (k) {
         case nlsat::atom::EQ: return k;
@@ -131,7 +126,7 @@ struct goal2nlsat::imp {
         m_qm.neg(d2);
         polynomial_ref p(m_pm);
         p = m_pm.addmul(d1, m_pm.mk_unit(), p1, d2, m_pm.mk_unit(), p2);
-        TRACE("goal2nlsat_bug", tout << "p: " << p << "\nk: " << k << "\n";);
+        TRACE("goal2nlsat_bug", tout << mk_pp(f, m) << " p: " << p << "\nk: " << k << "\n";);
         if (is_const(p)) {
             int sign;
             if (is_zero(p))
@@ -199,11 +194,10 @@ struct goal2nlsat::imp {
                 switch (to_app(f)->get_decl_kind()) {
                 case OP_TRUE:
                 case OP_FALSE:
-                    TRACE("goal2nlsat", tout << "f: " << mk_ismt2_pp(f, m) << "\n";);
+                    TRACE("goal2nlsat", tout << "f: " << mk_pp(f, m) << "\n";);
                     throw tactic_exception("apply simplify before applying nlsat");
                 case OP_AND:
                 case OP_OR:
-                case OP_IFF:
                 case OP_XOR:
                 case OP_NOT:
                 case OP_IMPLIES:
@@ -257,6 +251,7 @@ struct goal2nlsat::imp {
     }
 
     void operator()(goal const & g) {
+        TRACE("goal2nlsat", g.display(tout););
         if (has_term_ite(g))
             throw tactic_exception("eliminate term-ite before applying nlsat");
         unsigned sz = g.size();
@@ -264,27 +259,22 @@ struct goal2nlsat::imp {
             process(g.form(i), g.dep(i));
         }
     }
+
 };
 
 struct goal2nlsat::scoped_set_imp {
     goal2nlsat & m_owner; 
     scoped_set_imp(goal2nlsat & o, imp & i):m_owner(o) {
-        #pragma omp critical (tactic_cancel)
-        {
-            m_owner.m_imp = &i;
-        }
+        m_owner.m_imp = &i;        
     }
     
     ~scoped_set_imp() {
-        #pragma omp critical (tactic_cancel)
-        {
-            m_owner.m_imp = 0;
-        }
+        m_owner.m_imp = nullptr;
     }
 };
 
 goal2nlsat::goal2nlsat() {
-    m_imp = 0;
+    m_imp = nullptr;
 }
 
 goal2nlsat::~goal2nlsat() {
@@ -302,8 +292,154 @@ void goal2nlsat::operator()(goal const & g, params_ref const & p, nlsat::solver
     scoped_set_imp setter(*this, local_imp);
     local_imp(g);
 }
+
     
-void goal2nlsat::set_cancel(bool f) {
-    if (m_imp)
-        m_imp->set_cancel(f);
+
+struct nlsat2goal::imp {
+    ast_manager& m;
+    arith_util   a;
+    u_map<expr*> const* m_x2t;
+public:
+    imp(ast_manager& m):m(m),a(m) {}
+
+    expr_ref operator()(nlsat::solver& s, u_map<expr*> const& b2a, u_map<expr*> const& x2t, nlsat::literal l) {
+        m_x2t = &x2t;
+        expr_ref result(m);
+        expr* t;
+        if (b2a.find(l.var(), t)) {
+            result = t;
+        }
+        else {
+            nlsat::atom const* at = s.bool_var2atom(l.var());
+            SASSERT(at != 0);
+            if (at->is_ineq_atom()) {
+                nlsat::ineq_atom const* ia = to_ineq_atom(at);
+                unsigned sz = ia->size();
+                expr_ref_vector ps(m);
+                bool is_int = true;
+                for (unsigned i = 0; is_int && i < sz; ++i) {
+                    is_int = poly_is_int(ia->p(i));
+                }
+                for (unsigned i = 0; i < sz; ++i) {
+                    polynomial::polynomial* p = ia->p(i);
+                    expr_ref t = poly2expr(s, p, is_int);
+                    if (ia->is_even(i)) {
+                        t = a.mk_power(t, a.mk_numeral(rational(2), a.is_int(t)));
+                    }
+                    ps.push_back(t);
+                }
+                result = a.mk_mul_simplify(ps);
+                expr_ref zero(m);
+                zero = a.mk_numeral(rational(0), a.is_int(result));
+                switch (ia->get_kind()) {
+                case nlsat::atom::EQ:
+                    result = m.mk_eq(result, zero);
+                    break;
+                case nlsat::atom::LT:
+                    if (l.sign()) {
+                        l.neg();
+                        result = a.mk_ge(result, zero);
+                    }
+                    else {
+                        result = a.mk_lt(result, zero);
+                    }
+                    break;
+                case nlsat::atom::GT:
+                    if (l.sign()) {
+                        l.neg();
+                        result = a.mk_le(result, zero);
+                    }
+                    else {
+                        result = a.mk_gt(result, zero);
+                    }
+                    break;
+                default:
+                    UNREACHABLE();
+                }
+            }
+            else {
+                //nlsat::root_atom const* ra = nlsat::to_root_atom(at);
+                //ra->i();
+                //expr_ref p = poly2expr(s, ra->p());
+                //expr*    x = m_x2t->find(ra->x());
+                std::ostringstream strm;
+                s.display(strm, l.sign()?~l:l);
+                result = m.mk_const(symbol(strm.str().c_str()), m.mk_bool_sort());
+            }
+        }
+
+        if (l.sign()) {
+            result = m.mk_not(result);
+        }
+        return result;
+    }
+
+    expr_ref poly2expr(nlsat::solver& s, polynomial::polynomial* p, bool is_int) {
+        expr_ref result(m);
+        unsigned sz = polynomial::manager::size(p);
+        expr_ref_vector args(m);
+        for (unsigned i = 0; i < sz; ++i) {
+            args.push_back(mono2expr(s, 
+                                     polynomial::manager::coeff(p, i), 
+                                     polynomial::manager::get_monomial(p, i), is_int));
+        }
+        result = a.mk_add_simplify(args);
+        return result;
+    }
+
+    expr_ref mono2expr(nlsat::solver& s, polynomial::numeral const& c, polynomial::monomial* mon, bool is_int) {
+        expr_ref result(m);
+        expr_ref_vector args(m);
+        unsigned sz = polynomial::manager::size(mon);
+        for (unsigned i = 0; i < sz; ++i) {
+            unsigned d = polynomial::manager::degree(mon, i);
+            expr* t = m_x2t->find(polynomial::manager::get_var(mon, i));
+            SASSERT(d >= 1);
+            if (d == 1) {
+                args.push_back(t);
+            }
+            else {
+                args.push_back(a.mk_power(t, a.mk_numeral(rational(d), a.is_int(t))));
+            }
+        }
+        if (!s.pm().m().is_one(c)) {
+            args.push_back(a.mk_numeral(c, is_int));
+        }
+        result = a.mk_mul_simplify(args);
+        return result;
+    }
+
+    bool poly_is_int(polynomial::polynomial* p) {
+        bool is_int = true;
+        unsigned sz = polynomial::manager::size(p);
+        for (unsigned i = 0; is_int && i < sz; ++i) {
+            is_int = mono_is_int(polynomial::manager::get_monomial(p, i));
+        }
+        return is_int;
+    }
+
+    bool mono_is_int(polynomial::monomial* mon) {
+        bool is_int = true;
+        unsigned sz = polynomial::manager::size(mon);
+        for (unsigned i = 0; is_int && i < sz; ++i) {
+            is_int = a.is_int(m_x2t->find(polynomial::manager::get_var(mon, i)));
+        }
+        return is_int;
+    }
+};
+
+
+nlsat2goal::nlsat2goal(ast_manager& m) {
+    m_imp = alloc(imp, m);
 }
+
+
+nlsat2goal::~nlsat2goal() {
+    dealloc(m_imp);
+}
+
+expr_ref nlsat2goal::operator()(nlsat::solver& s, u_map<expr*> const& b2a, u_map<expr*> const& x2t, nlsat::literal l) {
+    return (*m_imp)(s, b2a, x2t, l);
+}
+
+
