@@ -133,7 +133,7 @@ public:
         m_max_core_size(3),
         m_maximize_assignment(false),
         m_max_correction_set_size(3),
-        m_pivot_on_cs(true)
+        m_pivot_on_cs(true)       
     {
         switch(st) {
         case s_primal:
@@ -177,6 +177,7 @@ public:
         else {
             asum = mk_fresh_bool("soft");
             fml = m.mk_iff(asum, e);
+            m_defs.push_back(fml);
             add(fml);
         }
         new_assumption(asum, w);
@@ -328,8 +329,8 @@ public:
         verify_assumptions();
         m_lower.reset();
         for (soft& s : m_soft) {
-            s.is_true = m_model->is_true(s.s);
-            if (!s.is_true) {
+            s.set_value(m_model->is_true(s.s));
+            if (!s.is_true()) {
                 m_lower += s.weight;
             }
         }
@@ -374,7 +375,7 @@ public:
             get_mus_model(mdl);
             is_sat = minimize_core(_core);
             core.append(_core.size(), _core.c_ptr());
-            verify_core(core);
+            DEBUG_CODE(verify_core(core););
             ++m_stats.m_num_cores;
             if (is_sat != l_true) {
                 IF_VERBOSE(100, verbose_stream() << "(opt.maxres minimization failed)\n";);
@@ -382,6 +383,7 @@ public:
             }
             if (core.empty()) {
                 IF_VERBOSE(100, verbose_stream() << "(opt.maxres core is empty)\n";);
+                TRACE("opt", tout << "empty core\n";);
                 cores.reset();
                 m_lower = m_upper;
                 return l_true;
@@ -433,7 +435,9 @@ public:
         maxres& mr;
         compare_asm(maxres& mr):mr(mr) {}
         bool operator()(expr* a, expr* b) const {
-            return mr.get_weight(a) > mr.get_weight(b);
+            rational w1 = mr.get_weight(a);
+            rational w2 = mr.get_weight(b);
+            return w1 > w2 || (w1 == w2 && a->get_id() > b->get_id());
         }
     };
 
@@ -516,6 +520,10 @@ public:
         max_resolve(core, w);
         fml = mk_not(m, mk_and(m, core.size(), core.c_ptr()));
         add(fml);
+        // save small cores such that lex-combinations of maxres can reuse these cores.
+        if (core.size() <= 2) {
+            m_defs.push_back(fml);
+        }
         m_lower += w;
         if (m_st == s_primal_dual) {
             m_lower = std::min(m_lower, m_upper);
@@ -698,8 +706,7 @@ public:
                 fml = m.mk_implies(d, cls);
                 update_model(d, cls);
                 add(fml);
-                m_defs.push_back(fml);
-                
+                m_defs.push_back(fml);                
             }
             else {
                 d = cls;
@@ -733,7 +740,7 @@ public:
             m_correction_set_size = correction_set_size;
         }
 
-        TRACE("opt", tout << *mdl;);
+        TRACE("opt_verbose", tout << *mdl;);
 
         rational upper(0);
 
@@ -756,10 +763,10 @@ public:
         m_model = mdl;
         m_c.model_updated(mdl.get());
 
-        TRACE("opt", tout << "updated upper: " << upper << "\nmodel\n" << *m_model;);
+        TRACE("opt", tout << "updated upper: " << upper << "\n";);
 
         for (soft& s : m_soft) {
-            s.is_true = m_model->is_true(s.s);
+            s.set_value(m_model->is_true(s.s));
         }
        
         verify_assignment();
@@ -833,22 +840,29 @@ public:
 
     void commit_assignment() override {
         if (m_found_feasible_optimum) {
-            TRACE("opt", tout << "Committing feasible solution\n" << m_defs << " " << m_asms;);
             add(m_defs);
             add(m_asms);
+            TRACE("opt", tout << "Committing feasible solution\ndefs:" << m_defs << "\nasms:" << m_asms << "\n";);
+
         }
         // else: there is only a single assignment to these soft constraints.
     }
 
     void verify_core(exprs const& core) {
         return;
-        IF_VERBOSE(3, verbose_stream() << "verify core " << s().check_sat(core.size(), core.c_ptr()) << "\n";);                
+        IF_VERBOSE(1, verbose_stream() << "verify core " << s().check_sat(core.size(), core.c_ptr()) << "\n";);                
         ref<solver> _solver = mk_smt_solver(m, m_params, symbol());
         _solver->assert_expr(s().get_assertions());
         _solver->assert_expr(core);
         lbool is_sat = _solver->check_sat(0, nullptr);
-        IF_VERBOSE(0, verbose_stream() << "core status (l_false:) " << is_sat << "\n");
-        VERIFY(is_sat == l_false);
+        IF_VERBOSE(0, verbose_stream() << "core status (l_false:) " << is_sat << " core size " << core.size() << "\n");
+        CTRACE("opt", is_sat != l_false, 
+               for (expr* c : core) tout << "core: " << mk_pp(c, m) << "\n";
+               _solver->display(tout);
+               tout << "other solver\n";
+               s().display(tout);
+               );
+        VERIFY(is_sat == l_false || m.canceled());
     }
 
     void verify_assumptions() {
@@ -870,7 +884,7 @@ public:
         expr_ref n(m);
         for (soft& s : m_soft) {
             n = s.s;
-            if (!s.is_true) {
+            if (!s.is_true()) {
                 n = mk_not(m, n);
             }
             _solver->assert_expr(n);
