@@ -18,11 +18,12 @@ Notes:
 #ifndef OPT_CMDS_H_
 #define OPT_CMDS_H_
 
-#include "ast.h"
+#include "ast/ast.h"
+#include "opt/opt_context.h"
 
 class cmd_context;
 
-void install_opt_cmds(cmd_context & ctx);
+void install_opt_cmds(cmd_context & ctx, opt::context* opt = nullptr);
 
 
 #endif
