@@ -411,13 +411,14 @@ private:
                     }
                     m_buffer.push_back(0);
                     m_tokens.push_back(asymbol(symbol(m_buffer.c_ptr()), in.line()));
+                    IF_VERBOSE(10, verbose_stream() << "tok: " << m_tokens.back() << "\n");
                     continue;
                 }
             }
 
             if (is_num(c)) {
                 rational n(0);
-                unsigned div = 1;
+                rational div(1);
                 while (is_num(c) && !in.eof()) {
                     n = n*rational(10) + rational(c - '0');
                     in.next();
@@ -429,11 +430,11 @@ private:
                     while (is_num(c) && !in.eof()) {
                         n = n*rational(10) + rational(c - '0');
                         in.next();
-                        div *= 10;
+                        div *= rational(10);
                         c = in.ch();
                     }
                 }
-                if (div > 1) n = n / rational(div);
+                if (div > rational(1)) n = n / div;
                 if (neg) n.neg();
                 m_tokens.push_back(asymbol(n, in.line()));
                 IF_VERBOSE(10, verbose_stream() << "num: " << m_tokens.back() << "\n");
@@ -686,14 +687,22 @@ private:
         return peek(pos) == "<=" || peek(pos) == "=<";
     }
 
-    bool peek_minus_infty(unsigned pos) {
+    bool peek_minus_infty_long(unsigned pos) {
         return peek(pos) == "-" && (peek(pos+1) == "inf" || peek(pos+1) == "infinity");
     }
 
-    bool peek_plus_infty(unsigned pos) {
+    bool peek_minus_infty_short(unsigned pos) {
+        return peek(pos) == "-inf" || peek(pos) == "-infinity";
+    }
+
+    bool peek_plus_infty_long(unsigned pos) {
         return peek(pos) == "+" && (peek(pos+1) == "inf" || peek(pos+1) == "infinity");
     }
 
+    bool peek_plus_infty_short(unsigned pos) {
+        return peek(pos) == "+inf" || peek(pos) == "+infinity";
+    }
+
     void parse_indicator(symbol& var, rational& val) {
         if (peek(1) == "=" && tok.peek_num(2) && peek(3) == "->") {
             var = peek(0);
@@ -730,14 +739,22 @@ private:
             tok.next(3);
             parse_upper(v);
         }
-        else if (peek_minus_infty(0) && peek_le(2)) {
+        else if (peek_minus_infty_long(0) && peek_le(2)) {
             v = peek(3);
             tok.next(4);
             parse_upper(v);
         }
-        else if (peek_plus_infty(2) && peek_le(1)) {
+        else if (peek_minus_infty_short(0) && peek_le(1)) {
+            v = peek(2);
+            tok.next(3);
+            parse_upper(v);
+        }
+        else if (peek_plus_infty_long(2) && peek_le(1)) {
             tok.next(4);            
         }
+        else if (peek_plus_infty_short(2) && peek_le(1)) {
+            tok.next(3);
+        }
         else if (peek_le(1) && tok.peek_num(2)) {
             v = peek(0);
             tok.next(2);
@@ -756,9 +773,11 @@ private:
             update_upper(v, rhs);
             tok.next(2);
         }
-        else if (peek_le(0) && peek_plus_infty(1)) {
+        else if (peek_le(0) && peek_plus_infty_long(1)) {
             tok.next(3);            
         }
+        else if (peek_le(0) && peek_plus_infty_short(1)) {
+            tok.next(2);        }
 
     }
 
