@@ -31,6 +31,7 @@ Notes:
 #include <typeinfo>
 #include "opt/optsmt.h"
 #include "opt/opt_solver.h"
+#include "opt/opt_context.h"
 #include "ast/arith_decl_plugin.h"
 #include "smt/theory_arith.h"
 #include "ast/ast_pp.h"
@@ -171,6 +172,7 @@ namespace opt {
     }
 
     lbool optsmt::geometric_lex(unsigned obj_index, bool is_maximize) {
+        TRACE("opt", tout << "index: " << obj_index << " is-max: " << is_maximize << "\n";);
         arith_util arith(m);
         bool is_int = arith.is_int(m_objs[obj_index].get());
         lbool is_sat = l_true;
@@ -179,6 +181,7 @@ namespace opt {
         for (unsigned i = 0; i < obj_index; ++i) {
             commit_assignment(i);
         }
+        m_s->get_model(m_model);
 
         unsigned steps = 0;
         unsigned step_incs = 0;
@@ -189,10 +192,14 @@ namespace opt {
             SASSERT(delta_per_step.is_int());
             SASSERT(delta_per_step.is_pos());
             is_sat = m_s->check_sat(0, nullptr);
+            TRACE("opt", tout << "check " << is_sat << "\n";
+                  tout << "lower: " << m_lower[obj_index] << "\n";
+                  tout << "upper: " << m_upper[obj_index] << "\n";
+                  );
             if (is_sat == l_true) {                
                 m_s->maximize_objective(obj_index, bound);
                 m_s->get_model(m_model);
-                m_s->get_labels(m_labels);
+                SASSERT(m_model);
                 inf_eps obj = m_s->saved_objective_value(obj_index);
                 update_lower_lex(obj_index, obj, is_maximize);
                 if (!is_int || !m_lower[obj_index].is_finite()) {
@@ -211,7 +218,7 @@ namespace opt {
                     ++num_scopes;
                     bound = m_s->mk_ge(obj_index, obj + inf_eps(delta_per_step));
                 }
-                TRACE("opt", tout << delta_per_step << " " << bound << "\n";);
+                TRACE("opt", tout << "delta: " << delta_per_step << " " << bound << "\n";);
                 m_s->assert_expr(bound);
             }
             else if (is_sat == l_false && delta_per_step > rational::one()) {
@@ -220,13 +227,19 @@ namespace opt {
                 delta_per_step = rational::one();
                 SASSERT(num_scopes > 0);
                 --num_scopes;
-                m_s->pop(1);             
+                m_s->pop(1);                             
             }
             else {
                 break;
             }
         }
         m_s->pop(num_scopes);        
+
+        TRACE("opt", tout << is_sat << " " << num_scopes << "\n";);
+
+        if (is_sat == l_false && !m_model) {
+            return l_false;
+        }
         
         if (m.canceled() || is_sat == l_undef) {
             return l_undef;
@@ -355,12 +368,15 @@ namespace opt {
                            verbose_stream() << "(optsmt lower bound: " << v << ")\n";
                        else
                            verbose_stream() << "(optsmt upper bound: " << (-v) << ")\n";
-                       );
+                       );            
             expr_ref tmp(m);
             for (unsigned i = idx+1; i < m_vars.size(); ++i) {
                 m_s->maximize_objective(i, tmp);
                 m_lower[i] = m_s->saved_objective_value(i);
             }
+            m_best_model = m_model;
+            m_s->get_labels(m_labels);
+            m_context.set_model(m_model);
         }
     }
 
@@ -483,6 +499,7 @@ namespace opt {
 
     lbool optsmt::lex(unsigned obj_index, bool is_maximize) {
         TRACE("opt", tout << "optsmt:lex\n";);
+        m_context.get_base_model(m_best_model);
         solver::scoped_push _push(*m_s);
         SASSERT(obj_index < m_vars.size());
         if (is_maximize && m_optsmt_engine == symbol("farkas")) {
@@ -510,7 +527,6 @@ namespace opt {
             
             m_s->maximize_objective(obj_index, bound);
             m_s->get_model(m_model);
-            m_s->get_labels(m_labels);
             inf_eps obj = m_s->saved_objective_value(obj_index);
             update_lower_lex(obj_index, obj, is_maximize);
             TRACE("opt", tout << "strengthen bound: " << bound << "\n";);
@@ -574,8 +590,8 @@ namespace opt {
         return m_upper[i];
     }
 
-    void optsmt::get_model(model_ref& mdl, svector<symbol> & labels) {
-        mdl = m_model.get();
+    void optsmt::get_model(model_ref& mdl, svector<symbol> & labels) {        
+        mdl = m_best_model.get();
         labels = m_labels;
     }
 
