@@ -16,13 +16,53 @@ Author:
 Notes:
 
 --*/
-#include "pb_sls.h"
-#include "smt_literal.h"
-#include "ast_pp.h"
-#include "th_rewriter.h"
-#include "sat_sls.h"
+#include "opt/pb_sls.h"
+#include "smt/smt_literal.h"
+#include "ast/ast_pp.h"
+#include "ast/rewriter/th_rewriter.h"
+#include "sat/sat_types.h"
 
 namespace smt {
+
+    class index_set {
+
+        unsigned_vector m_elems;
+        unsigned_vector m_index;        
+    public:
+        unsigned num_elems() const { return m_elems.size(); }       
+        unsigned operator[](unsigned idx) const { return m_elems[idx]; }
+        void reset() { m_elems.reset(); m_index.reset(); }        
+        bool empty() const { return m_elems.empty(); }        
+
+        bool contains(unsigned idx) const {
+            return 
+                (idx < m_index.size()) && 
+                (m_index[idx] < m_elems.size()) && 
+                (m_elems[m_index[idx]] == idx);
+        }
+        
+        void insert(unsigned idx) {
+            m_index.reserve(idx+1);
+            if (!contains(idx)) {
+                m_index[idx] = m_elems.size();
+                m_elems.push_back(idx);
+            }
+        }
+        
+        void remove(unsigned idx) {
+            if (!contains(idx)) return;
+            unsigned pos = m_index[idx];
+            m_elems[pos] = m_elems.back();
+            m_index[m_elems[pos]] = pos;
+            m_elems.pop_back();
+        }
+        
+        unsigned choose(random_gen& rnd) const {
+            SASSERT(!empty());
+            return m_elems[rnd(num_elems())];
+        }
+    };
+
     struct pb_sls::imp {
 
         struct clause {
@@ -60,7 +100,6 @@ namespace smt {
         pb_util          pb;
         unsynch_mpz_manager mgr;
         th_rewriter      m_rewrite;
-        volatile bool    m_cancel;
         vector<clause>   m_clauses;      // clauses to be satisfied        
         expr_ref_vector  m_orig_clauses; // for debugging
         model_ref        m_orig_model;   // for debugging
@@ -74,8 +113,8 @@ namespace smt {
         expr_ref_vector  m_trail;
         obj_map<expr, unsigned> m_decl2var; // map declarations to Boolean variables.
         ptr_vector<expr> m_var2decl;        // reverse map
-        sat::index_set        m_hard_false;           // list of hard clauses that are false.
-        sat::index_set        m_soft_false;           // list of soft clauses that are false.
+        index_set        m_hard_false;           // list of hard clauses that are false.
+        index_set        m_soft_false;           // list of soft clauses that are false.
         unsigned         m_max_flips;            // maximal number of flips
         unsigned         m_non_greedy_percent;   // percent of moves to do non-greedy style
         random_gen       m_rng;
@@ -86,7 +125,6 @@ namespace smt {
             m(m),
             pb(m),
             m_rewrite(m),
-            m_cancel(false),
             m_orig_clauses(m),
             m_trail(m),
             one(mgr)
@@ -141,8 +179,7 @@ namespace smt {
             m_orig_model = mdl;
             for (unsigned i = 0; i < m_var2decl.size(); ++i) {
                 expr_ref tmp(m);
-                VERIFY(mdl->eval(m_var2decl[i], tmp));
-                m_assignment[i] = m.is_true(tmp);
+                m_assignment[i] = mdl->is_true(m_var2decl[i]);
             }
         }
 
@@ -157,7 +194,7 @@ namespace smt {
                 while (m_max_flips > 0) {
                     --m_max_flips;
                     literal lit = flip();
-                    if (m_cancel) {
+                    if (m.canceled()) {
                         return l_undef;
                     }
                     IF_VERBOSE(3, verbose_stream() 
@@ -203,9 +240,6 @@ namespace smt {
         bool get_value(literal l) {
             return l.sign() ^ m_assignment[l.var()];
         }
-        void set_cancel(bool f) {
-            m_cancel = f;
-        }
         void get_model(model_ref& mdl) {
             mdl = alloc(model, m);
             for (unsigned i = 1; i < m_var2decl.size(); ++i) {
@@ -309,8 +343,7 @@ namespace smt {
             for (unsigned i = 0; i < m_clauses.size(); ++i) {
                 if (!eval(m_clauses[i])) {
                     m_hard_false.insert(i);
-                    expr_ref tmp(m);
-                    VERIFY(m_orig_model->eval(m_orig_clauses[i].get(), tmp));                    
+                    expr_ref tmp = (*m_orig_model)(m_orig_clauses[i].get());
                     IF_VERBOSE(0,                               
                                verbose_stream() << "original evaluation: " << tmp << "\n";
                                verbose_stream() << mk_pp(m_orig_clauses[i].get(), m) << "\n";
@@ -370,10 +403,8 @@ namespace smt {
                 }
                 int new_break_count = flip(~lit);
                 if (-break_count != new_break_count) {
-                    verbose_stream() << lit << "\n";
-                    IF_VERBOSE(0, display(verbose_stream(), cls););
-                    display(verbose_stream());
-                    exit(0);
+                    IF_VERBOSE(0, display(verbose_stream() << lit << "\n", cls);
+                               display(verbose_stream()));
                 }
                 // VERIFY(-break_count == flip(~lit));
             }            
@@ -485,15 +516,13 @@ namespace smt {
 
         literal mk_aux_literal(expr* f) {           
             unsigned var;
-            expr_ref tmp(m);
             if (!m_decl2var.find(f, var)) {
                 var = m_hard_occ.size();
                 SASSERT(m_var2decl.size() == var);
                 SASSERT(m_soft_occ.size() == var);
                 m_hard_occ.push_back(unsigned_vector());
                 m_soft_occ.push_back(unsigned_vector());
-                VERIFY(m_orig_model->eval(f, tmp));
-                m_assignment.push_back(m.is_true(tmp));
+                m_assignment.push_back(m_orig_model->is_true(f));
                 m_decl2var.insert(f, var);
                 m_var2decl.push_back(f);
             }
@@ -719,9 +748,6 @@ namespace smt {
     lbool pb_sls::operator()() {
         return (*m_imp)();
     }
-    void pb_sls::set_cancel(bool f) {
-        m_imp->set_cancel(f);
-    }
     void pb_sls::collect_statistics(statistics& st) const {
         m_imp->collect_statistics(st);
     }
