@@ -19,11 +19,11 @@ Notes:
 #ifndef PB_SLS_H_
 #define PB_SLS_H_
 
-#include "pb_decl_plugin.h"
-#include "model.h"
-#include "lbool.h"
-#include "params.h"
-#include "statistics.h"
+#include "ast/pb_decl_plugin.h"
+#include "model/model.h"
+#include "util/lbool.h"
+#include "util/params.h"
+#include "util/statistics.h"
 
 namespace smt {
 
@@ -38,7 +38,6 @@ namespace smt {
         bool soft_holds(unsigned index);
         void set_model(model_ref& mdl);
         lbool operator()();
-        void set_cancel(bool f);
         void collect_statistics(::statistics& st) const;
         void get_model(model_ref& mdl);
         void updt_params(params_ref& p);
