@@ -20,10 +20,10 @@ Revision History:
 #define SMT2SCANNER_H_
 
 #include<iostream>
-#include"symbol.h"
-#include"vector.h"
-#include"rational.h"
-#include"cmd_context.h"
+#include "util/symbol.h"
+#include "util/vector.h"
+#include "util/rational.h"
+#include "cmd_context/cmd_context.h"
 
 namespace smt2 {
 
@@ -34,6 +34,7 @@ namespace smt2 {
         bool               m_interactive;
         int                m_spos; // position in the current line of the stream
         char               m_curr;  // current char;
+        bool               m_at_eof;
         
         int                m_line;  // line
         int                m_pos;   // start position of the token
@@ -91,6 +92,7 @@ namespace smt2 {
         token read_symbol();
         token read_quoted_symbol();
         void read_comment();
+        void read_multiline_comment();
         token read_number();
         token read_signed_number();
         token read_string();
