@@ -19,15 +19,15 @@ Revision History:
 #ifndef PATTERN_VALIDATION_H_
 #define PATTERN_VALIDATION_H_
 
-#include"ast.h"
-#include"uint_set.h"
-#include"vector.h"
+#include "ast/ast.h"
+#include "util/uint_set.h"
+#include "util/vector.h"
 
 class pattern_validator {
     family_id          m_bfid;
     family_id          m_lfid;
 
-    bool process(uint_set & found_vars, unsigned num_bindings, unsigned num_new_bindings, expr * n);
+    bool process(uint_set & found_vars, unsigned num_bindings, unsigned num_new_bindings, expr * n, unsigned line, unsigned pos);
 
 public:
     pattern_validator(ast_manager const & m):
@@ -35,8 +35,8 @@ public:
         m_lfid(m.get_label_family_id()) {
     }
 
-    bool operator()(unsigned num_bindings, unsigned num_new_bindings, expr * n);
-    bool operator()(unsigned num_new_bindings, expr * n) { return operator()(UINT_MAX, num_new_bindings, n); }
+    bool operator()(unsigned num_bindings, unsigned num_new_bindings, expr * n, unsigned line, unsigned pos);
+    bool operator()(unsigned num_new_bindings, expr * n, unsigned line, unsigned pos) { return operator()(UINT_MAX, num_new_bindings, n, line, pos); }
 };
 
 #endif /* PATTERN_VALIDATION_H_ */
