@@ -604,6 +604,11 @@ namespace qe {
         return m_imp->maximize(fmls, mdl, t, ge, gt);
     }
 
+    void arith_project_plugin::saturate(model& model, func_decl_ref_vector const& shared, expr_ref_vector& lits) {
+        UNREACHABLE();
+    }
+
+
     bool arith_project(model& model, app* var, expr_ref_vector& lits) {
         ast_manager& m = lits.get_manager();
         arith_project_plugin ap(m);
