@@ -18,23 +18,45 @@ Revision History:
 
 --*/
 
-#include "qe.h"
-#include "ast_pp.h"
-#include "expr_safe_replace.h"
-#include "bool_rewriter.h"
-#include "bv_decl_plugin.h"
-#include "arith_decl_plugin.h"
-#include "arith_eq_solver.h"
-#include "arith_rewriter.h"
-#include "th_rewriter.h"
-#include "factor_rewriter.h"
-#include "obj_pair_hashtable.h"
-#include "nlarith_util.h"
-#include "model_evaluator.h"
-#include "smt_kernel.h"
+#include "qe/qe.h"
+#include "ast/ast_pp.h"
+#include "ast/rewriter/expr_safe_replace.h"
+#include "ast/rewriter/bool_rewriter.h"
+#include "ast/bv_decl_plugin.h"
+#include "ast/arith_decl_plugin.h"
+#include "smt/arith_eq_solver.h"
+#include "ast/rewriter/arith_rewriter.h"
+#include "ast/rewriter/th_rewriter.h"
+#include "ast/rewriter/factor_rewriter.h"
+#include "util/obj_pair_hashtable.h"
+#include "qe/nlarith_util.h"
+#include "model/model_evaluator.h"
+#include "smt/smt_kernel.h"
+#include "qe/qe_arith.h"
 
 namespace qe {
 
+
+    static bool is_divides(arith_util& a, expr* e1, expr* e2, rational& k, expr_ref& p) {  
+        expr* t1, *t2;
+        if (a.is_mod(e2, t1, t2) && 
+            a.is_numeral(e1, k) && 
+            k.is_zero() &&
+            a.is_numeral(t2, k)) {
+            p = t1;
+            return true;
+        }
+        return false;
+    }
+
+    static bool is_divides(arith_util& a, expr* e, rational& k, expr_ref& t) {
+        expr* e1, *e2;
+        if (!a.get_manager().is_eq(e, e1, e2)) {
+            return false;
+        }
+        return is_divides(a, e1, e2, k, t) || is_divides(a, e2, e1, k, t);
+    }
+
     class bound {        
         rational   m_coeff;
         expr_ref  m_term;
@@ -83,6 +105,7 @@ namespace qe {
     public:
         arith_util        m_arith; // initialize before m_zero_i, etc.
         th_rewriter       simplify;
+        app_ref_vector    m_vars_added;
     private:
         arith_eq_solver   m_arith_solver;
         bv_util           m_bv;
@@ -104,6 +127,7 @@ namespace qe {
             m_ctx(ctx),
             m_arith(m),
             simplify(m),
+            m_vars_added(m),
             m_arith_solver(m),
             m_bv(m),
             m_zero_i(m_arith.mk_numeral(numeral(0), true), m),
@@ -266,23 +290,8 @@ namespace qe {
         // 
         // match 0 == p mod k, p mod k == 0
         //
-        bool is_divides(app* e, numeral& k, expr_ref& p) {
-            expr* e1, *e2;
-            if (!m.is_eq(e, e1, e2)) {
-                return false;
-            }
-            return is_divides(e1, e2, k, p) || is_divides(e2, e1, k, p);
-        }
-    
-        bool is_divides(expr* e1, expr* e2, numeral& k, expr_ref& p) {  
-            if (m_arith.is_mod(e2) && 
-                m_arith.is_numeral(e1, k) && 
-                k.is_zero() &&
-                m_arith.is_numeral(to_app(e2)->get_arg(1), k)) {
-                p = to_app(e2)->get_arg(0);
-                return true;
-            }
-            return false;
+        bool is_divides(expr* e, numeral& k, expr_ref& p) {
+            return qe::is_divides(m_arith, e, k, p);
         }
 
         bool is_not_divides(app* e, app_ref& n, numeral& k, expr_ref& p) {
@@ -307,7 +316,7 @@ namespace qe {
         void mk_bound_aux(rational const& a, expr* t, rational const& b, expr* s, expr_ref& result) {
             SASSERT(a.is_neg() == b.is_neg());
             expr_ref tt(t, m), ss(s, m), e(m);
-            // hack to fix wierd gcc compilation error
+            // hack to fix weird gcc compilation error
             rational abs_a(a);
             rational abs_b(b);
             if (abs_a.is_neg()) abs_a.neg();
@@ -776,6 +785,11 @@ namespace qe {
             }
         }
 
+        void add_var(app* v, bool track = true) {
+            m_ctx.add_var(v);
+            if (track) m_vars_added.push_back(v);
+        }
+
 
     private:
 
@@ -843,10 +857,11 @@ namespace qe {
                   << mk_pp(result, m) << "\n";);
         }
 
+
         void mk_big_or_symbolic(numeral up, app* x, expr* body, expr_ref& result) {
             app_ref z_bv(m);
             mk_big_or_symbolic(up, x, body, z_bv, result);
-            m_ctx.add_var(z_bv);
+            add_var(z_bv);
         }        
 
         void mk_big_or_symbolic_blast(numeral up, app* x, expr* body, expr_ref& result) {
@@ -876,7 +891,7 @@ namespace qe {
             
             while (m_arith.is_add(p)) {
                 bool found_x = false;
-                expr* next_p = 0;
+                expr* next_p = nullptr;
                 for (unsigned i = 0; i < to_app(p)->get_num_args(); ++i) {
                     expr* arg = to_app(p)->get_arg(i);
                     if (contains_x(arg)) {
@@ -992,9 +1007,9 @@ namespace qe {
         bool solve_linear(expr* p, expr* fml) {
             vector<numeral> values;
             unsigned num_vars = m_ctx.get_num_vars();
-            app*const* vars_ptr = m_ctx.get_vars();
+            app_ref_vector const& vars = m_ctx.get_vars();
             
-            if (!is_linear(p, num_vars, vars_ptr, values)) {
+            if (!is_linear(p, num_vars, vars.c_ptr(), values)) {
                 return false;
             }
 
@@ -1018,7 +1033,7 @@ namespace qe {
                 // it has coefficient 'm' = values[index].
                 SASSERT(values[index] >= rational(3));
                 z  = m.mk_fresh_const("x", m_arith.mk_int());
-                m_ctx.add_var(z);
+                add_var(z);
                 p1 = m_arith.mk_mul(m_arith.mk_numeral(values[index], true), z);
             }
             else {                
@@ -1167,21 +1182,20 @@ namespace qe {
         
 
         bool get_bound(contains_app& contains_x, app* a) {
-            ast_manager& m = m_util.get_manager();
-            app* x = contains_x.x();
-            if (m_mark.is_marked(a) ||
+            bool has_bound = 
+                m_mark.is_marked(a) ||
                 get_le_bound(contains_x, a) ||
                 get_lt_bound(contains_x, a) ||
                 get_divides(contains_x, a) ||
-                get_nested_divs(contains_x, a)) {
-                TRACE("qe_verbose", tout << "Bound for " << mk_pp(x, m) << " within " << mk_pp(a, m) << "\n";);
+                get_nested_divs(contains_x, a);
+            if (has_bound) {
                 m_mark.mark(a, true);
-                return true;
-            }
-            else {
-                TRACE("qe", tout << "No bound for " << mk_pp(x, m) << " within " << mk_pp(a, m) << "\n";);
-                return false;
             }
+            TRACE("qe_verbose", 
+                  ast_manager& m = m_util.get_manager();
+                  app* x = contains_x.x();
+                  tout << has_bound << " bound for " << mk_pp(x, m) << " within " << mk_pp(a, m) << "\n";);
+            return has_bound;
         }
 
         unsigned lt_size() { return m_lt_terms.size(); }
@@ -1273,7 +1287,7 @@ namespace qe {
             m_div_coeffs.reset();
             m_div_divisors.reset();
             m_div_atoms.reset();
-            m_div_z = 0;
+            m_div_z = nullptr;
             m_nested_div_terms.reset();
             m_nested_div_coeffs.reset();
             m_nested_div_divisors.reset();
@@ -1287,6 +1301,7 @@ namespace qe {
             ptr_vector<expr> todo;
             todo.push_back(a);
             rational k1, k2;
+            expr* e1 = nullptr, *e2 = nullptr;
             expr_ref rest(m);
             while (!todo.empty()) {
                 expr* e = todo.back();
@@ -1305,9 +1320,9 @@ namespace qe {
                     return false;
                 }
                 a = to_app(e);
-                if (m_util.m_arith.is_mod(e) && 
-                    m_util.m_arith.is_numeral(to_app(e)->get_arg(1), k1) &&
-                    m_util.get_coeff(contains_x, to_app(e)->get_arg(0), k2, rest)) {
+                if (m_util.m_arith.is_mod(e, e1, e2) && 
+                    m_util.m_arith.is_numeral(e2, k1) &&
+                    m_util.get_coeff(contains_x, e1, k2, rest)) {
                     app_ref z(m), z_bv(m);
                     m_util.mk_bounded_var(k1, z_bv, z);
                     m_nested_div_terms.push_back(rest);
@@ -1317,10 +1332,9 @@ namespace qe {
                     m_nested_div_z.push_back(z);
                     continue;
                 }
-                unsigned num_args = a->get_num_args();
-                for (unsigned i = 0; i < num_args; ++i) {
-                    todo.push_back(a->get_arg(i));
-                }                
+                for (expr* arg : *a) {
+                    todo.push_back(arg);
+                }
             }              
             return true;
         }
@@ -1469,16 +1483,18 @@ public:
         expr*    m_result;
         rational m_coeff;
         expr*    m_term;
+        ptr_vector<app> m_vars;
 
-        branch_formula(): m_fml(0), m_var(0), m_branch(0), m_result(0), m_term(0) {}
+        branch_formula(): m_fml(nullptr), m_var(nullptr), m_branch(0), m_result(nullptr), m_term(nullptr) {}
         
-        branch_formula(expr* fml, app* var, unsigned b, expr* r, rational coeff, expr* term):
+        branch_formula(expr* fml, app* var, unsigned b, expr* r, rational coeff, expr* term, app_ref_vector const& vars):
             m_fml(fml),
             m_var(var),
             m_branch(b),
             m_result(r),
             m_coeff(coeff),
-            m_term(term)
+            m_term(term),
+            m_vars(vars.size(), vars.c_ptr())
         {}
         
         unsigned mk_hash() const {
@@ -1520,14 +1536,14 @@ public:
             m_trail(m)
         {}
 
-        ~arith_plugin() {
+        ~arith_plugin() override {
             bounds_cache::iterator it = m_bounds_cache.begin(), end = m_bounds_cache.end();
             for (; it != end; ++it) {
                 dealloc(it->get_value());
             }
         }
 
-        virtual void assign(contains_app& contains_x, expr* fml, rational const& vl) {
+        void assign(contains_app& contains_x, expr* fml, rational const& vl) override {
             SASSERT(vl.is_unsigned());
             app* x = contains_x.x();
             unsigned v     = vl.get_unsigned();
@@ -1538,6 +1554,7 @@ public:
             if (get_cache(x, fml, v, result)) {
                 return;
             }
+            m_util.m_vars_added.reset();
             
             bounds_proc& bounds = get_bounds(x, fml);            
             bool is_lower = get_bound_sizes(bounds, x, t_size, e_size);            
@@ -1595,8 +1612,7 @@ public:
             expr_ref t(bounds.exprs(is_strict, is_lower)[index], m);
             rational a = bounds.coeffs(is_strict, is_lower)[index];
 
-            
-                                
+                                            
             mk_bounds(bounds, x, true,  is_eq, is_strict, is_lower, index, a, t, result);
             mk_bounds(bounds, x, false, is_eq, is_strict, is_lower, index, a, t, result);
 
@@ -1611,13 +1627,13 @@ public:
                   {
                       tout << vl << " " << mk_pp(bounds.atoms(is_strict, is_lower)[index],m) << "\n";
                       tout << mk_pp(fml, m) << "\n";
-                      tout << mk_pp(result, m) << "\n";
+                      tout << result << "\n";
                   }
                   );
         }
 
 
-        virtual bool get_num_branches(contains_app& contains_x, expr* fml, rational& nb) { 
+        bool get_num_branches(contains_app& contains_x, expr* fml, rational& nb) override {
             app* x = contains_x.x();
             if (!update_bounds(contains_x, fml)) {
                 return false;
@@ -1629,16 +1645,16 @@ public:
             return true;
         }
 
-        virtual void subst(contains_app& contains_x, rational const& vl, expr_ref& fml, expr_ref* def) {
+        void subst(contains_app& contains_x, rational const& vl, expr_ref& fml, expr_ref* def) override {
             SASSERT(vl.is_unsigned());            
-           if (def) {
+            if (def) {
                get_def(contains_x, vl.get_unsigned(), fml, *def);
             }
             VERIFY(get_cache(contains_x.x(), fml, vl.get_unsigned(), fml));
             TRACE("qe", tout << mk_pp(contains_x.x(), m) << " " << vl << "\n" << mk_pp(fml, m) << "\n";);
         } 
 
-        virtual bool project(contains_app& x, model_ref& model, expr_ref& fml) {
+        bool project(contains_app& x, model_ref& model, expr_ref& fml) override {
             if (!update_bounds(x, fml)) {
                 TRACE("qe", tout << mk_pp(x.x(), m) << " failed to update bounds\n";);
                 return false;
@@ -1652,19 +1668,19 @@ public:
         }
 
 
-        virtual unsigned get_weight(contains_app& contains_x, expr* fml) {
+        unsigned get_weight(contains_app& contains_x, expr* fml) override {
             return 2;
         }
 
-        virtual bool solve(conj_enum& conjs, expr* fml) {
+        bool solve(conj_enum& conjs, expr* fml) override {
             return m_util.solve(conjs, fml);
         }
 
-        virtual bool mk_atom(expr* e, bool p, expr_ref& result) {
+        bool mk_atom(expr* e, bool p, expr_ref& result) override {
             return m_util.mk_atom(e, p, result);
         }
 
-        virtual bool is_uninterpreted(app* f) {
+        bool is_uninterpreted(app* f) override {
             switch(f->get_decl_kind()) {
             case OP_NUM:
             case OP_LE:
@@ -1734,7 +1750,7 @@ public:
             x_subst x_t(m_util);
             bounds_proc& bounds = get_bounds(x, fml);    
             branch_formula bf;
-            VERIFY (m_subst.find(branch_formula(fml, x, v, 0, rational::zero(), 0), bf));
+            VERIFY (m_subst.find(branch_formula(fml, x, v, nullptr, rational::zero(), nullptr, m_util.m_vars_added), bf));
             x_t.set_term(bf.m_term);
             x_t.set_coeff(bf.m_coeff);
 
@@ -1933,7 +1949,7 @@ public:
                 }
             }
             assign(x, fml, vl);
-            subst(x, vl, fml, 0);
+            subst(x, vl, fml, nullptr);
             TRACE("qe", tout << mk_pp(fml, m) << "\n";);            
             return true;
         }
@@ -1956,7 +1972,7 @@ public:
                 vl = numeral(0);
             }
             assign(x, fml, vl);
-            subst(x, vl, fml, 0);
+            subst(x, vl, fml, nullptr);
             TRACE("qe", tout << mk_pp(fml, m) << "\n";);
             
             return true;
@@ -2016,16 +2032,19 @@ public:
             m_trail.push_back(fml);
             m_trail.push_back(result);
             if (term) m_trail.push_back(term);
-            m_subst.insert(branch_formula(fml, x, v, result, coeff, term));
+            m_subst.insert(branch_formula(fml, x, v, result, coeff, term, m_util.m_vars_added));
         }
         
         bool get_cache(app* x, expr* fml, unsigned v, expr_ref& result) {
             branch_formula bf;
-            if (!m_subst.find(branch_formula(fml, x, v, 0, rational::zero(), 0), bf)) {
+            if (!m_subst.find(branch_formula(fml, x, v, nullptr, rational::zero(), nullptr, m_util.m_vars_added), bf)) {
                 return false;
             }
             SASSERT(bf.m_result);
             result = bf.m_result;
+            for (app* v : bf.m_vars) {
+                m_util.add_var(v, false);
+            }            
             return true;
         }
 
@@ -2037,7 +2056,7 @@ public:
             if (!bounds.div_z(d, z_bv, z)) {
                 return;
             }
-            m_ctx.add_var(z_bv);
+            m_util.add_var(z_bv);
             
             //
             // assert 
@@ -2114,7 +2133,7 @@ public:
                 app* z1_bv = bounds.nested_div_z_bv(i);
                 app* z1 = bounds.nested_div_z(i);
 
-                m_ctx.add_var(z1_bv);
+                m_util.add_var(z1_bv);
 
                 //
                 // assert
@@ -2152,7 +2171,7 @@ public:
         }
 
         bounds_proc& get_bounds(app* x, expr* fml) {
-            bounds_proc* result = 0;
+            bounds_proc* result = nullptr;
             VERIFY (m_bounds_cache.find(x, fml, result));
             return *result;
         }      
@@ -2337,7 +2356,6 @@ public:
             unsigned sz = bounds.size(is_strict, !is_lower);
             bool is_strict_real = !is_eq_ctx && m_util.is_real(x) && !is_strict_ctx;                   
             bool strict_resolve = is_strict || is_strict_ctx || is_strict_real;
-            app* atm = bounds.atoms(is_strict_ctx, is_lower)[index];    
 
             for (unsigned i = 0; i < sz; ++i) {
                 app* e = bounds.atoms(is_strict, !is_lower)[i];
@@ -2355,6 +2373,7 @@ public:
                 m_ctx.add_constraint(true, mk_not(e), tmp);
 
                 TRACE("qe_verbose", 
+                      app* atm = bounds.atoms(is_strict_ctx, is_lower)[index];    
                       tout << mk_pp(atm, m) << " ";
                       tout << mk_pp(e, m) << " ==>\n";
                       tout << mk_pp(tmp, m) << "\n";
@@ -2388,7 +2407,7 @@ public:
         }
 
         bool update_bounds(contains_app& contains_x, expr* fml) {
-            bounds_proc* bounds = 0;
+            bounds_proc* bounds = nullptr;
             if (m_bounds_cache.find(contains_x.x(), fml, bounds)) {
                 return true;
             }
@@ -2437,7 +2456,7 @@ public:
             m_util.set_enable_linear(true); // (produce_models);
         }
 
-        virtual ~nlarith_plugin() {
+        ~nlarith_plugin() override {
             bcs_t::iterator it = m_cache.begin(), end = m_cache.end();
             for (; it != end; ++it) {
                 dealloc(it->get_value());
@@ -2448,7 +2467,7 @@ public:
             }                        
         }
 
-        virtual bool simplify(expr_ref& fml) { 
+        bool simplify(expr_ref& fml) override {
             expr_ref tmp(m), tmp2(m);
             m_factor_rw(fml, tmp);
             m_rewriter(tmp, tmp2);
@@ -2459,8 +2478,8 @@ public:
             return false; 
         }
                 
-        virtual void assign(contains_app& x, expr* fml, rational const& vl) {
-            nlarith::branch_conditions *brs;
+        void assign(contains_app& x, expr* fml, rational const& vl) override {
+            nlarith::branch_conditions *brs = nullptr;
             VERIFY (m_cache.find(x.x(), fml, brs));
             SASSERT(vl.is_unsigned());
             SASSERT(vl.get_unsigned() < brs->size());
@@ -2472,8 +2491,8 @@ public:
             m_ctx.add_constraint(true, result);
         }
         
-        virtual bool get_num_branches(contains_app& x, 
-                                      expr* fml, rational& num_branches) {
+        bool get_num_branches(contains_app& x,
+                              expr* fml, rational& num_branches) override {
             nlarith::branch_conditions *brs;
             if (m_cache.find(x.x(), fml, brs)) {
                 num_branches = rational(brs->size());
@@ -2496,8 +2515,8 @@ public:
             return true;
         }
         
-        virtual void subst(contains_app& x, rational const& vl, expr_ref& fml, expr_ref* def) {
-            nlarith::branch_conditions *brs;
+        void subst(contains_app& x, rational const& vl, expr_ref& fml, expr_ref* def) override {
+            nlarith::branch_conditions *brs = nullptr;
             VERIFY (m_cache.find(x.x(), fml, brs));
             SASSERT(vl.is_unsigned());
             SASSERT(vl.get_unsigned() < brs->size());
@@ -2515,8 +2534,8 @@ public:
         }
 
         
-        virtual unsigned get_weight(contains_app& x, expr* fml) { 
-            obj_map<app, unsigned>* weights = 0;
+        unsigned get_weight(contains_app& x, expr* fml) override {
+            obj_map<app, unsigned>* weights = nullptr;
             unsigned weight = 0;
             if (!m_weights.find(fml, weights)) {
                 weights = alloc(weight_m);
@@ -2534,12 +2553,12 @@ public:
             return UINT_MAX; 
         }
 
-        virtual bool solve(conj_enum& conjs, expr* fml) { return false; }
+        bool solve(conj_enum& conjs, expr* fml) override { return false; }
 
         // we don't need to modify the atom.
-        virtual bool mk_atom(expr* e, bool p, expr_ref& result) { return false;  }
+        bool mk_atom(expr* e, bool p, expr_ref& result) override { return false;  }
 
-        virtual bool is_uninterpreted(app* f) {
+        bool is_uninterpreted(app* f) override {
             if (m_produce_models) {
                 return true;
             }
