@@ -32,7 +32,6 @@ Revision History:
 #include "ast/rewriter/th_rewriter.h"
 #include "ast/for_each_expr.h"
 #include "ast/rewriter/expr_safe_replace.h"
-#include "util/cooperate.h"
 #include "ast/datatype_decl_plugin.h"
 
 #include "qe/qe_vartest.h"
@@ -69,6 +68,7 @@ namespace eq {
         ast_manager &   m;
         arith_util      a;
         datatype_util   dt;
+        bv_util         bv;
         is_variable_proc* m_is_variable;
         beta_reducer    m_subst;
         expr_ref_vector m_subst_map;
@@ -84,6 +84,37 @@ namespace eq {
         th_rewriter      m_rewriter;
         params_ref       m_params;
 
+        bool is_sub_extract(unsigned idx, expr* t) {
+            bool has_ground = false;            
+            if (bv.is_concat(t)) {
+                unsigned lo, hi;
+                ptr_buffer<expr> args;
+                args.append(to_app(t)->get_num_args(), to_app(t)->get_args());
+                for (unsigned i = 0; i < args.size(); ++i) {
+                    expr* arg = args[i];
+                    if (is_ground(arg)) {
+                        has_ground = true;
+                        continue;
+                    }
+                    if (bv.is_extract(arg, lo, hi, arg)) {
+                        if (is_var(arg) && to_var(arg)->get_idx() == idx) 
+                            continue;
+                    }
+                    if (bv.is_concat(arg)) {
+                        args.append(to_app(arg)->get_num_args(), to_app(arg)->get_args());
+                        continue;
+                    }
+                    return false;
+                }
+                return has_ground;
+            }
+            return false;
+        }
+
+        bool strict_occurs_var(unsigned idx, expr* t) {
+            return occurs_var(idx, t) && !is_sub_extract(idx, t);
+        }
+
         void der_sort_vars(ptr_vector<var> & vars, ptr_vector<expr> & definitions, unsigned_vector & order) {
             order.reset();
 
@@ -92,7 +123,7 @@ namespace eq {
             for (unsigned i = 0; i < definitions.size(); i++) {
                 var * v  = vars[i];
                 expr * t = definitions[i];
-                if (t == nullptr || has_quantifiers(t) || occurs_var(v->get_idx(), t))
+                if (t == nullptr || has_quantifiers(t) || strict_occurs_var(v->get_idx(), t))
                     definitions[i] = nullptr;
                 else
                     found = true; // found at least one candidate
@@ -112,6 +143,11 @@ namespace eq {
             for (unsigned i = 0; i < definitions.size(); i++) {
                 if (definitions[i] == nullptr)
                     continue;
+                if (is_sub_extract(vars[i]->get_idx(), definitions[i])) {
+                    order.push_back(i);
+                    done.mark(definitions[i]);
+                    continue;
+                }
                 var * v = vars[i];
                 SASSERT(v->get_idx() == i);
                 SASSERT(todo.empty());
@@ -153,7 +189,7 @@ namespace eq {
                                 done.mark(t);
                             }
                         }
-                            done.mark(t);
+                        done.mark(t);
                         todo.pop_back();
                         break;
                     case AST_QUANTIFIER:
@@ -255,6 +291,7 @@ namespace eq {
 
         bool is_var_eq(expr * e, ptr_vector<var>& vs, expr_ref_vector & ts) {
             expr* lhs = nullptr, *rhs = nullptr;
+            TRACE("qe_lite", tout << mk_pp(e, m) << "\n";);
 
             // (= VAR t), (iff VAR t), (iff (not VAR) t), (iff t (not VAR)) cases
             if (m.is_eq(e, lhs, rhs) && trivial_solve(lhs, rhs, e, vs, ts)) {
@@ -270,6 +307,7 @@ namespace eq {
                 if (res != e && m.is_eq(res, lhs, rhs) && is_variable(lhs)) {
                     vs.push_back(to_var(lhs));
                     ts.push_back(rhs);
+                    TRACE("qe_lite", tout << res << "\n";);
                     return true;
                 }
             }
@@ -314,7 +352,8 @@ namespace eq {
                 expr_ref r(m);
                 if (is_ground(cur)) r = cur; else m_subst(cur, r);
                 unsigned inx = sz - idx - 1;
-                CTRACE("topo_sort", m_subst_map.get(inx) != nullptr,
+                TRACE("qe_lite", tout << idx << " |-> " << r << "\n";);
+                CTRACE("top_sort", m_subst_map.get(inx) != nullptr,
                        tout << "inx is " << inx << "\n"
                        << "idx is " << idx << "\n"
                        << "sz is " << sz << "\n"
@@ -368,6 +407,7 @@ namespace eq {
                 break;
             }
             expr_ref new_e = m_subst(t, m_subst_map.size(), m_subst_map.c_ptr());
+            TRACE("qe_lite", tout << new_e << "\n";);
 
             // don't forget to update the quantifier patterns
             expr_ref_buffer  new_patterns(m);
@@ -483,30 +523,30 @@ namespace eq {
                                 m_var2pos[idx] = i;
                                 m_map[idx] = t;
                                 m_new_exprs.push_back(std::move(t));
-                          }
-                          // -- prefer ground
-                          else if (is_app(t) && to_app(t)->is_ground() &&
+                            }
+                            // -- prefer ground
+                            else if (is_app(t) && to_app(t)->is_ground() &&
                                    (!is_app(old_def) ||
                                     !to_app(old_def)->is_ground())) {
-                              m_pos2var[m_var2pos[idx]] = -1;
-                              m_pos2var[i] = idx;
-                              m_var2pos[idx] = i;
-                              m_map[idx] = t;
-                              m_new_exprs.push_back(std::move(t));
-                          }
-                          // -- prefer constants
-                          else if (is_uninterp_const(t)
-                                   /* && !is_uninterp_const(old_def) */){
-                              m_pos2var[m_var2pos[idx]] = -1;
-                              m_pos2var[i] = idx;
-                              m_var2pos[idx] = i;
-                              m_map[idx] = t;
-                              m_new_exprs.push_back(std::move(t));
-                          }
-                          TRACE ("qe_def",
-                                 tout << "Replacing definition of VAR " << idx << " from "
-                                 << mk_pp(old_def, m) << " to " << mk_pp(t, m)
-                                 << " inferred from: " << mk_pp(args[i], m) << "\n";);
+                                m_pos2var[m_var2pos[idx]] = -1;
+                                m_pos2var[i] = idx;
+                                m_var2pos[idx] = i;
+                                m_map[idx] = t;
+                                m_new_exprs.push_back(std::move(t));
+                            }
+                            // -- prefer constants
+                            else if (is_uninterp_const(t)
+                                     /* && !is_uninterp_const(old_def) */){
+                                m_pos2var[m_var2pos[idx]] = -1;
+                                m_pos2var[i] = idx;
+                                m_var2pos[idx] = i;
+                                m_map[idx] = t;
+                                m_new_exprs.push_back(std::move(t));
+                            }
+                            TRACE ("qe_def",
+                                   tout << "Replacing definition of VAR " << idx << " from "
+                                   << mk_pp(old_def, m) << " to " << mk_pp(t, m)
+                                   << " inferred from: " << mk_pp(args[i], m) << "\n";);
                         }
                     }
                 }
@@ -647,7 +687,6 @@ namespace eq {
         }
 
         void checkpoint() {
-            cooperate("der");
             if (m.canceled())
                 throw tactic_exception(m.limit().get_cancel_msg());
         }
@@ -657,6 +696,7 @@ namespace eq {
             m(m),
             a(m),
             dt(m),
+            bv(m),
             m_is_variable(nullptr),
             m_subst(m),
             m_subst_map(m),
@@ -671,6 +711,7 @@ namespace eq {
             m_solvers.reset();
             m_solvers.register_plugin(qe::mk_arith_solve_plugin(m, proc));            
             m_solvers.register_plugin(qe::mk_basic_solve_plugin(m, proc));            
+            m_solvers.register_plugin(qe::mk_bv_solve_plugin(m, proc));            
         }
 
         void operator()(quantifier * q, expr_ref & r, proof_ref & pr) {
@@ -694,7 +735,8 @@ namespace eq {
                 if (m.proofs_enabled() && r != q) {
                     pr = m.mk_transitivity(pr, curr_pr);
                 }
-            } while (q != r && is_quantifier(r));
+            } 
+            while (q != r && is_quantifier(r));
 
             m_new_exprs.reset();
         }
@@ -835,7 +877,6 @@ namespace ar {
         }
 
         void checkpoint() {
-            cooperate("der");
             if (m.canceled())
                 throw tactic_exception(m.limit().get_cancel_msg());
     }
@@ -2111,7 +2152,6 @@ namespace fm {
         }
 
         void checkpoint() {
-            cooperate("fm");
             if (m.canceled())
                 throw tactic_exception(m.limit().get_cancel_msg());
         }
@@ -2370,6 +2410,7 @@ void qe_lite::operator()(uint_set const& index_set, bool index_of_bound, expr_re
     (*m_impl)(index_set, index_of_bound, fmls);
 }
 
+namespace {
 class qe_lite_tactic : public tactic {
 
     struct imp {
@@ -2384,7 +2425,6 @@ class qe_lite_tactic : public tactic {
         void checkpoint() {
             if (m.canceled())
                 throw tactic_exception(m.limit().get_cancel_msg());
-            cooperate("qe-lite");
         }
 
         void debug_diff(expr* a, expr* b) {
@@ -2428,7 +2468,6 @@ class qe_lite_tactic : public tactic {
             tactic_report report("qe-lite", *g);
             proof_ref new_pr(m);
             expr_ref new_f(m);
-            bool produce_proofs = g->proofs_enabled();
 
             unsigned sz = g->size();
             for (unsigned i = 0; i < sz; i++) {
@@ -2440,7 +2479,7 @@ class qe_lite_tactic : public tactic {
                     continue;
                 new_f = f;
                 m_qe(new_f, new_pr);
-                if (produce_proofs) {
+                if (new_pr) {
                     expr* fact = m.get_fact(new_pr);
                     if (to_app(fact)->get_arg(0) != to_app(fact)->get_arg(1)) {
                         new_pr = m.mk_modus_ponens(g->pr(i), new_pr);
@@ -2494,7 +2533,6 @@ public:
         (*m_imp)(in, result);
     }
 
-
     void collect_statistics(statistics & st) const override {
         // m_imp->collect_statistics(st);
     }
@@ -2503,14 +2541,14 @@ public:
         // m_imp->reset_statistics();
     }
 
-
     void cleanup() override {
         ast_manager & m = m_imp->m;
-        dealloc(m_imp);
-        m_imp = alloc(imp, m, m_params);
+        m_imp->~imp();
+        m_imp = new (m_imp) imp(m, m_params);
     }
 
 };
+}
 
 tactic * mk_qe_lite_tactic(ast_manager & m, params_ref const & p) {
     return alloc(qe_lite_tactic, m, p);
