@@ -48,7 +48,7 @@ namespace qe {
 
     solve_plugin* mk_dt_solve_plugin(ast_manager& m, is_variable_proc& is_var);
 
-    // solve_plugin* mk_bv_solve_plugin(ast_manager& m, is_variable_proc& is_var);
+    solve_plugin* mk_bv_solve_plugin(ast_manager& m, is_variable_proc& is_var);
 
     // solve_plugin* mk_array_solve_plugin(ast_manager& m, is_variable_proc& is_var);
 }
