@@ -16,23 +16,20 @@ Author:
 Revision History:
 
 --*/
-#include"tactical.h"
-#include"filter_model_converter.h"
-#include"cooperate.h"
-#include"qe.h"
+#include "tactic/tactical.h"
+#include "qe/qe.h"
 
 class qe_tactic : public tactic {
+    statistics               m_st;
     struct     imp {
         ast_manager &            m;
         smt_params               m_fparams;
-        volatile bool            m_cancel;
         qe::expr_quant_elim      m_qe;
 
         imp(ast_manager & _m, params_ref const & p):
             m(_m),
             m_qe(m, m_fparams) {
             updt_params(p);
-            m_cancel = false;
         }
 
         void updt_params(params_ref const & p) {
@@ -45,24 +42,14 @@ class qe_tactic : public tactic {
             m_qe.collect_param_descrs(r);
         }
 
-        void set_cancel(bool f) {
-            m_cancel = f;
-            m_qe.set_cancel(f);
-        }
-
         void checkpoint() {
-            if (m_cancel)
-                throw tactic_exception(TACTIC_CANCELED_MSG);
-            cooperate("qe");
+            if (m.canceled()) 
+                throw tactic_exception(m.limit().get_cancel_msg());
         }
 
         void operator()(goal_ref const & g, 
-                        goal_ref_buffer & result, 
-                        model_converter_ref & mc, 
-                        proof_converter_ref & pc,
-                        expr_dependency_ref & core) {
+                        goal_ref_buffer & result) {
             SASSERT(g->is_well_sorted());
-            mc = 0; pc = 0; core = 0;
             tactic_report report("qe", *g);
             m_fparams.m_model = g->models_enabled();
             proof_ref new_pr(m);
@@ -78,17 +65,26 @@ class qe_tactic : public tactic {
                 if (!has_quantifiers(f))
                     continue;
                 m_qe(m.mk_true(), f, new_f);
-                new_pr = 0;
+                new_pr = nullptr;
                 if (produce_proofs) {
                     new_pr = m.mk_modus_ponens(g->pr(i), new_pr);
                 }
                 g->update(i, new_f, new_pr, g->dep(i));                
             }
             g->inc_depth();
+            g->elim_true();
             result.push_back(g.get());
             TRACE("qe", g->display(tout););
             SASSERT(g->is_well_sorted());
         }
+
+        void collect_statistics(statistics & st) const {
+            m_qe.collect_statistics(st);
+        }
+
+        void reset_statistics() {            
+        }
+
     };
     
     imp *      m_imp;
@@ -99,53 +95,48 @@ public:
         m_imp = alloc(imp, m, p);
     }
 
-    virtual tactic * translate(ast_manager & m) {
+    tactic * translate(ast_manager & m) override {
         return alloc(qe_tactic, m, m_params);
     }
         
-    virtual ~qe_tactic() {
+    ~qe_tactic() override {
         dealloc(m_imp);
     }
 
-    virtual void updt_params(params_ref const & p) {
+    void updt_params(params_ref const & p) override {
         m_params = p;
         m_imp->updt_params(p);
     }
 
    
-    virtual void collect_param_descrs(param_descrs & r) {
+    void collect_param_descrs(param_descrs & r) override {
         r.insert("qe_nonlinear", CPK_BOOL, "(default: false) enable virtual term substitution.");
         m_imp->collect_param_descrs(r);
     }
     
-    virtual void operator()(goal_ref const & in, 
-                            goal_ref_buffer & result, 
-                            model_converter_ref & mc, 
-                            proof_converter_ref & pc,
-                            expr_dependency_ref & core) {
-        (*m_imp)(in, result, mc, pc, core);
+    void operator()(goal_ref const & in, 
+                    goal_ref_buffer & result) override {
+        (*m_imp)(in, result);
+        m_st.reset();
+        m_imp->collect_statistics(m_st);
+        
     }
+
+    void collect_statistics(statistics & st) const override {
+        st.copy(m_st);
+    }
+
+    void reset_statistics() override {
+        m_st.reset();
+    }
+
     
-    virtual void cleanup() {
+    void cleanup() override {
         ast_manager & m = m_imp->m;
-        imp * d = m_imp;
-        #pragma omp critical (tactic_cancel)
-        {
-            m_imp = 0;
-        }
-        dealloc(d);
-        d = alloc(imp, m, m_params);
-        #pragma omp critical (tactic_cancel)
-        {
-            m_imp = d;
-        }
+        dealloc(m_imp);
+        m_imp = alloc(imp, m, m_params);
     }
     
-protected:
-    virtual void set_cancel(bool f) {
-        if (m_imp)
-            m_imp->set_cancel(f);
-    }
 };
 
 tactic * mk_qe_tactic(ast_manager & m, params_ref const & p) {
