@@ -19,9 +19,9 @@ Revision History:
 #ifndef DIMACS_H_
 #define DIMACS_H_
 
-#include"sat_types.h"
+#include "sat/sat_types.h"
 
-void parse_dimacs(std::istream & s, sat::solver & solver);
+bool parse_dimacs(std::istream & s, std::ostream& err, sat::solver & solver);
 
 #endif /* DIMACS_PARSER_H_ */
 
