@@ -19,7 +19,7 @@ Revision History:
 #ifndef SAT_CLAUSE_SET_H_
 #define SAT_CLAUSE_SET_H_
 
-#include"sat_clause.h"
+#include "sat/sat_clause.h"
 
 namespace sat {
 
