@@ -27,7 +27,8 @@ namespace sat {
     enum phase_selection {
         PS_ALWAYS_TRUE,
         PS_ALWAYS_FALSE,
-        PS_CACHING,
+        PS_BASIC_CACHING,
+        PS_SAT_CACHING,
         PS_RANDOM
     };
 
@@ -52,14 +53,6 @@ namespace sat {
         BH_LRB
     };
 
-    enum pb_solver {
-        PB_SOLVER,
-        PB_CIRCUIT,
-        PB_SORTING,
-        PB_TOTALIZER,
-        PB_SEGMENTED
-    };
-
     enum pb_resolve {
         PB_CARDINALITY,
         PB_ROUNDING
@@ -94,9 +87,13 @@ namespace sat {
     struct config {
         unsigned long long m_max_memory;
         phase_selection    m_phase;
-        unsigned           m_phase_caching_on;
-        unsigned           m_phase_caching_off;
+        unsigned           m_search_sat_conflicts;
+        unsigned           m_search_unsat_conflicts;
         bool               m_phase_sticky;
+        unsigned           m_rephase_base;
+        unsigned           m_reorder_base;
+        double             m_reorder_itau;
+        unsigned           m_reorder_activity_scale;
         bool               m_propagate_prefetch;
         restart_strategy   m_restart;
         bool               m_restart_fast;
@@ -104,6 +101,7 @@ namespace sat {
         double             m_restart_factor; // for geometric case
         double             m_restart_margin; // for ema
         unsigned           m_restart_max;
+        unsigned           m_activity_scale;
         double             m_fast_glue_avg;
         double             m_slow_glue_avg;
         unsigned           m_inprocess_max;
@@ -112,12 +110,16 @@ namespace sat {
         unsigned           m_burst_search;
         unsigned           m_max_conflicts;
         unsigned           m_num_threads;
+        bool               m_ddfw_search;
+        unsigned           m_ddfw_threads;
+        bool               m_prob_search;
         unsigned           m_local_search_threads;
         bool               m_local_search;
         local_search_mode  m_local_search_mode;
         bool               m_local_search_dbg_flips;
         unsigned           m_unit_walk_threads;
         bool               m_unit_walk;
+        bool               m_binspr;
         bool               m_lookahead_simplify;
         bool               m_lookahead_simplify_bca;
         cutoff_t           m_lookahead_cube_cutoff;
@@ -128,6 +130,7 @@ namespace sat {
         double             m_lookahead_cube_psat_clause_base;
         double             m_lookahead_cube_psat_trigger;
         reward_t           m_lookahead_reward;
+        bool               m_lookahead_double;
         bool               m_lookahead_global_autarky;
         double             m_lookahead_delta_fraction;
         bool               m_lookahead_use_learned;
@@ -150,18 +153,25 @@ namespace sat {
 
         bool               m_force_cleanup;
 
+        // backtracking
+        unsigned           m_backtrack_scopes;
+        unsigned           m_backtrack_init_conflicts;
 
         bool               m_minimize_lemmas;
         bool               m_dyn_sub_res;
         bool               m_core_minimize;
         bool               m_core_minimize_partial;
+
+        // drat proofs
         bool               m_drat;
+        bool               m_drat_binary;
         symbol             m_drat_file;
         bool               m_drat_check_unsat;
         bool               m_drat_check_sat;
+        bool               m_drat_activity;
         
-        pb_solver          m_pb_solver;
         bool               m_card_solver;
+        bool               m_xor_solver;
         pb_resolve         m_pb_resolve;
         pb_lemma_format    m_pb_lemma_format;
         
@@ -180,6 +190,7 @@ namespace sat {
         config(params_ref const & p);
         void updt_params(params_ref const & p);
         static void collect_param_descrs(param_descrs & d);
+
     };
 };
 
