@@ -23,9 +23,15 @@ Revision History:
 namespace sat {
     
     elim_eqs::elim_eqs(solver & s):
-        m_solver(s) {
+        m_solver(s),
+        m_to_delete(nullptr) {
     }
 
+    elim_eqs::~elim_eqs() {
+        dealloc(m_to_delete);
+    }
+
+
     inline literal norm(literal_vector const & roots, literal l) {
         if (l.sign()) 
             return ~roots[l.var()];
@@ -47,7 +53,7 @@ namespace sat {
                     literal l2 = it->get_literal();
                     literal r2 = norm(roots, l2);
                     if (r1 == r2) {
-                        m_solver.assign(r1, justification());
+                        m_solver.assign_unit(r1);
                         if (m_solver.inconsistent())
                             return;
                         // consume unit
@@ -86,6 +92,12 @@ namespace sat {
         m_new_bin.reset();
     }
 
+    void elim_eqs::drat_delete_clause() {
+        if (m_solver.m_config.m_drat) {
+            m_solver.m_drat.del(*m_to_delete->get());             
+        }
+    }
+
     void elim_eqs::cleanup_clauses(literal_vector const & roots, clause_vector & cs) {
         clause_vector::iterator it  = cs.begin();
         clause_vector::iterator it2 = it;
@@ -107,8 +119,16 @@ namespace sat {
                 it2++;
                 continue;
             }
-            if (!c.frozen())
+            if (!c.frozen()) {
                 m_solver.detach_clause(c);
+            }
+            
+            // save clause to be deleted for drat
+            if (m_solver.m_config.m_drat) {
+                if (!m_to_delete) m_to_delete = alloc(tmp_clause);
+                m_to_delete->set(sz, c.begin(), c.is_learned());
+            }
+
             // apply substitution
             for (i = 0; i < sz; i++) {   
                 literal lit = c[i];
@@ -124,60 +144,72 @@ namespace sat {
                         CTRACE("sat", l != norm(roots, l), tout << l << " " << norm(roots, l) << "\n"; tout.flush(););
                         SASSERT(l == norm(roots, l));
                     } });
+
             // remove duplicates, and check if it is a tautology
-            literal l_prev = null_literal;
             unsigned j = 0;
+            literal l_prev = null_literal;
             for (i = 0; i < sz; i++) {
                 literal l = c[i];
-                if (l == l_prev)
-                    continue;
-                if (l == ~l_prev)
+                if (l == ~l_prev) {
                     break;
+                }
+                if (l == l_prev) {
+                    continue;
+                }
+                SASSERT(l != ~l_prev);
                 l_prev = l;
                 lbool val = m_solver.value(l);
-                if (val == l_true)
-                    break; // clause was satisfied
-                if (val == l_false)
+                if (val == l_true) {
+                    break;
+                }
+                if (val == l_false) {
                     continue; // skip
+                }
                 c[j] = l;                
                 j++;
             }
+            TRACE("elim_eqs", tout << "after removing duplicates: " << c << " j: " << j << "\n";);
+
             if (i < sz) {
-                // clause is a tautology or was simplified to true
+                drat_delete_clause();
+                c.set_removed(true);
                 m_solver.del_clause(c);
                 continue; 
             }
-            if (j == 0) {
-                // empty clause
-                m_solver.set_conflict(justification());
+
+            switch (j) {
+            case 0:
+                m_solver.set_conflict();
                 for (; it != end; ++it) {
                     *it2 = *it;
                     it2++;
                 }
                 cs.set_end(it2);
-                return;
-            }
-            TRACE("elim_eqs", tout << "after removing duplicates: " << c << " j: " << j << "\n";);
-
-            SASSERT(j >= 1);
-            switch (j) {
+                return;                
             case 1:
-                m_solver.assign(c[0], justification());
+                m_solver.assign_unit(c[0]);
+                drat_delete_clause();
+                c.set_removed(true);
                 m_solver.del_clause(c);
                 break;
             case 2:
                 m_solver.mk_bin_clause(c[0], c[1], c.is_learned());
+                drat_delete_clause();
+                c.set_removed(true);
                 m_solver.del_clause(c);
                 break;
             default:
                 SASSERT(*it == &c);
                 if (j < sz) {
-                    if (m_solver.m_config.m_drat) m_solver.m_drat.del(c); 
                     c.shrink(j);
-                    if (m_solver.m_config.m_drat) m_solver.m_drat.add(c, true); 
                 }
-                else
+                else {
                     c.update_approx();
+                }
+                if (m_solver.m_config.m_drat) {
+                    m_solver.m_drat.add(c, true);
+                    drat_delete_clause();
+                }
 
                 DEBUG_CODE(for (literal l : c) VERIFY(l == norm(roots, l)););
                 
@@ -198,7 +230,8 @@ namespace sat {
             literal  l(v, false);
             literal r  = roots[v];
             SASSERT(v != r.var());
-            bool root_ok = !m_solver.is_external(v) || m_solver.set_root(l, r);
+            bool set_root = m_solver.set_root(l, r);
+            bool root_ok = !m_solver.is_external(v) || set_root;
             if (m_solver.is_assumption(v) || (m_solver.is_external(v) && (m_solver.is_incremental() || !root_ok))) {
                 // cannot really eliminate v, since we have to notify extension of future assignments
                 m_solver.mk_bin_clause(~l, r, false);
@@ -207,7 +240,7 @@ namespace sat {
             else {
                 model_converter::entry & e = mc.mk(model_converter::ELIM_VAR, v);
                 TRACE("save_elim", tout << "marking as deleted: " << v << " l: " << l << " r: " << r << "\n";);
-                m_solver.m_eliminated[v] = true;
+                m_solver.set_eliminated(v, true);
                 mc.insert(e, ~l, r);
                 mc.insert(e,  l, ~r);
             }
