@@ -23,6 +23,7 @@ Revision History:
 
 namespace sat {
     class solver;
+    class tmp_clause;
     
     class elim_eqs {
         struct bin {
@@ -32,6 +33,8 @@ namespace sat {
         };
         svector<bin> m_new_bin;
         solver & m_solver;
+        tmp_clause* m_to_delete;
+        void drat_delete_clause();
         void save_elim(literal_vector const & roots, bool_var_vector const & to_elim);
         void cleanup_clauses(literal_vector const & roots, clause_vector & cs);
         void cleanup_bin_watches(literal_vector const & roots);
@@ -39,6 +42,7 @@ namespace sat {
         bool check_clause(clause const& c, literal_vector const& roots) const;
     public:
         elim_eqs(solver & s);
+        ~elim_eqs();
         void operator()(literal_vector const & roots, bool_var_vector const & to_elim);
     };
 
