@@ -20,11 +20,17 @@ Notes:
 #ifndef HS_INC_SAT_SOLVER_H_
 #define HS_INC_SAT_SOLVER_H_
 
-#include "solver.h"
+#include "solver/solver.h"
 
-solver* mk_inc_sat_solver(ast_manager& m, params_ref const& p);
+class tactic;
 
-lbool inc_sat_check_sat(solver& s, unsigned sz, expr*const* soft, rational const* weights, rational const& max_weight);
+solver* mk_inc_sat_solver(ast_manager& m, params_ref const& p, bool incremental_mode = true);
+
+tactic* mk_psat_tactic(ast_manager& m, params_ref const& p);
+
+/*
+  ADD_TACTIC('psat', '(try to) solve goal using a parallel SAT solver.', 'mk_psat_tactic(m, p)')
+*/
 
 void  inc_sat_display(std::ostream& out, solver& s, unsigned sz, expr*const* soft, rational const* _weights);
 
