@@ -26,8 +26,13 @@ Revision History:
 #include "util/common_msgs.h"
 #include "util/vector.h"
 #include "util/uint_set.h"
+#include "util/stopwatch.h"
 #include<iomanip>
 
+class params_ref;
+class reslimit;
+class statistics;
+
 namespace sat {
 #define SAT_VB_LVL 10
 
@@ -119,11 +124,8 @@ namespace sat {
 
     typedef approx_set_tpl<bool_var, u2u, unsigned> var_approx_set;
 
-    enum phase {
-        POS_PHASE, NEG_PHASE, PHASE_NOT_AVAILABLE
-    };
-
     class solver;
+    class parallel;
     class lookahead;
     class unit_walk;
     class clause;
@@ -220,8 +222,11 @@ namespace sat {
 
     inline std::ostream & operator<<(std::ostream & out, mem_stat const & m) {
         double mem = static_cast<double>(memory::get_allocation_size())/static_cast<double>(1024*1024);
-        out << " :memory " << std::fixed << std::setprecision(2) << mem;
-        return out;
+        return out << std::fixed << std::setprecision(2) << mem;
+    }
+
+    inline std::ostream& operator<<(std::ostream& out, stopwatch const& sw) {
+        return out << " :time " << std::fixed << std::setprecision(2) << sw.get_seconds();
     }
 
     struct dimacs_lit {
@@ -253,6 +258,22 @@ namespace sat {
     inline std::ostream & operator<<(std::ostream & out, literal_vector const & ls) {
         return out << mk_lits_pp(ls.size(), ls.c_ptr());
     }
+
+    class i_local_search {
+    public:
+        virtual ~i_local_search() {}
+        virtual void add(solver const& s) = 0;
+        virtual void updt_params(params_ref const& p) = 0;
+        virtual void set_seed(unsigned s) = 0;
+        virtual lbool check(unsigned sz, literal const* assumptions, parallel* par) = 0;
+        virtual void reinit(solver& s) = 0;        
+        virtual unsigned num_non_binary_clauses() const = 0;
+        virtual reslimit& rlimit() = 0;
+        virtual model const& get_model() const = 0;
+        virtual void collect_statistics(statistics& st) const = 0;        
+        virtual double get_priority(bool_var v) const { return 0; }
+
+    };
 };
 
 #endif
