@@ -18,6 +18,7 @@ Revision History:
 --*/
 #include "sat/sat_watched.h"
 #include "sat/sat_clause.h"
+#include "sat/sat_extension.h"
 
 namespace sat {
 
@@ -85,7 +86,15 @@ namespace sat {
             }    
         }
         wlist.set_end(it2);
-        //VERIFY(found);
+#if 0
+        VERIFY(found);
+        for (watched const& w2 : wlist) {
+            if (w2 == w) {
+                std::cout << l1 << " " << l2 << "\n";
+            }
+            //VERIFY(w2 != w);
+        }
+#endif
     }
 
     void conflict_cleanup(watch_list::iterator it, watch_list::iterator it2, watch_list& wlist) {
@@ -96,7 +105,7 @@ namespace sat {
     }
 
 
-    std::ostream& display_watch_list(std::ostream & out, clause_allocator const & ca, watch_list const & wlist) {
+    std::ostream& display_watch_list(std::ostream & out, clause_allocator const & ca, watch_list const & wlist, extension* ext) {
         bool first = true;
         for (watched const& w : wlist) {
             if (first)
@@ -116,7 +125,12 @@ namespace sat {
                 out << "(" << w.get_blocked_literal() << " " << *(ca.get_clause(w.get_clause_offset())) << ")";
                 break;
             case watched::EXT_CONSTRAINT:
-                out << "ext: " << w.get_ext_constraint_idx();
+                if (ext) {
+                    ext->display_constraint(out, w.get_ext_constraint_idx());
+                }
+                else  {
+                    out << "ext: " << w.get_ext_constraint_idx();
+                }
                 break;
             default: 
                 UNREACHABLE();
