@@ -19,8 +19,8 @@ Notes:
 #ifndef ATOM2BOOL_VAR_H_
 #define ATOM2BOOL_VAR_H_
 
-#include"expr2var.h"
-#include"sat_types.h"
+#include "ast/expr2var.h"
+#include "sat/sat_types.h"
 
 /**
    \brief Mapping from atoms into SAT boolean variables.
@@ -31,6 +31,7 @@ public:
     void insert(expr * n, sat::bool_var v) { expr2var::insert(n, v); }
     sat::bool_var to_bool_var(expr * n) const;
     void mk_inv(expr_ref_vector & lit2expr) const;
+    void mk_var_inv(app_ref_vector & var2expr) const;
     // return true if the mapping contains uninterpreted atoms.
     bool interpreted_atoms() const { return expr2var::interpreted_vars(); }
 };
