@@ -62,7 +62,7 @@ public:
        \warning conversion throws a tactic_exception, if it is interrupted (by set_cancel),
        an unsupported operator is found, or memory consumption limit is reached (set with param :max-memory).
     */
-    void operator()(goal const & g, params_ref const & p, sat::solver & t, atom2bool_var & m, dep2asm_map& dep2asm, bool default_external = false, bool is_lemma = false);
+    void operator()(goal const & g, params_ref const & p, sat::solver_core & t, atom2bool_var & m, dep2asm_map& dep2asm, bool default_external = false, bool is_lemma = false);
 
     void get_interpreted_atoms(expr_ref_vector& atoms);
 
@@ -79,7 +79,7 @@ public:
         ast_manager&            m;
         sat::model_converter    m_smc;
         generic_model_converter_ref m_gmc;
-        app_ref_vector         m_var2expr;
+        app_ref_vector          m_var2expr;
 
         // flushes from m_smc to m_gmc;
         void flush_gmc();
@@ -88,7 +88,8 @@ public:
         mc(ast_manager& m);
         ~mc() override {}
         // flush model converter from SAT solver to this structure.
-        void flush_smc(sat::solver& s, atom2bool_var const& map);         
+        void flush_smc(sat::solver_core& s, atom2bool_var const& map);
+        void operator()(sat::model& m);
         void operator()(model_ref& md) override;
         void operator()(expr_ref& fml) override; 
         model_converter* translate(ast_translation& translator) override;
@@ -113,7 +114,7 @@ public:
        \warning conversion throws a tactic_exception, if it is interrupted (by set_cancel),
        or memory consumption limit is reached (set with param :max-memory).
     */
-    void operator()(sat::solver & t, atom2bool_var const & m, params_ref const & p, goal & s, ref<mc> & mc);
+    void operator()(sat::solver_core & t, atom2bool_var const & m, params_ref const & p, goal & s, ref<mc> & mc);
     
 };
 
