@@ -13,7 +13,9 @@ Author:
 #ifndef OPT_FRONTEND_H_
 #define OPT_FRONTEND_H_
 
-unsigned parse_opt(char const* file_name, bool is_wcnf);
+enum opt_format { opb_t, wcnf_t, lp_t };
+
+unsigned parse_opt(char const* file_name, opt_format f);
 
 #endif /* OPT_FRONTEND_H_ */
 
