@@ -19,8 +19,9 @@ Revision History:
 #ifndef MAM_H_
 #define MAM_H_
 
-#include"ast.h"
-#include"smt_types.h"
+#include "ast/ast.h"
+#include "smt/smt_types.h"
+#include <tuple>
 
 namespace smt {
     /**
@@ -57,7 +58,7 @@ namespace smt {
 
         virtual void display(std::ostream& out) = 0;
         
-        virtual void on_match(quantifier * q, app * pat, unsigned num_bindings, enode * const * bindings, unsigned max_generation, ptr_vector<enode> & used_enodes) = 0;
+        virtual void on_match(quantifier * q, app * pat, unsigned num_bindings, enode * const * bindings, unsigned max_generation, vector<std::tuple<enode *, enode *>> & used_enodes) = 0;
         
         virtual bool is_shared(enode * n) const = 0;
 
