@@ -17,7 +17,7 @@ Revision History:
 
 --*/
 
-#include"old_interval.h"
+#include "smt/old_interval.h"
 
 void ext_numeral::neg() {
     switch (m_kind) {
@@ -81,7 +81,7 @@ ext_numeral & ext_numeral::operator*=(ext_numeral const & other) {
         m_value.reset();
         return *this;
     }
-    
+
     if (is_infinite() || other.is_infinite()) {
         if (sign() == other.sign())
             m_kind = PLUS_INFINITY;
@@ -157,8 +157,8 @@ interval::interval(v_dependency_manager & m):
     m_upper(true),
     m_lower_open(true),
     m_upper_open(true),
-    m_lower_dep(0),
-    m_upper_dep(0) {
+    m_lower_dep(nullptr),
+    m_upper_dep(nullptr) {
 }
 
 /**
@@ -203,7 +203,7 @@ interval::interval(v_dependency_manager & m, rational const & val, v_dependency
     m_lower_dep(l_dep),
     m_upper_dep(u_dep) {
 }
- 
+
 /**
    \brief Create intervals (-oo, val], (-oo, val), [val, oo), (val, oo)
 */
@@ -215,12 +215,12 @@ interval::interval(v_dependency_manager & m, rational const & val, bool open, bo
         m_lower_dep  = d;
         m_upper      = ext_numeral(true);
         m_upper_open = true;
-        m_upper_dep  = 0;
+        m_upper_dep  = nullptr;
     }
     else {
         m_lower      = ext_numeral(false);
         m_lower_open = true;
-        m_lower_dep  = 0;
+        m_lower_dep  = nullptr;
         m_upper      = ext_numeral(val);
         m_upper_open = open;
         m_upper_dep  = d;
@@ -252,8 +252,8 @@ interval & interval::operator+=(interval const & other) {
     m_upper      += other.m_upper;
     m_lower_open |= other.m_lower_open;
     m_upper_open |= other.m_upper_open;
-    m_lower_dep   = m_lower.is_infinite() ? 0 : m_manager.mk_join(m_lower_dep, other.m_lower_dep);
-    m_upper_dep   = m_upper.is_infinite() ? 0 : m_manager.mk_join(m_upper_dep, other.m_upper_dep);
+    m_lower_dep   = m_lower.is_infinite() ? nullptr : m_manager.mk_join(m_lower_dep, other.m_lower_dep);
+    m_upper_dep   = m_upper.is_infinite() ? nullptr : m_manager.mk_join(m_upper_dep, other.m_upper_dep);
     return *this;
 }
 
@@ -271,8 +271,8 @@ interval & interval::operator-=(interval const & other) {
     return operator+=(tmp);
 }
 
-v_dependency * interval::join(v_dependency * d1, v_dependency * d2, v_dependency * d3, v_dependency * d4) { 
-    return m_manager.mk_join(m_manager.mk_join(d1, d2), m_manager.mk_join(d3,d4)); 
+v_dependency * interval::join(v_dependency * d1, v_dependency * d2, v_dependency * d3, v_dependency * d4) {
+    return m_manager.mk_join(m_manager.mk_join(d1, d2), m_manager.mk_join(d3,d4));
 }
 
 /**
@@ -283,7 +283,7 @@ v_dependency * interval::join_opt(v_dependency * d1, v_dependency * d2, v_depend
         return join(d1, d2);
     if (opt2 == d1 || opt2 == d2)
         return join(d1, d2);
-    if (opt1 == 0 || opt2 == 0)
+    if (opt1 == nullptr || opt2 == nullptr)
         return join(d1, d2);
     // TODO: more opts...
     return join(d1, d2, opt1);
@@ -318,7 +318,7 @@ interval & interval::operator*=(interval const & other) {
     v_dependency * d_d = other.m_upper_dep;
 
     TRACE("interval_bug", tout << "operator*= " << *this << " " << other << "\n";);
-    
+
     if (is_N()) {
         if (other.is_N()) {
             // x <= b <= 0, y <= d <= 0 --> b*d <= x*y
@@ -331,8 +331,8 @@ interval & interval::operator*=(interval const & other) {
             m_upper_open = a_o || c_o; SASSERT(a.is_neg() && c.is_neg());
             m_lower      = new_lower;
             m_upper      = new_upper;
-            m_lower_dep  = m_lower.is_infinite() ? 0 : join(b_d, d_d);
-            m_upper_dep  = m_upper.is_infinite() ? 0 : join_opt(a_d, c_d, b_d, d_d);
+            m_lower_dep  = m_lower.is_infinite() ? nullptr : join(b_d, d_d);
+            m_upper_dep  = m_upper.is_infinite() ? nullptr : join_opt(a_d, c_d, b_d, d_d);
         }
         else if (other.is_M()) {
             // a <= x <= b <= 0,  y <= d, d > 0 --> a*d <= x*y (uses the fact that b is not positive)
@@ -344,8 +344,8 @@ interval & interval::operator*=(interval const & other) {
             m_upper_open = a_o || c_o;
             m_lower      = new_lower;
             m_upper      = new_upper;
-            m_lower_dep  = m_lower.is_infinite() ? 0 : join(a_d, d_d, b_d);
-            m_upper_dep  = m_upper.is_infinite() ? 0 : join(a_d, c_d, b_d);
+            m_lower_dep  = m_lower.is_infinite() ? nullptr : join(a_d, d_d, b_d);
+            m_upper_dep  = m_upper.is_infinite() ? nullptr : join(a_d, c_d, b_d);
         }
         else {
             // a <= x <= b <= 0, 0 <= c <= y <= d --> a*d <= x*y (uses the fact that x is neg (b is not positive) or y is pos (c is not negative))
@@ -359,8 +359,8 @@ interval & interval::operator*=(interval const & other) {
             m_upper_open = (is_N0_old || other.is_P0()) ? false : (b_o || c_o);
             m_lower      = new_lower;
             m_upper      = new_upper;
-            m_lower_dep  = m_lower.is_infinite() ? 0 : join_opt(a_d, d_d, b_d, c_d);
-            m_upper_dep  = m_upper.is_infinite() ? 0 : join(b_d, c_d);
+            m_lower_dep  = m_lower.is_infinite() ? nullptr : join_opt(a_d, d_d, b_d, c_d);
+            m_upper_dep  = m_upper.is_infinite() ? nullptr : join(b_d, c_d);
         }
     }
     else if (is_M()) {
@@ -374,8 +374,8 @@ interval & interval::operator*=(interval const & other) {
             m_upper_open = a_o || c_o; SASSERT(a.is_neg() && c.is_neg());
             m_lower      = new_lower;
             m_upper      = new_upper;
-            m_lower_dep  = m_lower.is_infinite() ? 0 : join(b_d, c_d, d_d);
-            m_upper_dep  = m_upper.is_infinite() ? 0 : join(a_d, c_d, d_d);
+            m_lower_dep  = m_lower.is_infinite() ? nullptr : join(b_d, c_d, d_d);
+            m_upper_dep  = m_upper.is_infinite() ? nullptr : join(a_d, c_d, d_d);
         }
         else if (other.is_M()) {
             TRACE("interval_bug", tout << "(M, M)\n";);
@@ -404,8 +404,8 @@ interval & interval::operator*=(interval const & other) {
                 m_upper       = bd;
                 m_upper_open  = bd_o;
             }
-            m_lower_dep = m_lower.is_infinite() ? 0 : join(a_d, b_d, c_d, d_d);
-            m_upper_dep = m_upper.is_infinite() ? 0 : join(a_d, b_d, c_d, d_d);
+            m_lower_dep = m_lower.is_infinite() ? nullptr : join(a_d, b_d, c_d, d_d);
+            m_upper_dep = m_upper.is_infinite() ? nullptr : join(a_d, b_d, c_d, d_d);
         }
         else {
             // a < 0, a <= x, 0 <= c <= y <= d --> a*d <= x*y (uses the fact that c is not negative)
@@ -418,8 +418,8 @@ interval & interval::operator*=(interval const & other) {
             m_upper_open = b_o || d_o; SASSERT(b.is_pos() && d.is_pos());
             m_lower      = new_lower;
             m_upper      = new_upper;
-            m_lower_dep  = m_lower.is_infinite() ? 0 : join(a_d, d_d, c_d);
-            m_upper_dep  = m_upper.is_infinite() ? 0 : join(b_d, d_d, c_d);
+            m_lower_dep  = m_lower.is_infinite() ? nullptr : join(a_d, d_d, c_d);
+            m_upper_dep  = m_upper.is_infinite() ? nullptr : join(b_d, d_d, c_d);
         }
     }
     else {
@@ -435,8 +435,8 @@ interval & interval::operator*=(interval const & other) {
             m_upper_open = (is_P0_old || other.is_N0()) ? false : a_o || d_o;
             m_lower      = new_lower;
             m_upper      = new_upper;
-            m_lower_dep  = m_lower.is_infinite() ? 0 : join_opt(b_d, c_d, a_d, d_d);
-            m_upper_dep  = m_upper.is_infinite() ? 0 : join(a_d, d_d);
+            m_lower_dep  = m_lower.is_infinite() ? nullptr : join_opt(b_d, c_d, a_d, d_d);
+            m_upper_dep  = m_upper.is_infinite() ? nullptr : join(a_d, d_d);
         }
         else if (other.is_M()) {
             // 0 <= a <= x <= b,  c <= y --> b*c <= x*y (uses the fact that a is not negative)
@@ -448,11 +448,11 @@ interval & interval::operator*=(interval const & other) {
             m_upper_open = b_o || d_o;
             m_lower      = new_lower;
             m_upper      = new_upper;
-            m_lower_dep  = m_lower.is_infinite() ? 0 : join(b_d, c_d, a_d);
-            m_upper_dep  = m_upper.is_infinite() ? 0 : join(b_d, d_d, a_d);
+            m_lower_dep  = m_lower.is_infinite() ? nullptr : join(b_d, c_d, a_d);
+            m_upper_dep  = m_upper.is_infinite() ? nullptr : join(b_d, d_d, a_d);
         }
         else {
-            // 0 <= a <= x, 0 <= c <= y --> a*c <= x*y 
+            // 0 <= a <= x, 0 <= c <= y --> a*c <= x*y
             // x <= b, y <= d --> x*y <= b*d (uses the fact that x is pos (a is not negative) or y is pos (c is not negative))
             TRACE("interval_bug", tout << "(P, P)\n";);
             SASSERT(other.is_P());
@@ -462,12 +462,12 @@ interval & interval::operator*=(interval const & other) {
             m_upper_open = b_o || d_o; SASSERT(b.is_pos() && d.is_pos());
             m_lower      = new_lower;
             m_upper      = new_upper;
-            m_lower_dep  = m_lower.is_infinite() ? 0 : join(a_d, c_d);
-            m_upper_dep  = m_upper.is_infinite() ? 0 : join_opt(b_d, d_d, a_d, c_d);
+            m_lower_dep  = m_lower.is_infinite() ? nullptr : join(a_d, c_d);
+            m_upper_dep  = m_upper.is_infinite() ? nullptr : join_opt(b_d, d_d, a_d, c_d);
         }
     }
     TRACE("interval_bug", tout << "operator*= result: " << *this << "\n";);
-    CTRACE("interval", !(!(contains_zero1 || contains_zero2) || contains_zero()), 
+    CTRACE("interval", !(!(contains_zero1 || contains_zero2) || contains_zero()),
            tout << "contains_zero1: " << contains_zero1 << ", contains_zero2: " << contains_zero2 << ", contains_zero(): " << contains_zero() << "\n";);
     SASSERT(!(contains_zero1 || contains_zero2) || contains_zero());
     return *this;
@@ -482,7 +482,7 @@ bool interval::contains_zero() const {
           tout << "m_upper.is_zero:  " << m_upper.is_zero() << "\n";
           tout << "m_upper_open:     " << m_upper_open << "\n";
           tout << "result:           " << ((m_lower.is_neg() || (m_lower.is_zero() && !m_lower_open)) && (m_upper.is_pos() || (m_upper.is_zero() && !m_upper_open))) << "\n";);
-    return 
+    return
         (m_lower.is_neg() || (m_lower.is_zero() && !m_lower_open)) &&
         (m_upper.is_pos() || (m_upper.is_zero() && !m_upper_open));
 }
@@ -510,7 +510,7 @@ interval & interval::inv() {
         ext_numeral new_upper;
         if (m_lower.is_zero()) {
             SASSERT(m_lower_open);
-            ext_numeral plus_infinity(true); 
+            ext_numeral plus_infinity(true);
             new_upper = plus_infinity;
         }
         else {
@@ -561,15 +561,24 @@ interval & interval::operator/=(interval const & other) {
         TRACE("interval", other.display_with_dependencies(tout););
         if (other.m_lower.is_pos() || (other.m_lower.is_zero() && other.m_lower_open)) {
             // other.lower > 0
+            //     x in ([0, 0] / [other.lo, other.up]), for other.lo > 0
+            // <=>
+            //     x >= 0: because y*x >= 0 & y > 0
+            //     x <= 0: because y*x <= 0 & y > 0
             m_lower_dep  = join(m_lower_dep, other.m_lower_dep);
             m_upper_dep  = join(m_upper_dep, other.m_lower_dep);
         }
         else {
             // assertion must hold since !other.contains_zero()
-            SASSERT(other.m_upper.is_neg() || (other.m_upper.is_zero() && other.m_upper_open));
+            SASSERT(other.m_upper.is_neg() || (other.m_upper.is_zero() && other.m_upper_open));            
             // other.upper < 0
-            m_lower_dep  = join(m_lower_dep, other.m_upper_dep);
-            m_upper_dep  = join(m_upper_dep, other.m_upper_dep);
+            //     x in ([0, 0] / [other.lo, other.up]), for up < 0
+            // <=>
+            //     x >= 0: because y*x <= 0 & y < 0
+            //     x <= 0: because y*x >= 0 & y < 0
+            v_dependency* lower_dep = m_lower_dep;
+            m_lower_dep  = join(m_upper_dep, other.m_upper_dep);
+            m_upper_dep  = join(lower_dep, other.m_upper_dep);
         }
         return *this;
     }
@@ -590,18 +599,18 @@ void interval::expt(unsigned n) {
             // 0 < a <= x <= b --> x^n <= b^n (use lower and upper bound -- need the fact that x is positive)
             m_lower.expt(n);
             m_upper.expt(n);
-            m_upper_dep = m_upper.is_infinite() ? 0 : m_manager.mk_join(m_lower_dep, m_upper_dep);
+            m_upper_dep = m_upper.is_infinite() ? nullptr : m_manager.mk_join(m_lower_dep, m_upper_dep);
         }
         else if (m_upper.is_neg()) {
             // [l, u]^n = [u^n, l^n] if u < 0
             // a <= x <= b < 0   -->  x^n <= a^n (use lower and upper bound -- need the fact that x is negative)
-            // x <= b < 0        -->  b^n <= x^n 
+            // x <= b < 0        -->  b^n <= x^n
             std::swap(m_lower, m_upper);
             std::swap(m_lower_open, m_upper_open);
             std::swap(m_lower_dep, m_upper_dep);
             m_lower.expt(n);
             m_upper.expt(n);
-            m_upper_dep = m_upper.is_infinite() ? 0 : m_manager.mk_join(m_lower_dep, m_upper_dep);
+            m_upper_dep = m_upper.is_infinite() ? nullptr : m_manager.mk_join(m_lower_dep, m_upper_dep);
         }
         else {
             // [l, u]^n = [0, max{l^n, u^n}] otherwise
@@ -614,10 +623,10 @@ void interval::expt(unsigned n) {
                 m_upper      = m_lower;
                 m_upper_open = m_lower_open;
             }
-            m_upper_dep  = m_upper.is_infinite() ? 0 : m_manager.mk_join(m_lower_dep, m_upper_dep); 
+            m_upper_dep  = m_upper.is_infinite() ? nullptr : m_manager.mk_join(m_lower_dep, m_upper_dep);
             m_lower      = ext_numeral(0);
             m_lower_open = false;
-            m_lower_dep  = 0;
+            m_lower_dep  = nullptr;
         }
     }
     else {
