@@ -19,7 +19,7 @@ Revision History:
 #ifndef DYN_ACK_PARAMS_H_
 #define DYN_ACK_PARAMS_H_
 
-#include"params.h"
+#include "util/params.h"
 
 enum dyn_ack_strategy {
     DACK_DISABLED,
@@ -47,6 +47,8 @@ public:
     }
 
     void updt_params(params_ref const & _p);
+
+    void display(std::ostream & out) const;
 };
 
 
