@@ -19,6 +19,8 @@ Revision History:
 #ifndef THEORY_DATATYPE_PARAMS_H_
 #define THEORY_DATATYPE_PARAMS_H_
 
+#include "smt/params/smt_params_helper.hpp"
+
 struct theory_datatype_params {
     unsigned   m_dt_lazy_splits;
 
@@ -26,11 +28,12 @@ struct theory_datatype_params {
         m_dt_lazy_splits(1) {
     }
 
-#if 0
-    void register_params(ini_params & p) {
-        p.register_unsigned_param("dt_lazy_splits", m_dt_lazy_splits, "How lazy datatype splits are performed: 0- eager, 1- lazy for infinite types, 2- lazy");
+    void updt_params(params_ref const & _p) {
+        smt_params_helper p(_p);
+        m_dt_lazy_splits = p.dt_lazy_splits(); 
     }
-#endif
+
+    void display(std::ostream & out) const { out << "m_dt_lazy_splits=" << m_dt_lazy_splits << std::endl; }
 };
 
 
