@@ -23,8 +23,6 @@ void theory_pb_params::updt_params(params_ref const & _p) {
     smt_params_helper p(_p);
     m_pb_conflict_frequency = p.pb_conflict_frequency();
     m_pb_learn_complements = p.pb_learn_complements();
-    m_pb_enable_compilation = p.pb_enable_compilation();
-    m_pb_enable_simplex = p.pb_enable_simplex();
 }
 
 #define DISPLAY_PARAM(X) out << #X"=" << X << std::endl;
@@ -32,6 +30,4 @@ void theory_pb_params::updt_params(params_ref const & _p) {
 void theory_pb_params::display(std::ostream & out) const {
     DISPLAY_PARAM(m_pb_conflict_frequency);
     DISPLAY_PARAM(m_pb_learn_complements);
-    DISPLAY_PARAM(m_pb_enable_compilation);
-    DISPLAY_PARAM(m_pb_enable_simplex);
-}
\ No newline at end of file
+}
