@@ -27,11 +27,11 @@ namespace smt {
     enum failure {
         OK,
         UNKNOWN,
-        TIMEOUT,    
         MEMOUT,     
         CANCELED,      //!< External cancel flag was set
         NUM_CONFLICTS, //!< Maximum number of conflicts was reached
         THEORY,        //!< Theory is incomplete
+        RESOURCE_LIMIT,
         QUANTIFIERS    //!< Logical context contains universal quantifiers.
     };
 
