@@ -110,7 +110,7 @@ namespace smt {
         if (is_quantifier(n))
             return true;
         SASSERT(is_app(n));
-        if (m_manager.is_bool(n)) {
+        if (m.is_bool(n)) {
             if (b_internalized(n))
                 return true;
         }
@@ -137,13 +137,13 @@ namespace smt {
 
         SASSERT(def_int);
 
-        if (m_manager.is_term_ite(n)) {
+        if (m.is_term_ite(n)) {
             ts_visit_child(to_app(n)->get_arg(0), true, tcolors, fcolors, todo, visited);
             ts_visit_child(to_app(n)->get_arg(1), false, tcolors, fcolors, todo, visited);
             ts_visit_child(to_app(n)->get_arg(2), false, tcolors, fcolors, todo, visited);
             return visited;
         }
-        bool new_gate_ctx = m_manager.is_bool(n) && (is_gate(m_manager, n) || m_manager.is_not(n));
+        bool new_gate_ctx = m.is_bool(n) && (is_gate(m, n) || m.is_not(n));
         unsigned j = to_app(n)->get_num_args();
         while (j > 0) {
             --j;
@@ -170,7 +170,7 @@ namespace smt {
             case Grey:
                 SASSERT(ts_visit_children(curr, gate_ctx, tcolors, fcolors, todo));
                 set_color(tcolors, fcolors, curr, gate_ctx, Black);
-                if (n != curr && !m_manager.is_not(curr))
+                if (n != curr && !m.is_not(curr))
                     sorted_exprs.push_back(expr_bool_pair(curr, gate_ctx));
                 break;
             case Black:
@@ -190,8 +190,8 @@ namespace smt {
        \remark pr is 0 if proofs are disabled.
     */
     void context::internalize_assertion(expr * n, proof * pr, unsigned generation) {
-        TRACE("internalize_assertion", tout << mk_pp(n, m_manager) << "\n";); 
-        TRACE("internalize_assertion_ll", tout << mk_ll_pp(n, m_manager) << "\n";); 
+        TRACE("internalize_assertion", tout << mk_pp(n, m) << "\n";); 
+        TRACE("internalize_assertion_ll", tout << mk_ll_pp(n, m) << "\n";); 
         TRACE("generation", tout << "generation: " << m_generation << "\n";);
         TRACE("incompleteness_bug", tout << "[internalize-assertion]: #" << n->get_id() << "\n";);
         flet<unsigned> l(m_generation, generation);
@@ -201,7 +201,7 @@ namespace smt {
             // stack overflow.
             // a caveat is that theory internalizers do rely on recursive descent so
             // internalization over these follows top-down
-            TRACE("deep_internalize", tout << "expression is deep: #" << n->get_id() << "\n" << mk_ll_pp(n, m_manager););
+            TRACE("deep_internalize", tout << "expression is deep: #" << n->get_id() << "\n" << mk_ll_pp(n, m););
             svector<expr_bool_pair> sorted_exprs;
             top_sort_expr(n, sorted_exprs);
             TRACE("deep_internalize", for (auto & kv : sorted_exprs) tout << "#" << kv.first->get_id() << " " << kv.second << "\n"; );
@@ -209,12 +209,12 @@ namespace smt {
                 expr* e = kv.first;
                 if (!is_app(e) || 
                     to_app(e)->get_family_id() == null_family_id || 
-                    to_app(e)->get_family_id() == m_manager.get_basic_family_id()) 
+                    to_app(e)->get_family_id() == m.get_basic_family_id()) 
                     internalize(e, kv.second);
             }
         }
-        SASSERT(m_manager.is_bool(n));
-        if (is_gate(m_manager, n)) {
+        SASSERT(m.is_bool(n));
+        if (is_gate(m, n)) {
             switch(to_app(n)->get_decl_kind()) {
             case OP_AND: {
                 for (expr * arg : *to_app(n)) {
@@ -265,7 +265,7 @@ namespace smt {
             }
             mark_as_relevant(n);
         }
-        else if (m_manager.is_distinct(n)) {
+        else if (m.is_distinct(n)) {
             assert_distinct(to_app(n), pr);
             mark_as_relevant(n);
         }
@@ -291,22 +291,22 @@ namespace smt {
 #define DISTINCT_SZ_THRESHOLD 32
 
     void context::assert_distinct(app * n, proof * pr) {
-        TRACE("assert_distinct", tout << mk_pp(n, m_manager) << "\n";);
+        TRACE("assert_distinct", tout << mk_pp(n, m) << "\n";);
         unsigned num_args = n->get_num_args();
-        if (num_args == 0 || num_args <= DISTINCT_SZ_THRESHOLD || m_manager.proofs_enabled()) {
+        if (num_args == 0 || num_args <= DISTINCT_SZ_THRESHOLD || m.proofs_enabled()) {
             assert_default(n, pr);
             return;
         }
-        sort * s = m_manager.get_sort(n->get_arg(0));
-        sort_ref u(m_manager.mk_fresh_sort("distinct-elems"), m_manager);
-        func_decl_ref f(m_manager.mk_fresh_func_decl("distinct-aux-f", "", 1, &s, u), m_manager);
+        sort * s = m.get_sort(n->get_arg(0));
+        sort_ref u(m.mk_fresh_sort("distinct-elems"), m);
+        func_decl_ref f(m.mk_fresh_func_decl("distinct-aux-f", "", 1, &s, u), m);
         for (expr * arg : *n) {
-            app_ref fapp(m_manager.mk_app(f, arg), m_manager);
-            app_ref val(m_manager.mk_fresh_const("unique-value", u), m_manager);
+            app_ref fapp(m.mk_app(f, arg), m);
+            app_ref val(m.mk_fresh_const("unique-value", u), m);
             enode * e   = mk_enode(val, false, false, true);
             e->mark_as_interpreted();
-            app_ref eq(m_manager.mk_eq(fapp, val), m_manager);
-            TRACE("assert_distinct", tout << "eq: " << mk_pp(eq, m_manager) << "\n";);
+            app_ref eq(m.mk_eq(fapp, val), m);
+            TRACE("assert_distinct", tout << "eq: " << mk_pp(eq, m) << "\n";);
             assert_default(eq, nullptr);
             mark_as_relevant(eq.get());
             // TODO: we may want to hide the auxiliary values val and the function f from the model.
@@ -331,12 +331,12 @@ namespace smt {
        - gate_ctx is true if the expression is in the context of a logical gate.
     */
     void context::internalize(expr * n, bool gate_ctx) {
-        TRACE("internalize", tout << "internalizing:\n" << mk_pp(n, m_manager) << "\n";);
-        TRACE("internalize_bug", tout << "internalizing:\n" << mk_bounded_pp(n, m_manager) << "\n";);
+        TRACE("internalize", tout << "internalizing:\n" << mk_pp(n, m) << "\n";);
+        TRACE("internalize_bug", tout << "internalizing:\n" << mk_bounded_pp(n, m) << "\n";);
         if (is_var(n)) {
             throw default_exception("Formulas should not contain unbound variables");
         }
-        if (m_manager.is_bool(n)) {
+        if (m.is_bool(n)) {
             SASSERT(is_quantifier(n) || is_app(n));
             internalize_formula(n, gate_ctx);
         }
@@ -355,12 +355,12 @@ namespace smt {
        \brief Internalize the given formula into the logical context.
     */
     void context::internalize_formula(expr * n, bool gate_ctx) {
-        TRACE("internalize_bug", tout << "internalize formula: #" << n->get_id() << ", gate_ctx: " << gate_ctx << "\n" << mk_pp(n, m_manager) << "\n";);
-        SASSERT(m_manager.is_bool(n));
-        if (m_manager.is_true(n) || m_manager.is_false(n))
+        TRACE("internalize_bug", tout << "internalize formula: #" << n->get_id() << ", gate_ctx: " << gate_ctx << "\n" << mk_pp(n, m) << "\n";);
+        SASSERT(m.is_bool(n));
+        if (m.is_true(n) || m.is_false(n))
             return;
 
-        if (m_manager.is_not(n) && gate_ctx) {
+        if (m.is_not(n) && gate_ctx) {
             // a boolean variable does not need to be created if n a NOT gate is in
             // the context of a gate.
             internalize(to_app(n)->get_arg(0), true);
@@ -396,9 +396,9 @@ namespace smt {
             return;
         }
 
-        if (m_manager.is_eq(n) && !m_manager.is_iff(n))
+        if (m.is_eq(n) && !m.is_iff(n))
             internalize_eq(to_app(n), gate_ctx);
-        else if (m_manager.is_distinct(n))
+        else if (m.is_distinct(n))
             internalize_distinct(to_app(n), gate_ctx);
         else if (is_app(n) && internalize_theory_atom(to_app(n), gate_ctx))
             return;
@@ -412,15 +412,15 @@ namespace smt {
        \brief Internalize an equality.
     */
     void context::internalize_eq(app * n, bool gate_ctx) {
-        TRACE("internalize", tout << mk_pp(n, m_manager) << "\n";);
+        TRACE("internalize", tout << mk_pp(n, m) << "\n";);
         SASSERT(!b_internalized(n));
-        SASSERT(m_manager.is_eq(n));
+        SASSERT(m.is_eq(n));
         internalize_formula_core(n, gate_ctx);
         bool_var v        = get_bool_var(n);
         bool_var_data & d = get_bdata(v);
         d.set_eq_flag();
         
-        sort * s    = m_manager.get_sort(n->get_arg(0));
+        sort * s    = m.get_sort(n->get_arg(0));
         theory * th = m_theories.get_plugin(s->get_family_id());
         if (th)
             th->internalize_eq_eh(n, v);
@@ -430,10 +430,10 @@ namespace smt {
        \brief Internalize distinct constructor.
     */
     void context::internalize_distinct(app * n, bool gate_ctx) {
-        TRACE("distinct", tout << "internalizing distinct: " << mk_pp(n, m_manager) << "\n";);
+        TRACE("distinct", tout << "internalizing distinct: " << mk_pp(n, m) << "\n";);
         SASSERT(!b_internalized(n));
-        SASSERT(m_manager.is_distinct(n));
-        expr_ref def(m_manager.mk_distinct_expanded(n->get_num_args(), n->get_args()), m_manager);
+        SASSERT(m.is_distinct(n));
+        expr_ref def(m.mk_distinct_expanded(n->get_num_args(), n->get_args()), m);
         internalize(def, true);
         bool_var v    = mk_bool_var(n);
         literal l(v);
@@ -456,10 +456,10 @@ namespace smt {
     bool context::internalize_theory_atom(app * n, bool gate_ctx) {
         SASSERT(!b_internalized(n));
         theory * th  = m_theories.get_plugin(n->get_family_id());
-        TRACE("datatype_bug", tout << "internalizing theory atom:\n" << mk_pp(n, m_manager) << "\n";);
+        TRACE("datatype_bug", tout << "internalizing theory atom:\n" << mk_pp(n, m) << "\n";);
         if (!th || !th->internalize_atom(n, gate_ctx))
             return false;
-        TRACE("datatype_bug", tout << "internalization succeeded\n" << mk_pp(n, m_manager) << "\n";);
+        TRACE("datatype_bug", tout << "internalization succeeded\n" << mk_pp(n, m) << "\n";);
         SASSERT(b_internalized(n));
         TRACE("internalize_theory_atom", tout << "internalizing theory atom: #" << n->get_id() << "\n";);
         bool_var v        = get_bool_var(n);
@@ -524,8 +524,8 @@ namespace smt {
        context. 
     */
     void context::internalize_quantifier(quantifier * q, bool gate_ctx) {
-        TRACE("internalize_quantifier", tout << mk_pp(q, m_manager) << "\n";);
-        CTRACE("internalize_quantifier_zero", q->get_weight() == 0, tout << mk_pp(q, m_manager) << "\n";);
+        TRACE("internalize_quantifier", tout << mk_pp(q, m) << "\n";);
+        CTRACE("internalize_quantifier_zero", q->get_weight() == 0, tout << mk_pp(q, m) << "\n";);
         SASSERT(gate_ctx); // limitation of the current implementation
         SASSERT(!b_internalized(q));
         SASSERT(is_forall(q));
@@ -543,22 +543,22 @@ namespace smt {
     }
 
     void context::internalize_lambda(quantifier * q) {
-        TRACE("internalize_quantifier", tout << mk_pp(q, m_manager) << "\n";);
+        TRACE("internalize_quantifier", tout << mk_pp(q, m) << "\n";);
         SASSERT(is_lambda(q));
-        app_ref lam_name(m_manager.mk_fresh_const("lambda", m_manager.get_sort(q)), m_manager);
-        app_ref eq(m_manager), lam_app(m_manager);
-        expr_ref_vector vars(m_manager);
+        app_ref lam_name(m.mk_fresh_const("lambda", m.get_sort(q)), m);
+        app_ref eq(m), lam_app(m);
+        expr_ref_vector vars(m);
         vars.push_back(lam_name);
         unsigned sz = q->get_num_decls();
         for (unsigned i = 0; i < sz; ++i) {
-            vars.push_back(m_manager.mk_var(sz - i - 1, q->get_decl_sort(i)));
+            vars.push_back(m.mk_var(sz - i - 1, q->get_decl_sort(i)));
         }
-        array_util autil(m_manager);
+        array_util autil(m);
         lam_app = autil.mk_select(vars.size(), vars.c_ptr());
-        eq = m_manager.mk_eq(lam_app, q->get_expr());
-        quantifier_ref fa(m_manager);
-        expr * patterns[1] = { m_manager.mk_pattern(lam_app) };
-        fa = m_manager.mk_forall(sz, q->get_decl_sorts(), q->get_decl_names(), eq, 0, m_manager.lambda_def_qid(), symbol::null, 1, patterns);
+        eq = m.mk_eq(lam_app, q->get_expr());
+        quantifier_ref fa(m);
+        expr * patterns[1] = { m.mk_pattern(lam_app) };
+        fa = m.mk_forall(sz, q->get_decl_sorts(), q->get_decl_names(), eq, 0, m.lambda_def_qid(), symbol::null, 1, patterns);
         internalize_quantifier(fa, true);
         if (!e_internalized(lam_name)) internalize_uninterpreted(lam_name);
         m_app2enode.setx(q->get_id(), get_enode(lam_name), nullptr);
@@ -571,15 +571,15 @@ namespace smt {
         SASSERT(!b_internalized(n));
         SASSERT(!e_internalized(n));
 
-        CTRACE("resolve_conflict_crash", m_manager.is_not(n), tout << mk_ismt2_pp(n, m_manager) << "\ngate_ctx: " << gate_ctx << "\n";);
+        CTRACE("resolve_conflict_crash", m.is_not(n), tout << mk_ismt2_pp(n, m) << "\ngate_ctx: " << gate_ctx << "\n";);
 
-        bool _is_gate  = is_gate(m_manager, n) || m_manager.is_not(n);
+        bool _is_gate  = is_gate(m, n) || m.is_not(n);
         // process args
         for (expr * arg : *n) {
             internalize(arg, _is_gate);
         }
         
-        CTRACE("internalize_bug", b_internalized(n), tout << mk_ll_pp(n, m_manager) << "\n";);
+        CTRACE("internalize_bug", b_internalized(n), tout << mk_ll_pp(n, m) << "\n";);
         
         bool is_new_var = false;
         bool_var v;
@@ -602,7 +602,7 @@ namespace smt {
         // 1) it is not in the context of a gate, or
         // 2) it has arguments and it is not a gate (i.e., uninterpreted predicate or equality).
         if (!e_internalized(n) && (!gate_ctx || (!_is_gate && n->get_num_args() > 0))) {
-            bool suppress_args = _is_gate || m_manager.is_not(n);
+            bool suppress_args = _is_gate || m.is_not(n);
             bool merge_tf      = !gate_ctx;
             mk_enode(n, suppress_args, merge_tf, true);
             set_enode_flag(v, is_new_var);
@@ -617,7 +617,7 @@ namespace smt {
         // Now, if v is assigned before being associated with an enode, then
         // v is not going to be inserted in m_atom_propagation_queue, and
         // propagate_bool_var_enode() method is not going to be invoked for v.
-        if (is_new_var && n->get_family_id() == m_manager.get_basic_family_id()) {
+        if (is_new_var && n->get_family_id() == m.get_basic_family_id()) {
             switch (n->get_decl_kind()) {
             case OP_NOT:
                 SASSERT(!gate_ctx);
@@ -632,7 +632,7 @@ namespace smt {
                 add_or_rel_watches(to_app(n));
                 break;
             case OP_EQ:
-                if (m_manager.is_iff(n))
+                if (m.is_iff(n))
                     mk_iff_cnstr(to_app(n));
                 break;
             case OP_ITE:
@@ -758,7 +758,7 @@ namespace smt {
             return;
         }
 
-        if (m_manager.is_term_ite(n)) {
+        if (m.is_term_ite(n)) {
             internalize_ite_term(n);
             return; // it is not necessary to apply sort constraint
         }
@@ -781,8 +781,8 @@ namespace smt {
         expr * c  = n->get_arg(0);
         expr * t  = n->get_arg(1);
         expr * e  = n->get_arg(2);
-        app_ref eq1(mk_eq_atom(n, t), m_manager);
-        app_ref eq2(mk_eq_atom(n, e), m_manager);
+        app_ref eq1(mk_eq_atom(n, t), m);
+        app_ref eq2(mk_eq_atom(n, e), m);
         mk_enode(n, 
                  true /* suppress arguments, I don't want to apply CC on ite terms */,
                  false /* it is a term, so it should not be merged with true/false */,
@@ -796,12 +796,12 @@ namespace smt {
         literal eq1_lit = get_literal(eq1);
         literal eq2_lit = get_literal(eq2);
         TRACE("internalize_ite_term_bug",
-              tout << mk_ismt2_pp(n, m_manager) << "\n";
-              tout << mk_ismt2_pp(c, m_manager) << "\n";
-              tout << mk_ismt2_pp(t, m_manager) << "\n";
-              tout << mk_ismt2_pp(e, m_manager) << "\n";              
-              tout << mk_ismt2_pp(eq1, m_manager) << "\n";              
-              tout << mk_ismt2_pp(eq2, m_manager) << "\n";              
+              tout << mk_ismt2_pp(n, m) << "\n";
+              tout << mk_ismt2_pp(c, m) << "\n";
+              tout << mk_ismt2_pp(t, m) << "\n";
+              tout << mk_ismt2_pp(e, m) << "\n";              
+              tout << mk_ismt2_pp(eq1, m) << "\n";              
+              tout << mk_ismt2_pp(eq2, m) << "\n";              
               tout << "literals:\n" << c_lit << " " << eq1_lit << " " << eq2_lit << "\n";);
         mk_gate_clause(~c_lit, eq1_lit);
         mk_gate_clause( c_lit, eq2_lit);
@@ -851,7 +851,7 @@ namespace smt {
     */
     bool_var context::mk_bool_var(expr * n) {
         SASSERT(!b_internalized(n));
-        //SASSERT(!m_manager.is_not(n));
+        //SASSERT(!m.is_not(n));
         unsigned id = n->get_id();
         bool_var v  = m_b_internalized_stack.size();
 #ifndef _EXTERNAL_RELEASE 
@@ -859,13 +859,13 @@ namespace smt {
             char const * header = "(iff z3@";
             int  id_sz = 6;
             std::cerr.width(id_sz);
-            std::cerr << header << std::left << v << " " << mk_pp(n, m_manager, static_cast<unsigned>(strlen(header)) + id_sz + 1) << ")\n";
+            std::cerr << header << std::left << v << " " << mk_pp(n, m, static_cast<unsigned>(strlen(header)) + id_sz + 1) << ")\n";
         }
         if (m_fparams.m_display_ll_bool_var2expr) {
-            std::cerr << v << " ::=\n" << mk_ll_pp(n, m_manager) << "<END-OF-FORMULA>\n";
+            std::cerr << v << " ::=\n" << mk_ll_pp(n, m) << "<END-OF-FORMULA>\n";
         }
 #endif
-        TRACE("mk_bool_var", tout << "creating boolean variable: " << v << " for:\n" << mk_pp(n, m_manager) << " " << n->get_id() << "\n";);
+        TRACE("mk_bool_var", tout << "creating boolean variable: " << v << " for:\n" << mk_pp(n, m) << " " << n->get_id() << "\n";);
         TRACE("mk_var_bug", tout << "mk_bool: " << v << "\n";);                
         set_bool_var(id, v);
         m_bdata.reserve(v+1);
@@ -909,7 +909,7 @@ namespace smt {
         unsigned n_id         = n->get_id();
         bool_var v            = get_bool_var_of_id(n_id);
         m_bool_var2expr[v]    = nullptr;
-        TRACE("undo_mk_bool_var", tout << "undo_bool: " << v << "\n" << mk_pp(n, m_manager) << "\n" << "m_bdata.size: " << m_bdata.size()
+        TRACE("undo_mk_bool_var", tout << "undo_bool: " << v << "\n" << mk_pp(n, m) << "\n" << "m_bdata.size: " << m_bdata.size()
               << " m_assignment.size: " << m_assignment.size() << "\n";);
         TRACE("mk_var_bug", tout << "undo_mk_bool: " << v << "\n";);
         // bool_var_data & d     = m_bdata[v];
@@ -927,7 +927,7 @@ namespace smt {
        in the egraph.
     */
     enode * context::mk_enode(app * n, bool suppress_args, bool merge_tf, bool cgc_enabled) {
-        TRACE("mk_enode_detail", tout << mk_pp(n, m_manager) << "\nsuppress_args: " << suppress_args << ", merge_tf: " << 
+        TRACE("mk_enode_detail", tout << mk_pp(n, m) << "\nsuppress_args: " << suppress_args << ", merge_tf: " << 
               merge_tf << ", cgc_enabled: " << cgc_enabled << "\n";);
         SASSERT(!e_internalized(n));
         unsigned id          = n->get_id();
@@ -938,9 +938,9 @@ namespace smt {
             CTRACE("cached_generation", generation != m_generation,
                    tout << "cached_generation: #" << n->get_id() << " " << generation << " " << m_generation << "\n";);
         }
-        enode * e           = enode::mk(m_manager, m_region, m_app2enode, n, generation, suppress_args, merge_tf, m_scope_lvl, cgc_enabled, true);
+        enode * e           = enode::mk(m, m_region, m_app2enode, n, generation, suppress_args, merge_tf, m_scope_lvl, cgc_enabled, true);
         TRACE("mk_enode_detail", tout << "e.get_num_args() = " << e->get_num_args() << "\n";);
-        if (n->get_num_args() == 0 && m_manager.is_unique_value(n))
+        if (n->get_num_args() == 0 && m.is_unique_value(n))
             e->mark_as_interpreted();
         TRACE("mk_var_bug", tout << "mk_enode: " << id << "\n";);
         TRACE("generation", tout << "mk_enode: " << id << " " << generation << "\n";);
@@ -980,14 +980,14 @@ namespace smt {
         }
         SASSERT(e_internalized(n));
         m_stats.m_num_mk_enode++;
-        TRACE("mk_enode", tout << "created enode: #" << e->get_owner_id() << " for:\n" << mk_pp(n, m_manager) << "\n";
+        TRACE("mk_enode", tout << "created enode: #" << e->get_owner_id() << " for:\n" << mk_pp(n, m) << "\n";
               if (e->get_num_args() > 0) {
                   tout << "is_true_eq: " << e->is_true_eq() << " in cg_table: " << m_cg_table.contains_ptr(e) << " is_cgr: " 
                        << e->is_cgr() << "\n";
               });
 
-        if (m_manager.has_trace_stream())
-            m_manager.trace_stream() << "[attach-enode] #" << n->get_id() << " " << m_generation << "\n";        
+        if (m.has_trace_stream())
+            m.trace_stream() << "[attach-enode] #" << n->get_id() << " " << m_generation << "\n";        
 
         return e;
     }
@@ -996,7 +996,7 @@ namespace smt {
         SASSERT(!m_e_internalized_stack.empty());
         m_stats.m_num_del_enode++;
         expr * n              = m_e_internalized_stack.back();
-        TRACE("undo_mk_enode", tout << "undo_enode: #" << n->get_id() << "\n" << mk_pp(n, m_manager) << "\n";);
+        TRACE("undo_mk_enode", tout << "undo_enode: #" << n->get_id() << "\n" << mk_pp(n, m) << "\n";);
         TRACE("mk_var_bug", tout << "undo_mk_enode: " << n->get_id() << "\n";);
         unsigned n_id         = n->get_id();
         SASSERT(is_app(n));
@@ -1012,7 +1012,7 @@ namespace smt {
             SASSERT(m_decl2enodes[decl_id].back() == e);
             m_decl2enodes[decl_id].pop_back();
         }
-        e->del_eh(m_manager);
+        e->del_eh(m);
         SASSERT(m_e_internalized_stack.size() == m_enodes.size());
         m_enodes.pop_back();
         m_e_internalized_stack.pop_back();
@@ -1033,15 +1033,15 @@ namespace smt {
        \brief Return the literal associated with n.
     */
     literal context::get_literal(expr * n) const {
-        if (m_manager.is_not(n)) {
-            CTRACE("get_literal_bug", !b_internalized(to_app(n)->get_arg(0)), tout << mk_ll_pp(n, m_manager) << "\n";);
+        if (m.is_not(n)) {
+            CTRACE("get_literal_bug", !b_internalized(to_app(n)->get_arg(0)), tout << mk_ll_pp(n, m) << "\n";);
             SASSERT(b_internalized(to_app(n)->get_arg(0)));
             return literal(get_bool_var(to_app(n)->get_arg(0)), true);
         }
-        else if (m_manager.is_true(n)) {
+        else if (m.is_true(n)) {
             return true_literal;
         }
-        else if (m_manager.is_false(n)) {
+        else if (m.is_false(n)) {
             return false_literal;
         }
         else {
@@ -1378,7 +1378,7 @@ namespace smt {
             bool save_atoms     = lemma && iscope_lvl > m_base_lvl;
             bool reinit         = save_atoms;
             SASSERT(!lemma || j == 0 || !j->in_region());
-            clause * cls = clause::mk(m_manager, num_lits, lits, k, j, del_eh, save_atoms, m_bool_var2expr.c_ptr());
+            clause * cls = clause::mk(m, num_lits, lits, k, j, del_eh, save_atoms, m_bool_var2expr.c_ptr());
             m_clause_proof.add(*cls);
             if (lemma) {
                 cls->set_activity(activity);
@@ -1455,7 +1455,7 @@ namespace smt {
         justification * js = nullptr;
         TRACE("mk_th_axiom", display_literals_verbose(tout, num_lits, lits) << "\n";);
 
-        if (m_manager.proofs_enabled()) {
+        if (m.proofs_enabled()) {
             js = mk_justification(theory_axiom_justification(tid, m_region, num_lits, lits, num_params, params));
         }
         if (m_fparams.m_smtlib_dump_lemmas) {
@@ -1483,20 +1483,20 @@ namespace smt {
             literal l      = lits[i];
             bool_var v     = l.var();
             expr * atom    = m_bool_var2expr[v]; 
-            new_lits.push_back(l.sign() ? m_manager.mk_not(atom) : atom);
+            new_lits.push_back(l.sign() ? m.mk_not(atom) : atom);
         }
         if (root_gate)
-            new_lits.push_back(m_manager.mk_not(root_gate));
+            new_lits.push_back(m.mk_not(root_gate));
         SASSERT(num_lits > 1);
-        expr * fact        = m_manager.mk_or(new_lits.size(), new_lits.c_ptr());
-        return m_manager.mk_def_axiom(fact);
+        expr * fact        = m.mk_or(new_lits.size(), new_lits.c_ptr());
+        return m.mk_def_axiom(fact);
         
     }
 
     void context::mk_gate_clause(unsigned num_lits, literal * lits) {
-        if (m_manager.proofs_enabled()) {
+        if (m.proofs_enabled()) {
             proof * pr = mk_clause_def_axiom(num_lits, lits, nullptr);
-            TRACE("gate_clause", tout << mk_ll_pp(pr, m_manager););
+            TRACE("gate_clause", tout << mk_ll_pp(pr, m););
             mk_clause(num_lits, lits, mk_justification(justification_proof_wrapper(*this, pr)));
         }
         else {
@@ -1520,15 +1520,15 @@ namespace smt {
     }
 
     void context::mk_root_clause(unsigned num_lits, literal * lits, proof * pr) {
-        if (m_manager.proofs_enabled()) {
-            SASSERT(m_manager.get_fact(pr));
-            expr * fact = m_manager.get_fact(pr);
-            if (!m_manager.is_or(fact)) {
-                proof * def = mk_clause_def_axiom(num_lits, lits, m_manager.get_fact(pr));
-                TRACE("gate_clause", tout << mk_ll_pp(def, m_manager) << "\n";
-                      tout << mk_ll_pp(pr, m_manager););
+        if (m.proofs_enabled()) {
+            SASSERT(m.get_fact(pr));
+            expr * fact = m.get_fact(pr);
+            if (!m.is_or(fact)) {
+                proof * def = mk_clause_def_axiom(num_lits, lits, m.get_fact(pr));
+                TRACE("gate_clause", tout << mk_ll_pp(def, m) << "\n";
+                      tout << mk_ll_pp(pr, m););
                 proof * prs[2] = { def, pr };
-                pr  = m_manager.mk_unit_resolution(2, prs);
+                pr  = m.mk_unit_resolution(2, prs);
             }
             mk_clause(num_lits, lits, mk_justification(justification_proof_wrapper(*this, pr)));
         }
