@@ -217,12 +217,22 @@ namespace smt {
         /**
            \brief return the next case split literal.
         */
-        expr* next_decision();
+        expr_ref next_cube();
+
+        /**
+           \brief retrieve depth of variables from decision stack.
+        */
+        void get_levels(ptr_vector<expr> const& vars, unsigned_vector& depth);
+
+        /**
+           \brief retrieve trail of assignment stack.
+        */
+        expr_ref_vector get_trail();
 
         /**
            \brief (For debubbing purposes) Prints the state of the kernel
         */
-        void display(std::ostream & out) const;
+        std::ostream& display(std::ostream & out) const;
 
         /**
            \brief Collect runtime statistics.
