@@ -21,11 +21,12 @@ Revision History:
 #ifndef SMT_MODEL_CHECKER_H_
 #define SMT_MODEL_CHECKER_H_
 
-#include"ast.h"
-#include"obj_hashtable.h"
-#include"qi_params.h"
-#include"smt_params.h"
-#include"region.h"
+#include "util/obj_hashtable.h"
+#include "ast/ast.h"
+#include "ast/array_decl_plugin.h"
+#include "ast/normal_forms/defined_names.h"
+#include "smt/params/qi_params.h"
+#include "smt/params/smt_params.h"
 
 class proto_model;
 class model;
@@ -37,9 +38,10 @@ namespace smt {
     class quantifier_manager;
 
     class model_checker {
-        ast_manager &                               m_manager;
+        ast_manager &                               m; // _manager;
         qi_params const &                           m_params;
-        // copy of smt_params for auxiliary context. 
+        array_util                                  m_autil;
+        // copy of smt_params for auxiliary context.
         // the idea is to use a different configuration for the aux context (e.g., disable relevancy)
         scoped_ptr<smt_params>                      m_fparams;
         quantifier_manager *                        m_qm;
@@ -49,31 +51,36 @@ namespace smt {
         scoped_ptr<context>                         m_aux_context; // Auxiliary context used for model checking quantifiers.
         unsigned                                    m_max_cexs;
         unsigned                                    m_iteration_idx;
+        bool                                        m_has_rec_fun;
         proto_model *                               m_curr_model;
         obj_map<expr, expr *>                       m_value2expr;
-        bool                                        m_cancel;
+        expr_ref_vector                             m_fresh_exprs;
+
         friend class instantiation_set;
 
         void init_aux_context();
+        void init_value2expr();
         expr * get_term_from_ctx(expr * val);
+        expr * get_type_compatible_term(expr * val);
+        expr_ref replace_value_from_ctx(expr * e);
         void restrict_to_universe(expr * sk, obj_hashtable<expr> const & universe);
         void assert_neg_q_m(quantifier * q, expr_ref_vector & sks);
         bool add_blocking_clause(model * cex, expr_ref_vector & sks);
         bool check(quantifier * q);
-        
+        bool check_rec_fun(quantifier* q, bool strict_rec_fun);
+        bool has_rec_under_quantifiers();
+        void check_quantifiers(bool strict_rec_fun, bool& found_relevant, unsigned& num_failures);
+
         struct instance {
             quantifier * m_q;
             unsigned     m_generation;
-            expr *       m_bindings[0];
-            static unsigned get_obj_size(unsigned num_bindings) { return sizeof(instance) + num_bindings * sizeof(expr*); }
-            instance(quantifier * q, expr * const * bindings, unsigned gen):m_q(q), m_generation(gen) {
-                memcpy(m_bindings, bindings, q->get_num_decls() * sizeof(expr*));
-            }
+            expr *       m_def;
+            unsigned     m_bindings_offset;
+            instance(quantifier * q, unsigned offset, expr* def, unsigned gen):m_q(q), m_generation(gen), m_def(def), m_bindings_offset(offset) {}
         };
 
-        region                                     m_new_instances_region;
-        expr_ref_vector                            m_new_instances_bindings;
-        ptr_vector<instance>                       m_new_instances;
+        svector<instance>                          m_new_instances;
+        expr_ref_vector                            m_pinned_exprs;
         bool add_instance(quantifier * q, model * cex, expr_ref_vector & sks, bool use_inv);
         void reset_new_instances();
         void assert_new_instances();
@@ -82,7 +89,8 @@ namespace smt {
 
         struct is_model_value {};
         expr_mark m_visited;
-        bool contains_model_value(expr* e);
+        bool contains_model_value(expr * e);
+        void add_instance(quantifier * q, expr_ref_vector const & bindings, unsigned max_generation, expr * def);
 
     public:
         model_checker(ast_manager & m, qi_params const & p, model_finder & mf);
@@ -98,8 +106,6 @@ namespace smt {
 
         void reset();
 
-        void set_cancel(bool f) { m_cancel = f; }
-
         void operator()(expr* e);
 
     };
