@@ -17,7 +17,7 @@ Revision History:
 
 --*/
 #include "ast/ast_pp.h"
-#include "ast/ast_smt2_pp.h"
+#include "ast/ast_ll_pp.h"
 #include "smt/smt_quantifier.h"
 #include "smt/smt_context.h"
 #include "smt/smt_quantifier_stat.h"
@@ -32,6 +32,98 @@ namespace smt {
 
     quantifier_manager_plugin * mk_default_plugin();
 
+    void log_single_justification(std::ostream & out, enode *en, obj_hashtable<enode> &visited, context &ctx, ast_manager &m);
+
+    /**
+         \brief Ensures that all relevant proof steps to explain why the enode is equal to the root of its
+        equivalence class are in the log and up-to-date.
+    */
+    void quantifier_manager::log_justification_to_root(std::ostream & out, enode *en, obj_hashtable<enode> &visited, context &ctx, ast_manager &m) {
+        enode *root = en->get_root();
+        for (enode *it = en; it != root; it = it->get_trans_justification().m_target) {
+            if (visited.find(it) == visited.end()) visited.insert(it);
+            else break;
+
+            if (!it->m_proof_is_logged) {
+                log_single_justification(out, it, visited, ctx, m);
+                it->m_proof_is_logged = true;
+            } else if (it->get_trans_justification().m_justification.get_kind() == smt::eq_justification::kind::CONGRUENCE) {
+
+                // When the justification of an argument changes m_proof_is_logged is not reset => We need to check if the proofs of all arguments are logged.
+                const unsigned num_args = it->get_num_args();
+                enode *target = it->get_trans_justification().m_target;
+
+                for (unsigned i = 0; i < num_args; ++i) {
+                    log_justification_to_root(out, it->get_arg(i), visited, ctx, m);
+                    log_justification_to_root(out, target->get_arg(i), visited, ctx, m);
+                }
+                it->m_proof_is_logged = true;
+            }
+        }
+        if (!root->m_proof_is_logged) {
+            out << "[eq-expl] #" << root->get_owner_id() << " root\n";
+            root->m_proof_is_logged = true;
+        }
+    }
+
+    /**
+         \brief Logs a single equality explanation step and, if necessary, recursively calls log_justification_to_root to log
+        equalities needed by the step (e.g. argument equalities for congruence steps).
+    */
+    void log_single_justification(std::ostream & out, enode *en, obj_hashtable<enode> &visited, context &ctx, ast_manager &m) {
+        smt::literal lit;
+        unsigned num_args;
+        enode *target = en->get_trans_justification().m_target;
+        theory_id th_id;
+
+        switch (en->get_trans_justification().m_justification.get_kind()) {
+        case smt::eq_justification::kind::EQUATION:
+            lit = en->get_trans_justification().m_justification.get_literal();
+            out << "[eq-expl] #" << en->get_owner_id() << " lit #" << ctx.bool_var2expr(lit.var())->get_id() << " ; #" << target->get_owner_id() << "\n";
+            break;
+        case smt::eq_justification::kind::AXIOM:
+            out << "[eq-expl] #" << en->get_owner_id() << " ax ; #" << target->get_owner_id() << "\n";
+            break;
+        case smt::eq_justification::kind::CONGRUENCE:
+            if (!en->get_trans_justification().m_justification.used_commutativity()) {
+                num_args = en->get_num_args();
+
+                for (unsigned i = 0; i < num_args; ++i) {
+                    quantifier_manager::log_justification_to_root(out, en->get_arg(i), visited, ctx, m);
+                    quantifier_manager::log_justification_to_root(out, target->get_arg(i), visited, ctx, m);
+                }
+
+                out << "[eq-expl] #" << en->get_owner_id() << " cg";
+                for (unsigned i = 0; i < num_args; ++i) {
+                    out << " (#" << en->get_arg(i)->get_owner_id() << " #" << target->get_arg(i)->get_owner_id() << ")";
+                }
+                out << " ; #" << target->get_owner_id() << "\n";
+
+                break;
+            } else {
+
+                // The e-graph only supports commutativity for binary functions
+                out << "[eq-expl] #" << en->get_owner_id()
+                    << " cg (#" << en->get_arg(0)->get_owner_id() << " #" << target->get_arg(1)->get_owner_id()
+                    << ") (#" << en->get_arg(1)->get_owner_id() << " #" << target->get_arg(0)->get_owner_id()
+                    << ") ; #" << target->get_owner_id() << "\n";
+                break;
+            }
+        case smt::eq_justification::kind::JUSTIFICATION:
+            th_id = en->get_trans_justification().m_justification.get_justification()->get_from_theory();
+            if (th_id != null_theory_id) {
+                symbol const theory = m.get_family_name(th_id);
+                out << "[eq-expl] #" << en->get_owner_id() << " th " << theory.str() << " ; #" << target->get_owner_id() << "\n";
+            } else {
+                out << "[eq-expl] #" << en->get_owner_id() << " unknown ; #" << target->get_owner_id() << "\n";
+            }
+            break;
+        default:
+            out << "[eq-expl] #" << en->get_owner_id() << " unknown ; #" << target->get_owner_id() << "\n";
+            break;
+        }
+    }
+
     struct quantifier_manager::imp {
         quantifier_manager &                   m_wrapper;
         context &                              m_context;
@@ -81,7 +173,7 @@ namespace smt {
             if (num_instances > 0) {
                 out << "[quantifier_instances] ";
                 out.width(10);
-                out << q->get_qid().str().c_str() << " : ";
+                out << q->get_qid().str() << " : ";
                 out.width(6);
                 out << num_instances << " : ";
                 out.width(3);
@@ -105,138 +197,58 @@ namespace smt {
             return m_plugin->is_shared(n);
         }
 
-        /**
-           \brief Ensures that all relevant proof steps to explain why the enode is equal to the root of its
-           equivalence class are in the log and up-to-date.
-        */
-        void log_justification_to_root(std::ostream & out, enode *en, obj_hashtable<enode> &visited) {
-            enode* root = en->get_root();
-            for (enode *it = en; it != root && !visited.contains(it); it = it->get_trans_justification().m_target) {
-
-                visited.insert(it);
-
-                if (!it->m_proof_is_logged) {
-                    log_single_justification(out, it, visited);
-                    it->m_proof_is_logged = true;
-                } 
-                else if (it->get_trans_justification().m_justification.get_kind() == smt::eq_justification::kind::CONGRUENCE) {
-
-                    // When the justification of an argument changes m_proof_is_logged 
-                    // is not reset => We need to check if the proofs of all arguments are logged.
-                    const unsigned num_args = it->get_num_args();
-                    enode *target = it->get_trans_justification().m_target;
-
-                    for (unsigned i = 0; i < num_args; ++i) {
-                        log_justification_to_root(out, it->get_arg(i), visited);
-                        log_justification_to_root(out, target->get_arg(i), visited);
-                    }
-                    SASSERT(it->m_proof_is_logged);
-                }
-            }
-            if (!root->m_proof_is_logged) {
-                out << "[eq-expl] #" << root->get_owner_id() << " root\n";
-                root->m_proof_is_logged = true;
-            }
-        }
-
-        /**
-          \brief Logs a single equality explanation step and, if necessary, recursively calls log_justification_to_root to log
-          equalities needed by the step (e.g. argument equalities for congruence steps).
-        */
-        void log_single_justification(std::ostream & out, enode *en, obj_hashtable<enode> &visited) {
-            smt::literal lit;
-            unsigned num_args;
-            enode *target = en->get_trans_justification().m_target;
-            theory_id th_id;
-
-            switch (en->get_trans_justification().m_justification.get_kind()) {
-            case smt::eq_justification::kind::EQUATION:
-                lit = en->get_trans_justification().m_justification.get_literal();
-                out << "[eq-expl] #" << en->get_owner_id() << " lit #" << m_context.bool_var2expr(lit.var())->get_id() << " ; #" << target->get_owner_id() << "\n";
-                break;
-            case smt::eq_justification::kind::AXIOM:
-                out << "[eq-expl] #" << en->get_owner_id() << " ax ; #" << target->get_owner_id() << "\n";
-                break;
-            case smt::eq_justification::kind::CONGRUENCE:
-                if (!en->get_trans_justification().m_justification.used_commutativity()) {
-                    num_args = en->get_num_args();
-
-                    for (unsigned i = 0; i < num_args; ++i) {
-                        log_justification_to_root(out, en->get_arg(i), visited);
-                        log_justification_to_root(out, target->get_arg(i), visited);
-                    }
-
-                    out << "[eq-expl] #" << en->get_owner_id() << " cg";
-                    for (unsigned i = 0; i < num_args; ++i) {
-                        out << " (#" << en->get_arg(i)->get_owner_id() << " #" << target->get_arg(i)->get_owner_id() << ")";
-                    }
-                    out << " ; #" << target->get_owner_id() << "\n";
-
-                    break;
-                } else {
-                    out << "[eq-expl] #" << en->get_owner_id() << " nyi ; #" << target->get_owner_id() << "\n";
-                    break;
-                }
-            case smt::eq_justification::kind::JUSTIFICATION:
-                th_id = en->get_trans_justification().m_justification.get_justification()->get_from_theory();
-                if (th_id != null_theory_id) {
-                    symbol const theory = m().get_family_name(th_id);
-                    out << "[eq-expl] #" << en->get_owner_id() << " th " << theory.str() << " ; #" << target->get_owner_id() << "\n";
-                } else {
-                    out << "[eq-expl] #" << en->get_owner_id() << " unknown ; #" << target->get_owner_id() << "\n";
-                }
-                break;
-            default:
-                out << "[eq-expl] #" << en->get_owner_id() << " unknown ; #" << target->get_owner_id() << "\n";
-                break;
-            }
-        }
-
         void log_add_instance(
-            fingerprint* f,
-            quantifier * q, app * pat,
-            unsigned num_bindings,
-            enode * const * bindings,
-            vector<std::tuple<enode *, enode *>> & used_enodes) {
+             fingerprint* f,
+             quantifier * q, app * pat,
+             unsigned num_bindings,
+             enode * const * bindings,
+             vector<std::tuple<enode *, enode *>> & used_enodes) {
 
-            std::ostream & out = trace_stream();
-            
-            obj_hashtable<enode> visited;
-            
-            // In the term produced by the quantifier instantiation the root of 
-            // the equivalence class of the terms bound to the quantified variables
-            // is used. We need to make sure that all of these equalities appear in the log.
-            for (unsigned i = 0; i < num_bindings; ++i) {
-                log_justification_to_root(out, bindings[i], visited);
-            }
-            
-            for (auto n : used_enodes) {
-                enode *orig = std::get<0>(n);
-                enode *substituted = std::get<1>(n);
-                if (orig != nullptr) {
-                    log_justification_to_root(out, orig, visited);
-                    log_justification_to_root(out, substituted, visited);
+            if (pat == nullptr) {
+                trace_stream() << "[inst-discovered] MBQI " << static_cast<void*>(f) << " #" << q->get_id();
+                for (unsigned i = 0; i < num_bindings; ++i) {
+                    trace_stream() << " #" << bindings[num_bindings - i - 1]->get_owner_id();
                 }
-            }
-            
-            // At this point all relevant equalities for the match are logged.
-            out << "[new-match] " << static_cast<void*>(f) << " #" << q->get_id() << " #" << pat->get_id();
-            for (unsigned i = 0; i < num_bindings; i++) {
-                // I don't want to use mk_pp because it creates expressions for pretty printing.
-                // This nasty side-effect may change the behavior of Z3.
-                out << " #" << bindings[i]->get_owner_id();
-            }
-            out << " ;";
-            for (auto n : used_enodes) {
-                enode *orig = std::get<0>(n);
-                enode *substituted = std::get<1>(n);
-                if (orig == nullptr)
-                    out << " #" << substituted->get_owner_id();
-                else {
-                    out << " (#" << orig->get_owner_id() << " #" << substituted->get_owner_id() << ")";
+                trace_stream() << "\n";
+            } else {
+                std::ostream & out = trace_stream();
+
+                obj_hashtable<enode> already_visited;
+
+                // In the term produced by the quantifier instantiation the root of the equivalence class of the terms bound to the quantified variables
+                // is used. We need to make sure that all of these equalities appear in the log.
+                for (unsigned i = 0; i < num_bindings; ++i) {
+                    log_justification_to_root(out, bindings[i], already_visited, m_context, m());
                 }
+
+                for (auto n : used_enodes) {
+                    enode *orig = std::get<0>(n);
+                    enode *substituted = std::get<1>(n);
+                    if (orig != nullptr) {
+                        log_justification_to_root(out, orig, already_visited, m_context, m());
+                        log_justification_to_root(out, substituted, already_visited, m_context, m());
+                    }
+                }
+
+                // At this point all relevant equalities for the match are logged.
+                out << "[new-match] " << static_cast<void*>(f) << " #" << q->get_id() << " #" << pat->get_id();
+                for (unsigned i = 0; i < num_bindings; i++) {
+                    // I don't want to use mk_pp because it creates expressions for pretty printing.
+                    // This nasty side-effect may change the behavior of Z3.
+                    out << " #" << bindings[num_bindings - i - 1]->get_owner_id();
+                }
+                out << " ;";
+                for (auto n : used_enodes) {
+                    enode *orig = std::get<0>(n);
+                    enode *substituted = std::get<1>(n);
+                    if (orig == nullptr)
+                        out << " #" << substituted->get_owner_id();
+                    else {
+                        out << " (#" << orig->get_owner_id() << " #" << substituted->get_owner_id() << ")";
+                    }
+                }
+                out << "\n";
             }
-            out << "\n";
         }
 
         bool add_instance(quantifier * q, app * pat,
@@ -260,13 +272,13 @@ namespace smt {
                 m_qi_queue.insert(f, pat, max_generation, min_top_generation, max_top_generation); // TODO
                 m_num_instances++;
             }
-            TRACE("quantifier",
-                  tout << mk_pp(q, m()) << " ";
+
+            CTRACE("quantifier_", f != nullptr, 
+                  tout << expr_ref(q, m()) << " ";
                   for (unsigned i = 0; i < num_bindings; ++i) {
-                      tout << mk_pp(bindings[i]->get_owner(), m()) << " ";
+                      tout << expr_ref(bindings[i]->get_owner(), m()) << " ";
                   }
                   tout << "\n";
-                  tout << "inserted: " << (f != nullptr) << "\n";
                   );
 
             return f != nullptr;
@@ -646,7 +658,7 @@ namespace smt {
                     m_lazy_mam->add_pattern(q, mp);
                 }
                 else {
-                    TRACE("quantifier", tout << "adding:\n" << mk_ismt2_pp(mp, m) << "\n";);
+                    TRACE("quantifier", tout << "adding:\n" << expr_ref(mp, m) << "\n";);
                     m_mam->add_pattern(q, mp);
                 }
                 if (!unary)
