@@ -33,6 +33,7 @@ Revision History:
 #include "smt/theory_dl.h"
 #include "smt/theory_seq_empty.h"
 #include "smt/theory_seq.h"
+#include "smt/theory_special_relations.h"
 #include "smt/theory_pb.h"
 #include "smt/theory_fpa.h"
 #include "smt/theory_str.h"
@@ -316,6 +317,7 @@ namespace smt {
                 // }
                 // else {
                 TRACE("rdl_bug", tout << "using theory_mi_arith\n";);
+                //setup_lra_arith();
                 m_context.register_plugin(alloc(smt::theory_mi_arith, m_manager, m_params));
                 // }
             }
@@ -407,7 +409,7 @@ namespace smt {
         m_params.m_nnf_cnf          = false;
         m_params.m_arith_eq_bounds  = true;
         m_params.m_arith_eq2ineq    = true;
-        m_params.m_phase_selection  = PS_ALWAYS_FALSE;
+        // m_params.m_phase_selection  = PS_THEORY;
         m_params.m_restart_strategy = RS_GEOMETRIC;
         m_params.m_restart_factor   = 1.5;
         m_params.m_restart_adaptive = false;
@@ -440,7 +442,7 @@ namespace smt {
             }
         }
         m_params.m_arith_eq_bounds  = true;
-        m_params.m_phase_selection  = PS_ALWAYS_FALSE;
+        // m_params.m_phase_selection  = PS_THEORY;
         m_params.m_restart_strategy = RS_GEOMETRIC;
         m_params.m_restart_factor   = 1.5;
         m_params.m_restart_adaptive = false;
@@ -454,13 +456,14 @@ namespace smt {
     }
 
     void setup::setup_QF_LRA() {
-        TRACE("setup", tout << "setup_QF_LRA(st)\n";);
+        TRACE("setup", tout << "setup_QF_LRA()\n";);
         m_params.m_relevancy_lvl       = 0;
         m_params.m_arith_eq2ineq       = true;
         m_params.m_arith_reflect       = false;
         m_params.m_arith_propagate_eqs = false;
         m_params.m_eliminate_term_ite  = true;
         m_params.m_nnf_cnf             = false;
+        m_params.m_phase_selection       = PS_THEORY;
         setup_lra_arith();
     }
 
@@ -476,13 +479,10 @@ namespace smt {
             m_params.m_relevancy_lvl       = 2;
             m_params.m_relevancy_lemma     = false;
         }
-        if (st.m_cnf) {
-            m_params.m_phase_selection = PS_CACHING_CONSERVATIVE2;
-        }
-        else {
+        m_params.m_phase_selection       = PS_THEORY;
+        if (!st.m_cnf) {
             m_params.m_restart_strategy      = RS_GEOMETRIC;
             m_params.m_arith_stronger_lemmas = false;
-            m_params.m_phase_selection       = PS_ALWAYS_FALSE;
             m_params.m_restart_adaptive      = false;
         }
         m_params.m_arith_small_lemma_size = 32;
@@ -532,7 +532,6 @@ namespace smt {
         } 
         else {
             m_params.m_eliminate_term_ite   = true;
-            m_params.m_phase_selection      = PS_CACHING;
             m_params.m_restart_adaptive     = false;
             m_params.m_restart_strategy     = RS_GEOMETRIC;
             m_params.m_restart_factor       = 1.5;
@@ -734,6 +733,13 @@ namespace smt {
         else if (m_params.m_string_solver == "auto") {
             setup_unknown();
         }
+ 
+        else if (m_params.m_string_solver == "empty") {
+            m_context.register_plugin(alloc(smt::theory_seq_empty, m_manager));
+        }
+        else if (m_params.m_string_solver == "none") {
+            // don't register any solver.
+        }
         else {
             throw default_exception("invalid parameter for smt.string_solver, valid options are 'z3str3', 'seq', 'auto'");
         }
@@ -753,6 +759,7 @@ namespace smt {
     }
 
     void setup::setup_lra_arith() {
+        // m_context.register_plugin(alloc(smt::theory_mi_arith, m_manager, m_params));
         m_context.register_plugin(alloc(smt::theory_lra, m_manager, m_params));
     }
 
@@ -836,7 +843,7 @@ namespace smt {
                 m_context.register_plugin(alloc(smt::theory_mi_arith, m_manager, m_params));
             break;
         case AS_NEW_ARITH:
-            if (st.m_num_non_linear != 0) 
+            if (st.m_num_non_linear != 0 && st.m_has_int) 
                 m_context.register_plugin(alloc(smt::theory_mi_arith, m_manager, m_params));
             else 
                 setup_lra_arith();
@@ -898,6 +905,12 @@ namespace smt {
         else if (m_params.m_string_solver == "seq") {
             setup_seq();
         } 
+        else if (m_params.m_string_solver == "empty") {
+            m_context.register_plugin(alloc(smt::theory_seq_empty, m_manager));
+        }
+        else if (m_params.m_string_solver == "none") {
+            // don't register any solver.
+        }
         else if (m_params.m_string_solver == "auto") {
             if (st.m_has_seq_non_str) {
                 setup_seq();
@@ -934,6 +947,10 @@ namespace smt {
         m_context.register_plugin(alloc(smt::theory_jobscheduler, m_manager));
     }
 
+    void setup::setup_special_relations() {
+        m_context.register_plugin(alloc(smt::theory_special_relations, m_manager));
+    }
+
     void setup::setup_unknown() {
         static_features st(m_manager);
         ptr_vector<expr> fmls;
@@ -949,6 +966,7 @@ namespace smt {
         setup_seq_str(st);
         setup_card();
         setup_fpa();
+        if (st.m_has_sr) setup_special_relations();
     }
 
     void setup::setup_unknown(static_features & st) {
@@ -965,6 +983,7 @@ namespace smt {
             setup_card();
             setup_fpa();
             setup_recfuns();
+            if (st.m_has_sr) setup_special_relations();
             return;
         }
 
