@@ -22,7 +22,7 @@ Revision History:
 #ifndef SMT_VALUE_SORT_H_
 #define SMT_VALUE_SORT_H_
 
-#include "ast.h"
+#include "ast/ast.h"
 
 
 namespace smt {
