@@ -19,7 +19,7 @@ Notes:
 #ifndef CTX_SOLVER_SIMPLIFY_TACTIC_H_
 #define CTX_SOLVER_SIMPLIFY_TACTIC_H_
 
-#include"tactical.h"
+#include "tactic/tactical.h"
 
 tactic * mk_ctx_solver_simplify_tactic(ast_manager & m, params_ref const & p = params_ref());
 /*
