@@ -19,14 +19,14 @@ Revision History:
 #ifndef THEORY_ARITH_DEF_H_
 #define THEORY_ARITH_DEF_H_
 
-#include"theory_arith.h"
-#include"theory_arith_core.h"
-#include"theory_arith_aux.h"
-#include"theory_arith_inv.h"
-#include"theory_arith_pp.h"
-#include"theory_arith_int.h"
-#include"theory_arith_eq.h"
-#include"theory_arith_nl.h"
+#include "smt/theory_arith.h"
+#include "smt/theory_arith_core.h"
+#include "smt/theory_arith_aux.h"
+#include "smt/theory_arith_inv.h"
+#include "smt/theory_arith_pp.h"
+#include "smt/theory_arith_int.h"
+#include "smt/theory_arith_eq.h"
+#include "smt/theory_arith_nl.h"
 
 #endif /* THEORY_ARITH_DEF_H_ */
 
