@@ -789,7 +789,14 @@ namespace smt {
             bound  = m_util.mk_eq(var2expr(v), m_util.mk_numeral(rational(0), true));
         TRACE("non_linear", tout << "new bound:\n" << mk_pp(bound, get_manager()) << "\n";);
         context & ctx = get_context();
+        ast_manager & m = get_manager();
+        if (m.has_trace_stream()) {
+            app_ref body(m);
+            body = m.mk_or(bound, m.mk_not(bound));
+            log_axiom_instantiation(body);
+        }
         ctx.internalize(bound, true);
+        if (m.has_trace_stream()) m.trace_stream() << "[end-of-instance]\n";
         ctx.mark_as_relevant(bound);
         literal l     = ctx.get_literal(bound);
         SASSERT(!l.sign());
@@ -804,15 +811,13 @@ namespace smt {
     bool theory_arith<Ext>::is_monomial_linear(expr * m) const {
         SASSERT(is_pure_monomial(m));
         unsigned num_nl_vars = 0;
-        for (unsigned i = 0; i < to_app(m)->get_num_args(); i++) {
-            expr * arg = to_app(m)->get_arg(i);
+        for (expr* arg : *to_app(m)) {
             theory_var _var = expr2var(arg);
             if (!is_fixed(_var)) {
                 num_nl_vars++;
             }
-            else {
-                if (lower_bound(_var).is_zero())
-                     return true;
+            else if (lower_bound(_var).is_zero()) {
+                return true;
             }
         }
         return num_nl_vars <= 1;
@@ -1212,10 +1217,8 @@ namespace smt {
             m_var2num_occs.insert(VAR, occs);                                                           \
         }
 
-        typename sbuffer<coeff_expr>::const_iterator it  = p.begin();
-        typename sbuffer<coeff_expr>::const_iterator end = p.end();
-        for (; it != end; ++it) {
-            expr * m = it->second;
+        for (coeff_expr const& kv : p) {
+            expr * m = kv.second;
             if (is_pure_monomial(m)) {
                 unsigned num_vars = get_num_vars_in_monomial(m);
                 for (unsigned i = 0; i < num_vars; i++) {
@@ -1230,8 +1233,8 @@ namespace smt {
                 ADD_OCC(m);
             }
             else {
-                TRACE("non_linear", tout << mk_pp(m, get_manager()) << "\n";);
-                UNREACHABLE();
+                ctx.internalize(m, false);
+                ADD_OCC(m);
             }
         }
 
@@ -1248,7 +1251,6 @@ namespace smt {
     template<typename Ext>
     expr * theory_arith<Ext>::p2expr(sbuffer<coeff_expr> & p) {
         SASSERT(!p.empty());
-        TRACE("p2expr_bug", display_coeff_exprs(tout, p););
         ptr_buffer<expr> args;
         for (coeff_expr const& ce : p) {
             rational const & c = ce.first;
@@ -1270,6 +1272,7 @@ namespace smt {
         SASSERT(!args.empty());
         expr * r = mk_nary_add(args.size(), args.c_ptr());
         m_nl_new_exprs.push_back(r);
+        TRACE("p2expr_bug", display_coeff_exprs(tout, p); tout << mk_pp(r, get_manager()) << "\n";);
         return r;
     }
 
@@ -1480,14 +1483,15 @@ namespace smt {
                 r.push_back(coeff_expr(kv.first, f));
             }
         }
+
         expr * s = cross_nested(e, nullptr);
         if (!r.empty()) {
             expr * q = horner(r, var);
             // TODO: improve here
-            s        = m_util.mk_add(q, s);
+            s = m_util.mk_add(q, s);
         }
 
-        expr * result   = s;
+        expr * result = s;
         if (d != 0) {
             expr * xd = power(var, d);
             result = m_util.mk_mul(xd, s);
@@ -1530,7 +1534,7 @@ namespace smt {
         rational a, b;
         unsigned n  = UINT_MAX;
         unsigned nm = UINT_MAX;
-        if (in_monovariate_monomials(p, var, i1, a, n, i2, b, nm)) {
+        if (in_monovariate_monomials(p, var, i1, a, n, i2, b, nm) && n != nm) {
             CTRACE("in_monovariate_monomials", n == nm,
                    for (unsigned i = 0; i < p.size(); i++) {
                        if (i > 0) tout << " + "; tout << p[i].first << "*" << mk_pp(p[i].second, get_manager());
@@ -1690,11 +1694,9 @@ namespace smt {
 
         TRACE("non_linear", tout << "check problematic row:\n"; display_row(tout, r); display_row(tout, r, false););
         sbuffer<coeff_expr> p;
-        typename vector<row_entry>::const_iterator it  = r.begin_entries();
-        typename vector<row_entry>::const_iterator end = r.end_entries();
-        for (; it != end; ++it) {
-            if (!it->is_dead())
-                p.push_back(coeff_expr(it->m_coeff.to_rational() * c, var2expr(it->m_var)));
+        for (row_entry const& re : r) {
+            if (!re.is_dead())
+                p.push_back(coeff_expr(re.m_coeff.to_rational() * c, var2expr(re.m_var)));
         }
         SASSERT(!p.empty());
         CTRACE("cross_nested_bug", !c.is_one(), tout << "c: " << c << "\n"; display_row(tout, r); tout << "---> p (coeffs, exprs):\n"; display_coeff_exprs(tout, p););
@@ -1924,7 +1926,7 @@ namespace smt {
         derived_bound  b(null_theory_var, inf_numeral(0), B_LOWER);
         dependency2new_bound(d, b);
         set_conflict(b, ante, "arith_nl");
-        TRACE("non_linear", for (literal lit : b.m_lits) tout << lit << " "; tout << "\n";); 
+        TRACE("non_linear", for (literal lit : b.m_lits) get_context().display_literal_verbose(tout, lit) << "\n"; tout << "\n";); 
     }
 
     /**
