@@ -19,35 +19,38 @@ Revision History:
 #ifndef THEORY_ARITH_PP_H_
 #define THEORY_ARITH_PP_H_
 
-#include"theory_arith.h"
-#include"ast_smt_pp.h"
-#include"stats.h"
+#include "smt/theory_arith.h"
+#include "ast/ast_smt_pp.h"
+#include "util/stats.h"
 
 namespace smt {
     template<typename Ext>
     void theory_arith<Ext>::collect_statistics(::statistics & st) const {
         st.update("arith conflicts", m_stats.m_conflicts);
-        st.update("add rows", m_stats.m_add_rows);
-        st.update("pivots", m_stats.m_pivots);
-        st.update("assert lower", m_stats.m_assert_lower);
-        st.update("assert upper", m_stats.m_assert_upper);
-        st.update("assert diseq", m_stats.m_assert_diseq);
-        st.update("bound prop", m_stats.m_bound_props);
-        st.update("fixed eqs", m_stats.m_fixed_eqs);
-        st.update("offset eqs", m_stats.m_offset_eqs);
-        st.update("gcd tests", m_stats.m_gcd_tests);
-        st.update("ineq splits", m_stats.m_branches);
-        st.update("gomory cuts", m_stats.m_gomory_cuts);
-        st.update("max-min", m_stats.m_max_min);
-        st.update("grobner", m_stats.m_gb_compute_basis);
-        st.update("pseudo nonlinear", m_stats.m_nl_linear);
-        st.update("nonlinear bounds", m_stats.m_nl_bounds);
-        st.update("nonlinear horner", m_stats.m_nl_cross_nested);
+        st.update("arith add rows", m_stats.m_add_rows);
+        st.update("arith pivots", m_stats.m_pivots);
+        st.update("arith assert lower", m_stats.m_assert_lower);
+        st.update("arith assert upper", m_stats.m_assert_upper);
+        st.update("arith assert diseq", m_stats.m_assert_diseq);
+        st.update("arith bound prop", m_stats.m_bound_props);
+        st.update("arith fixed eqs", m_stats.m_fixed_eqs);
+        st.update("arith offset eqs", m_stats.m_offset_eqs);
+        st.update("arith gcd tests", m_stats.m_gcd_tests);
+        st.update("arith ineq splits", m_stats.m_branches);
+        st.update("arith gomory cuts", m_stats.m_gomory_cuts);
+        st.update("arith patches", m_stats.m_patches);
+        st.update("arith patches_succ", m_stats.m_patches_succ);
+        st.update("arith max-min", m_stats.m_max_min);
+        st.update("arith grobner", m_stats.m_gb_compute_basis);
+        st.update("arith pseudo nonlinear", m_stats.m_nl_linear);
+        st.update("arith nonlinear bounds", m_stats.m_nl_bounds);
+        st.update("arith nonlinear horner", m_stats.m_nl_cross_nested);
         m_arith_eq_adapter.collect_statistics(st);
     }
 
     template<typename Ext>
     void theory_arith<Ext>::display(std::ostream & out) const {
+        if (get_num_vars() == 0) return;
         out << "Theory arithmetic:\n";
         display_vars(out);
         display_nl_monomials(out);
@@ -388,8 +391,19 @@ namespace smt {
     void theory_arith<Ext>::display_vars(std::ostream & out) const {
         out << "vars:\n";
         int n = get_num_vars();
-        for (theory_var v = 0; v < n; v++)
+        int inf_vars = 0;
+        int int_inf_vars = 0;
+        for (theory_var v = 0; v < n; v++) {
+            if ((lower(v) && lower(v)->get_value() > get_value(v))
+                || (upper(v) && upper(v)->get_value() < get_value(v)))
+                inf_vars++;
+            if (is_int(v) && !get_value(v).is_int())
+                int_inf_vars++;
+        }
+        out << "infeasibles = " << inf_vars << " int_inf = " << int_inf_vars << std::endl;
+        for (theory_var v = 0; v < n; v++) {
             display_var(out, v);
+        }
     }
 
     template<typename Ext>
@@ -399,33 +413,42 @@ namespace smt {
         out << "\n";
     }
 
+    template<typename Ext>
+    std::ostream& theory_arith<Ext>::antecedents_t::display(theory_arith& th, std::ostream & out) const {
+        th.get_context().display_literals_verbose(out, lits().size(), lits().c_ptr());
+        if (!lits().empty()) out << "\n";
+        ast_manager& m = th.get_manager();
+        for (auto const& e : m_eqs) {
+            out << mk_pp(e.first->get_owner(), m) << " ";
+            out << mk_pp(e.second->get_owner(), m) << "\n";            
+        }
+        return out;
+    }
+
     template<typename Ext>
     void theory_arith<Ext>::display_deps(std::ostream & out, v_dependency* dep) {
         ptr_vector<void> bounds;
         m_dep_manager.linearize(dep, bounds);
         m_tmp_lit_set.reset();
         m_tmp_eq_set.reset();
-        ptr_vector<void>::const_iterator it  = bounds.begin();
-        ptr_vector<void>::const_iterator end = bounds.end();
-        for (; it != end; ++it) {
-            bound * b = static_cast<bound*>(*it);
-            out << " ";
-            b->display(*this, out);
+        for (void *_b : bounds) {
+            bound * b = static_cast<bound*>(_b);
+            b->display(*this, out << "\n");
         }
     }
 
     template<typename Ext>
     void theory_arith<Ext>::display_interval(std::ostream & out, interval const& i) {
         i.display(out);
-        display_deps(out << " lo:", i.get_lower_dependencies());
-        display_deps(out << " hi:", i.get_upper_dependencies());
+        display_deps(out << "\nlo:", i.get_lower_dependencies());
+        display_deps(out << "\nhi:", i.get_upper_dependencies());
     }
 
     template<typename Ext>
     void theory_arith<Ext>::display_atoms(std::ostream & out) const {
         out << "atoms:\n";
-        for (unsigned i = 0; i < m_atoms.size(); i++)
-            display_atom(out, m_atoms[i], false);
+        for (atom * a : m_atoms) 
+            display_atom(out, a, false);
     }
 
     template<typename Ext>
@@ -491,7 +514,7 @@ namespace smt {
                 pp.add_assumption(eq);
             }
             else {
-                if (lower(v) != 0) {
+                if (lower(v) != nullptr) {
                     inf_numeral k_inf = lower_bound(v);
                     rational k = k_inf.get_rational().to_rational();
                     expr_ref ineq(m);
@@ -501,7 +524,7 @@ namespace smt {
                         ineq = m_util.mk_lt(m_util.mk_numeral(k, is_int(v)), n);
                     pp.add_assumption(ineq);
                 }
-                if (upper(v) != 0) {
+                if (upper(v) != nullptr) {
                     inf_numeral k_inf = upper_bound(v);
                     rational k = k_inf.get_rational().to_rational();
                     expr_ref ineq(m);
@@ -513,7 +536,7 @@ namespace smt {
                 }
             }
         }
-        pp.display(out, m.mk_true());
+        pp.display_smt2(out, m.mk_true());
     }
 
     template<typename Ext>
@@ -521,7 +544,7 @@ namespace smt {
         char buffer[128];
         static int id = 0;
 #ifdef _WINDOWS
-        sprintf_s(buffer, ARRAYSIZE(buffer), "arith_%d.smt", id);
+        sprintf_s(buffer, Z3_ARRAYSIZE(buffer), "arith_%d.smt", id);
 #else
         sprintf(buffer, "arith_%d.smt", id);
 #endif
