@@ -18,14 +18,14 @@ Notes:
 
 --*/
 
-#include "inf_rational.h"
-#include "inf_eps_rational.h"
-#include "arith_decl_plugin.h"
+#include "util/inf_rational.h"
+#include "util/inf_eps_rational.h"
+#include "ast/arith_decl_plugin.h"
 
 #ifndef THEORY_OPT_H_
 #define THEORY_OPT_H_
 
-class filter_model_converter;
+class generic_model_converter;
 namespace smt {
     class theory_opt {
     public:
@@ -33,7 +33,6 @@ namespace smt {
         virtual inf_eps value(theory_var) = 0;
         virtual inf_eps maximize(theory_var v, expr_ref& blocker, bool& has_shared) = 0; 
         virtual theory_var add_objective(app* term) = 0;
-        virtual expr_ref mk_ge(filter_model_converter& fm, theory_var v, inf_eps const& val) { UNREACHABLE(); return expr_ref(*((ast_manager*)0)); }
         bool is_linear(ast_manager& m, expr* term);
         bool is_numeral(arith_util& a, expr* term);
     };
