@@ -158,8 +158,6 @@ namespace smt {
         m_watch_sz = 0;
         m_watch_sum.reset();
         m_num_propagations = 0;
-        m_compilation_threshold = UINT_MAX;
-        m_compiled = l_false;
         m_args[0].reset();
         m_args[0].m_k.reset();
         m_args[1].reset();
@@ -253,7 +251,7 @@ namespace smt {
         context& ctx = th.get_context();
         unsigned sz = size();
         unsigned bound = k();
-        TRACE("pb", tout << "assign: " << m_lit << " " << ~alit << " " << bound << "\n";);
+        TRACE("pb", th.display(tout << "assign: " << m_lit << " " << ~alit << " " << bound << "\n", *this, true););
 
         SASSERT(0 < bound && bound < sz);
         SASSERT(ctx.get_assignment(alit) == l_false);
@@ -430,10 +428,6 @@ namespace smt {
 
     void theory_pb::card::inc_propagations(theory_pb& th) {
         ++m_num_propagations;
-        if (m_compiled == l_false && m_num_propagations >= m_compilation_threshold) {
-            // m_compiled = l_undef;
-            // th.m_to_compile.push_back(&c);
-        }
     }
 
     // ------------------------
@@ -443,8 +437,6 @@ namespace smt {
         theory(m.mk_family_id("pb")),
         m_params(p),
         pb(m),
-        m_max_compiled_coeff(rational(8)),
-        m_cardinality_lemma(false),
         m_restart_lim(3),
         m_restart_inc(0),
         m_antecedent_exprs(m),
@@ -452,7 +444,6 @@ namespace smt {
     {        
         m_learn_complements  = p.m_pb_learn_complements;
         m_conflict_frequency = p.m_pb_conflict_frequency;
-        m_enable_compilation = p.m_pb_enable_compilation;
     }
 
     theory_pb::~theory_pb() {
@@ -467,7 +458,6 @@ namespace smt {
         context& ctx = get_context();
         ast_manager& m = get_manager();
 
-        TRACE("pb", tout << mk_pp(atom, m) << "\n";);
         if (ctx.b_internalized(atom)) {
             return true;
         }
@@ -494,17 +484,25 @@ namespace smt {
         ctx.set_var_theory(abv, get_id());
         literal lit(abv);
 
-
         if (pb.is_eq(atom)) {
-            expr_ref_vector args(m);
+            expr_ref_vector args(m), nargs(m);
             vector<rational> coeffs;
-            unsigned n = atom->get_num_args();
-            for (unsigned i = 0; i < n; ++i) {
+            rational sum(0);
+            for (unsigned i = 0; i < num_args; ++i) {
                 args.push_back(atom->get_arg(i));
-                coeffs.push_back(pb.get_coeff(atom, i));
+                nargs.push_back(::mk_not(m, atom->get_arg(i)));
+                rational c = pb.get_coeff(atom, i);
+                coeffs.push_back(c);
+                sum += c;
             }
-            expr_ref le(pb.mk_le(n, coeffs.c_ptr(), args.c_ptr(), pb.get_k(atom)), m);
-            expr_ref ge(pb.mk_ge(n, coeffs.c_ptr(), args.c_ptr(), pb.get_k(atom)), m);
+            rational k = pb.get_k(atom);
+            // ax + by + cz <= k
+            // <=>
+            // -ax - by - cz >= -k
+            // <=>
+            // a(1-x) + b(1-y) + c(1-z) >= a + b + c - k
+            expr_ref le(pb.mk_ge(num_args, coeffs.c_ptr(), nargs.c_ptr(), sum - k), m);
+            expr_ref ge(pb.mk_ge(num_args, coeffs.c_ptr(), args.c_ptr(), k), m);
             ctx.internalize(le, false);
             ctx.internalize(ge, false);
             literal le_lit = ctx.get_literal(le);
@@ -522,7 +520,6 @@ namespace smt {
         numeral& k = c->m_args[0].m_k;
         arg_t& args = c->m_args[0];
 
-
         // extract literals and coefficients.
         for (unsigned i = 0; i < num_args; ++i) {
             expr* arg = atom->get_arg(i);
@@ -539,27 +536,23 @@ namespace smt {
                 break;
             }
         }
+
         if (pb.is_at_most_k(atom) || pb.is_le(atom)) {
-            // turn W <= k into -W >= -k
-            for (unsigned i = 0; i < args.size(); ++i) {
-                args[i].second = -args[i].second;
-            }
             k = -k;
+            for (auto& a : args) {
+                a.first.neg();
+                k += a.second;
+            }
         }
-        else {
-            SASSERT(pb.is_at_least_k(atom) || pb.is_ge(atom) || pb.is_eq(atom));
-        }
-        TRACE("pb", display(tout, *c, true););        
         c->unique();
         lbool is_true = c->normalize();
         c->prune();
         c->post_prune();
 
-
-        TRACE("pb", display(tout, *c); tout << " := " << lit << "\n";);        
+        TRACE("pb", display(tout, *c); tout << " := " << lit << " " << is_true << "\n";);        
         switch (is_true) {
         case l_false: 
-            lit = ~lit;
+            lit.neg();
             // fall-through
         case l_true: 
             ctx.mk_th_axiom(get_id(), 1, &lit);
@@ -584,32 +577,13 @@ namespace smt {
         // maximal coefficient:
         scoped_mpz& max_watch = c->m_max_watch;
         max_watch.reset();
-        for (unsigned i = 0; i < args.size(); ++i) {
-            mpz const& num = args[i].second.to_mpq().numerator();
+        for (auto const& a : args) {
+            mpz const& num = a.second.to_mpq().numerator();
             if (m_mpz_mgr.lt(max_watch, num)) {
                 max_watch = num;
             }
         }
 
-        // pre-compile threshold for cardinality
-        bool enable_compile = m_enable_compilation && c->is_ge() && !c->k().is_one();
-        for (unsigned i = 0; enable_compile && i < args.size(); ++i) {
-            enable_compile = (args[i].second <= m_max_compiled_coeff);
-        }
-        if (enable_compile) {
-            unsigned log = 1, n = 1;
-            while (n <= args.size()) {
-                ++log;
-                n *= 2;
-            }
-            unsigned th = args.size()*log*log; 
-            c->m_compilation_threshold = th;
-            IF_VERBOSE(2, verbose_stream() << "(smt.pb setting compilation threshold to " << th << ")\n";);
-            TRACE("pb", tout << "compilation threshold: " << th << "\n";);
-        }
-        else {
-            c->m_compilation_threshold = UINT_MAX;
-        }
         init_watch_ineq(*c);
         init_watch(abv);
         m_var_infos[abv].m_ineq = c;
@@ -723,10 +697,11 @@ namespace smt {
             ineqs = alloc(ptr_vector<ineq>);
             m_var_infos[lit.var()].m_lit_watch[lit.sign()] = ineqs;
         }
-        for (auto* c1 : *ineqs) {
-            //if (c1 == c) return;
-            SASSERT (c1 != c);
-        }
+        DEBUG_CODE(
+            for (auto* c1 : *ineqs) {
+                //if (c1 == c) return;
+                SASSERT (c1 != c);
+            });
         ineqs->push_back(c);
     }
 
@@ -965,10 +940,10 @@ namespace smt {
         justification* js = nullptr;
         c.inc_propagations(*this);
         if (!resolve_conflict(c, lits)) {
-			if (proofs_enabled()) {
-				js = alloc(theory_lemma_justification, get_id(), ctx, lits.size(), lits.c_ptr());
-			}
-            ctx.mk_clause(lits.size(), lits.c_ptr(), js, CLS_AUX_LEMMA, nullptr);
+            if (proofs_enabled()) {
+                js = alloc(theory_lemma_justification, get_id(), ctx, lits.size(), lits.c_ptr());
+            }
+            ctx.mk_clause(lits.size(), lits.c_ptr(), js, CLS_TH_LEMMA, nullptr);
         }
         SASSERT(ctx.inconsistent());
     }
@@ -980,7 +955,8 @@ namespace smt {
         }
         c.inc_propagations(*this);
         m_stats.m_num_propagations++;
-        TRACE("pb", tout << "#prop: " << c.num_propagations() << " - " << c.lit() << " => " << l << "\n";);
+        TRACE("pb", tout << "#prop: " << c.num_propagations() << " - " << c.lit() << " => " << l << "\n";
+              display(tout, c, true) << "\n";);
         SASSERT(validate_unit_propagation(c));
         ctx.assign(l, ctx.mk_justification(card_justification(c, l, get_id())));
     }
@@ -995,12 +971,10 @@ namespace smt {
     // 
 
     void theory_pb::collect_statistics(::statistics& st) const {
+        st.update("pb resolves", m_stats.m_num_resolves);
         st.update("pb conflicts", m_stats.m_num_conflicts);
         st.update("pb propagations", m_stats.m_num_propagations);
         st.update("pb predicates", m_stats.m_num_predicates);        
-        st.update("pb compilations", m_stats.m_num_compiles);
-        st.update("pb compiled clauses", m_stats.m_num_compiled_clauses);
-        st.update("pb compiled vars", m_stats.m_num_compiled_vars);
     }
     
     void theory_pb::reset_eh() {
@@ -1013,8 +987,6 @@ namespace smt {
         m_card_trail.reset();
         m_card_lim.reset();
         m_stats.reset();
-        m_to_compile.reset();
-        m_cardinality_lemma = false;
     }
 
     void theory_pb::new_eq_eh(theory_var v1, theory_var v2) {
@@ -1045,12 +1017,8 @@ namespace smt {
         }
         ineq* c = m_var_infos[v].m_ineq;
         if (c != nullptr) {
-            if (c->is_ge()) {
-                assign_ineq(*c, is_true);
-            }
-            else {
-                assign_eq(*c, is_true);
-            }
+            VERIFY(c->is_ge());
+            assign_ineq(*c, is_true);
         }
 
         ptr_vector<card>* cards = m_var_infos[v].m_lit_cwatch[nlit.sign()];
@@ -1194,10 +1162,15 @@ namespace smt {
         // perform unit propagation
         if (maxsum >= c.mpz_k() && maxsum - mininc < c.mpz_k()) { 
             literal_vector& lits = get_unhelpful_literals(c, true);
+            // for (literal lit : lits) SASSERT(ctx.get_assignment(lit) == l_true);
             lits.push_back(c.lit());
+            // SASSERT(ctx.get_assignment(c.lit()) == l_true);
             for (unsigned i = 0; i < sz; ++i) {
-                DEBUG_CODE(validate_assign(c, lits, c.lit(i)););
-                add_assign(c, lits, c.lit(i));                               
+                literal lit = c.lit(i);
+                if (ctx.get_assignment(lit) == l_undef) {
+                    DEBUG_CODE(validate_assign(c, lits, lit););
+                    add_assign(c, lits, c.lit(i));               
+                }
             }
         }
     }
@@ -1207,7 +1180,7 @@ namespace smt {
     */
     void theory_pb::assign_eq(ineq& c, bool is_true) {
         SASSERT(c.is_eq());
-        
+        UNREACHABLE();
     }
 
 
@@ -1362,31 +1335,9 @@ namespace smt {
 
     void theory_pb::inc_propagations(ineq& c) {
         ++c.m_num_propagations;
-        if (c.m_compiled == l_false && c.m_num_propagations >= c.m_compilation_threshold) {
-            c.m_compiled = l_undef;
-            m_to_compile.push_back(&c);
-        }
     }
 
     void theory_pb::restart_eh() {
-        for (unsigned i = 0; i < m_to_compile.size(); ++i) {
-            compile_ineq(*m_to_compile[i]);
-        }
-        m_to_compile.reset();
-
-        return;
-
-        if (m_restart_lim <= m_restart_inc) {
-            m_restart_inc = 0;
-            if (gc()) {
-                m_restart_lim = 3;
-            }
-            else {
-                m_restart_lim *= 4;
-                m_restart_lim /= 3;
-            }
-        }
-        ++m_restart_inc;
     }
 
     bool theory_pb::gc() {
@@ -1424,12 +1375,8 @@ namespace smt {
         for (unsigned i = 0; i < lemmas.size(); ++i) {
             clause* cl = lemmas[i];
             if (!cl->deleted()) {
-                unsigned sz = cl->get_num_literals();
-                for (unsigned j = 0; j < sz; ++j) {
-                    literal lit = cl->get_literal(j);
+                for (literal lit : *cl) {
                     if (m_occs.contains(lit.var())) {
-                        //std::cout << "deleting clause " << lit << " " << sz << "\n";
-                        //ctx.mark_as_deleted(cl);
                         break;
                     }
                 }
@@ -1451,76 +1398,6 @@ namespace smt {
     }
 
 
-    void theory_pb::compile_ineq(ineq& c) {
-        ++m_stats.m_num_compiles;
-        context& ctx = get_context();
-        // only cardinality constraints are compiled.
-        SASSERT(c.m_compilation_threshold < UINT_MAX);
-        DEBUG_CODE(for (unsigned i = 0; i < c.size(); ++i) SASSERT(c.coeff(i).is_int()); );
-        unsigned k = c.k().get_unsigned();
-        unsigned num_args = c.size();
-
-
-        literal thl = c.lit();
-        literal at_least_k;
-
-        literal_vector in;
-        for (unsigned i = 0; i < num_args; ++i) {
-            rational n = c.coeff(i);
-            literal lit = c.lit(i);
-            lbool val = ctx.get_assignment(lit); 
-            if (val != l_undef  && ctx.get_assign_level(lit) == ctx.get_base_level()) {
-                if (val == l_true) {
-                    unsigned m = n.get_unsigned();
-                    if (k < m) {
-                        return;
-                    }
-                    k -= m;
-                }
-                continue;
-            }
-            while (n.is_pos()) {
-                in.push_back(c.lit(i));
-                n -= rational::one();
-            }
-        }
-        
-        TRACE("pb", tout << in << " >= " << k << "\n";);
-
-
-        psort_expr ps(ctx, *this);
-        psort_nw<psort_expr> sortnw(ps);
-        sortnw.m_stats.reset();
-
-        if (ctx.get_assignment(thl) == l_true  && 
-            ctx.get_assign_level(thl) == ctx.get_base_level()) {
-            at_least_k = sortnw.ge(false, k, in.size(), in.c_ptr());            
-            TRACE("pb", tout << ~thl << " " << at_least_k << "\n";);
-            ctx.mk_clause(~thl, at_least_k, justify(~thl, at_least_k));
-        }
-        else {
-            literal at_least_k = sortnw.ge(true, k, in.size(), in.c_ptr());
-            TRACE("pb", tout << ~thl << " " << at_least_k << "\n";);
-            ctx.mk_clause(~thl, at_least_k, justify(~thl, at_least_k));
-            ctx.mk_clause(~at_least_k, thl, justify(thl, ~at_least_k));
-        }
-        m_stats.m_num_compiled_vars += sortnw.m_stats.m_num_compiled_vars;
-        m_stats.m_num_compiled_clauses += sortnw.m_stats.m_num_compiled_clauses;
-
-        IF_VERBOSE(2, verbose_stream() 
-                   << "(smt.pb compile sorting network bound: " 
-                   << k << " literals: " << in.size() 
-                   << " clauses: " << sortnw.m_stats.m_num_compiled_clauses 
-                   << " vars: " << sortnw.m_stats.m_num_compiled_vars << ")\n";);
-
-        // auxiliary clauses get removed when popping scopes.
-        // we have to recompile the circuit after back-tracking.
-        c.m_compiled = l_false;
-        ctx.push_trail(value_trail<context, lbool>(c.m_compiled));
-        c.m_compiled = l_true;
-    }
-
-
     void theory_pb::init_search_eh() {
     }
 
@@ -1540,7 +1417,6 @@ namespace smt {
             clear_watch(*c);
             m_var_infos[v].m_ineq = nullptr;
             m_ineqs_trail.pop_back();
-            m_to_compile.erase(c);
             dealloc(c);
         }
         m_ineqs_lim.resize(new_lim);
@@ -1659,7 +1535,7 @@ namespace smt {
         if (proofs_enabled()) {                                         
             js = alloc(theory_lemma_justification, get_id(), ctx, lits.size(), lits.c_ptr());
         }
-        ctx.mk_clause(lits.size(), lits.c_ptr(), js, CLS_AUX_LEMMA, nullptr);
+        ctx.mk_clause(lits.size(), lits.c_ptr(), js, CLS_TH_LEMMA, nullptr);
     }
 
 
@@ -1723,6 +1599,9 @@ namespace smt {
                 value += coeff;
             }
         }
+        if (value >= 0) {
+            display_resolved_lemma(verbose_stream() << "not validated\n");
+        }
         // std::cout << "bound: " << m_bound << " value " << value << " coeffs: " << m_active_vars.size() << " lemma is " << (value >= 0 ? "sat" : "unsat") << "\n";    
         return value < 0;
     }
@@ -1808,15 +1687,20 @@ namespace smt {
         return arg_max;
     }
 
+    /**
+       \brief retrieve asserting literal. Prefer p, otherwise find a literal with maximal level.
+       Note that an asserting literal should be false with respect to the resolvent inequality.
+     */
     literal theory_pb::get_asserting_literal(literal p) {
-        if (get_abs_coeff(p.var()) != 0) {
-            return p;
-        }
         context& ctx = get_context();
         unsigned lvl = 0;
+        TRACE("pb", tout << p << " " << ctx.get_assignment(p) << "\n";);
+
+        if (ctx.get_assignment(p) == l_false && get_abs_coeff(p.var()) != 0 && p == literal(p.var(), get_coeff(p.var()) < 0)) {
+            return p;
+        }
         
-        for (unsigned i = 0; i < m_active_vars.size(); ++i) { 
-            bool_var v = m_active_vars[i];
+        for (bool_var v : m_active_vars) {
             literal lit(v, get_coeff(v) < 0);
             if (ctx.get_assignment(lit) == l_false && ctx.get_assign_level(lit) > lvl) {
                 p = lit;
@@ -1859,82 +1743,6 @@ namespace smt {
         return true;
     }
 
-    void theory_pb::add_cardinality_lemma() {
-        context& ctx = get_context();
-        normalize_active_coeffs();
-        int s = 0;
-        int new_bound = 0;
-        if (!init_arg_max()) {
-            return;
-        }
-        // TBD: can be optimized
-        while (s < m_bound) {
-            int coeff;
-            int arg = arg_max(coeff);
-            if (arg == -1) break;
-            s += coeff;
-            ++new_bound;
-        }
-        int slack = m_active_coeffs.empty() ? m_bound : (std::min(m_bound, static_cast<int>(m_active_coeffs[0]) - 1));
-        reset_arg_max();
-
-        while (slack > 0) {
-            bool found = false;
-            int v = 0;
-            int coeff = 0;
-            for (unsigned i = 0; !found && i < m_active_vars.size(); ++i) {            
-                bool_var v = m_active_vars[i];
-                coeff = get_abs_coeff(v);
-                if (0 < coeff && coeff < slack) {
-                    found = true;
-                }
-            }
-            if (!found) {
-                break;
-            }
-            slack -= coeff;
-            m_coeffs[v] = 0; // deactivate coefficient.
-        }
-        for (unsigned i = 0; i < m_active_vars.size(); ++i) {
-            bool_var v = m_active_vars[i];
-            int coeff = get_coeff(v);
-            if (coeff < 0) {
-                m_coeffs[v] = -1;
-            }
-            else if (coeff > 0) {
-                m_coeffs[v] = 1;
-            }
-        }        
-
-        m_bound = new_bound;
-        if (!validate_lemma()) {
-            return;
-        }
-        SASSERT(m_bound > 0);
-        if (m_bound > static_cast<int>(m_active_vars.size())) {
-            return;
-        }           
-        if (m_bound == static_cast<int>(m_active_vars.size())) {
-            return;
-        }
-
-        m_antecedent_exprs.reset();
-        m_antecedent_signs.reset();
-        m_cardinality_exprs.reset();
-        m_cardinality_signs.reset();
-        for (unsigned i = 0; i < m_antecedents.size(); ++i) {
-            literal lit = m_antecedents[i];
-            m_antecedent_exprs.push_back(ctx.bool_var2expr(lit.var()));
-            m_antecedent_signs.push_back(lit.sign());
-        }
-        for (unsigned i = 0; i < m_active_vars.size(); ++i) {
-            bool_var v = m_active_vars[i];
-            m_cardinality_exprs.push_back(ctx.bool_var2expr(v));
-            m_cardinality_signs.push_back(get_coeff(v) < 0);
-        }
-        m_cardinality_lemma = true;
-    }
-
     void theory_pb::normalize_active_coeffs() {
         while (!m_active_var_set.empty()) m_active_var_set.erase();
         unsigned i = 0, j = 0, sz = m_active_vars.size();
@@ -2006,56 +1814,17 @@ namespace smt {
         }
         if (g >= 2) {
             normalize_active_coeffs();
-            for (unsigned i = 0; i < m_active_vars.size(); ++i) {
-                m_coeffs[m_active_vars[i]] /= g;                
+            for (auto v : m_active_vars) {
+                m_coeffs[v] /= static_cast<int>(g);                
             }
             m_bound = (m_bound + g - 1) / g;
             TRACE("pb", display_resolved_lemma(tout << "cut\n"););
         }
     }
 
-    bool theory_pb::can_propagate() { return m_cardinality_lemma; }
+    bool theory_pb::can_propagate() { return false; }
     
-    void theory_pb::propagate() {
-        context& ctx = get_context();
-        ast_manager& m = get_manager();
-        if (!m_cardinality_lemma) {
-            return;
-        }
-        m_cardinality_lemma = false;
-        if (ctx.inconsistent()) {
-            return;
-        }
-        m_antecedents.reset();
-
-        for (unsigned i = 0; i < m_antecedent_exprs.size(); ++i) {
-            expr* a = m_antecedent_exprs[i].get();
-            if (!ctx.b_internalized(a)) {
-                // std::cout << "not internalized " << mk_pp(a, m) << "\n";
-                return;
-            }
-            m_antecedents.push_back(~literal(ctx.get_bool_var(a), m_antecedent_signs[i]));
-        }
-        for (unsigned i = 0; i < m_cardinality_exprs.size(); ++i) {
-            expr* a = m_cardinality_exprs[i].get();
-            if (!ctx.b_internalized(a)) {
-                // std::cout << "not internalized " << mk_pp(a, m) << "\n";
-                return;
-            }
-            if (m_cardinality_signs[i]) {
-                m_cardinality_exprs[i] = m.mk_not(a);
-            }
-        }        
-        app_ref atl(pb.mk_at_least_k(m_cardinality_exprs.size(), m_cardinality_exprs.c_ptr(), m_bound), m);
-        VERIFY(internalize_card(atl, false));
-        bool_var abv = ctx.get_bool_var(atl);
-        m_antecedents.push_back(literal(abv));
-        justification* js = nullptr;
-        if (proofs_enabled()) {
-            js = nullptr;
-        }
-        ctx.mk_clause(m_antecedents.size(), m_antecedents.c_ptr(), js, CLS_AUX_LEMMA, nullptr);
-    }
+    void theory_pb::propagate() { }
 
     bool theory_pb::resolve_conflict(card& c, literal_vector const& confl) {
        
@@ -2065,9 +1834,7 @@ namespace smt {
         context& ctx = get_context();
         ast_manager& m = get_manager();
         m_conflict_lvl = 0;
-        m_cardinality_lemma = false;
-        for (unsigned i = 0; i < confl.size(); ++i) {
-            literal lit = confl[i];
+        for (literal lit : confl) {
             SASSERT(ctx.get_assignment(lit) == l_false);
             m_conflict_lvl = std::max(m_conflict_lvl, ctx.get_assign_level(lit));            
         }
@@ -2075,8 +1842,6 @@ namespace smt {
             return false;
         }
 
-        // std::cout << c.lit() << "\n";
-
         reset_coeffs();
         m_num_marks = 0;
         m_bound = c.k();
@@ -2165,6 +1930,7 @@ namespace smt {
                 process_antecedent(~js.get_literal(), offset);
                 break;
             case b_justification::AXIOM:
+                bound = 0;
                 break;
             case b_justification::JUSTIFICATION: {
                 justification* j = js.get_justification(); 
@@ -2175,7 +1941,11 @@ namespace smt {
                 }
                 if (pbj == nullptr) {
                     TRACE("pb", tout << "skip justification for " << conseq << "\n";);
-                    inc_coeff(conseq, offset);
+                    bound = 0;
+                    // this is possible when conseq is an assumption.
+                    // The justification of conseq is itself, 
+                    // don't increment the cofficient here because it assumes
+                    // conseq is justified further. it isnt'. conseq becomes part of the lemma.
                 }
                 else {
                     card& c2 = pbj->get_card();
@@ -2183,7 +1953,6 @@ namespace smt {
                     bound = c2.k();
                 }
                 
-                // std::cout << " offset: " << offset << " bound: " << bound << "\n";
                 break;
             }
             default:
@@ -2229,71 +1998,48 @@ namespace smt {
         }
 
         int slack = -m_bound;
-        for (unsigned i = 0; i < m_active_vars.size(); ++i) { 
-            bool_var v = m_active_vars[i];
+        for (bool_var v : m_active_vars) {
             slack += get_abs_coeff(v);
         }
 
-#if 1
-        //std::cout << slack << " " << m_bound << "\n";
         unsigned i = 0;
         literal_vector const& alits = ctx.assigned_literals();
 
         literal alit = get_asserting_literal(~conseq);
         slack -= get_abs_coeff(alit.var());
 
-        for (i = alits.size(); 0 <= slack && i > 0; ) {
-            --i;
+        for (i = alits.size(); 0 <= slack && i-- > 0; ) {
             literal lit = alits[i];
             bool_var v = lit.var();
             // -3*x >= k 
             if (m_active_var_set.contains(v) && v != alit.var()) {
                 int coeff = get_coeff(v);
-                //std::cout << coeff << " " << lit << "\n";
                 if (coeff < 0 && !lit.sign()) {
                     slack += coeff;
                     m_antecedents.push_back(lit);
-                    //std::cout << "ante: " << lit << "\n";
                 }
                 else if (coeff > 0 && lit.sign()) {
                     slack -= coeff;
                     m_antecedents.push_back(lit);
-                    //std::cout << "ante: " << lit << "\n";
                 }
             }
         }
         SASSERT(slack < 0);
 
-#else 
+        ++m_stats.m_num_resolves;
 
-        literal alit = get_asserting_literal(~conseq);
-        slack -= get_abs_coeff(alit.var());
-
-        for (unsigned i = 0; 0 <= slack; ++i) { 
-            SASSERT(i < m_active_vars.size());
-            bool_var v = m_active_vars[i];
-            literal lit(v, get_coeff(v) < 0);
-            if (v != alit.var() && ctx.get_assignment(lit) == l_false) {
-                m_antecedents.push_back(~lit);
-                slack -= get_abs_coeff(v);
-            }
-            if (slack < 0) {
-                std::cout << i << " " << m_active_vars.size() << "\n";
-            }
-        }
-#endif
         SASSERT(validate_antecedents(m_antecedents));
+        TRACE("pb", tout << "assign " << m_antecedents << " ==> " << alit << "\n";);
         ctx.assign(alit, ctx.mk_justification(theory_propagation_justification(get_id(), ctx.get_region(), m_antecedents.size(), m_antecedents.c_ptr(), alit, 0, nullptr)));
 
         DEBUG_CODE(
             m_antecedents.push_back(~alit);
             expr_ref_vector args(m);
-            for (unsigned i = 0; i < m_antecedents.size(); ++i) {
-                args.push_back(literal2expr(m_antecedents[i]));
+            for (literal lit : m_antecedents) {
+                args.push_back(literal2expr(lit));
             }
             B = m.mk_not(m.mk_and(args.size(), args.c_ptr()));
             validate_implies(A, B); );
-        // add_cardinality_lemma();
         return true;
     }
 
@@ -2357,21 +2103,58 @@ namespace smt {
     }
 
     void theory_pb::validate_final_check() {
-        for (unsigned i = 0; i < m_var_infos.size(); ++i) {
-            ineq* c = m_var_infos[i].m_ineq;
-            if (c) {
-                validate_final_check(*c);
+        TRACE("pb", tout << "validate " << m_var_infos.size() << "\n";);
+        for (auto & vi : m_var_infos) {
+            if (vi.m_ineq) {
+                validate_final_check(*vi.m_ineq);
+            }
+            if (vi.m_card) {
+                validate_final_check(*vi.m_card);
             }
         }
     }
 
+    void theory_pb::validate_final_check(card& c) {
+        context& ctx = get_context();
+        if (ctx.get_assignment(c.lit()) == l_undef) {
+            TRACE("pb", display(tout << "is undef ", c, true););
+            return;
+        }
+        if (!ctx.is_relevant(c.lit())) {
+            TRACE("pb", display(tout << "not relevant ", c, true););
+            return;
+        }
+
+        unsigned sum = 0, maxsum = 0;
+        for (unsigned i = 0; i < c.size(); ++i) {
+            switch(ctx.get_assignment(c.lit(i))) {
+            case l_true:
+                ++sum;
+            case l_undef:
+                ++maxsum;
+                break;
+            case l_false:
+                break;
+            }
+        }
+        TRACE("pb_verbose", display(tout << "validate: ", c, true);
+              tout << "sum: " << sum << " " << maxsum << " ";
+              tout << ctx.get_assignment(c.lit()) << "\n";);
+
+        SASSERT(sum <= maxsum);
+        SASSERT((sum >= c.k()) == (ctx.get_assignment(c.lit()) == l_true));
+        SASSERT((maxsum < c.k()) == (ctx.get_assignment(c.lit()) == l_false));        
+    }
+
     void theory_pb::validate_final_check(ineq& c) {
         context& ctx = get_context();
 
         if (ctx.get_assignment(c.lit()) == l_undef) {
+            TRACE("pb", tout << c.lit() << " is undef\n";);
             return;
         }
         if (!ctx.is_relevant(c.lit())) {
+            TRACE("pb", tout << c.lit() << " is not relevant\n";);
             return;
         }
         numeral sum = numeral::zero(), maxsum = numeral::zero();
@@ -2399,8 +2182,8 @@ namespace smt {
 
     bool theory_pb::validate_antecedents(literal_vector const& lits) {
         context& ctx = get_context();
-        for (unsigned i = 0; i < lits.size(); ++i) {
-            if (ctx.get_assignment(lits[i]) != l_true) {
+        for (literal lit : lits) {
+            if (ctx.get_assignment(lit) != l_true) {
                 return false;
             }
         }
@@ -2410,7 +2193,9 @@ namespace smt {
     bool theory_pb::validate_unit_propagation(card const& c) {
         context& ctx = get_context();
         for (unsigned i = c.k(); i < c.size(); ++i) {
-            VERIFY(ctx.get_assignment(c.lit(i)) == l_false);
+            if (ctx.get_assignment(c.lit(i)) != l_false) {
+                return false;
+            }
         }
         return true;
     }
@@ -2440,15 +2225,11 @@ namespace smt {
 
     void theory_pb::display_resolved_lemma(std::ostream& out) const {
         context& ctx = get_context();
-        bool_var v;
-        unsigned lvl;
         out << "num marks: " << m_num_marks << "\n";
         out << "conflict level: " << m_conflict_lvl << "\n";
-        for (unsigned i = 0; i < m_resolved.size(); ++i) {
-            v = m_resolved[i].var();
-            lvl = ctx.get_assign_level(v);
-            out << lvl << ": " << m_resolved[i] << " ";
-            ctx.display(out, ctx.get_justification(v));
+        for (literal r : m_resolved) {
+            out << ctx.get_assign_level(r) << ": " << r << " ";
+            ctx.display(out, ctx.get_justification(r.var()));
         }
 
         if (!m_antecedents.empty()) {
@@ -2456,8 +2237,7 @@ namespace smt {
         }
         uint_set seen;
         bool first = true;
-        for (unsigned i = 0; i < m_active_vars.size(); ++i) {
-            bool_var v = m_active_vars[i];
+        for (bool_var v: m_active_vars) {
             if (seen.contains(v)) {
                 continue;
             }
@@ -2469,18 +2249,14 @@ namespace smt {
             if (!first) {
                 out << " + ";
             }
-            if (coeff == 1) {
-                out << literal(v);
+            literal lit(v, coeff < 0);
+            if (coeff > 1) {
+                out << coeff << " * ";
             }
-            else if (coeff == -1) {
-                out << literal(v, true);
-            }
-            else if (coeff > 0) {
-                out << coeff << " * " << literal(v);
-            }
-            else {
-                out << (-coeff) << " * " << literal(v, true);
+            else if (coeff < -1) {
+                out << (-coeff) << " * ";
             }
+            out << lit << "(" << ctx.get_assignment(lit) << "@" << ctx.get_assign_level(lit) << ")";
             first = false;
         }
         out << " >= " << m_bound << "\n";
@@ -2556,7 +2332,7 @@ namespace smt {
             result.append(m_dependencies.size(), m_dependencies.c_ptr());
         }
 
-        app * mk_value(model_generator & mg, ptr_vector<expr> & values) override {
+        app * mk_value(model_generator & mg, expr_ref_vector const& values) override {
             ast_manager& m = mg.get_manager();
             SASSERT(values.size() == m_dependencies.size());
             SASSERT(values.size() == m_app->get_num_args());
