@@ -777,11 +777,11 @@ namespace smt {
         enforce_parity();
         m_graph.set_to_zero(to_var(m_zero), neg(to_var(m_zero)));
         compute_delta();   
-        DEBUG_CODE(validate_model(););
+        DEBUG_CODE(model_validate(););
     }
 
     template<typename Ext>    
-    void theory_utvpi<Ext>::validate_model() {
+    void theory_utvpi<Ext>::model_validate() {
         context& ctx = get_context();
         unsigned sz = m_atoms.size();
         for (unsigned i = 0; i < sz; ++i) {
