@@ -21,7 +21,7 @@ Notes:
 #ifndef COMBINED_SOLVER_H_
 #define COMBINED_SOLVER_H_
 
-#include"params.h"
+#include "util/params.h"
 
 class solver;
 class solver_factory;
