@@ -62,7 +62,7 @@ public:
 
     void push_core() override;
     void pop_core(unsigned n) override;
-    lbool check_sat_core(unsigned num_assumptions, expr * const * assumptions) override;
+    lbool check_sat_core2(unsigned num_assumptions, expr * const * assumptions) override;
 
     void collect_statistics(statistics & st) const override;
     void get_unsat_core(expr_ref_vector & r) override;
@@ -80,11 +80,21 @@ public:
 
     expr_ref_vector cube(expr_ref_vector& vars, unsigned ) override {
         set_reason_unknown("cubing is not supported on tactics");
+        IF_VERBOSE(1, verbose_stream() << "cubing is not supported on tactics\n");
         return expr_ref_vector(get_manager());
     }
 
     model_converter_ref get_model_converter() const override { return m_mc; }
 
+    void get_levels(ptr_vector<expr> const& vars, unsigned_vector& depth) override {
+        throw default_exception("cannot retrieve depth from solvers created using tactics");
+    }
+
+    expr_ref_vector get_trail() override {
+        throw default_exception("cannot retrieve trail from solvers created using tactcis");
+    }
+
+
 };
 
 ast_manager& tactic2solver::get_manager() const { return m_assertions.get_manager(); }
@@ -136,7 +146,7 @@ void tactic2solver::pop_core(unsigned n) {
     m_result = nullptr;
 }
 
-lbool tactic2solver::check_sat_core(unsigned num_assumptions, expr * const * assumptions) {
+lbool tactic2solver::check_sat_core2(unsigned num_assumptions, expr * const * assumptions) {
     if (m_tactic.get() == nullptr)
         return l_false;
     ast_manager & m = m_assertions.m();
@@ -183,12 +193,14 @@ lbool tactic2solver::check_sat_core(unsigned num_assumptions, expr * const * ass
     }
     catch (z3_error & ex) {
         TRACE("tactic2solver", tout << "exception: " << ex.msg() << "\n";);
+        m_result->m_proof = pr;
         throw ex;
     }
     catch (z3_exception & ex) {
         TRACE("tactic2solver", tout << "exception: " << ex.msg() << "\n";);
         m_result->set_status(l_undef);
         m_result->m_unknown = ex.msg();
+        m_result->m_proof = pr;
     }
     m_tactic->collect_statistics(m_result->m_stats);
     m_tactic->collect_statistics(m_stats);
