@@ -22,22 +22,25 @@ Notes:
 #ifndef TACTIC2SOLVER_H_
 #define TACTIC2SOLVER_H_
 
-#include"params.h"
+#include "util/params.h"
+
 class ast_manager;
 class tactic;
-class tactic_factory;
 class solver;
 class solver_factory;
 
+typedef tactic* (*tactic_factory)(ast_manager&, const params_ref&);
+
 solver * mk_tactic2solver(ast_manager & m, 
-                          tactic * t = 0, 
+                          tactic * t = nullptr,
                           params_ref const & p = params_ref(), 
                           bool produce_proofs = false, 
                           bool produce_models = true, 
                           bool produce_unsat_cores = false, 
                           symbol const & logic = symbol::null);
 
+
 solver_factory * mk_tactic2solver_factory(tactic * t);
-solver_factory * mk_tactic_factory2solver_factory(tactic_factory * f);
+solver_factory * mk_tactic_factory2solver_factory(tactic_factory f);
 
 #endif
