@@ -30,7 +30,7 @@ Notes:
 --*/
 #ifndef ARITH_BOUNDS_TACTIC_H_
 #define ARITH_BOUNDS_TACTIC_H_
-#include "tactic.h"
+#include "tactic/tactic.h"
 
 tactic * mk_arith_bounds_tactic(ast_manager & m, params_ref const & p = params_ref());
 
