@@ -33,8 +33,10 @@ class bv2int_rewriter_ctx {
     expr_ref_vector          m_trail;
     
 public:
-    bv2int_rewriter_ctx(ast_manager& m, params_ref const& p) : 
-        m_max_size(UINT_MAX), m_side_conditions(m), m_trail(m) { update_params(p); }
+    bv2int_rewriter_ctx(ast_manager& m, params_ref const& p, unsigned max_size) : 
+		m_max_size(max_size), m_side_conditions(m), m_trail(m) {
+		update_params(p); 
+	}
 
     void reset() { m_side_conditions.reset(); m_trail.reset(); m_power2.reset(); }
     void add_side_condition(expr* e) { m_side_conditions.push_back(e); }
