@@ -24,7 +24,7 @@ Notes:
 #ifndef FIX_DL_VAR_TACTIC_H_
 #define FIX_DL_VAR_TACTIC_H_
 
-#include"params.h"
+#include "util/params.h"
 class ast_manager;
 class tactic;
 
