@@ -21,17 +21,16 @@ Author:
 Revision History:
 
 --*/
-#include"fm_tactic.h"
-#include"tactical.h"
-#include"arith_decl_plugin.h"
-#include"for_each_expr.h"
-#include"cooperate.h"
-#include"ast_smt2_pp.h"
-#include"ast_pp.h"
-#include"id_gen.h"
-#include"model_evaluator.h"
-#include"model_v2_pp.h"
-#include"simplify_tactic.h"
+#include "tactic/arith/fm_tactic.h"
+#include "tactic/tactical.h"
+#include "ast/arith_decl_plugin.h"
+#include "ast/for_each_expr.h"
+#include "ast/ast_smt2_pp.h"
+#include "ast/ast_pp.h"
+#include "util/id_gen.h"
+#include "model/model_evaluator.h"
+#include "model/model_v2_pp.h"
+#include "tactic/core/simplify_tactic.h"
 
 class fm_tactic : public tactic {
     typedef ptr_vector<app> clauses;
@@ -44,7 +43,6 @@ class fm_tactic : public tactic {
         ast_manager &         m;
         ptr_vector<func_decl> m_xs;
         vector<clauses>       m_clauses;
-        volatile bool         m_cancel;
 
         enum r_kind {
             NONE,
@@ -55,7 +53,7 @@ class fm_tactic : public tactic {
         bool is_false(model_ref & md, app * p) {
             SASSERT(is_uninterp_const(p));
             expr * val = md->get_const_interp(p->get_decl());
-            if (val == 0) {
+            if (val == nullptr) {
                 // if it is don't care, then set to false
                 md->register_decl(p->get_decl(), m.mk_false());
                 return true;
@@ -63,7 +61,7 @@ class fm_tactic : public tactic {
             return m.is_false(val);
         }
 
-        r_kind process(func_decl * x, expr * cls, arith_util & u, model_evaluator & ev, rational & r) {
+        r_kind process(func_decl * x, expr * cls, arith_util & u, model& ev, rational & r) {
             unsigned num_lits;
             expr * const * lits;
             if (m.is_or(cls)) {
@@ -81,9 +79,7 @@ class fm_tactic : public tactic {
                 expr * l = lits[i];
                 expr * atom;
                 if (is_uninterp_const(l) || (m.is_not(l, atom) && is_uninterp_const(atom))) {
-                    expr_ref val(m);
-                    ev(l, val);
-                    if (m.is_true(val))
+                    if (ev.is_true(l)) 
                         return NONE; // clause was satisfied
                 }
                 else {
@@ -132,7 +128,7 @@ class fm_tactic : public tactic {
                         }
                         else {
                             expr_ref val(m);
-                            ev(monomial, val);
+                            val = ev(monomial);
                             SASSERT(u.is_numeral(val));
                             rational tmp;
                             u.is_numeral(val, tmp);
@@ -157,6 +153,7 @@ class fm_tactic : public tactic {
                     r = c;
                 }
             }
+            (void)found;
             SASSERT(found);
             return is_lower ? LOWER : UPPER;
         }
@@ -164,7 +161,7 @@ class fm_tactic : public tactic {
     public:
         fm_model_converter(ast_manager & _m):m(_m) {}
 
-        virtual ~fm_model_converter() {
+        ~fm_model_converter() override {
             m.dec_array_ref(m_xs.size(), m_xs.c_ptr());
             vector<clauses>::iterator it  = m_clauses.begin();
             vector<clauses>::iterator end = m_clauses.end();
@@ -180,11 +177,13 @@ class fm_tactic : public tactic {
             m_clauses.back().swap(c);
         }
 
-        virtual void operator()(model_ref & md, unsigned goal_idx) {
+        void get_units(obj_map<expr, bool>& units) override { units.reset(); }
+
+        void operator()(model_ref & md) override {
             TRACE("fm_mc", model_v2_pp(tout, *md); display(tout););
-            m_cancel = false;
-            model_evaluator ev(*(md.get()));
-            ev.set_model_completion(true);
+            model::scoped_model_completion _sc(*md, true);
+            //model_evaluator ev(*(md.get()));
+            //ev.set_model_completion(true);
             arith_util u(m);
             unsigned i = m_xs.size();
             while (i > 0) {
@@ -199,8 +198,8 @@ class fm_tactic : public tactic {
                 clauses::iterator it  = m_clauses[i].begin();
                 clauses::iterator end = m_clauses[i].end();
                 for (; it != end; ++it) {
-                    if (m_cancel) throw tactic_exception(TACTIC_CANCELED_MSG);
-                    switch (process(x, *it, u, ev, val)) {
+                    if (m.canceled()) throw tactic_exception(m.limit().get_cancel_msg());
+                    switch (process(x, *it, u, *md, val)) {
                     case NONE: 
                         TRACE("fm_mc", tout << "no bound for:\n" << mk_ismt2_pp(*it, m) << "\n";);
                         break;
@@ -244,11 +243,8 @@ class fm_tactic : public tactic {
             TRACE("fm_mc", model_v2_pp(tout, *md););
         }
 
-        virtual void cancel() {
-            m_cancel = true;
-        }
 
-        virtual void display(std::ostream & out) {
+        void display(std::ostream & out) override {
             out << "(fm-model-converter";
             SASSERT(m_xs.size() == m_clauses.size());
             unsigned sz = m_xs.size();
@@ -265,7 +261,7 @@ class fm_tactic : public tactic {
             out << ")\n";
         }
 
-        virtual model_converter * translate(ast_translation & translator) {
+        model_converter * translate(ast_translation & translator) override {
             ast_manager & to_m = translator.to();
             fm_model_converter * res = alloc(fm_model_converter, to_m);
             unsigned sz = m_xs.size();
@@ -394,7 +390,6 @@ class fm_tactic : public tactic {
         obj_hashtable<func_decl> m_forbidden_set; // variables that cannot be eliminated because occur in non OCC ineq part
         goal_ref                 m_new_goal;
         ref<fm_model_converter>  m_mc;
-        volatile bool            m_cancel;
         id_gen                   m_id_gen;
         bool                     m_produce_models;
         bool                     m_fm_real_only;
@@ -784,7 +779,6 @@ class fm_tactic : public tactic {
             m_var2expr(m),
             m_inconsistent_core(m) {
             updt_params(p);
-            m_cancel = false;
         }
         
         ~imp() {
@@ -801,9 +795,6 @@ class fm_tactic : public tactic {
             m_fm_occ         = p.get_bool("fm_occ", false);
         }
         
-        void set_cancel(bool f) {
-            m_cancel = f;
-        }
         
         struct forbidden_proc {
             imp & m_owner;
@@ -837,7 +828,7 @@ class fm_tactic : public tactic {
             reset_constraints();
             m_bvar2expr.reset();
             m_bvar2sign.reset();
-            m_bvar2expr.push_back(0); // bvar 0 is not used
+            m_bvar2expr.push_back(nullptr); // bvar 0 is not used
             m_bvar2sign.push_back(0);
             m_expr2var.reset();
             m_is_int.reset();
@@ -847,11 +838,11 @@ class fm_tactic : public tactic {
             m_expr2var.reset();
             m_lowers.reset();
             m_uppers.reset();
-            m_new_goal = 0;
-            m_mc = 0;
+            m_new_goal = nullptr;
+            m_mc = nullptr;
             m_counter = 0;
             m_inconsistent = false;
-            m_inconsistent_core = 0;
+            m_inconsistent_core = nullptr;
             init_forbidden_set(g);
         }
         
@@ -887,7 +878,7 @@ class fm_tactic : public tactic {
                 // 0 <= 0 -- > true
                 if (c.m_c.is_pos() || (!c.m_strict && c.m_c.is_zero()))
                     return m.mk_true();
-                ineq = 0;
+                ineq = nullptr;
             }
             else {
                 bool int_cnstr = all_int(c);
@@ -1002,7 +993,7 @@ class fm_tactic : public tactic {
             sbuffer<var>     xs;
             buffer<rational> as;
             rational         c;
-            bool             strict;
+            bool             strict = false;
             unsigned         num;
             expr * const *   args;
             if (m.is_or(f)) {
@@ -1124,7 +1115,7 @@ class fm_tactic : public tactic {
             }
             else {
                 TRACE("add_constraint_bug", tout << "all variables are forbidden "; display(tout, *c); tout << "\n";);
-                m_new_goal->assert_expr(to_expr(*c), 0, c->m_dep);
+                m_new_goal->assert_expr(to_expr(*c), nullptr, c->m_dep);
                 del_constraint(c);
                 return false;
             }
@@ -1139,7 +1130,7 @@ class fm_tactic : public tactic {
                 if (is_occ(f))
                     add_constraint(f, g.dep(i));
                 else
-                    m_new_goal->assert_expr(f, 0, g.dep(i));
+                    m_new_goal->assert_expr(f, nullptr, g.dep(i));
             }
         }
 
@@ -1239,7 +1230,7 @@ class fm_tactic : public tactic {
         }
         
         // An integer variable x may be eliminated, if 
-        //   1- All variables in the contraints it occur are integer.
+        //   1- All variables in the constraints it occur are integer.
         //   2- The coefficient of x in all lower bounds (or all upper bounds) is unit.
         bool can_eliminate(var x) const {
             if (!is_int(x))
@@ -1376,7 +1367,7 @@ class fm_tactic : public tactic {
                       display(tout, l);
                       tout << "\n";
                       display(tout, u); tout << "\n";);
-                return 0; // no constraint needs to be created.
+                return nullptr; // no constraint needs to be created.
             }
             
             new_lits.reset();
@@ -1420,7 +1411,7 @@ class fm_tactic : public tactic {
                       display(tout, l);
                       tout << "\n";
                       display(tout, u); tout << "\n";);
-                return 0;
+                return nullptr;
             }
 
             expr_dependency * new_dep = m.mk_join(l.m_dep, u.m_dep);
@@ -1432,7 +1423,7 @@ class fm_tactic : public tactic {
                       display(tout, u); tout << "\n";);
                 m_inconsistent      = true;
                 m_inconsistent_core = new_dep;
-                return 0;
+                return nullptr;
             }
             
             constraint * new_cnstr = mk_constraint(new_lits.size(),
@@ -1502,7 +1493,7 @@ class fm_tactic : public tactic {
                     constraint const & l_c = *(l[i]);
                     constraint const & u_c = *(u[j]);
                     constraint * new_c = resolve(l_c, u_c, x);
-                    if (new_c != 0) {
+                    if (new_c != nullptr) {
                         num_new_cnstrs++;
                         new_constraints.push_back(new_c);
                     }
@@ -1537,7 +1528,7 @@ class fm_tactic : public tactic {
                         c->m_dead = true;
                         expr * new_f = to_expr(*c);
                         TRACE("fm_bug", tout << "asserting...\n" << mk_ismt2_pp(new_f, m) << "\nnew_dep: " << c->m_dep << "\n";);
-                        m_new_goal->assert_expr(new_f, 0, c->m_dep);
+                        m_new_goal->assert_expr(new_f, nullptr, c->m_dep);
                     }
                 }
             }
@@ -1551,20 +1542,15 @@ class fm_tactic : public tactic {
         }
         
         void checkpoint() {
-            cooperate("fm");
-            if (m_cancel)
-                throw tactic_exception(TACTIC_CANCELED_MSG);
+            if (m.canceled())
+                throw tactic_exception(m.limit().get_cancel_msg());
             if (memory::get_allocation_size() > m_max_memory)
                 throw tactic_exception(TACTIC_MAX_MEMORY_MSG);
         }
         
-        virtual void operator()(goal_ref const & g, 
-                                goal_ref_buffer & result, 
-                                model_converter_ref & mc, 
-                                proof_converter_ref & pc,
-                                expr_dependency_ref & core) {
+        void operator()(goal_ref const & g, 
+                        goal_ref_buffer & result) {
             SASSERT(g->is_well_sorted());
-            mc = 0; pc = 0; core = 0;
             tactic_report report("fm", *g);
             fail_if_proof_generation("fm", g);
             m_produce_models = g->models_enabled();
@@ -1580,7 +1566,7 @@ class fm_tactic : public tactic {
             
             if (m_inconsistent) {
                 m_new_goal->reset();
-                m_new_goal->assert_expr(m.mk_false(), 0, m_inconsistent_core);
+                m_new_goal->assert_expr(m.mk_false(), nullptr, m_inconsistent_core);
             }
             else {
                 TRACE("fm", display(tout););
@@ -1604,7 +1590,7 @@ class fm_tactic : public tactic {
                         eliminated++;
                     if (m_inconsistent) {
                         m_new_goal->reset();
-                        m_new_goal->assert_expr(m.mk_false(), 0, m_inconsistent_core);
+                        m_new_goal->assert_expr(m.mk_false(), nullptr, m_inconsistent_core);
                         break;
                     }
                 }
@@ -1612,7 +1598,7 @@ class fm_tactic : public tactic {
                 report_tactic_progress(":fm-cost", m_counter);
                 if (!m_inconsistent) {
                     copy_remaining();
-                    mc = m_mc.get();
+                    m_new_goal->add(concat(g->mc(), m_mc.get()));
                 }
             }
             reset_constraints();
@@ -1652,20 +1638,20 @@ public:
         m_imp = alloc(imp, m, p);
     }
 
-    virtual tactic * translate(ast_manager & m) {
+    tactic * translate(ast_manager & m) override {
         return alloc(fm_tactic, m, m_params);
     }
 
-    virtual ~fm_tactic() {
+    ~fm_tactic() override {
         dealloc(m_imp);
     }
 
-    virtual void updt_params(params_ref const & p) {
+    void updt_params(params_ref const & p) override {
         m_params = p;
         m_imp->updt_params(p);
     }
 
-    virtual void collect_param_descrs(param_descrs & r) {
+    void collect_param_descrs(param_descrs & r) override {
         insert_produce_models(r);
         insert_max_memory(r);
         r.insert("fm_real_only", CPK_BOOL, "(default: true) consider only real variables for fourier-motzkin elimination.");
@@ -1676,26 +1662,16 @@ public:
         r.insert("fm_extra", CPK_UINT, "(default: 0) max. increase on the number of inequalities for each FM variable elimination step.");
     }
 
-    virtual void set_cancel(bool f) {
-        if (m_imp)
-            m_imp->set_cancel(f);
-    }
 
-    virtual void cleanup() {
+    void cleanup() override {
         imp * d = alloc(imp, m_imp->m, m_params);
-        #pragma omp critical (tactic_cancel)
-        {
-            std::swap(d, m_imp);
-        }
+        std::swap(d, m_imp);        
         dealloc(d);
     }
 
-    virtual void operator()(goal_ref const & in, 
-                            goal_ref_buffer & result, 
-                            model_converter_ref & mc, 
-                            proof_converter_ref & pc,
-                            expr_dependency_ref & core) {
-        (*m_imp)(in, result, mc, pc, core);
+    void operator()(goal_ref const & in, 
+                    goal_ref_buffer & result) override {
+        (*m_imp)(in, result);
     }
 };
 
