@@ -24,7 +24,7 @@ Revision History:
 #ifndef FM_TACTIC_H_
 #define FM_TACTIC_H_
 
-#include"params.h"
+#include "util/params.h"
 class ast_manager;
 class tactic;
 
