@@ -20,7 +20,7 @@ Notes:
 #ifndef LIA2CARD_TACTIC_H_
 #define LIA2CARD_TACTIC_H_
 
-#include"params.h"
+#include "util/params.h"
 class ast_manager;
 class tactic;
 
