@@ -19,7 +19,7 @@ Revision History:
 #ifndef LIA2PB_TACTIC_H_
 #define LIA2PB_TACTIC_H_
 
-#include"params.h"
+#include "util/params.h"
 class ast_manager;
 class tactic;
 
