@@ -20,7 +20,7 @@ Notes:
 #ifndef NLA2BV_TACTIC_H_
 #define NLA2BV_TACTIC_H_
 
-#include"params.h"
+#include "util/params.h"
 class ast_manager;
 class tactic;
 
