@@ -21,7 +21,7 @@ Revision History:
 #ifndef NORMALIZE_BOUNDS_TACTIC_H_
 #define NORMALIZE_BOUNDS_TACTIC_H_
 
-#include"params.h"
+#include "util/params.h"
 class ast_manager;
 class tactic;
 
