@@ -33,7 +33,7 @@ Notes:
 #ifndef PROPAGATE_INEQS_TACTIC_H_
 #define PROPAGATE_INEQS_TACTIC_H_
 
-#include"params.h"
+#include "util/params.h"
 class ast_manager;
 class tactic;
 
