@@ -30,14 +30,13 @@ Author:
 Revision History:
 
 --*/
-#include"tactical.h"
-#include"th_rewriter.h"
-#include"extension_model_converter.h"
-#include"filter_model_converter.h"
-#include"arith_decl_plugin.h"
-#include"expr_substitution.h"
-#include"dec_ref_util.h"
-#include"ast_smt2_pp.h"
+#include "tactic/tactical.h"
+#include "ast/rewriter/th_rewriter.h"
+#include "tactic/generic_model_converter.h"
+#include "ast/arith_decl_plugin.h"
+#include "ast/expr_substitution.h"
+#include "util/dec_ref_util.h"
+#include "ast/ast_smt2_pp.h"
 
 class recover_01_tactic : public tactic {
     struct imp {
@@ -66,14 +65,11 @@ class recover_01_tactic : public tactic {
             updt_params_core(p);
         }
         
-        void set_cancel(bool f) {
-            m_rw.set_cancel(f);
-        }
 
         bool save_clause(expr * c) {
             if (!m.is_or(c))
                 return false;
-            func_decl * x = 0;
+            func_decl * x = nullptr;
             app * cls   = to_app(c);
             if (cls->get_num_args() <= 1 || cls->get_num_args() >= m_cls_max_size)
                 return false;
@@ -87,7 +83,7 @@ class recover_01_tactic : public tactic {
                 else if (m.is_not(lit, arg) && is_uninterp_const(arg)) {
                     // negative literal
                 }
-                else if (x == 0 && m.is_eq(lit, lhs, rhs)) {
+                else if (x == nullptr && m.is_eq(lit, lhs, rhs)) {
                     // x = k  literal
                     if (is_uninterp_const(lhs) && m_util.is_numeral(rhs)) {
                         x = to_app(lhs)->get_decl();
@@ -104,7 +100,7 @@ class recover_01_tactic : public tactic {
                 }
             }
             
-            if (x != 0) {
+            if (x != nullptr) {
                 var2clauses::obj_map_entry * entry = m_var2clauses.insert_if_not_there2(x, ptr_vector<app>());
                 if (entry->get_data().m_value.empty() || entry->get_data().m_value.back()->get_num_args() == cls->get_num_args()) {
                     entry->get_data().m_value.push_back(cls);
@@ -115,8 +111,7 @@ class recover_01_tactic : public tactic {
         }
         
         // temporary fields used by operator() and process
-        extension_model_converter * mc1;
-        filter_model_converter *    mc2;
+        generic_model_converter *   gmc;
         expr_substitution *         subst;
         goal_ref                    new_goal;
         obj_map<expr, expr *>       bool2int;
@@ -137,7 +132,7 @@ class recover_01_tactic : public tactic {
                     }
                 }
             }
-            return 0;
+            return nullptr;
         }
         
         // Find coeff (the k of literal (x = k)) of clause cls.
@@ -202,14 +197,14 @@ class recover_01_tactic : public tactic {
             SASSERT(is_uninterp_const(atom));
             expr * var;
             if (!bool2int.find(atom, var)) {
-                var = m.mk_fresh_const(0, m_util.mk_int());
+                var = m.mk_fresh_const(nullptr, m_util.mk_int());
                 new_goal->assert_expr(m_util.mk_le(m_util.mk_numeral(rational(0), true), var));
                 new_goal->assert_expr(m_util.mk_le(var, m_util.mk_numeral(rational(1), true)));
                 expr * bool_def = m.mk_eq(var, m_util.mk_numeral(rational(1), true));
                 subst->insert(atom, bool_def);
                 if (m_produce_models) {
-                    mc2->insert(to_app(var)->get_decl());
-                    mc1->insert(to_app(atom)->get_decl(), bool_def);
+                    gmc->hide(var);
+                    gmc->add(to_app(atom)->get_decl(), bool_def);
                 }
                 m.inc_ref(atom);
                 m.inc_ref(var);
@@ -228,7 +223,7 @@ class recover_01_tactic : public tactic {
             if (clauses.size() < expected_num_clauses) // using < instead of != because we tolerate duplicates
                 return false;
             app * zero_cls = find_zero_cls(x, clauses);
-            if (zero_cls == 0)
+            if (zero_cls == nullptr)
                 return false;
             
             buffer<bool>     found; // marks which idx were found
@@ -291,21 +286,18 @@ class recover_01_tactic : public tactic {
             TRACE("recover_01", tout << x->get_name() << " --> " << mk_ismt2_pp(x_def, m) << "\n";);
             subst->insert(m.mk_const(x), x_def);
             if (m_produce_models) {
-                mc1->insert(x, x_def);
+                gmc->add(x, x_def);
             }
             return true;
         }
     
         void operator()(goal_ref const & g, 
-                        goal_ref_buffer & result, 
-                        model_converter_ref & mc, 
-                        proof_converter_ref & pc,
-                        expr_dependency_ref & core) {
+                        goal_ref_buffer & result) {
             SASSERT(g->is_well_sorted());
             fail_if_proof_generation("recover-01", g);
             fail_if_unsat_core_generation("recover-01", g);
             m_produce_models      = g->models_enabled();
-            mc = 0; pc = 0; core = 0; result.reset();
+            result.reset();
             tactic_report report("recover-01", *g);
             
             bool saved = false;
@@ -313,7 +305,9 @@ class recover_01_tactic : public tactic {
             SASSERT(new_goal->depth() == g->depth());
             SASSERT(new_goal->prec() == g->prec());
             new_goal->inc_depth();
-            
+            new_goal->add(g->mc());
+            new_goal->add(g->pc());
+
             unsigned sz = g->size();
             for (unsigned i = 0; i < sz; i++) {
                 expr * f = g->form(i);
@@ -331,9 +325,8 @@ class recover_01_tactic : public tactic {
             }
             
             if (m_produce_models) {
-                mc1 = alloc(extension_model_converter, m);
-                mc2 = alloc(filter_model_converter, m);
-                mc  = concat(mc2, mc1);
+                gmc = alloc(generic_model_converter, m, "recover_01");
+                new_goal->add(gmc);
             }
             
             dec_ref_key_values(m, bool2int);
@@ -342,25 +335,20 @@ class recover_01_tactic : public tactic {
             bool recovered = false;
             expr_substitution _subst(m);
             subst = &_subst;
-            var2clauses::iterator it  = m_var2clauses.begin();
-            var2clauses::iterator end = m_var2clauses.end();
-            for (; it != end; ++it) {
-                if (process(it->m_key, it->m_value)) {
+            for (auto& kv : m_var2clauses) {
+                if (process(kv.m_key, kv.m_value)) {
                     recovered = true;
                     counter++;
                 }
                 else {
-                    ptr_vector<app>::iterator it2   = it->m_value.begin();
-                    ptr_vector<app>::iterator end2  = it->m_value.end();
-                    for (; it2 != end2; ++it2) {
-                        new_goal->assert_expr(*it2);
+                    for (app* a : kv.m_value) {
+                        new_goal->assert_expr(a);
                     }
                 }
             }
             
             if (!recovered) {
                 result.push_back(g.get());
-                mc = 0;
                 return;
             }
             
@@ -393,51 +381,39 @@ public:
         m_imp = alloc(imp, m, p);
     }
 
-    virtual tactic * translate(ast_manager & m) {
+    tactic * translate(ast_manager & m) override {
         return alloc(recover_01_tactic, m, m_params);
     }
     
-    virtual ~recover_01_tactic() {
+    ~recover_01_tactic() override {
         dealloc(m_imp);
     }
 
-    virtual void updt_params(params_ref const & p) {
+    void updt_params(params_ref const & p) override {
         m_params = p;
         m_imp->updt_params(p);
     }
 
-    virtual void collect_param_descrs(param_descrs & r) { 
+    void collect_param_descrs(param_descrs & r) override {
         th_rewriter::get_param_descrs(r);
         r.insert("recover_01_max_bits", CPK_UINT, "(default: 10) maximum number of bits to consider in a clause.");
     }
 
     void operator()(goal_ref const & g, 
-                    goal_ref_buffer & result, 
-                    model_converter_ref & mc, 
-                    proof_converter_ref & pc,
-                    expr_dependency_ref & core) {
+                    goal_ref_buffer & result) override {
         try {
-            (*m_imp)(g, result, mc, pc, core);
+            (*m_imp)(g, result);
         }
         catch (rewriter_exception & ex) {
             throw tactic_exception(ex.msg());
         }
     }
     
-    virtual void cleanup() {
+    void cleanup() override {
         imp * d = alloc(imp, m_imp->m, m_params);
-        #pragma omp critical (tactic_cancel)
-        {
-            std::swap(d, m_imp);
-        }
+        std::swap(d, m_imp);        
         dealloc(d);
     }
-
-protected:
-    virtual void set_cancel(bool f) {
-        if (m_imp)
-            m_imp->set_cancel(f);
-    }
 };
 
 tactic * mk_recover_01_tactic(ast_manager & m, params_ref const & p) {
