@@ -33,7 +33,7 @@ Revision History:
 #ifndef RECOVER_01_TACTIC_H_
 #define RECOVER_01_TACTIC_H_
 
-#include"params.h"
+#include "util/params.h"
 class ast_manager;
 class tactic;
 
