@@ -25,7 +25,7 @@ Notes:
 #ifndef BV1_BLASTER_TACTIC_H_
 #define BV1_BLASTER_TACTIC_H_
 
-#include"params.h"
+#include "util/params.h"
 class ast_manager;
 class tactic;
 
