@@ -16,28 +16,25 @@ Author:
 Revision History:
 
 --*/
-#include"cofactor_elim_term_ite.h"
-#include"mk_simplified_app.h"
-#include"rewriter_def.h"
-#include"cooperate.h"
-#include"for_each_expr.h"
-#include"ast_smt2_pp.h"
-#include"ast_ll_pp.h"
-#include"tactic.h"
+#include "tactic/core/cofactor_elim_term_ite.h"
+#include "ast/rewriter/mk_simplified_app.h"
+#include "ast/rewriter/rewriter_def.h"
+#include "ast/for_each_expr.h"
+#include "ast/ast_smt2_pp.h"
+#include "ast/ast_ll_pp.h"
+#include "tactic/tactic.h"
 
 struct cofactor_elim_term_ite::imp {
     ast_manager &      m;
     params_ref         m_params;
     unsigned long long m_max_memory;
     bool               m_cofactor_equalities;
-    volatile bool      m_cancel;
 
     void checkpoint() { 
-        cooperate("cofactor ite");
         if (memory::get_allocation_size() > m_max_memory)
             throw tactic_exception(TACTIC_MAX_MEMORY_MSG);
-        if (m_cancel)
-            throw tactic_exception(TACTIC_CANCELED_MSG);
+        if (m.canceled())
+            throw tactic_exception(m.limit().get_cancel_msg());
     }
 
     // Collect atoms that contain term if-then-else
@@ -88,7 +85,6 @@ struct cofactor_elim_term_ite::imp {
                 case OP_TRUE:
                 case OP_FALSE:
                 case OP_ITE:
-                case OP_IFF:
                     return;
                 case OP_EQ:
                 case OP_DISTINCT:
@@ -199,7 +195,7 @@ struct cofactor_elim_term_ite::imp {
                     switch (arg->get_kind()) {
                     case AST_VAR:
                     case AST_QUANTIFIER:
-                        // ingore quantifiers
+                        // ignore quantifiers
                         break;
                     case AST_APP:
                         if (to_app(arg)->get_num_args() > 0) {
@@ -220,7 +216,7 @@ struct cofactor_elim_term_ite::imp {
                 break;
             }
         }
-        return 0;
+        return nullptr;
     }
 
     /**
@@ -266,7 +262,7 @@ struct cofactor_elim_term_ite::imp {
                     switch (arg->get_kind()) {
                     case AST_VAR:
                     case AST_QUANTIFIER:
-                        // ingore quantifiers
+                        // ignore quantifiers
                         break;
                     case AST_APP:
                         if (to_app(arg)->get_num_args() > 0) {
@@ -287,7 +283,7 @@ struct cofactor_elim_term_ite::imp {
                 break;
             }
         }
-        expr * best = 0;
+        expr * best = nullptr;
         unsigned best_occs = 0;
         obj_map<expr, unsigned>::iterator it  = occs.begin();
         obj_map<expr, unsigned>::iterator end = occs.end();
@@ -315,9 +311,6 @@ struct cofactor_elim_term_ite::imp {
         r.insert("cofactor_equalities", CPK_BOOL, "(default: true) use equalities to rewrite bodies of ite-expressions. This is potentially expensive.");
     }
 
-    void set_cancel(bool f) {
-        m_cancel = f;
-    }
 
     struct cofactor_rw_cfg : public default_rewriter_cfg {
         ast_manager &         m;
@@ -333,7 +326,7 @@ struct cofactor_elim_term_ite::imp {
         bool                  m_strict_upper;
         app *                 m_upper;
 
-        cofactor_rw_cfg(ast_manager & _m, imp & owner, obj_hashtable<expr> * has_term_ite = 0):
+        cofactor_rw_cfg(ast_manager & _m, imp & owner, obj_hashtable<expr> * has_term_ite = nullptr):
             m(_m),
             m_owner(owner),
             m_has_term_ite(has_term_ite),
@@ -353,13 +346,13 @@ struct cofactor_elim_term_ite::imp {
             if (m.is_not(t)) {
                 m_atom = to_app(t)->get_arg(0);
                 m_sign = true;
-                m_term = 0;
+                m_term = nullptr;
                 // TODO: bounds
             }
             else {
                 m_atom = t;
                 m_sign = false;
-                m_term = 0;
+                m_term = nullptr;
                 expr * lhs;
                 expr * rhs;
                 if (m_owner.m_cofactor_equalities && m.is_eq(t, lhs, rhs)) {
@@ -381,19 +374,19 @@ struct cofactor_elim_term_ite::imp {
         bool rewrite_patterns() const { return false; }
         
         br_status reduce_app(func_decl * f, unsigned num, expr * const * args, expr_ref & result, proof_ref & result_pr) {
-            result_pr = 0;
+            result_pr = nullptr;
             return m_mk_app.mk_core(f, num, args, result);
         }
 
         bool get_subst(expr * s, expr * & t, proof * & pr) {
-            pr = 0;
+            pr = nullptr;
             
             if (s == m_atom) {
                 t = m_sign ? m.mk_false() : m.mk_true();
                 return true;
             }
             
-            if (s == m_term && m_value != 0) {
+            if (s == m_term && m_value != nullptr) {
                 t = m_value;
                 return true;
             }
@@ -410,7 +403,7 @@ struct cofactor_elim_term_ite::imp {
     struct cofactor_rw : rewriter_tpl<cofactor_rw_cfg> {
         cofactor_rw_cfg m_cfg;
     public:
-        cofactor_rw(ast_manager & m, imp & owner, obj_hashtable<expr> * has_term_ite = 0):
+        cofactor_rw(ast_manager & m, imp & owner, obj_hashtable<expr> * has_term_ite = nullptr):
             rewriter_tpl<cofactor_rw_cfg>(m, false, m_cfg),
             m_cfg(m, owner, has_term_ite) {
         }
@@ -448,7 +441,7 @@ struct cofactor_elim_term_ite::imp {
         
         bool get_subst(expr * s, expr * & t, proof * & t_pr) { 
             if (m_candidates.contains(s)) {
-                t_pr = 0;
+                t_pr = nullptr;
 
                 if (m_cache.find(s, t))
                     return true;
@@ -459,7 +452,7 @@ struct cofactor_elim_term_ite::imp {
                 while (true) {
                     // expr * c = m_owner.get_best(curr);
                     expr * c = m_owner.get_first(curr);
-                    if (c == 0) {
+                    if (c == nullptr) {
                         m_cache.insert(s, curr);
                         m_cache_domain.push_back(curr);
                         t = curr.get();
@@ -536,7 +529,7 @@ struct cofactor_elim_term_ite::imp {
             while (true) {
                 expr * c = m_owner.get_best(curr);
                 // expr * c = m_owner.get_first(curr);
-                if (c == 0) {
+                if (c == nullptr) {
                     r = curr.get();
                     return;
                 }
@@ -611,7 +604,7 @@ struct cofactor_elim_term_ite::imp {
                 unsigned num = to_app(t)->get_num_args();
                 for (unsigned i = 0; i < num; i++) {
                     expr * arg = to_app(t)->get_arg(i);
-                    expr * new_arg = 0;
+                    expr * new_arg = nullptr;
                     TRACE("cofactor_bug", tout << "collecting child: " << arg->get_id() << "\n";);
                     m_cache.find(arg, new_arg);
                     SASSERT(new_arg != 0);
@@ -649,7 +642,7 @@ struct cofactor_elim_term_ite::imp {
                 m_cache_domain.push_back(new_t);
                 m_frames.pop_back();
             }
-            expr * result = 0;
+            expr * result = nullptr;
             m_cache.find(t, result);
             r = result;
         }
@@ -659,7 +652,6 @@ struct cofactor_elim_term_ite::imp {
         m(_m),
         m_params(p),
         m_cofactor_equalities(true) {
-        m_cancel = false;
         updt_params(p);
     }
 
@@ -698,18 +690,11 @@ void cofactor_elim_term_ite::operator()(expr * t, expr_ref & r) {
     m_imp->operator()(t, r);
 }
     
-void cofactor_elim_term_ite::set_cancel(bool f) {
-    if (m_imp)
-        m_imp->set_cancel(f);
-}
 
 void cofactor_elim_term_ite::cleanup() {
     ast_manager & m = m_imp->m;    
     imp * d = alloc(imp, m, m_params);
-    #pragma omp critical (tactic_cancel)
-    {
-        std::swap(d, m_imp);
-    }
+    std::swap(d, m_imp);    
     dealloc(d);
 }
 
