@@ -24,7 +24,6 @@ Revision History:
 #include "tactic/tactical.h"
 #include "tactic/core/occf_tactic.h"
 #include "tactic/generic_model_converter.h"
-#include "util/cooperate.h"
 
 class occf_tactic : public tactic {
     struct     imp {
@@ -38,7 +37,6 @@ class occf_tactic : public tactic {
         void checkpoint() {
             if (m.canceled())
                 throw tactic_exception(TACTIC_CANCELED_MSG);
-            cooperate("occf");
         }
 
         bool is_literal(expr * t) const {
