@@ -31,56 +31,14 @@ Notes:
 
 
 --*/
-#include "pb_preprocess_tactic.h"
-#include "tactical.h"
-#include "for_each_expr.h"
-#include "pb_decl_plugin.h"
-#include "th_rewriter.h"
-#include "expr_substitution.h"
-#include "ast_pp.h"
-
-class pb_preproc_model_converter : public model_converter {
-    ast_manager& m;
-    pb_util      pb;
-    expr_ref_vector                  m_refs;
-    svector<std::pair<app*, expr*> > m_const;
-
-public:
-    pb_preproc_model_converter(ast_manager& m):m(m), pb(m), m_refs(m) {}
-
-    virtual void operator()(model_ref & mdl, unsigned goal_idx) {
-        SASSERT(goal_idx == 0);
-        for (unsigned i = 0; i < m_const.size(); ++i) {
-            mdl->register_decl(m_const[i].first->get_decl(), m_const[i].second);
-        }
-    }
-
-    void set_value(expr* e, bool p) {
-        while (m.is_not(e, e)) {
-            p = !p;
-        }
-        SASSERT(is_app(e));
-        set_value_p(to_app(e), p?m.mk_true():m.mk_false());        
-    }
-
-    virtual model_converter * translate(ast_translation & translator) {
-        pb_preproc_model_converter* mc = alloc(pb_preproc_model_converter, translator.to());
-        for (unsigned i = 0; i < m_const.size(); ++i) {
-            mc->set_value_p(translator(m_const[i].first), translator(m_const[i].second));
-        }
-        return mc;
-    }
-
-private:
-    void set_value_p(app* e, expr* v) {
-        SASSERT(e->get_num_args() == 0);  
-        SASSERT(is_uninterp_const(e));
-        m_const.push_back(std::make_pair(e, v));
-        m_refs.push_back(e);
-        m_refs.push_back(v);
-    }
-
-};
+#include "tactic/core/pb_preprocess_tactic.h"
+#include "tactic/tactical.h"
+#include "tactic/generic_model_converter.h"
+#include "ast/for_each_expr.h"
+#include "ast/pb_decl_plugin.h"
+#include "ast/rewriter/th_rewriter.h"
+#include "ast/expr_substitution.h"
+#include "ast/ast_pp.h"
 
 class pb_preprocess_tactic : public tactic {
     struct rec { unsigned_vector pos, neg; rec() { } };
@@ -114,52 +72,52 @@ class pb_preprocess_tactic : public tactic {
         for (unsigned i = 0; i < m_other.size(); ++i) {
             out << "ot " << m_other[i] << ": " << mk_pp(g->form(m_other[i]), m) << "\n";
         }
-                
-        var_map::iterator it  = m_vars.begin();
-        var_map::iterator end = m_vars.end();
-        for (; it != end; ++it) {
-            app* e = it->m_key;
-            unsigned_vector const& pos = it->m_value.pos;
-            unsigned_vector const& neg = it->m_value.neg;
+         
+        for (auto const& kv : m_vars) {
+            app* e = kv.m_key;
+            unsigned_vector const& pos = kv.m_value.pos;
+            unsigned_vector const& neg = kv.m_value.neg;
             out << mk_pp(e, m) << ": ";
-            for (unsigned i = 0; i < pos.size(); ++i) {
-                out << "p: " << pos[i] << " ";
+            for (unsigned p : pos) {
+                out << "p: " << p << " ";
             }
-            for (unsigned i = 0; i < neg.size(); ++i) {
-                out << "n: " << neg[i] << " ";
+            for (unsigned n : neg) {
+                out << "n: " << n << " ";
             }
             out << "\n";
         }
     }
 
+    void set_value(generic_model_converter& mc, expr* e, bool p) {
+        while (m.is_not(e, e)) {
+            p = !p;
+        }
+        SASSERT(is_app(e));
+        mc.add(to_app(e), p?m.mk_true():m.mk_false());        
+    }
+
+
 public:
     pb_preprocess_tactic(ast_manager& m, params_ref const& p = params_ref()): 
         m(m), pb(m), m_r(m) {}
 
-    virtual ~pb_preprocess_tactic() {}
+    ~pb_preprocess_tactic() override {}
 
-    virtual tactic * translate(ast_manager & m) {
+    tactic * translate(ast_manager & m) override {
         return alloc(pb_preprocess_tactic, m);
     }
     
-    virtual void operator()(
+    void operator()(
         goal_ref const & g, 
-        goal_ref_buffer & result, 
-        model_converter_ref & mc, 
-        proof_converter_ref & pc,
-        expr_dependency_ref & core) {
+        goal_ref_buffer & result) override {
         SASSERT(g->is_well_sorted());
-        pc = 0; core = 0;
 
-        if (g->unsat_core_enabled()) {
-            throw tactic_exception("pb-preprocess does not support cores");
-        }
         if (g->proofs_enabled()) {
             throw tactic_exception("pb-preprocess does not support proofs");
         }
 
-        pb_preproc_model_converter* pp = alloc(pb_preproc_model_converter, m);
-        mc = pp;
+        generic_model_converter* pp = alloc(generic_model_converter, m, "pb-preprocess");
+        g->add(pp);
 
         g->inc_depth();        
         result.push_back(g.get());       
@@ -167,7 +125,7 @@ public:
         // decompose(g);
     }
 
-    bool simplify(goal_ref const& g, pb_preproc_model_converter& mc) {
+    bool simplify(goal_ref const& g, generic_model_converter& mc) {
         reset();
         normalize(g);
         if (g->inconsistent()) {
@@ -202,13 +160,14 @@ public:
         while (it != m_vars.end()) {            
             app * e = it->m_key;
             rec const& r = it->m_value;
+            TRACE("pb", tout << mk_pp(e, m) << " " << r.pos.size() << " " << r.neg.size() << "\n";);
             if (r.pos.empty()) {                
                 replace(r.neg, e, m.mk_false(), g);
-                mc.set_value(e, false);
+                set_value(mc, e, false);
             }
             else if (r.neg.empty()) {
                 replace(r.pos, e, m.mk_true(), g);
-                mc.set_value(e, true);
+                set_value(mc, e, true);
             }
             if (g->inconsistent()) return false;
             ++it;
@@ -249,11 +208,11 @@ public:
             unsigned_vector const& pos = neg?r.neg:r.pos;
             for (unsigned j = 0; j < pos.size(); ++j) {
                 unsigned k = pos[j];
-                if (k == i) continue;
+                if (k == m_ge[i]) continue;
                 if (!to_ge(g->form(k), args2, coeffs2, k2)) continue;
                 if (subsumes(args1, coeffs1, k1, args2, coeffs2, k2)) {
                     IF_VERBOSE(3, verbose_stream() << "replace " << mk_pp(g->form(k), m) << "\n";);
-                    g->update(k, m.mk_true());                    
+                    g->update(k, m.mk_true(), nullptr, m.mk_join(g->dep(m_ge[i]), g->dep(k)));
                     m_progress = true;
                 }
             }
@@ -264,18 +223,15 @@ public:
         return m_progress;
     }
 
-    virtual void set_cancel(bool f) {
+    void updt_params(params_ref const & p) override {
     }
 
-    virtual void updt_params(params_ref const & p) {
-    }
-
-    virtual void cleanup() {
+    void cleanup() override {
     }
 
 private:
 
-    void reset() {
+    void reset() override {
         m_ge.reset();
         m_other.reset();
         m_vars.reset();
@@ -302,7 +258,7 @@ private:
                     args.push_back(negate(to_app(r)->get_arg(j)));
                 }
                 tmp = pb.mk_ge(args.size(), coeffs.c_ptr(), args.c_ptr(), sum - k + rational::one());
-                g->update(i, tmp);
+                g->update(i, tmp, g->pr(i), g->dep(i));
             }
         }
     }
@@ -317,8 +273,8 @@ private:
     }
 
     /**
-       \brief decompose large sums into smaller sums by intoducing
-       auxilary variables.
+       \brief decompose large sums into smaller sums by introducing
+       auxiliary variables.
     */
     void decompose(goal_ref const& g) {
         expr_ref fml1(m), fml2(m);
@@ -334,12 +290,12 @@ private:
                 for (unsigned j = 0; j < cuts.size(); ++j) {
                     unsigned end = cuts[j];
                     fml1 = decompose_cut(a, start, end, cut_args, cut_coeffs); 
-                    g->assert_expr(fml1);
+                    g->assert_expr(fml1, nullptr, g->dep(i));
                     start = end;
                     TRACE("pb", tout << fml1 << "\n";);
                 }
                 fml2 = pb.mk_ge(cut_args.size(), cut_coeffs.c_ptr(), cut_args.c_ptr(), pb.get_k(e));
-                g->update(i, fml2);
+                g->update(i, fml2, nullptr, g->dep(i));
                 TRACE("pb", tout << fml2 << "\n";);
             }
         }
@@ -513,7 +469,7 @@ private:
 
     // Implement very special case of resolution.
     
-    void resolve(pb_preproc_model_converter& mc, unsigned idx1, 
+    void resolve(generic_model_converter& mc, unsigned idx1, 
                  unsigned_vector const& positions, app* e, bool pos, goal_ref const& g) {
         if (positions.size() != 1) return;
         unsigned idx2 = positions[0];
@@ -562,7 +518,7 @@ private:
             else {
                 args2[j] = m.mk_true();
             }
-            mc.set_value(arg, j != min_index);
+            set_value(mc, arg, j != min_index);
         }
         
         tmp1 = pb.mk_ge(args2.size(), coeffs2.c_ptr(), args2.c_ptr(), k2);
@@ -580,8 +536,12 @@ private:
                    verbose_stream() << "resolve: " << mk_pp(fml1, m) << "\n" << mk_pp(fml2, m) << "\n" << tmp1 << "\n";
                    verbose_stream() << "to\n" << mk_pp(fml2, m) << " -> " << tmp2 << "\n";);
 
-        g->update(idx1, m.mk_true()); // proof & dependencies
-        g->update(idx2, tmp2);        // proof & dependencies
+        TRACE("pb",
+              tout << "resolve: " << mk_pp(fml1, m) << "\n" << mk_pp(fml2, m) << "\n" << tmp1 << "\n";
+              tout << "to\n" << mk_pp(fml2, m) << " -> " << tmp2 << "\n";);
+
+        g->update(idx2, tmp2, nullptr, m.mk_join(g->dep(idx1), g->dep(idx2)));
+        g->update(idx1, m.mk_true(), nullptr, nullptr);
         m_progress = true;
         //IF_VERBOSE(0, if (!g->inconsistent()) display_annotation(verbose_stream(), g););
     }
@@ -637,19 +597,23 @@ private:
         for (unsigned i = 0; i < positions.size(); ++i) {
             unsigned idx = positions[i];
             expr_ref f(m);
+            proof_ref new_pr(m);
             f = g->form(idx);
             if (!m.is_true(f)) {
-                m_r(f, tmp);
+                m_r(f, tmp, new_pr);
                 if (tmp != f) {
                     TRACE("pb", tout << mk_pp(f, m) << " -> " << tmp 
                           << " by " << mk_pp(e, m) << " |-> " << mk_pp(v, m) << "\n";);
                     IF_VERBOSE(3, verbose_stream() << "replace " << mk_pp(f, m) << " -> " << tmp << "\n";);
-                    g->update(idx, tmp); // proof & dependencies.
+                    if (g->proofs_enabled()) {
+                        new_pr = m.mk_modus_ponens(g->pr(idx), new_pr);
+                    }
+                    g->update(idx, tmp, new_pr, g->dep(idx)); 
                     m_progress = true;
                 }
             }
         }
-        m_r.set_substitution(0);
+        m_r.set_substitution(nullptr);
     }
 
     bool subsumes(expr_ref_vector const& args1, 
