@@ -19,8 +19,8 @@ Notes:
 #ifndef SIMPLIFY_TACTIC_H_
 #define SIMPLIFY_TACTIC_H_
 
-#include"tactic.h"
-#include"tactical.h"
+#include "tactic/tactic.h"
+#include "tactic/tactical.h"
 
 class simplify_tactic : public tactic {
     struct     imp;
@@ -28,25 +28,21 @@ class simplify_tactic : public tactic {
     params_ref m_params;
 public:
     simplify_tactic(ast_manager & m, params_ref const & ref = params_ref());
-    virtual ~simplify_tactic();
+    ~simplify_tactic() override;
+
+    void updt_params(params_ref const & p) override;
 
-    virtual void updt_params(params_ref const & p);
     static void get_param_descrs(param_descrs & r);
-    virtual void collect_param_descrs(param_descrs & r) { get_param_descrs(r); }
     
-    virtual void operator()(goal_ref const & in, 
-                            goal_ref_buffer & result, 
-                            model_converter_ref & mc, 
-                            proof_converter_ref & pc,
-                            expr_dependency_ref & core);
+    void collect_param_descrs(param_descrs & r) override { get_param_descrs(r); }
     
-    virtual void cleanup();
+    void operator()(goal_ref const & in, goal_ref_buffer & result) override;
+    
+    void cleanup() override;
 
     unsigned get_num_steps() const;
 
-    virtual tactic * translate(ast_manager & m) { return alloc(simplify_tactic, m, m_params); }
-protected:
-    virtual void set_cancel(bool f);
+    tactic * translate(ast_manager & m) override { return alloc(simplify_tactic, m, m_params); }
 
 };
 
