@@ -20,7 +20,7 @@ Notes:
 #ifndef TSEITIN_CNF_TACTIC_H_
 #define TSEITIN_CNF_TACTIC_H_
 
-#include"params.h"
+#include "util/params.h"
 class ast_manager;
 class tactic;
 
