@@ -24,7 +24,7 @@ Revision History:
 #ifndef EQUIV_PROOF_CONVERTER_H_
 #define EQUIV_PROOF_CONVERTER_H_
 
-#include "replace_proof_converter.h"
+#include "tactic/replace_proof_converter.h"
 
 class equiv_proof_converter : public proof_converter {
     ast_manager&            m;
@@ -33,13 +33,13 @@ public:
 
     equiv_proof_converter(ast_manager& m): m(m), m_replace(m) {}
 
-    virtual ~equiv_proof_converter() {}
+    ~equiv_proof_converter() override {}
 
-    virtual void operator()(ast_manager & m, unsigned num_source, proof * const * source, proof_ref & result) {
-        m_replace(m, num_source, source, result);
+    proof_ref operator()(ast_manager & m, unsigned num_source, proof * const * source) override {
+        return m_replace(m, num_source, source);
     }
 
-    virtual proof_converter * translate(ast_translation & translator) {
+    proof_converter * translate(ast_translation & translator) override {
         return m_replace.translate(translator);
     }
 
@@ -47,6 +47,7 @@ public:
 
     ast_manager& get_manager() { return m; }
 
+    void display(std::ostream & out) override {}
 };
 
 #endif
