@@ -78,9 +78,9 @@ public:
         m_rewriter.pop(n);
     }
 
-    lbool check_sat_core(unsigned num_assumptions, expr * const * assumptions) override {
+    lbool check_sat_core2(unsigned num_assumptions, expr * const * assumptions) override {
         m_solver->updt_params(get_params());
-        return m_solver->check_sat(num_assumptions, assumptions);
+        return m_solver->check_sat_core(num_assumptions, assumptions);
     }
 
     void updt_params(params_ref const & p) override { solver::updt_params(p); m_solver->updt_params(p);  }
@@ -178,7 +178,13 @@ public:
         return r;
     }
 
+    void get_levels(ptr_vector<expr> const& vars, unsigned_vector& depth) override {
+        m_solver->get_levels(vars, depth);
+    }
 
+    expr_ref_vector get_trail() override {
+        return m_solver->get_trail();
+    }
 
     unsigned get_num_assertions() const override {
         return m_solver->get_num_assertions();
