@@ -87,7 +87,7 @@ class fpa2bv_tactic : public tactic {
                         // Inject auxiliary lemmas that fix e to the one and only NaN value,
                         // that is (= e (fp #b0 #b1...1 #b0...01)), so that the value propagation
                         // has a value to propagate.
-                        expr * sgn, *sig, *exp;
+                        expr_ref sgn(m), sig(m), exp(m);
                         m_conv.split_fp(new_curr, sgn, exp, sig);
                         result.back()->assert_expr(m.mk_eq(sgn, m_conv.bu().mk_numeral(0, 1)));
                         result.back()->assert_expr(m.mk_eq(exp, m_conv.bu().mk_bv_neg(m_conv.bu().mk_numeral(1, m_conv.bu().get_bv_size(exp)))));
