@@ -20,7 +20,7 @@ Notes:
 #ifndef QFFP_TACTIC_H_
 #define QFFP_TACTIC_H_
 
-#include"params.h"
+#include "util/params.h"
 class ast_manager;
 class tactic;
 
