@@ -18,7 +18,7 @@ Revision History:
 #ifndef GOAL_NUM_OCCURS_H_
 #define GOAL_NUM_OCCURS_H_
 
-#include"num_occurs.h"
+#include "ast/num_occurs.h"
 
 class goal;
 
