@@ -19,8 +19,8 @@ Revision History:
 #ifndef GOAL_SHARED_OCCS_H_
 #define GOAL_SHARED_OCCS_H_
 
-#include"goal.h"
-#include"shared_occs.h"
+#include "tactic/goal.h"
+#include "ast/shared_occs.h"
 
 /**
    \brief Functor for computing the set of shared occurrences in a goal.
