@@ -20,7 +20,7 @@ Notes:
 #ifndef SLS_POWERS_H_
 #define SLS_POWERS_H_
 
-#include"mpz.h"
+#include "util/mpz.h"
 
 class powers : public u_map<mpz*> {
     unsynch_mpz_manager & m;
@@ -46,4 +46,4 @@ public:
     }
 };  
 
-#endif
\ No newline at end of file
+#endif
