@@ -21,13 +21,13 @@ Notes:
 #define SLS_TRACKER_H_
 
 #include<math.h>
-#include"for_each_expr.h"
-#include"ast_smt2_pp.h"
-#include"bv_decl_plugin.h"
-#include"model.h"
+#include "ast/for_each_expr.h"
+#include "ast/ast_smt2_pp.h"
+#include "ast/bv_decl_plugin.h"
+#include "model/model.h"
 
-#include"sls_params.hpp"
-#include"sls_powers.h"
+#include "tactic/sls/sls_params.hpp"
+#include "tactic/sls/sls_powers.h"
 
 class sls_tracker {
     ast_manager         & m_manager;
@@ -40,8 +40,25 @@ class sls_tracker {
     mpz                   m_zero, m_one, m_two;
         
     struct value_score { 
-    value_score() : m(0), value(unsynch_mpz_manager::mk_z(0)), score(0.0), score_prune(0.0), has_pos_occ(0), has_neg_occ(0), distance(0), touched(1) {};
+    value_score() : m(nullptr), value(unsynch_mpz_manager::mk_z(0)), score(0.0), score_prune(0.0), has_pos_occ(0), has_neg_occ(0), distance(0), touched(1) {};
+        value_score(value_score && other) :
+            m(other.m),
+            value(std::move(other.value)),
+            score(other.score),
+            score_prune(other.score_prune),
+            has_pos_occ(other.has_pos_occ),
+            has_neg_occ(other.has_neg_occ),
+            distance(other.distance),
+            touched(other.touched) {}
         ~value_score() { if (m) m->del(value); }
+        void operator=(value_score && other) {
+            this->~value_score();
+            new (this) value_score(std::move(other));
+        }
+        value_score& operator=(value_score& other) {
+            UNREACHABLE();
+            return *this;
+        }
         unsynch_mpz_manager * m;
         mpz value;
         double score;
@@ -50,15 +67,6 @@ class sls_tracker {
         unsigned has_neg_occ;
         unsigned distance; // max distance from any root
         unsigned touched;
-        value_score & operator=(const value_score & other) {
-            SASSERT(m == 0 || m == other.m);
-            if (m) m->set(value, 0); else m = other.m;
-            m->set(value, other.value);
-            score = other.score;
-            distance = other.distance;
-            touched = other.touched;
-            return *this;
-        }
     };
 
 public:
@@ -68,7 +76,7 @@ private:
     typedef obj_map<expr, value_score> scores_type;    
     typedef obj_map<expr, ptr_vector<expr> > uplinks_type;    
     typedef obj_map<expr, ptr_vector<func_decl> > occ_type;
-    obj_hashtable<expr>	  m_top_expr;
+    obj_hashtable<expr>      m_top_expr;
     scores_type           m_scores;
     uplinks_type          m_uplinks;
     entry_point_type      m_entry_points;
@@ -85,11 +93,11 @@ private:
     unsigned              m_touched;
     double                m_scale_unsat;
     unsigned              m_paws_init;
-    obj_map<expr, unsigned>	m_where_false;
-    expr**					m_list_false;
+    obj_map<expr, unsigned>    m_where_false;
+    expr**                    m_list_false;
     unsigned              m_track_unsat;
     obj_map<expr, unsigned> m_weights;
-    double				  m_top_sum;
+    double                  m_top_sum;
     obj_hashtable<expr>   m_temp_seen;
 
 public:    
@@ -294,7 +302,7 @@ public:
         if (!m_scores.contains(n)) {
             value_score vs;
             vs.m = & m_mpz_manager;
-            m_scores.insert(n, vs);
+            m_scores.insert(n, std::move(vs));
         }
 
         // Update uplinks
@@ -450,7 +458,7 @@ public:
             m_list_false = new expr*[sz];
             for (unsigned i = 0; i < sz; i++)
             {
-        	    if (m_mpz_manager.eq(get_value(as[i]), m_zero))
+                if (m_mpz_manager.eq(get_value(as[i]), m_zero))
                     break_assertion(as[i]);
             }
         }
@@ -462,9 +470,9 @@ public:
 
             // initialize weights
             if (!m_weights.contains(e))
-        		m_weights.insert(e, m_paws_init);
+                m_weights.insert(e, m_paws_init);
 
-            // positive/negative occurences used for early pruning
+            // positive/negative occurrences used for early pruning
             setup_occs(as[i]);
         }
 
@@ -539,7 +547,7 @@ public:
                     rational r_val;
                     unsigned bv_sz;
                     m_bv_util.is_numeral(val, r_val, bv_sz);
-                    mpq q = r_val.to_mpq();
+                    const mpq& q = r_val.to_mpq();
                     SASSERT(m_mpz_manager.is_one(q.denominator()));
                     set_value(fd, q.numerator());
                 }
@@ -630,7 +638,7 @@ public:
         if (m_bv_util.is_bv_sort(s))
             return get_random_bv(s);
         else if (m_manager.is_bool(s))
-            return get_random_bool();
+            return m_mpz_manager.dup(get_random_bool());
         else
             NOT_IMPLEMENTED_YET(); // This only works for bit-vectors for now.
     }    
@@ -653,9 +661,7 @@ public:
         TRACE("sls", tout << "Abandoned model:" << std::endl; show_model(tout); );
 
         for (entry_point_type::iterator it = m_entry_points.begin(); it != m_entry_points.end(); it++) {
-            mpz temp = m_zero;
-            set_value(it->m_value, temp);
-            m_mpz_manager.del(temp);
+            set_value(it->m_value, m_zero);
         }
     }              
 
@@ -931,7 +937,7 @@ public:
             rational q;
             if (!m_bv_util.is_numeral(n, q, bv_sz))
                 NOT_IMPLEMENTED_YET();
-            mpq temp = q.to_mpq();
+            const mpq& temp = q.to_mpq();
             SASSERT(m_mpz_manager.is_one(temp.denominator()));
             m_mpz_manager.set(result, temp.numerator());
         }
@@ -997,7 +1003,7 @@ public:
     }
 
     ptr_vector<func_decl> & get_unsat_constants_walksat(expr * e) {
-            if (!e || m_temp_constants.size())
+            if (!e || !m_temp_constants.empty())
                 return m_temp_constants;
             ptr_vector<func_decl> const & this_decls = m_constants_occ.find(e);
             unsigned sz = this_decls.size();
@@ -1032,14 +1038,13 @@ public:
             if (m_mpz_manager.neq(get_value(as[0]), m_one))
                 return as[0];
             else
-                return 0;
+                return nullptr;
         }
         m_temp_constants.reset();
 
         unsigned pos = -1;
         if (m_ucb)
         {
-            value_score vscore;
             double max = -1.0;
             // Andreas: Commented things here might be used for track_unsat data structures as done in SLS for SAT. But seems to have no benefit.
             /* for (unsigned i = 0; i < m_where_false.size(); i++) {
@@ -1048,7 +1053,7 @@ public:
                 expr * e = as[i];
                 if (m_mpz_manager.neq(get_value(e), m_one))
                 {
-                    vscore = m_scores.find(e);
+                    value_score & vscore = m_scores.find(e);
                     // Andreas: Select the assertion with the greatest ucb score. Potentially add some noise.
                     // double q = vscore.score + m_ucb_constant * sqrt(log((double)m_touched) / vscore.touched);
                     double q = vscore.score + m_ucb_constant * sqrt(log((double)m_touched) / vscore.touched) + m_ucb_noise * get_random_uint(8); 
@@ -1056,7 +1061,7 @@ public:
                 }
             }
             if (pos == static_cast<unsigned>(-1))
-                return 0;
+                return nullptr;
 
             m_touched++;
             m_scores.find(as[pos]).touched++;
@@ -1075,9 +1080,9 @@ public:
 
             unsigned cnt_unsat = 0;
             for (unsigned i = 0; i < sz; i++)
-                if (m_mpz_manager.neq(get_value(as[i]), m_one) && (get_random_uint(16) % ++cnt_unsat == 0)) pos = i;	
+                if (m_mpz_manager.neq(get_value(as[i]), m_one) && (get_random_uint(16) % ++cnt_unsat == 0)) pos = i;    
             if (pos == static_cast<unsigned>(-1))
-                return 0;
+                return nullptr;
         }
         
         m_last_pos = pos;
@@ -1087,15 +1092,15 @@ public:
     expr * get_new_unsat_assertion(ptr_vector<expr> const & as) {
         unsigned sz = as.size();
         if (sz == 1)
-            return 0;
+            return nullptr;
         m_temp_constants.reset();
         
         unsigned cnt_unsat = 0, pos = -1;
         for (unsigned i = 0; i < sz; i++)
-            if ((i != m_last_pos) && m_mpz_manager.neq(get_value(as[i]), m_one) && (get_random_uint(16) % ++cnt_unsat == 0)) pos = i;	
+            if ((i != m_last_pos) && m_mpz_manager.neq(get_value(as[i]), m_one) && (get_random_uint(16) % ++cnt_unsat == 0)) pos = i;
 
         if (pos == static_cast<unsigned>(-1))
-            return 0;
+            return nullptr;
         return as[pos];
     }
 };
