@@ -19,10 +19,14 @@ Notes:
 #ifndef QFAUFLIA_TACTIC_H_
 #define QFAUFLIA_TACTIC_H_
 
-#include"params.h"
+#include "util/params.h"
 class ast_manager;
 class tactic;
 
 tactic * mk_qfauflia_tactic(ast_manager & m, params_ref const & p = params_ref());
 
+/*
+  ADD_TACTIC("qfauflia",  "builtin strategy for solving QF_AUFLIA problems.", "mk_qfauflia_tactic(m, p)")
+*/
+
 #endif
