@@ -19,10 +19,14 @@ Notes:
 #ifndef QFIDL_TACTIC_H_
 #define QFIDL_TACTIC_H_
 
-#include"params.h"
+#include "util/params.h"
 class ast_manager;
 class tactic;
 
 tactic * mk_qfidl_tactic(ast_manager & m, params_ref const & p = params_ref());
 
+/*
+    ADD_TACTIC("qfidl", "builtin strategy for solving QF_IDL problems.", "mk_qfidl_tactic(m, p)")
+*/
+
 #endif
