@@ -19,10 +19,12 @@ Notes:
 #ifndef QFLIA_TACTIC_H_
 #define QFLIA_TACTIC_H_
 
-#include"params.h"
+#include "util/params.h"
 class ast_manager;
 class tactic;
 
+tactic * mk_preamble_tactic(ast_manager& m);
+
 tactic * mk_qflia_tactic(ast_manager & m, params_ref const & p = params_ref());
 /*
   ADD_TACTIC("qflia", "builtin strategy for solving QF_LIA problems.", "mk_qflia_tactic(m, p)")
