@@ -19,12 +19,10 @@ Notes:
 #ifndef UFBV_TACTIC_H_
 #define UFBV_TACTIC_H_
 
-#include"params.h"
+#include "util/params.h"
 class ast_manager;
 class tactic;
 
-tactic * mk_ufbv_preprocessor_tactic(ast_manager & m, params_ref const & p = params_ref());
-
 tactic * mk_ufbv_tactic(ast_manager & m, params_ref const & p = params_ref());
 
 /*
