@@ -4,8 +4,8 @@ Copyright (c) 2015 Microsoft Corporation
 
 --*/
 
-#include "arith_eq_solver.h"
-#include "smt_params.h"
+#include "smt/arith_eq_solver.h"
+#include "smt/params/smt_params.h"
 
 typedef rational numeral;
 typedef vector<numeral> row;
