@@ -20,7 +20,7 @@ Revision History:
 #ifndef EXPR_DELTA_H_
 #define EXPR_DELTA_H_
 
-#include "ast.h"
+#include "ast/ast.h"
 
 class expr_delta {
     ast_manager&    m_manager;
@@ -33,7 +33,7 @@ public:
     
     //
     // Create the n'th delta in dfs mode.
-    // resturn 'true' if a delta was obtained.
+    // return 'true' if a delta was obtained.
     //
     bool delta_dfs(unsigned n, expr_ref_vector& result);
 
