@@ -19,14 +19,11 @@ Revision History:
 #ifndef EXPR_RAND_H_
 #define EXPR_RAND_H_
 
-#include"ast.h"
-#include"obj_hashtable.h"
+#include "ast/ast.h"
+#include "util/obj_hashtable.h"
 
 class expr_rand {
     ast_manager& m_manager;
-    unsigned     m_num_vars;
-    unsigned     m_num_apps;
-    unsigned     m_num_nodes;
     unsigned     m_max_steps;
     random_gen   m_random;
 
