@@ -151,7 +151,7 @@ public:
         if (m_data == nullptr) {
             return 0;  
         }
-        return static_cast<unsigned>(reinterpret_cast<size_t *>(m_data)[SIZE_IDX]); 
+        return static_cast<unsigned>(reinterpret_cast<size_t *>(m_data)[ARRAY_SIZE_IDX]); 
     }
     
     bool empty() const { return m_data == nullptr; }
