@@ -17,8 +17,8 @@ Notes:
 #ifndef DICTIONARY_H_
 #define DICTIONARY_H_
 
-#include"map.h"
-#include"symbol.h"
+#include "util/map.h"
+#include "util/symbol.h"
 
 template<typename T>
 class dictionary : public map<symbol, T, symbol_hash_proc, symbol_eq_proc> {
