@@ -20,7 +20,7 @@ Revision History:
 #ifndef MACHINE_H_
 #define MACHINE_H_
 
-#ifdef _AMD64_
+#if defined(__LP64__) || defined(_WIN64)
 #define PTR_ALIGNMENT 3
 #else
 #define PTR_ALIGNMENT 2
