@@ -19,8 +19,8 @@ Revision History:
 #ifndef MPBQI_H_
 #define MPBQI_H_
 
-#include"mpbq.h"
-#include"basic_interval.h"
+#include "util/mpbq.h"
+#include "util/basic_interval.h"
 
 class mpbqi_manager : public basic_interval_manager<mpbq_manager, false> {
     typedef basic_interval_manager<mpbq_manager, false> super;
