@@ -19,7 +19,7 @@ Revision History:
 #ifndef OBJ_PAIR_SET_H_
 #define OBJ_PAIR_SET_H_
 
-#include"chashtable.h"
+#include "util/chashtable.h"
 
 template<typename T1, typename T2>
 class obj_pair_set {
@@ -45,6 +45,12 @@ public:
     bool contains(T1 * t1, T2 * t2) const { return m_set.contains(obj_pair(t1, t2)); }
     bool contains(obj_pair const & p) const { return m_set.contains(p); }
     void reset() { m_set.reset(); }
+    bool empty() const { return m_set.empty(); }
+
+    typedef typename chashtable<obj_pair, hash_proc, eq_proc>::iterator iterator;
+
+    iterator begin() { return m_set.begin(); }
+    iterator end() { return m_set.end(); }
 };
 
 #endif
