@@ -19,7 +19,7 @@ Revision History:
 #ifndef PLUGIN_MANAGER_H_
 #define PLUGIN_MANAGER_H_
 
-#include"util.h"
+#include "util/util.h"
 
 template<typename Plugin>
 class plugin_manager {
@@ -27,7 +27,12 @@ class plugin_manager {
     ptr_vector<Plugin>   m_plugins;
 public:
     ~plugin_manager() {
+        reset();
+    }
+
+    void reset() {
         std::for_each(m_plugins.begin(), m_plugins.end(), delete_proc<Plugin>());
+        release();
     }
 
     /**
@@ -48,11 +53,13 @@ public:
     
     Plugin * get_plugin(family_id fid) const {
         if (fid == null_family_id) {
-            return 0;
+            return nullptr;
         }
         return m_fid2plugins.get(fid, 0);
     }
 
+    ptr_vector<Plugin> const& plugins() const { return m_plugins; }
+
     typename ptr_vector<Plugin>::const_iterator begin() const { 
         return m_plugins.begin(); 
     }
