@@ -53,7 +53,7 @@ public:
 
     rational(mpz const & z) { m().set(m_val, z); }
 
-    rational(double  z) { UNREACHABLE(); }
+    explicit rational(double  z) { UNREACHABLE(); }
     
     explicit rational(char const * v) { m().set(m_val, v); }
 
@@ -93,6 +93,12 @@ public:
     
     void display_decimal(std::ostream & out, unsigned prec, bool truncate = false) const { return m().display_decimal(out, m_val, prec, truncate); }
 
+    void display_smt2(std::ostream & out) const { return m().display_smt2(out, m_val, false); }
+
+    void display_hex(std::ostream & out, unsigned num_bits) const { SASSERT(is_int()); return m().display_hex(out, m_val.numerator(), num_bits); }
+
+    void display_bin(std::ostream & out, unsigned num_bits) const { SASSERT(is_int()); return m().display_bin(out, m_val.numerator(), num_bits); }
+
     bool is_uint64() const { return m().is_uint64(m_val); }
 
     bool is_int64() const { return m().is_int64(m_val); }
@@ -415,6 +421,8 @@ public:
         }
         return num_bits;
     }
+
+    static bool limit_denominator(rational &num, rational const& limit);
 };
 
 inline bool operator!=(rational const & r1, rational const & r2) { 
