@@ -18,6 +18,18 @@ Revision History:
 --*/
 #include "util/rlimit.h"
 #include "util/common_msgs.h"
+#include "util/mutex.h"
+
+
+static DECLARE_MUTEX(g_rlimit_mux);
+
+void initialize_rlimit() {
+    ALLOC_MUTEX(g_rlimit_mux);
+}
+
+void finalize_rlimit() {
+    DEALLOC_MUTEX(g_rlimit_mux);
+}
 
 reslimit::reslimit():
     m_cancel(0),
@@ -32,12 +44,14 @@ uint64_t reslimit::count() const {
 
 bool reslimit::inc() {
     ++m_count;
-    return (m_cancel == 0 && (m_limit == 0 || m_count <= m_limit)) || m_suspend;
+    bool r = (m_cancel == 0 && (m_limit == 0 || m_count <= m_limit)) || m_suspend;
+    return r;
 }
 
 bool reslimit::inc(unsigned offset) {
     m_count += offset;
-    return (m_cancel == 0 && (m_limit == 0 || m_count <= m_limit)) || m_suspend;
+    bool r = (m_cancel == 0 && (m_limit == 0 || m_count <= m_limit)) || m_suspend;
+    return r;
 }
 
 void reslimit::push(unsigned delta_limit) {
@@ -69,48 +83,34 @@ char const* reslimit::get_cancel_msg() const {
 }
 
 void reslimit::push_child(reslimit* r) {
-    #pragma omp critical (reslimit_cancel)
-    {
-        m_children.push_back(r);
-    }
+    lock_guard lock(*g_rlimit_mux);
+    m_children.push_back(r);    
 }
 
 void reslimit::pop_child() {
-    #pragma omp critical (reslimit_cancel)
-    {
-        m_children.pop_back();
-    }
+    lock_guard lock(*g_rlimit_mux);
+    m_children.pop_back();    
 }
 
 void reslimit::cancel() {
-    #pragma omp critical (reslimit_cancel)
-    {
-        set_cancel(m_cancel+1);
-    }
+    lock_guard lock(*g_rlimit_mux);
+    set_cancel(m_cancel+1);    
 }
 
-
 void reslimit::reset_cancel() {
-    #pragma omp critical (reslimit_cancel)
-    {
-        set_cancel(0);
-    }
+    lock_guard lock(*g_rlimit_mux);
+    set_cancel(0);    
 }
 
 void reslimit::inc_cancel() {
-    #pragma omp critical (reslimit_cancel)
-    {
-        set_cancel(m_cancel+1);
-    }
+    lock_guard lock(*g_rlimit_mux);
+    set_cancel(m_cancel+1);    
 }
 
-
 void reslimit::dec_cancel() {
-    #pragma omp critical (reslimit_cancel)
-    {
-        if (m_cancel > 0) {
-            set_cancel(m_cancel-1);
-        }
+    lock_guard lock(*g_rlimit_mux);
+    if (m_cancel > 0) {
+        set_cancel(m_cancel-1);
     }
 }
 
