@@ -16,11 +16,17 @@ Author:
 Revision History:
 
 --*/
-#ifndef RLIMIT_H_
-#define RLIMIT_H_
+#pragma once
 
 #include "util/vector.h"
 
+void initialize_rlimit();
+void finalize_rlimit();
+/*
+  ADD_INITIALIZER('initialize_rlimit();')
+  ADD_FINALIZER('finalize_rlimit();')
+*/
+
 class reslimit {
     volatile unsigned   m_cancel;
     bool            m_suspend;
@@ -71,6 +77,11 @@ public:
         m_suspend = r.m_suspend;
         r.m_suspend = true;
     }
+
+    scoped_suspend_rlimit(reslimit& r, bool do_suspend): m_limit(r) {
+        m_suspend = r.m_suspend;
+        r.m_suspend |= do_suspend;
+    }
     ~scoped_suspend_rlimit() {
         m_limit.m_suspend = m_suspend;
     }
@@ -83,6 +94,3 @@ struct scoped_limits {
     ~scoped_limits() { for (unsigned i = 0; i < m_sz; ++i) m_limit.pop_child(); }
     void push_child(reslimit* lim) { m_limit.push_child(lim); ++m_sz; }
 };
-
-
-#endif
