@@ -20,9 +20,9 @@ Revision History:
 #include<iostream>
 #include "util/scoped_ctrl_c.h"
 
-scoped_ctrl_c * scoped_ctrl_c::g_obj = nullptr;
+static scoped_ctrl_c * g_obj = nullptr;
 
-void scoped_ctrl_c::on_ctrl_c(int) {
+static void on_ctrl_c(int) {
     if (g_obj->m_first) {
         g_obj->m_cancel_eh(CTRL_C_EH_CALLER);
         if (g_obj->m_once) {
