@@ -19,7 +19,7 @@ Revision History:
 #ifndef SCOPED_TIMER_H_
 #define SCOPED_TIMER_H_
 
-#include"event_handler.h"
+#include "util/event_handler.h"
 
 class scoped_timer {
     struct imp;
