@@ -19,7 +19,7 @@ Notes:
 #ifndef SMT2_UTIL_H_
 #define SMT2_UTIL_H_
 
-#include"symbol.h"
+#include "util/symbol.h"
 
 bool is_smt2_simple_symbol_char(char c);
 bool is_smt2_quoted_symbol(char const * s);
