@@ -21,14 +21,14 @@ Revision History:
 
 #include<string.h>
 
-#include"hashtable.h"
-#include"hash.h"
+#include "util/hashtable.h"
+#include "util/hash.h"
 
 struct str_hash_proc { 
     unsigned operator()(char const * s) const { return string_hash(s, static_cast<unsigned>(strlen(s)), 17); } 
 };
 struct str_eq_proc { bool operator()(char const * s1, char const * s2) const { return strcmp(s1, s2) == 0; } }; 
-typedef ptr_hashtable<char, str_hash_proc, str_eq_proc> str_hashtable;
+typedef ptr_hashtable<const char, str_hash_proc, str_eq_proc> str_hashtable;
 
 #endif /* STR_HASHTABLE_H_ */
 
